@@ -44,6 +44,7 @@ if platform_family? 'debian'
     command 'apt-get update'
   end
 
+  ENV['DEBIAN_FRONTEND'] = 'noninteractive'
   package 'openssl'
 
   # dpkg returns 1 if dependencies are not satisfied, which they will not be
@@ -56,7 +57,7 @@ if platform_family? 'debian'
 
   # install the tools so we can test install_compass
   execute 'install mongo tools' do
-    command 'dpkg -i `find . -name "*tools*.deb"`'
+    command 'dpkg -i `find . -name "*tools-extra*.deb"`'
     cwd homedir
     returns [0, 1]
   end
@@ -88,7 +89,7 @@ if platform_family? 'rhel'
 
   # install the tools so we can test install_compass
   execute 'install mongo tools' do
-    command 'yum install -y `find . -name "*tools*.rpm"`'
+    command 'yum install -y `find . -name "*tools-extra*.rpm"`'
     cwd homedir
   end
 
@@ -115,23 +116,13 @@ if platform_family? 'suse'
   EOD
   end
 
-  %w(
-     SLES12-Pool
-     SLES12-Updates
-  ).each do |repo|
-    execute "add #{repo}" do
-      command "zypper addrepo --check --refresh --name \"#{repo}\" http://smt-ec2.susecloud.net/repo/SUSE/Products/SLE-SERVER/12/x86_64/product?credentials=SMT-http_smt-ec2_susecloud_net 'SMT-http_smt-ec2_susecloud_net:#{repo}'"
-      not_if "zypper lr | grep #{repo}"
-    end
-  end
-
   execute 'install mongod' do
-    command 'zypper -n install `find . -name "*server*.rpm"`'
+    command 'zypper --no-gpg-checks -n install `find . -name "*server*.rpm"`'
     cwd homedir
   end
 
   execute 'install mongo' do
-    command 'zypper -n install `find . -name "*shell*.rpm"`'
+    command 'zypper --no-gpg-checks -n install `find . -name "*shell*.rpm"`'
     cwd homedir
   end
 end
