@@ -251,6 +251,8 @@ class GDBDumper(Dumper):
             (pinfo.name, pinfo.pid)
         mongodb_javascript_stack = "mongodb-javascript-stack"
         mongod_dump_sessions = "mongod-dump-sessions"
+        mongodb_dump_mutexes = "mongodb-dump-mutexes"
+        mongodb_dump_recovery_units = "mongodb-dump-recovery-units"
 
         if not logger.mongo_process_filename:
             raw_stacks_commands = []
@@ -287,6 +289,8 @@ class GDBDumper(Dumper):
             mongodb_waitsfor_graph,
             mongodb_javascript_stack,
             mongod_dump_sessions,
+            mongodb_dump_mutexes,
+            mongodb_dump_recovery_units,
             "set confirm off",
             "quit",
         ]
