@@ -1,7 +1,5 @@
 """Driver of the test execution framework."""
 
-from __future__ import absolute_import
-
 import threading
 import time
 
@@ -9,13 +7,14 @@ from . import fixtures
 from . import hook_test_archival as archival
 from . import hooks as _hooks
 from . import job as _job
+from .queue_element import queue_elem_factory
 from . import report as _report
 from . import testcases
 from .. import config as _config
 from .. import errors
 from .. import utils
 from ..core import network
-from ..utils import queue as _queue
+from ..utils.queue import Queue
 
 
 class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
@@ -35,8 +34,11 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
 
         if _config.SHELL_CONN_STRING is not None:
             # Specifying the shellConnString command line option should override the fixture
-            # specified in the YAML configuration to be the no-op fixture.
-            self.fixture_config = {"class": fixtures.NOOP_FIXTURE_CLASS}
+            # specified in the YAML configuration to be the external fixture.
+            self.fixture_config = {
+                "class": fixtures.EXTERNAL_FIXTURE_CLASS,
+                "shell_conn_string": _config.SHELL_CONN_STRING
+            }
         else:
             self.fixture_config = fixture
 
@@ -49,20 +51,41 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
                                                       archive)
 
         self._suite = suite
-
-        # Only start as many jobs as we need. Note this means that the number of jobs we run may
-        # not actually be _config.JOBS or self._suite.options.num_jobs.
-        jobs_to_start = self._suite.options.num_jobs
-        self.num_tests = len(suite.tests)
-
-        if self.num_tests < jobs_to_start:
-            self.logger.info(
-                "Reducing the number of jobs from %d to %d since there are only %d test(s) to run.",
-                self._suite.options.num_jobs, self.num_tests, self.num_tests)
-            jobs_to_start = self.num_tests
+        self.num_tests = len(suite.tests) * suite.options.num_repeat_tests
+        self.test_queue_logger = self.logger.new_testqueue_logger(suite.test_kind)
 
         # Must be done after getting buildlogger configuration.
-        self._jobs = [self._make_job(job_num) for job_num in xrange(jobs_to_start)]
+        self._jobs = self._create_jobs(self.num_tests)
+
+    def _num_jobs_to_start(self, suite, num_tests):
+        """
+        Determine the number of jobs to start.
+
+        :param suite: Test suite being run.
+        :return: Number of jobs to start.
+        """
+        options = suite.options
+        num_jobs_to_start = options.num_jobs
+
+        if num_tests < num_jobs_to_start:
+            self.logger.info(
+                "Reducing the number of jobs from %d to %d since there are only %d test(s) to run.",
+                options.num_jobs, num_tests, num_tests)
+            num_jobs_to_start = num_tests
+
+        return num_jobs_to_start
+
+    def _create_jobs(self, num_tests):
+        """
+        Create job objects to consume and run tests.
+
+        Only start as many jobs as we need. Note this means that the number of jobs we run may
+        not actually be _config.JOBS or self._suite.options.num_jobs.
+
+        :return: List of jobs.
+        """
+        n_jobs_to_start = self._num_jobs_to_start(self._suite, num_tests)
+        return [self._make_job(job_num) for job_num in range(n_jobs_to_start)]
 
     def run(self):
         """Execute the test suite.
@@ -74,14 +97,15 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
         self.logger.info("Starting execution of %ss...", self._suite.test_kind)
 
         return_code = 0
+        # The first run of the job will set up the fixture.
+        setup_flag = threading.Event()
+        # We reset the internal state of the PortAllocator so that ports used by the fixture during
+        # a test suite run earlier can be reused during this current test suite.
+        network.PortAllocator.reset()
         teardown_flag = None
         try:
-            if not self._setup_fixtures():
-                return_code = 2
-                return
-
-            num_repeats = self._suite.options.num_repeats
-            while num_repeats > 0:
+            num_repeat_suites = self._suite.options.num_repeat_suites
+            while num_repeat_suites > 0:
                 test_queue = self._make_test_queue()
 
                 partial_reports = [job.report for job in self._jobs]
@@ -91,11 +115,18 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
                 # finish running their last test. This avoids having a large number of processes
                 # still running if an Evergreen task were to time out from a hang/deadlock being
                 # triggered.
-                teardown_flag = threading.Event() if num_repeats == 1 else None
-                (report, interrupted) = self._run_tests(test_queue, teardown_flag)
+                teardown_flag = threading.Event() if num_repeat_suites == 1 else None
+                (report, interrupted) = self._run_tests(test_queue, setup_flag, teardown_flag)
 
                 self._suite.record_test_end(report)
 
+                if setup_flag and setup_flag.is_set():
+                    self.logger.error("Setup of one of the job fixtures failed")
+                    return_code = 2
+                    return
+                # Remove the setup flag once the first suite ran.
+                setup_flag = None
+
                 # If the user triggered a KeyboardInterrupt, then we should stop.
                 if interrupted:
                     raise errors.UserInterrupt("Received interrupt from user")
@@ -105,7 +136,7 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
 
                 sb = []  # String builder.
                 self._suite.summarize_latest(sb)
-                self.logger.info("Summary: %s", "\n    ".join(sb))
+                self.logger.info("Summary of latest execution: %s", "\n    ".join(sb))
 
                 if not report.wasSuccessful():
                     return_code = 1
@@ -116,45 +147,21 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
                 test_results_num = len(test_report["results"])
                 # There should be at least as many tests results as expected number of tests.
                 if test_results_num < self.num_tests:
-                    raise errors.ResmokeError("{} reported tests is less than {} expected tests"
-                                              .format(test_results_num, self.num_tests))
+                    raise errors.ResmokeError(
+                        "{} reported tests is less than {} expected tests".format(
+                            test_results_num, self.num_tests))
 
                 # Clear the report so it can be reused for the next execution.
                 for job in self._jobs:
                     job.report.reset()
-                num_repeats -= 1
+                num_repeat_suites -= 1
         finally:
             if not teardown_flag:
                 if not self._teardown_fixtures():
                     return_code = 2
             self._suite.return_code = return_code
 
-    def _setup_fixtures(self):
-        """Set up a fixture for each job."""
-
-        # We reset the internal state of the PortAllocator before calling job.fixture.setup() so
-        # that ports used by the fixture during a test suite run earlier can be reused during this
-        # current test suite.
-        network.PortAllocator.reset()
-
-        for job in self._jobs:
-            try:
-                job.fixture.setup()
-            except:  # pylint: disable=bare-except
-                self.logger.exception("Encountered an error while setting up %s.", job.fixture)
-                return False
-
-        # Once they have all been started, wait for them to become available.
-        for job in self._jobs:
-            try:
-                job.fixture.await_ready()
-            except:  # pylint: disable=bare-except
-                self.logger.exception("Encountered an error while waiting for %s to be ready",
-                                      job.fixture)
-                return False
-        return True
-
-    def _run_tests(self, test_queue, teardown_flag):
+    def _run_tests(self, test_queue, setup_flag, teardown_flag):
         """Start a thread for each Job instance and block until all of the tests are run.
 
         Returns a (combined report, user interrupted) pair, where the
@@ -168,8 +175,9 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
         try:
             # Run each Job instance in its own thread.
             for job in self._jobs:
-                thr = threading.Thread(target=job, args=(test_queue, interrupt_flag),
-                                       kwargs=dict(teardown_flag=teardown_flag))
+                thr = threading.Thread(
+                    target=job, args=(test_queue, interrupt_flag), kwargs=dict(
+                        setup_flag=setup_flag, teardown_flag=teardown_flag))
                 # Do not wait for tests to finish executing if interrupted by the user.
                 thr.daemon = True
                 thr.start()
@@ -188,10 +196,20 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
         except (KeyboardInterrupt, SystemExit):
             interrupt_flag.set()
             user_interrupted = True
-        else:
-            # Only wait for all the Job instances if not interrupted by the user.
+
+        wait_secs = 2.0
+        self.logger.debug("Waiting for threads to complete")
+
+        timer = threading.Timer(wait_secs, self._log_timeout_warning, args=[wait_secs])
+        timer.daemon = True
+        timer.start()
+        try:
             for thr in threads:
                 thr.join()
+        finally:
+            timer.cancel()
+
+        self.logger.debug("Threads are completed!")
 
         reports = [job.report for job in self._jobs]
         combined_report = _report.TestReport.combine(*reports)
@@ -209,13 +227,9 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
         """
         success = True
         for job in self._jobs:
-            try:
-                job.fixture.teardown(finished=True)
-            except errors.ServerFailure as err:
-                self.logger.warn("Teardown of %s was not successful: %s", job.fixture, err)
-                success = False
-            except:  # pylint: disable=bare-except
-                self.logger.exception("Encountered an error while tearing down %s.", job.fixture)
+            if not job.manager.teardown_fixture(self.logger):
+                self.logger.warning("Teardown of %s of job %s was not successful", job.fixture,
+                                    job.job_num)
                 success = False
         return success
 
@@ -224,7 +238,6 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
 
         fixture_config = {}
         fixture_class = fixtures.NOOP_FIXTURE_CLASS
-
         if self.fixture_config is not None:
             fixture_config = self.fixture_config.copy()
             fixture_class = fixture_config.pop("class")
@@ -249,7 +262,12 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
         return hooks
 
     def _make_job(self, job_num):
-        """Return a Job instance with its own fixture, hooks, and test report."""
+        """
+        Create a Job instance with its own fixture, hooks, and test report.
+
+        :param job_num: instance number of job being created.
+        :return: Job instance.
+        """
         job_logger = self.logger.new_job_logger(self._suite.test_kind, job_num)
 
         fixture = self._make_fixture(job_num, job_logger)
@@ -257,25 +275,57 @@ class TestSuiteExecutor(object):  # pylint: disable=too-many-instance-attributes
 
         report = _report.TestReport(job_logger, self._suite.options)
 
-        return _job.Job(job_logger, fixture, hooks, report, self.archival, self._suite.options)
+        return _job.Job(job_num, job_logger, fixture, hooks, report, self.archival,
+                        self._suite.options, self.test_queue_logger)
+
+    def _num_times_to_repeat_tests(self):
+        """
+        Determine the number of times to repeat the tests.
+
+        :return: Number of times to repeat the tests.
+        """
+        if self._suite.options.num_repeat_tests:
+            return self._suite.options.num_repeat_tests
+        return 1
+
+    def _create_queue_elem_for_test_name(self, test_name):
+        """
+        Create the appropriate queue_elem to run the given test_name.
+
+        :param test_name: Name of test to be queued.
+        :return: queue_elem representing the test_name to be run.
+        """
+        test_case = testcases.make_test_case(self._suite.test_kind, self.test_queue_logger,
+                                             test_name, **self.test_config)
+        return queue_elem_factory(test_case, self.test_config, self._suite.options)
 
     def _make_test_queue(self):
-        """Return a queue of TestCase instances.
-
-        Use a multi-consumer queue instead of a unittest.TestSuite so
-        that the test cases can be dispatched to multiple threads.
         """
+        Create a queue of test cases to run.
+
+        Each test case will be added to the queue via a queue_element. For normal execution,
+        we will add a queue_element for each run of the test (if we are repeating the tests twice,
+        we will add 2 queue_elements of each test to the queue). If we are repeating execution for
+        a specified time period, we will add each test to the queue, but as a QueueElemRepeatTime
+        object, which will requeue itself if it has not run for the expected duration.
+
+        Use a multi-consumer queue instead of a unittest.TestSuite so that the test cases can
+        be dispatched to multiple threads.
+
+        :return: Queue of testcases to run.
+        """
+        queue = Queue()
 
-        test_queue_logger = self.logger.new_testqueue_logger(self._suite.test_kind)
         # Put all the test cases in a queue.
-        queue = _queue.Queue()
-        for test_name in self._suite.tests:
-            test_case = testcases.make_test_case(self._suite.test_kind, test_queue_logger,
-                                                 test_name, **self.test_config)
-            queue.put(test_case)
-
-        # Add sentinel value for each job to indicate when there are no more items to process.
-        for _ in xrange(len(self._jobs)):
-            queue.put(None)
+        for _ in range(self._num_times_to_repeat_tests()):
+            for test_name in self._suite.tests:
+                queue_elem = self._create_queue_elem_for_test_name(test_name)
+                queue.put(queue_elem)
 
         return queue
+
+    def _log_timeout_warning(self, seconds):
+        """Log a message if any thread fails to terminate after `seconds`."""
+        self.logger.warning(
+            '*** Still waiting for processes to terminate after %s seconds. Try using ctrl-\\ '
+            'to send a SIGQUIT on Linux or ctrl-c again on Windows ***', seconds)
