@@ -3,7 +3,7 @@
 Exhaustive test for authorization of commands with user-defined roles.
 
 The test logic implemented here operates on the test cases defined
-in jstests/auth/commands.js.
+in jstests/auth/lib/commands_lib.js.
 
 @tags: [requires_sharding]
 
@@ -47,7 +47,7 @@ function testProperAuthorization(conn, t, testcase, privileges) {
 
     if (!testcase.expectFail && res.ok != 1 && res.code != commandNotSupportedCode) {
         // don't error if the test failed with code commandNotSupported since
-        // some storage engines (e.g wiredTiger) don't support some commands (e.g. touch)
+        // some storage engines don't support some commands.
         out = "command failed with " + tojson(res) + " on db " + testcase.runOnDb +
             " with privileges " + tojson(privileges);
     } else if (testcase.expectFail && res.code == authErrCode) {
@@ -211,7 +211,8 @@ authCommandsLib.runTests(conn, impls);
 MongoRunner.stopMongod(conn);
 
 // run all tests sharded
-// TODO: Remove 'shardAsReplicaSet: false' when SERVER-32672 is fixed.
+// TODO: SERVER-43897 Make commands_user_defined_roles.js and commands_builtin_roles.js start shards
+// as replica sets.
 conn = new ShardingTest({
     shards: 2,
     mongos: 1,
