@@ -23,8 +23,8 @@ var $config = extendWorkload($config, function($config, $super) {
 
     // The variables used by the random_moveChunk_base config in order to move chunks.
     $config.data.shardKey = {a: 1};
-
     $config.data.newShardKey = {a: 1, b: 1};
+    $config.data.newShardKeyFields = ["a", "b"];
 
     // Use a CountDownLatch as if it were a std::atomic<long long> shared between all of the
     // threads. The collection name is suffixed with the current this.latch.getCount() value
@@ -41,7 +41,7 @@ var $config = extendWorkload($config, function($config, $super) {
     };
 
     $config.data.getCurrentOrPreviousLatchCollName = function(collName) {
-        const latchNumber = (Math.random() < 0.5)
+        const latchNumber = (Random.rand() < 0.5)
             ? this.latch.getCount()
             : Math.min(this.latch.getCount() + 1, this.latchCount);
 
@@ -95,7 +95,7 @@ var $config = extendWorkload($config, function($config, $super) {
             throw e;
         }
 
-        this.shardKeyField[latchCollName] = ['a', 'b'];
+        this.shardKeyField[latchCollName] = this.newShardKeyFields;
         this.latch.countDown();
     };
 
@@ -111,10 +111,18 @@ var $config = extendWorkload($config, function($config, $super) {
         }
     };
 
+    // Occasionally flush the router's cached metadata to verify the metadata for the refined
+    // collections can be successfully loaded.
+    $config.states.flushRouterConfig = function flushRouterConfig(db, collName, connCache) {
+        assert.commandWorked(db.adminCommand({flushRouterConfig: db.getName()}));
+    };
+
     $config.transitions = {
-        init: {moveChunk: 0.5, refineCollectionShardKey: 0.5},
-        moveChunk: {moveChunk: 0.5, refineCollectionShardKey: 0.5},
-        refineCollectionShardKey: {moveChunk: 0.5, refineCollectionShardKey: 0.5},
+        init: {moveChunk: 0.4, refineCollectionShardKey: 0.4, flushRouterConfig: 0.2},
+        moveChunk: {moveChunk: 0.4, refineCollectionShardKey: 0.4, flushRouterConfig: 0.2},
+        refineCollectionShardKey:
+            {moveChunk: 0.4, refineCollectionShardKey: 0.4, flushRouterConfig: 0.2},
+        flushRouterConfig: {moveChunk: 0.5, refineCollectionShardKey: 0.5},
     };
 
     $config.setup = function setup(db, collName, cluster) {
