@@ -6,6 +6,7 @@
  *  requires_fastcount,
  *  requires_collstats,
  *  requires_capped,
+ *  requires_fcv_44,
  * ]
  */
 
@@ -160,6 +161,7 @@ assert.eq(0, noCollStats.count, "All properties should be 0 on nonexistant colle
 assert.eq(0, noCollStats.storageSize, "All properties should be 0 on nonexistant collections");
 assert.eq(0, noCollStats.nindexes, "All properties should be 0 on nonexistant collections");
 assert.eq(0, noCollStats.totalIndexSize, "All properties should be 0 on nonexistant collections");
+assert.eq(0, noCollStats.totalSize, "All properties should be 0 on nonexistant collections");
 
 // scale - passed to stats() as sole numerical argument or part of an options object.
 t.drop();
