@@ -92,7 +92,6 @@ const kNonRetryableCommands = new Set([
     "captrunc",
     "clone",
     "cloneCollectionAsCapped",
-    "collMod",
     "convertToCapped",
     "create",
     "createIndexes",
@@ -857,8 +856,8 @@ function shouldRetryWithNetworkErrorOverride(
     if (!res.ok) {
         if (RetryableWritesUtil.isRetryableCode(res.code)) {
             // Don't decrement retries, because the command returned before the connection was
-            // closed, so a subsequent attempt will receive a network error (or NotMaster error)
-            // and need to retry.
+            // closed, so a subsequent attempt will receive a network error (or NotWritablePrimary
+            // error) and need to retry.
             logError("Retrying failed response with retryable code");
             return kContinue;
         }
