@@ -1,15 +1,28 @@
 // Helper functions for testing index builds.
 
-class IndexBuildTest {
+var IndexBuildTest = class {
     /**
      * Starts an index build in a separate mongo shell process with given options.
+     * Ensures the index build worked or failed with one of the expected failures.
      */
-    static startIndexBuild(conn, ns, keyPattern, options) {
+    static startIndexBuild(conn, ns, keyPattern, options, expectedFailures) {
         options = options || {};
-        return startParallelShell('const coll = db.getMongo().getCollection("' + ns + '");' +
-                                      'assert.commandWorked(coll.createIndex(' +
-                                      tojson(keyPattern) + ', ' + tojson(options) + '));',
-                                  conn.port);
+        expectedFailures = expectedFailures || [];
+
+        if (Array.isArray(keyPattern)) {
+            return startParallelShell(
+                'const coll = db.getMongo().getCollection("' + ns + '");' +
+                    'assert.commandWorkedOrFailedWithCode(coll.createIndexes(' +
+                    JSON.stringify(keyPattern) + ', ' + tojson(options) + '), ' +
+                    JSON.stringify(expectedFailures) + ');',
+                conn.port);
+        } else {
+            return startParallelShell('const coll = db.getMongo().getCollection("' + ns + '");' +
+                                          'assert.commandWorkedOrFailedWithCode(coll.createIndex(' +
+                                          tojson(keyPattern) + ', ' + tojson(options) + '), ' +
+                                          JSON.stringify(expectedFailures) + ');',
+                                      conn.port);
+        }
     }
 
     /**
@@ -18,9 +31,8 @@ class IndexBuildTest {
      * index build.
      * Accepts optional filter that can be used to customize the db.currentOp() query.
      */
-
     static getIndexBuildOpId(database, collectionName, indexName, filter) {
-        let pipeline = [{$currentOp: {allUsers: true}}];
+        let pipeline = [{$currentOp: {allUsers: true, idleConnections: true}}];
         if (filter) {
             pipeline.push({$match: filter});
         }
@@ -28,6 +40,8 @@ class IndexBuildTest {
                            .aggregate(pipeline, {readConcern: {level: "local"}})
                            .toArray();
         let indexBuildOpId = -1;
+        let indexBuildObj = {};
+        let indexBuildNamespace = "";
 
         result.forEach(function(op) {
             if (op.op != 'command') {
@@ -43,10 +57,16 @@ class IndexBuildTest {
                 cmdBody.indexes.forEach((index) => {
                     if (!indexName || index.name === indexName) {
                         indexBuildOpId = op.opid;
+                        indexBuildObj = index;
+                        indexBuildNamespace = op.ns;
                     }
                 });
             }
         });
+        if (indexBuildOpId != -1) {
+            jsTestLog("found in progress index build: " + tojson(indexBuildObj) + " on namespace " +
+                      indexBuildNamespace + " opid: " + indexBuildOpId);
+        }
         return indexBuildOpId;
     }
 
@@ -94,7 +114,7 @@ class IndexBuildTest {
      * An optional 'onOperationFn' callback accepts an operation to perform any additional checks.
      */
     static assertIndexBuildCurrentOpContents(database, opId, onOperationFn) {
-        const inprog = database.currentOp({opid: opId}).inprog;
+        const inprog = database.currentOp({opid: opId, "$all": true}).inprog;
         assert.eq(1,
                   inprog.length,
                   'unable to find opid ' + opId +
@@ -110,12 +130,13 @@ class IndexBuildTest {
      * Runs listIndexes command on collection.
      * If 'options' is provided, these will be sent along with the command request.
      * Asserts that all the indexes on this collection fit within the first batch of results.
+     * Returns a map of index specs keyed by name.
      */
     static assertIndexes(coll, numIndexes, readyIndexes, notReadyIndexes, options) {
         notReadyIndexes = notReadyIndexes || [];
         options = options || {};
 
-        let res = coll.runCommand("listIndexes", options);
+        let res = assert.commandWorked(coll.runCommand("listIndexes", options));
         assert.eq(numIndexes,
                   res.cursor.firstBatch.length,
                   'unexpected number of indexes in collection: ' + tojson(res));
@@ -158,6 +179,8 @@ class IndexBuildTest {
                        'unexpected buildUUID field in ' + name + ' index spec: ' + tojson(spec));
             }
         }
+
+        return indexMap;
     }
 
     /**
@@ -175,4 +198,23 @@ class IndexBuildTest {
         assert.commandWorked(
             conn.adminCommand({configureFailPoint: 'hangAfterStartingIndexBuild', mode: 'off'}));
     }
-}
+
+    /**
+     * Returns true if two phase index builds are supported.
+     */
+    static supportsTwoPhaseIndexBuild(conn) {
+        const adminDB = conn.getDB('admin');
+        const serverStatus = assert.commandWorked(adminDB.serverStatus());
+        const storageEngineSection = serverStatus.storageEngine;
+        return storageEngineSection.supportsTwoPhaseIndexBuild;
+    }
+
+    /**
+     * Returns true if majority commit quorum is supported by two phase index builds.
+     */
+    static indexBuildCommitQuorumEnabled(conn) {
+        return assert
+            .commandWorked(conn.adminCommand({getParameter: 1, enableIndexBuildCommitQuorum: 1}))
+            .enableIndexBuildCommitQuorum;
+    }
+};
