@@ -54,7 +54,7 @@ print("2: shut down slave");
 replTest.stop(slaveId);
 
 print("3: write to master");
-assert.writeOK(mdb.foo.insert({a: 1001}, {writeConcern: {w: 1}}));
+assert.commandWorked(mdb.foo.insert({a: 1001}, {writeConcern: {w: 1}}));
 
 print("4: modify master's minvalid");
 var local = master.getDB("local");
