@@ -1,7 +1,7 @@
 // @tags: [requires_replication, requires_sharding]
 
 var name = "sharding_rs_arb1";
-var replTest = new ReplSetTest({name: name, nodes: 3});
+var replTest = new ReplSetTest({name: name, nodes: 3, nodeOptions: {shardsvr: ""}});
 replTest.startSet();
 var port = replTest.ports;
 replTest.initiate({
