@@ -1,6 +1,8 @@
 /**
  * Test that oplog (on both primary and secondary) rolls over when its size exceeds the configured
  * maximum. This test runs on wiredTiger storage engine and inMemory storage engine (if available).
+ *
+ * @tags: [multiversion_incompatible]
  */
 (function() {
 "use strict";
@@ -8,9 +10,19 @@
 function doTest(storageEngine) {
     jsTestLog("Testing with storageEngine: " + storageEngine);
 
+    // Pause the oplog cap maintainer thread for this test until oplog truncation is needed. The
+    // truncation thread can hold a mutex for a short period of time which prevents new oplog stones
+    // from being created during an insertion if the mutex cannot be obtained immediately. Instead,
+    // the next insertion will attempt to create a new oplog stone, which this test does not do.
     const replSet = new ReplSetTest({
         // Set the syncdelay to 1s to speed up checkpointing.
-        nodeOptions: {syncdelay: 1, setParameter: {logComponentVerbosity: tojson({storage: 1})}},
+        nodeOptions: {
+            syncdelay: 1,
+            setParameter: {
+                logComponentVerbosity: tojson({storage: 1}),
+                'failpoint.hangOplogCapMaintainerThread': tojson({mode: 'alwaysOn'})
+            }
+        },
         nodes: [{}, {rsConfig: {priority: 0, votes: 0}}]
     });
     // Set max oplog size to 1MB.
@@ -22,6 +34,10 @@ function doTest(storageEngine) {
     const secondary = replSet.getSecondary();
     const secondaryOplog = secondary.getDB("local").oplog.rs;
 
+    // Verify that the oplog cap maintainer thread is paused.
+    checkLog.containsJson(primary, 5095500);
+    checkLog.containsJson(secondary, 5095500);
+
     const coll = primary.getDB("test").foo;
     // 400KB each so that oplog can keep at most two insert oplog entries.
     const longString = new Array(400 * 1024).join("a");
@@ -96,6 +112,17 @@ function doTest(storageEngine) {
                 .operationTime;
         jsTestLog("Third insert timestamp: " + thirdInsertTimestamp);
 
+        // Verify that there are three oplog entries while the oplog cap maintainer thread is
+        // paused.
+        assert.eq(3, numInsertOplogEntry(primaryOplog));
+        assert.eq(3, numInsertOplogEntry(secondaryOplog));
+
+        // Let the oplog cap maintainer thread start truncating the oplog.
+        assert.commandWorked(primary.adminCommand(
+            {configureFailPoint: "hangOplogCapMaintainerThread", mode: "off"}));
+        assert.commandWorked(secondary.adminCommand(
+            {configureFailPoint: "hangOplogCapMaintainerThread", mode: "off"}));
+
         // Test that oplog entry of the initial insert rolls over on both primary and secondary.
         // Use assert.soon to wait for oplog cap maintainer thread to run.
         assert.soon(() => {
@@ -110,6 +137,12 @@ function doTest(storageEngine) {
         assert.eq(res.oplogTruncation.truncateCount, 1, tojson(res.oplogTruncation));
         assert.gt(res.oplogTruncation.totalTimeTruncatingMicros, 0, tojson(res.oplogTruncation));
     } else {
+        // Let the oplog cap maintainer thread start truncating the oplog.
+        assert.commandWorked(primary.adminCommand(
+            {configureFailPoint: "hangOplogCapMaintainerThread", mode: "off"}));
+        assert.commandWorked(secondary.adminCommand(
+            {configureFailPoint: "hangOplogCapMaintainerThread", mode: "off"}));
+
         // Only test that oplog truncation will eventually happen.
         let numInserted = 2;
         assert.soon(function() {
