@@ -1,10 +1,14 @@
 /**
  * Tests that when a node restarts during initial sync before it can clone the FCV document, it is
  * still able to start up successfully and restart initial sync.
+ *
+ * This test asserts a certain FCV is cloned through initial sync.
+ * @tags: [multiversion_incompatible]
  */
 
 (function() {
-load("jstests/libs/feature_compatibility_version.js");
+
+load("jstests/libs/fail_point_util.js");
 
 rst = new ReplSetTest({nodes: 1});
 rst.startSet();
@@ -17,14 +21,12 @@ const versionCollName = "system.version";
 const nss = adminDbName + "." + versionCollName;
 
 // Hang initial sync before cloning the FCV document.
-let secondary = rst.add({rsConfig: {priority: 0}});
-assert.commandWorked(secondary.getDB('admin').runCommand({
-    configureFailPoint: 'initialSyncHangBeforeCollectionClone',
-    mode: 'alwaysOn',
-    data: {namespace: nss}
-}));
+let secondary = rst.add({rsConfig: {priority: 0, votes: 0}});
+let failPoint = configureFailPoint(secondary,
+                                   'hangBeforeClonerStage',
+                                   {cloner: 'CollectionCloner', stage: 'count', namespace: nss});
 rst.reInitiate();
-checkLog.contains(secondary, "initialSyncHangBeforeCollectionClone fail point enabled.");
+failPoint.wait();
 
 jsTestLog("Restarting secondary in the early stages of initial sync.");
 rst.restart(secondary);
