@@ -25,7 +25,7 @@ var primaryColl = db[collName];
 var collNss = primaryColl.getFullName();
 
 jsTestLog("Writing data to collection.");
-assert.writeOK(primaryColl.insert({_id: 'readOp'}, {"writeConcern": {"w": 2}}));
+assert.commandWorked(primaryColl.insert({_id: 'readOp'}, {"writeConcern": {"w": 2}}));
 
 TestData.dbName = dbName;
 TestData.collName = collName;
