@@ -6,12 +6,7 @@
 (function() {
 "use strict";
 
-// TODO: SERVER-33830 remove shardAsReplicaSet: false
-var st = new ShardingTest({
-    name: 'migrateBig_balancer',
-    shards: 2,
-    other: {enableBalancer: true, shardAsReplicaSet: false}
-});
+var st = new ShardingTest({name: 'migrateBig_balancer', shards: 2, other: {enableBalancer: true}});
 var mongos = st.s;
 var admin = mongos.getDB("admin");
 var db = mongos.getDB("test");
@@ -36,7 +31,7 @@ for (var i = 0; i < 40; i++) {
     bulk.insert({data: dataObj});
 }
 
-assert.writeOK(bulk.execute());
+assert.commandWorked(bulk.execute());
 assert.eq(40, coll.count(), "prep1");
 
 assert.commandWorked(admin.runCommand({shardcollection: "" + coll, key: {_id: 1}}));
