@@ -98,11 +98,16 @@ testBasicVerboseOnly(outputVerbose);
 var config = mongos.getDB("config");
 var configCopy = standalone.getDB("configCopy");
 config.getCollectionInfos().forEach(function(c) {
+    // It's illegal to copy the system collections.
+    if (c.name == "system.indexBuilds") {
+        return;
+    }
+
     // Create collection with options.
     assert.commandWorked(configCopy.createCollection(c.name, c.options));
     // Clone the docs.
     config.getCollection(c.name).find().hint({_id: 1}).forEach(function(d) {
-        assert.writeOK(configCopy.getCollection(c.name).insert(d));
+        assert.commandWorked(configCopy.getCollection(c.name).insert(d));
     });
     // Build the indexes.
     config.getCollection(c.name).getIndexes().forEach(function(i) {
@@ -179,11 +184,11 @@ function testCollDetails(args) {
     assert.commandWorked(admin.runCommand(cmdObj));
 
     if (args.hasOwnProperty("unique")) {
-        assert.writeOK(mongos.getDB("config").collections.update({_id: collName},
-                                                                 {$set: {"unique": args.unique}}));
+        assert.commandWorked(mongos.getDB("config").collections.update(
+            {_id: collName}, {$set: {"unique": args.unique}}));
     }
     if (args.hasOwnProperty("noBalance")) {
-        assert.writeOK(mongos.getDB("config").collections.update(
+        assert.commandWorked(mongos.getDB("config").collections.update(
             {_id: collName}, {$set: {"noBalance": args.noBalance}}));
     }
 
@@ -217,7 +222,7 @@ function testCollDetails(args) {
         mongos.getCollection(collName).drop();
     } catch (e) {
         // Ignore drop errors because they are from the illegal values in the collection entry
-        assert.writeOK(mongos.getDB("config").collections.remove({_id: collName}));
+        assert.commandWorked(mongos.getDB("config").collections.remove({_id: collName}));
     }
 
     testCollDetailsNum++;
