@@ -451,6 +451,7 @@ let testCases = {
         useLogs: true,
     },
     handshake: {skip: "does not accept read or write concern"},
+    hello: {skip: "does not accept read or write concern"},
     hostInfo: {skip: "does not accept read or write concern"},
     httpClientRequest: {skip: "does not accept read or write concern"},
     insert: {
@@ -463,8 +464,8 @@ let testCases = {
     },
     internalRenameIfOptionsAndIndexesMatch: {skip: "internal command"},
     invalidateUserCache: {skip: "does not accept read or write concern"},
-    isMaster: {skip: "does not accept read or write concern"},
     isdbgrid: {skip: "does not accept read or write concern"},
+    isMaster: {skip: "does not accept read or write concern"},
     killAllSessions: {skip: "does not accept read or write concern"},
     killAllSessionsByPattern: {skip: "does not accept read or write concern"},
     killCursors: {skip: "does not accept read or write concern"},
@@ -477,6 +478,7 @@ let testCases = {
     listShards: {skip: "does not accept read or write concern"},
     lockInfo: {skip: "does not accept read or write concern"},
     logApplicationMessage: {skip: "does not accept read or write concern"},
+    logMessage: {skip: "does not accept read or write concern"},
     logRotate: {skip: "does not accept read or write concern"},
     logout: {skip: "does not accept read or write concern"},
     makeSnapshot: {skip: "does not accept read or write concern"},
