@@ -20,7 +20,7 @@ while (inserted < (20 * 1024 * 1024)) {
     bulk.insert({_id: num++, s: bigString});
     inserted += bigString.length;
 }
-assert.writeOK(bulk.execute());
+assert.commandWorked(bulk.execute());
 
 assert.commandWorked(s.s0.adminCommand({shardcollection: "test.foo", key: {_id: 1}}));
 assert.lt(20, s.config.chunks.count({"ns": "test.foo"}), "setup2");
