@@ -1,7 +1,7 @@
 /*
  * Tests that changing the shard key value of a document using update and findAndModify works
  * correctly when the doc will change shards.
- * @tags: [uses_transactions, uses_multi_shard_transaction]
+ * @tags: [requires_find_command, uses_transactions, uses_multi_shard_transaction]
  */
 
 (function() {
@@ -22,7 +22,6 @@ st.ensurePrimaryShard(kDbName, shard0);
 function changeShardKeyWhenFailpointsSet(session, sessionDB, runInTxn, isFindAndModify) {
     let docsToInsert = [{"x": 4, "a": 3}, {"x": 100}, {"x": 300, "a": 3}, {"x": 500, "a": 6}];
     shardCollectionMoveChunks(st, kDbName, ns, {"x": 1}, docsToInsert, {"x": 100}, {"x": 300});
-    cleanupOrphanedDocs(st, ns);
 
     // Assert that the document is not updated when the delete fails
     assert.commandWorked(st.rs1.getPrimary().getDB(kDbName).adminCommand({
@@ -162,6 +161,17 @@ changeShardKeyOptions.forEach(function(updateConfig) {
                                    [{"$set": {"x": 30}}, {"$set": {"x": 600}}],
                                    upsert);
 
+    assertCanUnsetSKField(st,
+                          kDbName,
+                          ns,
+                          session,
+                          sessionDB,
+                          runInTxn,
+                          isFindAndModify,
+                          {"x": 300},
+                          {"$unset": {"x": 1}},
+                          upsert);
+
     // Failure cases. These tests do not take 'upsert' as an option so we do not need to test
     // them for both upsert true and false.
     if (!upsert) {
@@ -177,10 +187,6 @@ changeShardKeyOptions.forEach(function(updateConfig) {
             st, kDbName, ns, session, sessionDB, runInTxn, isFindAndModify, {"x": 300}, {
                 "$set": {"x": [30]}
             });
-        assertCannotUnsetSKField(
-            st, kDbName, ns, session, sessionDB, runInTxn, isFindAndModify, {"x": 300}, {
-                "$unset": {"x": 1}
-            });
 
         if (!isFindAndModify) {
             assertCannotUpdateWithMultiTrue(
@@ -236,6 +242,9 @@ changeShardKeyOptions.forEach(function(updateConfig) {
                                    [{"x": 30, "y": 80}, {"x": 600, "y": 3}],
                                    upsert);
 
+    assertCanUnsetSKField(
+        st, kDbName, ns, session, sessionDB, runInTxn, isFindAndModify, {"x": 300}, {}, upsert);
+
     // Failure cases. These tests do not take 'upsert' as an option so we do not need to test
     // them for both upsert true and false.
     if (!upsert) {
@@ -255,8 +264,6 @@ changeShardKeyOptions.forEach(function(updateConfig) {
             st, kDbName, ns, session, sessionDB, runInTxn, isFindAndModify, {"x": 300}, {
                 "x": [30]
             });
-        assertCannotUnsetSKField(
-            st, kDbName, ns, session, sessionDB, runInTxn, isFindAndModify, {"x": 300}, {});
     }
 });
 
@@ -297,7 +304,6 @@ mongos.getDB(kDbName).foo.drop();
 // ----Assert that specifying writeConcern succeeds----
 
 shardCollectionMoveChunks(st, kDbName, ns, {"x": 1}, docsToInsert, {"x": 100}, {"x": 300});
-cleanupOrphanedDocs(st, ns);
 
 assert.commandWorked(sessionDB.foo.update({x: 4}, {$set: {x: 1000}}, {writeConcern: {w: 1}}));
 
@@ -315,7 +321,6 @@ mongos.getDB(kDbName).foo.drop();
 // ----Assert retryable write result has WCE when the internal commitTransaction fails----
 
 shardCollectionMoveChunks(st, kDbName, ns, {"x": 1}, docsToInsert, {"x": 100}, {"x": 300});
-cleanupOrphanedDocs(st, ns);
 
 // Turn on failcommand fail point to fail CoordinateCommitTransaction
 assert.commandWorked(st.rs0.getPrimary().getDB(kDbName).adminCommand({
@@ -364,7 +369,6 @@ assertCannotUpdateInBulkOpWhenDocsMoveShards(st, kDbName, ns, session, sessionDB
 
 // Update two docs, updating one twice
 shardCollectionMoveChunks(st, kDbName, ns, {"x": 1}, docsToInsert, {"x": 100}, {"x": 300});
-cleanupOrphanedDocs(st, ns);
 
 session.startTransaction();
 let id = mongos.getDB(kDbName).foo.find({"x": 500}).toArray()[0]._id;
@@ -385,7 +389,6 @@ mongos.getDB(kDbName).foo.drop();
 // Check that doing $inc on doc A, then updating shard key for doc A, then $inc again only incs
 // once
 shardCollectionMoveChunks(st, kDbName, ns, {"x": 1}, docsToInsert, {"x": 100}, {"x": 300});
-cleanupOrphanedDocs(st, ns);
 
 session.startTransaction();
 assert.commandWorked(sessionDB.foo.update({"x": 500}, {"$inc": {"a": 1}}));
@@ -401,7 +404,6 @@ assert.eq(1, mongos.getDB(kDbName).foo.find({"x": 30, "a": 7}).itcount());
 mongos.getDB(kDbName).foo.drop();
 
 shardCollectionMoveChunks(st, kDbName, ns, {"x": 1}, docsToInsert, {"x": 100}, {"x": 300});
-cleanupOrphanedDocs(st, ns);
 
 // Insert and $inc before moving doc
 session.startTransaction();
@@ -423,7 +425,6 @@ mongos.getDB(kDbName).foo.drop();
 // ----Assert correct behavior when update is sent directly to a shard----
 
 shardCollectionMoveChunks(st, kDbName, ns, {"x": 1}, docsToInsert, {"x": 100}, {"x": 300});
-cleanupOrphanedDocs(st, ns);
 
 //
 // For Op-style updates.
