@@ -31,6 +31,7 @@
 
 #include <cmath>
 #include <cstdint>
+#include <fmt/format.h>
 #include <string.h>  // strlen
 #include <string>
 #include <vector>
@@ -52,6 +53,9 @@ class BSONObj;
 class BSONElement;
 class BSONObjBuilder;
 class Timestamp;
+class ExtendedCanonicalV200Generator;
+class ExtendedRelaxedV200Generator;
+class LegacyStrictGenerator;
 
 typedef BSONElement be;
 typedef BSONObj bo;
@@ -208,13 +212,37 @@ public:
                   int depth = 0) const;
 
     std::string jsonString(JsonStringFormat format,
+                           bool includeSeparator,
                            bool includeFieldNames = true,
-                           int pretty = 0) const;
+                           int pretty = 0,
+                           size_t writeLimit = 0,
+                           BSONObj* outTruncationResult = nullptr) const;
 
-    void jsonStringStream(JsonStringFormat format,
-                          bool includeFieldNames,
-                          int pretty,
-                          std::stringstream& s) const;
+    BSONObj jsonStringBuffer(JsonStringFormat format,
+                             bool includeSeparator,
+                             bool includeFieldNames,
+                             int pretty,
+                             fmt::memory_buffer& buffer,
+                             size_t writeLimit = 0) const;
+
+    BSONObj jsonStringGenerator(ExtendedCanonicalV200Generator const& generator,
+                                bool includeSeparator,
+                                bool includeFieldNames,
+                                int pretty,
+                                fmt::memory_buffer& buffer,
+                                size_t writeLimit = 0) const;
+    BSONObj jsonStringGenerator(ExtendedRelaxedV200Generator const& generator,
+                                bool includeSeparator,
+                                bool includeFieldNames,
+                                int pretty,
+                                fmt::memory_buffer& buffer,
+                                size_t writeLimit = 0) const;
+    BSONObj jsonStringGenerator(LegacyStrictGenerator const& generator,
+                                bool includeSeparator,
+                                bool includeFieldNames,
+                                int pretty,
+                                fmt::memory_buffer& buffer,
+                                size_t writeLimit = 0) const;
 
     operator std::string() const {
         return toString();
@@ -339,11 +367,24 @@ public:
         return ConstDataView(value()).read<LittleEndian<long long>>();
     }
 
-    /** Retrieve int value for the element safely.  Zero returned if not a number. */
+    /**
+     * Retrieves the value of this element as a 32 bit integer. If the BSON type is non-numeric,
+     * returns zero. If the element holds a double, truncates the fractional part.
+     *
+     * Results in undefined behavior if called on a double that is NaN, +/-infinity, or too
+     * large/small to be represented as an int.  Use 'safeNumberLong()' to safely convert an
+     * arbitrary BSON element to an integer without risk of UB.
+     */
     int numberInt() const;
-    /** Retrieve long value for the element safely.  Zero returned if not a number.
-     *  Behavior is not defined for double values that are NaNs, or too large/small
-     *  to be represented by long longs */
+
+    /**
+     * Retrieves the value of this element as a 64 bit integer. If the BSON type is non-numeric,
+     * returns zero. If the element holds a double, truncates the fractional part.
+     *
+     * Results in undefined behavior if called on a double that is NaN, +/-infinity, or too
+     * large/small to be repsented as a long. Use 'safeNumberLong()' to safely convert an arbitrary
+     * BSON element to an integer without risk of UB.
+     */
     long long numberLong() const;
 
     /** Like numberLong() but with well-defined behavior for doubles that
@@ -467,23 +508,7 @@ public:
         return ConstDataView(value() + 4).read<LittleEndian<int>>();
     }
 
-    /** Get the scope SavedContext of a CodeWScope data element.
-     *
-     *  This function is DEPRECATED, since it can error if there are
-     *  null chars in the codeWScopeCode. However, some existing indexes
-     *  may be based on an incorrect ordering derived from this function,
-     *  so it may still need to be used in certain cases.
-     *   */
-    const char* codeWScopeScopeDataUnsafe() const {
-        // This can error if there are null chars in the codeWScopeCode
-        return codeWScopeCode() + strlen(codeWScopeCode()) + 1;
-    }
-
     /* Get the scope SavedContext of a CodeWScope data element.
-     *
-     * This is the corrected version of codeWScopeScopeDataUnsafe(),
-     * but note that existing uses might rely on the behavior of
-     * that function so be careful in choosing which version to use.
      */
     const char* codeWScopeScopeData() const {
         return codeWScopeCode() + codeWScopeCodeLen();
@@ -703,13 +728,12 @@ public:
 
     // @param maxLen don't scan more than maxLen bytes
     explicit BSONElement(const char* d) : data(d) {
-        if (eoo()) {
-            fieldNameSize_ = 0;
-            totalSize = 1;
-        } else {
-            fieldNameSize_ = strlen(d + 1 /*skip type*/) + 1 /*include NUL byte*/;
-            totalSize = computeSize();
-        }
+        // While we should skip the type, and add 1 for the terminating null byte, just include
+        // the type byte in the strlen call: the extra byte cancels out. As an extra bonus, this
+        // also handles the EOO case, where the type byte is 0.
+        uint8_t type = *d;
+        fieldNameSize_ = strlen(d);
+        totalSize = computeSize(type, d, fieldNameSize_);
     }
 
     struct CachedSizeTag {};  // Opts in to next constructor.
@@ -730,7 +754,7 @@ public:
                 fieldNameSize_ = fieldNameSize;
             }
             if (totalSize == -1) {
-                this->totalSize = computeSize();
+                this->totalSize = computeSize(*d, d, fieldNameSize_);
             } else {
                 this->totalSize = totalSize;
             }
@@ -747,6 +771,9 @@ public:
     bool coerce(Decimal128* out) const;
     bool coerce(std::vector<std::string>* out) const;
 
+    template <typename T>
+    Status tryCoerce(T* out) const;
+
     /**
      * Constant double representation of 2^63, the smallest value that will overflow a long long.
      *
@@ -758,6 +785,14 @@ public:
     static const double kLongLongMaxPlusOneAsDouble;
 
 private:
+    template <typename Generator>
+    BSONObj _jsonStringGenerator(const Generator& g,
+                                 bool includeSeparator,
+                                 bool includeFieldNames,
+                                 int pretty,
+                                 fmt::memory_buffer& buffer,
+                                 size_t writeLimit) const;
+
     const char* data;
     int fieldNameSize_;  // internal size includes null terminator
     int totalSize;
@@ -778,7 +813,7 @@ private:
     }
 
     // Only called from constructors.
-    int computeSize() const;
+    static int computeSize(int8_t type, const char* data, int fieldNameSize);
 };
 
 inline bool BSONElement::trueValue() const {
@@ -846,8 +881,6 @@ inline double BSONElement::numberDouble() const {
     }
 }
 
-/** Retrieve int value for the element safely.  Zero returned if not a number. Converted to int if
- * another numeric type. */
 inline int BSONElement::numberInt() const {
     switch (type()) {
         case NumberDouble:
@@ -863,7 +896,6 @@ inline int BSONElement::numberInt() const {
     }
 }
 
-/** Retrieve long value for the element safely.  Zero returned if not a number. */
 inline long long BSONElement::numberLong() const {
     switch (type()) {
         case NumberDouble:
@@ -917,6 +949,61 @@ inline long long BSONElement::safeNumberLong() const {
     }
 }
 
+/**
+ * Attempt to coerce the BSONElement to a primitive type.
+ * For integral targets, we do additional checking that the
+ * source file is a finite real number and fits within the
+ * target type.
+ */
+template <typename T>
+Status BSONElement::tryCoerce(T* out) const {
+    if constexpr (std::is_integral<T>::value && !std::is_same<bool, T>::value) {
+        if (type() == NumberDouble) {
+            double d = numberDouble();
+            if (!std::isfinite(d)) {
+                return {ErrorCodes::BadValue, "Unable to coerce NaN/Inf to integral type"};
+            }
+            if ((d > std::numeric_limits<T>::max()) || (d < std::numeric_limits<T>::lowest())) {
+                return {ErrorCodes::BadValue, "Out of bounds coercing to integral value"};
+            }
+        } else if (type() == NumberDecimal) {
+            Decimal128 d = numberDecimal();
+            if (!d.isFinite()) {
+                return {ErrorCodes::BadValue, "Unable to coerce NaN/Inf to integral type"};
+            }
+            if (d.isGreater(Decimal128(std::numeric_limits<T>::max())) ||
+                d.isLess(Decimal128(std::numeric_limits<T>::lowest()))) {
+                return {ErrorCodes::BadValue, "Out of bounds coercing to integral value"};
+            }
+        } else if (type() == mongo::Bool) {
+            *out = Bool();
+            return Status::OK();
+        }
+
+        long long val;
+        if (!coerce(&val)) {
+            return {ErrorCodes::BadValue, "Unable to coerce value to integral type"};
+        }
+
+        if (std::is_same<long long, T>::value) {
+            *out = val;
+            return Status::OK();
+        }
+
+        if ((val > std::numeric_limits<T>::max()) || (val < std::numeric_limits<T>::lowest())) {
+            return {ErrorCodes::BadValue, "Out of bounds coercing to integral value"};
+        }
+
+        *out = static_cast<T>(val);
+        return Status::OK();
+    }
+
+    if (!coerce(out)) {
+        return {ErrorCodes::BadValue, "Unable to coerce value to correct type"};
+    }
+
+    return Status::OK();
+}
 /**
  * This safeNumberLongForHash() function does the same thing as safeNumberLong, but it preserves
  * edge-case behavior from older versions. It's provided for use by hash functions that need to
