@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -28,19 +27,21 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/db/jsobj.h"
 
 #include "mongo/base/data_range.h"
 #include "mongo/bson/bson_validate.h"
 #include "mongo/bson/bsonelement_comparator_interface.h"
+#include "mongo/bson/generator_extended_canonical_2_0_0.h"
+#include "mongo/bson/generator_extended_relaxed_2_0_0.h"
+#include "mongo/bson/generator_legacy_strict.h"
 #include "mongo/db/json.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/allocator.h"
 #include "mongo/util/hex.h"
-#include "mongo/util/log.h"
-#include "mongo/util/mongoutils/str.h"
-#include "mongo/util/stringutils.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -84,8 +85,6 @@ int compareObjects(const BSONObj& firstObj,
 
 }  // namespace
 
-using namespace std;
-
 /* BSONObj ------------------------------------------------------------*/
 
 void BSONObj::_assertInvalid(int maxSize) const {
@@ -103,8 +102,29 @@ void BSONObj::_assertInvalid(int maxSize) const {
 }
 
 BSONObj BSONObj::copy() const {
-    auto storage = SharedBuffer::allocate(objsize());
-    memcpy(storage.get(), objdata(), objsize());
+    // The undefined behavior checks in this function are best-effort and attempt to detect
+    // undefined behavior as early as possible. We cannot make any guarantees about detection
+    // because we are observing undefined state, and we assume the compiler does not make either
+    // of the following optimizations: a) optimizing away the call to objsize() on freed memory, and
+    // b) optimizing away the two sequential calls to objsize() as one.
+    // The behavior of this function must degrade as gracefully as possible under violation of
+    // those assumptions, and preserving any currently observed behavior does not form an argument
+    // against the later application of such optimizations.
+    int size = objsize();
+    _validateUnownedSize(size);
+    auto storage = SharedBuffer::allocate(size);
+
+    // If the call to objsize() changes between this call and the previous one, this indicates that
+    // that the memory we are reading has changed, and we must exit immediately to avoid further
+    // undefined behavior.
+    if (int sizeAfter = objsize(); sizeAfter != size) {
+        LOGV2_FATAL(
+            31323,
+            "BSONObj::copy() - size {sizeAfter} differs from previously observed size {size}",
+            "sizeAfter"_attr = sizeAfter,
+            "size"_attr = size);
+    }
+    memcpy(storage.get(), objdata(), size);
     return BSONObj(std::move(storage));
 }
 
@@ -114,31 +134,134 @@ BSONObj BSONObj::getOwned() const {
     return copy();
 }
 
-string BSONObj::jsonString(JsonStringFormat format, int pretty, bool isArray) const {
-    if (isEmpty())
-        return isArray ? "[]" : "{}";
+BSONObj BSONObj::getOwned(const BSONObj& obj) {
+    return obj.getOwned();
+}
 
-    StringBuilder s;
-    s << (isArray ? "[ " : "{ ");
-    BSONObjIterator i(*this);
-    BSONElement e = i.next();
-    if (!e.eoo())
-        while (1) {
-            s << e.jsonString(format, !isArray, pretty ? pretty + 1 : 0);
-            e = i.next();
-            if (e.eoo())
-                break;
-            s << ",";
-            if (pretty) {
-                s << '\n';
-                for (int x = 0; x < pretty; x++)
-                    s << "  ";
-            } else {
-                s << " ";
+BSONObj BSONObj::redact() const {
+    _validateUnownedSize(objsize());
+
+    // Helper to get an "internal function" to be able to do recursion
+    struct redactor {
+        void operator()(BSONObjBuilder& builder, const BSONObj& obj) {
+            for (BSONElement e : obj) {
+                if (e.type() == Object || e.type() == Array) {
+                    BSONObjBuilder subBuilder = builder.subobjStart(e.fieldNameStringData());
+                    operator()(subBuilder, e.Obj());
+                    subBuilder.done();
+                } else {
+                    builder.append(e.fieldNameStringData(), "###"_sd);
+                }
             }
         }
-    s << (isArray ? " ]" : " }");
-    return s.str();
+    };
+
+    BSONObjBuilder builder;
+    redactor()(builder, *this);
+    return builder.obj();
+}
+
+void BSONObj::_validateUnownedSize(int size) const {
+    // Only for unowned objects, the size is validated in the constructor, so it is an error for
+    // the size to ever be invalid. This means that the unowned memory we are reading has
+    // changed, and we must exit immediately to avoid further undefined behavior.
+    if (!isOwned() && (size < kMinBSONLength || size > BufferMaxSize)) {
+        LOGV2_FATAL(31322,
+                    "BSONObj::_validateUnownedSize() - size {size} of unowned BSONObj is invalid "
+                    "and differs from previously validated size.",
+                    "size"_attr = size);
+    }
+}
+
+template <typename Generator>
+BSONObj BSONObj::_jsonStringGenerator(const Generator& g,
+                                      int pretty,
+                                      bool isArray,
+                                      fmt::memory_buffer& buffer,
+                                      size_t writeLimit) const {
+    if (isEmpty()) {
+        const auto empty = isArray ? "[]"_sd : "{}"_sd;
+        buffer.append(empty.rawData(), empty.rawData() + empty.size());
+        return BSONObj();
+    }
+    buffer.push_back(isArray ? '[' : '{');
+
+    BSONObjIterator i(*this);
+    BSONElement e = i.next();
+    BSONObj truncation;
+    if (!e.eoo()) {
+        bool writeSeparator = false;
+        while (1) {
+            truncation = e.jsonStringGenerator(
+                g, writeSeparator, !isArray, pretty ? (pretty + 1) : 0, buffer, writeLimit);
+            e = i.next();
+            if (!truncation.isEmpty() || e.eoo()) {
+                g.writePadding(buffer);
+                break;
+            }
+            writeSeparator = true;
+        }
+    }
+
+    if (pretty)
+        fmt::format_to(buffer, "\n{:<{}}", "", (pretty - 1) * 4);
+    buffer.push_back(isArray ? ']' : '}');
+    return truncation;
+}
+
+BSONObj BSONObj::jsonStringGenerator(ExtendedCanonicalV200Generator const& generator,
+                                     int pretty,
+                                     bool isArray,
+                                     fmt::memory_buffer& buffer,
+                                     size_t writeLimit) const {
+    return _jsonStringGenerator(generator, pretty, isArray, buffer, writeLimit);
+}
+BSONObj BSONObj::jsonStringGenerator(ExtendedRelaxedV200Generator const& generator,
+                                     int pretty,
+                                     bool isArray,
+                                     fmt::memory_buffer& buffer,
+                                     size_t writeLimit) const {
+    return _jsonStringGenerator(generator, pretty, isArray, buffer, writeLimit);
+}
+BSONObj BSONObj::jsonStringGenerator(LegacyStrictGenerator const& generator,
+                                     int pretty,
+                                     bool isArray,
+                                     fmt::memory_buffer& buffer,
+                                     size_t writeLimit) const {
+    return _jsonStringGenerator(generator, pretty, isArray, buffer, writeLimit);
+}
+
+std::string BSONObj::jsonString(JsonStringFormat format,
+                                int pretty,
+                                bool isArray,
+                                size_t writeLimit,
+                                BSONObj* outTruncationResult) const {
+    fmt::memory_buffer buffer;
+    BSONObj truncation = jsonStringBuffer(format, pretty, isArray, buffer);
+    if (outTruncationResult) {
+        *outTruncationResult = truncation;
+    }
+    return fmt::to_string(buffer);
+}
+
+BSONObj BSONObj::jsonStringBuffer(JsonStringFormat format,
+                                  int pretty,
+                                  bool isArray,
+                                  fmt::memory_buffer& buffer,
+                                  size_t writeLimit) const {
+    auto withGenerator = [&](auto&& gen) {
+        return jsonStringGenerator(gen, pretty, isArray, buffer, writeLimit);
+    };
+
+    if (format == ExtendedCanonicalV2_0_0) {
+        return withGenerator(ExtendedCanonicalV200Generator());
+    } else if (format == ExtendedRelaxedV2_0_0) {
+        return withGenerator(ExtendedRelaxedV200Generator(dateFormatIsLocalTimezone()));
+    } else if (format == LegacyStrict) {
+        return withGenerator(LegacyStrictGenerator());
+    } else {
+        MONGO_UNREACHABLE;
+    }
 }
 
 bool BSONObj::valid(BSONVersion version) const {
@@ -212,7 +335,7 @@ bool BSONObj::isFieldNamePrefixOf(const BSONObj& otherObj) const {
     while (a.more() && b.more()) {
         BSONElement x = a.next();
         BSONElement y = b.next();
-        if (!str::equals(x.fieldName(), y.fieldName())) {
+        if (x.fieldNameStringData() != y.fieldNameStringData()) {
             return false;
         }
     }
@@ -220,8 +343,7 @@ bool BSONObj::isFieldNamePrefixOf(const BSONObj& otherObj) const {
     return !a.more();
 }
 
-BSONObj BSONObj::extractFieldsUnDotted(const BSONObj& pattern) const {
-    BSONObjBuilder b;
+void BSONObj::extractFieldsUndotted(BSONObjBuilder* b, const BSONObj& pattern) const {
     BSONObjIterator i(pattern);
     while (i.moreWithEOO()) {
         BSONElement e = i.next();
@@ -229,13 +351,17 @@ BSONObj BSONObj::extractFieldsUnDotted(const BSONObj& pattern) const {
             break;
         BSONElement x = getField(e.fieldName());
         if (!x.eoo())
-            b.appendAs(x, "");
+            b->appendAs(x, "");
     }
+}
+
+BSONObj BSONObj::extractFieldsUndotted(const BSONObj& pattern) const {
+    BSONObjBuilder b;
+    extractFieldsUndotted(&b, pattern);
     return b.obj();
 }
 
-BSONObj BSONObj::filterFieldsUndotted(const BSONObj& filter, bool inFilter) const {
-    BSONObjBuilder b;
+void BSONObj::filterFieldsUndotted(BSONObjBuilder* b, const BSONObj& filter, bool inFilter) const {
     BSONObjIterator i(*this);
     while (i.moreWithEOO()) {
         BSONElement e = i.next();
@@ -243,8 +369,13 @@ BSONObj BSONObj::filterFieldsUndotted(const BSONObj& filter, bool inFilter) cons
             break;
         BSONElement x = filter.getField(e.fieldName());
         if ((x.eoo() && !inFilter) || (!x.eoo() && inFilter))
-            b.append(e);
+            b->append(e);
     }
+}
+
+BSONObj BSONObj::filterFieldsUndotted(const BSONObj& filter, bool inFilter) const {
+    BSONObjBuilder b;
+    filterFieldsUndotted(&b, filter, inFilter);
     return b.obj();
 }
 
@@ -272,24 +403,10 @@ BSONElement BSONObj::getFieldUsingIndexNames(StringData fieldName, const BSONObj
     return BSONElement();
 }
 
-/* grab names of all the fields in this object */
-int BSONObj::getFieldNames(set<string>& fields) const {
-    int n = 0;
-    BSONObjIterator i(*this);
-    while (i.moreWithEOO()) {
-        BSONElement e = i.next();
-        if (e.eoo())
-            break;
-        fields.insert(e.fieldName());
-        n++;
-    }
-    return n;
-}
-
 /* note: addFields always adds _id even if not specified
    returns n added not counting _id unless requested.
 */
-int BSONObj::addFields(BSONObj& from, set<string>& fields) {
+int BSONObj::addFields(BSONObj& from, std::set<std::string>& fields) {
     verify(isEmpty() && !isOwned()); /* partial implementation for now... */
 
     BSONObjBuilder b;
@@ -331,7 +448,7 @@ bool BSONObj::couldBeArray() const {
             break;
 
         // TODO:  If actually important, may be able to do int->char* much faster
-        if (strcmp(e.fieldName(), ((string)(str::stream() << index)).c_str()) != 0)
+        if (strcmp(e.fieldName(), static_cast<std::string>(str::stream() << index).c_str()) != 0)
             return false;
         index++;
     }
@@ -384,6 +501,25 @@ BSONObj BSONObj::replaceFieldNames(const BSONObj& names) const {
     return b.obj();
 }
 
+BSONObj BSONObj::stripFieldNames(const BSONObj& obj) {
+    if (!obj.hasFieldNames())
+        return obj;
+
+    BSONObjBuilder bb;
+    for (auto e : obj) {
+        bb.appendAs(e, StringData());
+    }
+    return bb.obj();
+}
+
+bool BSONObj::hasFieldNames() const {
+    for (auto e : *this) {
+        if (e.fieldName()[0])
+            return true;
+    }
+    return false;
+}
+
 Status BSONObj::storageValidEmbedded() const {
     BSONObjIterator i(*this);
 
@@ -391,26 +527,26 @@ Status BSONObj::storageValidEmbedded() const {
     bool first = true;
     while (i.more()) {
         BSONElement e = i.next();
-        const char* name = e.fieldName();
+        StringData name = e.fieldNameStringData();
 
         // Cannot start with "$", unless dbref which must start with ($ref, $id)
-        if (str::startsWith(name, '$')) {
+        if (name.startsWith("$")) {
             if (first &&
                 // $ref is a collection name and must be a String
-                str::equals(name, "$ref") &&
-                e.type() == String && str::equals(i.next().fieldName(), "$id")) {
+                (name == "$ref") && e.type() == String &&
+                (i.next().fieldNameStringData() == "$id")) {
                 first = false;
                 // keep inspecting fields for optional "$db"
                 e = i.next();
-                name = e.fieldName();  // "" if eoo()
+                name = e.fieldNameStringData();  // "" if eoo()
 
                 // optional $db field must be a String
-                if (str::equals(name, "$db") && e.type() == String) {
+                if ((name == "$db") && e.type() == String) {
                     continue;  // this element is fine, so continue on to siblings (if any more)
                 }
 
                 // Can't start with a "$", all other checks are done below (outside if blocks)
-                if (str::startsWith(name, '$')) {
+                if (name.startsWith("$")) {
                     return Status(ErrorCodes::DollarPrefixedFieldName,
                                   str::stream() << name << " is not valid for storage.");
                 }
@@ -447,18 +583,6 @@ Status BSONObj::storageValidEmbedded() const {
     return Status::OK();
 }
 
-void BSONObj::dump() const {
-    LogstreamBuilder builder = log();
-    builder << hex;
-    const char* p = objdata();
-    for (int i = 0; i < objsize(); i++) {
-        builder << i << '\t' << (0xff & ((unsigned)*p));
-        if (*p >= 'A' && *p <= 'z')
-            builder << '\t' << *p;
-        p++;
-    }
-}
-
 void BSONObj::getFields(unsigned n, const char** fieldNames, BSONElement* fields) const {
     BSONObjIterator i(*this);
     while (i.more()) {
@@ -628,10 +752,13 @@ void BSONObj::toString(
     s << (isArray ? " ]" : " }");
 }
 
-Status DataType::Handler<BSONObj>::store(
-    const BSONObj& bson, char* ptr, size_t length, size_t* advanced, std::ptrdiff_t debug_offset) {
+Status DataType::Handler<BSONObj>::store(const BSONObj& bson,
+                                         char* ptr,
+                                         size_t length,
+                                         size_t* advanced,
+                                         std::ptrdiff_t debug_offset) noexcept try {
     if (bson.objsize() > static_cast<int>(length)) {
-        mongoutils::str::stream ss;
+        str::stream ss;
         ss << "buffer too small to write bson of size (" << bson.objsize()
            << ") at offset: " << debug_offset;
         return Status(ErrorCodes::Overflow, ss);
@@ -646,6 +773,8 @@ Status DataType::Handler<BSONObj>::store(
     }
 
     return Status::OK();
+} catch (const DBException& e) {
+    return e.toStatus();
 }
 
 std::ostream& operator<<(std::ostream& s, const BSONObj& o) {
@@ -664,7 +793,7 @@ public:
     bool operator()(const char* s1, const char* s2) const;
 
 private:
-    LexNumCmp _cmp;
+    str::LexNumCmp _cmp;
 };
 
 BSONIteratorSorted::ElementFieldCmp::ElementFieldCmp(bool isArray) : _cmp(!isArray) {}
