@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -32,6 +31,7 @@
 
 #include <memory>
 
+#include "mongo/client/authenticate.h"
 #include "mongo/db/service_context.h"
 #include "mongo/executor/network_connection_hook.h"
 #include "mongo/executor/remote_command_request.h"
@@ -61,16 +61,30 @@ public:
                                   Milliseconds timeout);
 
     Future<executor::RemoteCommandResponse> runCommandRequest(
-        executor::RemoteCommandRequest request, const transport::BatonHandle& baton = nullptr);
+        executor::RemoteCommandRequest request, const BatonHandle& baton = nullptr);
     Future<rpc::UniqueReply> runCommand(OpMsgRequest request,
-                                        const transport::BatonHandle& baton = nullptr);
+                                        const BatonHandle& baton = nullptr,
+                                        bool fireAndForget = false);
+
+    Future<executor::RemoteCommandResponse> beginExhaustCommandRequest(
+        executor::RemoteCommandRequest request, const BatonHandle& baton = nullptr);
+    Future<executor::RemoteCommandResponse> runExhaustCommand(OpMsgRequest request,
+                                                              const BatonHandle& baton = nullptr);
+    Future<executor::RemoteCommandResponse> awaitExhaustCommand(const BatonHandle& baton = nullptr);
 
     Future<void> authenticate(const BSONObj& params);
 
+    Future<void> authenticateInternal(boost::optional<std::string> mechanismHint);
+
+    Future<bool> completeSpeculativeAuth(std::shared_ptr<SaslClientSession> session,
+                                         std::string authDB,
+                                         BSONObj specAuth,
+                                         auth::SpeculativeAuthType speculativeAuthtype);
+
     Future<void> initWireVersion(const std::string& appName,
                                  executor::NetworkConnectionHook* const hook);
 
-    void cancel(const transport::BatonHandle& baton = nullptr);
+    void cancel(const BatonHandle& baton = nullptr);
 
     bool isStillConnected();
 
@@ -80,10 +94,18 @@ public:
     const HostAndPort& local() const;
 
 private:
-    Future<Message> _call(Message request, const transport::BatonHandle& baton = nullptr);
-    BSONObj _buildIsMasterRequest(const std::string& appName);
+    Future<executor::RemoteCommandResponse> _continueReceiveExhaustResponse(
+        ClockSource::StopWatch stopwatch,
+        boost::optional<int32_t> msgId,
+        const BatonHandle& baton = nullptr);
+    Future<Message> _waitForResponse(boost::optional<int32_t> msgId,
+                                     const BatonHandle& baton = nullptr);
+    Future<void> _call(Message request, int32_t msgId, const BatonHandle& baton = nullptr);
+    BSONObj _buildIsMasterRequest(const std::string& appName,
+                                  executor::NetworkConnectionHook* hook);
     void _parseIsMasterResponse(BSONObj request,
                                 const std::unique_ptr<rpc::ReplyInterface>& response);
+    auth::RunCommandHook _makeAuthRunCommandHook();
 
     const HostAndPort _peer;
     transport::SessionHandle _session;
