@@ -183,6 +183,31 @@ TEST_F(ServerSelectorTestFixture, ShouldReturnNoneIfTopologyUnknown) {
     ASSERT_EQ(boost::none, selector.selectServers(topologyDescription, ReadPreferenceSetting()));
 }
 
+TEST_F(ServerSelectorTestFixture, ShouldBeAbleToSelectWithMaxStalenessFromClonedTopology) {
+    TopologyStateMachine stateMachine(sdamConfiguration);
+    auto topologyDescription = std::make_shared<TopologyDescription>(sdamConfiguration);
+    auto primary = ServerDescriptionBuilder()
+                       .withAddress(HostAndPort("s0"))
+                       .withType(ServerType::kRSPrimary)
+                       .withLastUpdateTime(Date_t::now())
+                       .withLastWriteDate(Date_t::now())
+                       .withRtt(Milliseconds{1})
+                       .withSetName("set")
+                       .withHost(HostAndPort("s0"))
+                       .withMinWireVersion(WireVersion::SUPPORTS_OP_MSG)
+                       .withMaxWireVersion(WireVersion::LATEST_WIRE_VERSION)
+                       .instance();
+    stateMachine.onServerDescription(*topologyDescription, primary);
+    topologyDescription = TopologyDescription::clone(topologyDescription);
+
+    const auto ninetySeconds = Seconds(90);
+    const auto readPref =
+        ReadPreferenceSetting(ReadPreference::Nearest, TagSets::emptySet, ninetySeconds);
+    auto result = selector.selectServers(topologyDescription, readPref);
+    ASSERT(result);
+    ASSERT_EQ(primary->getAddress(), (*result)[0]->getAddress());
+}
+
 TEST_F(ServerSelectorTestFixture, ShouldSelectRandomlyWhenMultipleOptionsAreAvailable) {
     TopologyStateMachine stateMachine(sdamConfiguration);
     auto topologyDescription = std::make_shared<TopologyDescription>(sdamConfiguration);
