@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -41,7 +40,7 @@
 #include "mongo/db/auth/role_graph.h"
 #include "mongo/db/auth/user_management_commands_parser.h"
 #include "mongo/db/update/update_driver.h"
-#include "mongo/util/mongoutils/str.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 
@@ -91,12 +90,9 @@ Status checkIdMatchesRoleName(const BSONElement& idElement, const RoleName& role
     if (firstDot == std::string::npos || idField.substr(0, firstDot) != roleName.getDB() ||
         idField.substr(firstDot + 1) != roleName.getRole()) {
         return Status(ErrorCodes::FailedToParse,
-                      mongoutils::str::stream()
-                          << "Role document _id fields must be encoded as the string "
-                             "dbname.rolename.  Found "
-                          << idField
-                          << " for "
-                          << roleName.getFullName());
+                      str::stream() << "Role document _id fields must be encoded as the string "
+                                       "dbname.rolename.  Found "
+                                    << idField << " for " << roleName.getFullName());
     }
     return Status::OK();
 }
@@ -198,6 +194,7 @@ Status handleOplogInsert(RoleGraph* roleGraph, const BSONObj& insertedObj) {
  */
 Status handleOplogUpdate(OperationContext* opCtx,
                          RoleGraph* roleGraph,
+                         const NamespaceString& nss,
                          const BSONObj& updatePattern,
                          const BSONObj& queryPattern) {
     RoleName roleToUpdate;
@@ -205,15 +202,13 @@ Status handleOplogUpdate(OperationContext* opCtx,
     if (!status.isOK())
         return status;
 
-    boost::intrusive_ptr<ExpressionContext> expCtx(new ExpressionContext(opCtx, nullptr));
+    boost::intrusive_ptr<ExpressionContext> expCtx(new ExpressionContext(opCtx, nullptr, nss));
     UpdateDriver driver(std::move(expCtx));
     driver.setFromOplogApplication(true);
 
     // Oplog updates do not have array filters.
     std::map<StringData, std::unique_ptr<ExpressionWithPlaceholder>> arrayFilters;
-    status = driver.parse(updatePattern, arrayFilters);
-    if (!status.isOK())
-        return status;
+    driver.parse(updatePattern, arrayFilters);
 
     mutablebson::Document roleDocument;
     status = RoleGraph::getBSONForRole(roleGraph, roleToUpdate, roleDocument.root());
@@ -230,7 +225,9 @@ Status handleOplogUpdate(OperationContext* opCtx,
 
     const bool validateForStorage = false;
     const FieldRefSet emptyImmutablePaths;
-    status = driver.update(StringData(), &roleDocument, validateForStorage, emptyImmutablePaths);
+    bool isInsert = false;
+    status = driver.update(
+        StringData(), &roleDocument, validateForStorage, emptyImmutablePaths, isInsert);
     if (!status.isOK())
         return status;
 
@@ -299,6 +296,10 @@ Status handleOplogCommand(RoleGraph* roleGraph, const BSONObj& cmdObj) {
         return Status::OK();
     }
 
+    if (cmdName == "commitTransaction" || cmdName == "abortTransaction") {
+        return Status::OK();
+    }
+
     if (cmdName == "dropIndexes" || cmdName == "deleteIndexes") {
         return Status::OK();
     }
@@ -307,26 +308,39 @@ Status handleOplogCommand(RoleGraph* roleGraph, const BSONObj& cmdObj) {
         // We don't care about these if they're not on the roles collection.
         return Status::OK();
     }
-
     if (cmdName == "createIndexes" &&
         cmdObj.firstElement().str() == rolesCollectionNamespace.coll()) {
         UnorderedFieldsBSONObjComparator instance;
-        if (instance.evaluate(cmdObj == (BSON("createIndexes"
-                                              << "system.roles"
-                                              << "v"
-                                              << 2
-                                              << "name"
-                                              << "role_1_db_1"
-                                              << "key"
-                                              << BSON("role" << 1 << "db" << 1)
-                                              << "unique"
-                                              << true)))) {
+        if (instance.evaluate(
+                cmdObj ==
+                (BSON("createIndexes"
+                      << "system.roles"
+                      << "v" << 2 << "name"
+                      << "role_1_db_1"
+                      << "key" << BSON("role" << 1 << "db" << 1) << "unique" << true)))) {
             return Status::OK();
         }
     }
+    if (cmdName == "startIndexBuild" || cmdName == "abortIndexBuild" ||
+        cmdName == "commitIndexBuild") {
+        if (cmdObj.firstElement().str() != rolesCollectionNamespace.coll()) {
+            return Status::OK();
+        }
+        for (auto indexSpecElem : cmdObj["indexes"].Array()) {
+            UnorderedFieldsBSONObjComparator instance;
+            auto indexSpec = indexSpecElem.Obj();
+            if (instance.evaluate(
+                    indexSpec ==
+                    (BSON("v" << 2 << "name"
+                              << "role_1_db_1"
+                              << "key" << BSON("role" << 1 << "db" << 1) << "unique" << true)))) {
+                return Status::OK();
+            }
+        }
+    }
 
-    if ((cmdName == "collMod") && (cmdObj.nFields() == 1)) {
-        // We also don't care about empty modifications even if they are on roles collection
+    if (cmdName == "collMod" && cmdObj.nFields() == 1) {
+        // We don't care about empty modifications, even if they are on roles collection.
         return Status::OK();
     }
 
@@ -353,8 +367,7 @@ Status RoleGraph::handleLogOp(OperationContext* opCtx,
         return Status::OK();
     if (op[0] == '\0' || op[1] != '\0') {
         return Status(ErrorCodes::BadValue,
-                      mongoutils::str::stream() << "Unrecognized \"op\" field value \"" << op
-                                                << '"');
+                      str::stream() << "Unrecognized \"op\" field value \"" << op << '"');
     }
 
     if (ns.db() != AuthorizationManager::rolesCollectionNamespace.db())
@@ -379,7 +392,7 @@ Status RoleGraph::handleLogOp(OperationContext* opCtx,
                 return Status(ErrorCodes::InternalError,
                               "Missing query pattern in update oplog entry.");
             }
-            return handleOplogUpdate(opCtx, this, o, *o2);
+            return handleOplogUpdate(opCtx, this, ns, o, *o2);
         case 'd':
             return handleOplogDelete(this, o);
         case 'n':
@@ -389,8 +402,7 @@ Status RoleGraph::handleLogOp(OperationContext* opCtx,
                           "Namespace admin.system.roles is not a valid target for commands");
         default:
             return Status(ErrorCodes::BadValue,
-                          mongoutils::str::stream() << "Unrecognized \"op\" field value \"" << op
-                                                    << '"');
+                          str::stream() << "Unrecognized \"op\" field value \"" << op << '"');
     }
 }
 
