@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kAccessControl
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
 #include "mongo/platform/basic.h"
 
@@ -52,8 +52,8 @@
 #include "mongo/db/auth/security_file.h"
 #include "mongo/db/auth/user.h"
 #include "mongo/db/server_options.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/icu.h"
-#include "mongo/util/log.h"
 #include "mongo/util/password_digest.h"
 
 namespace mongo {
@@ -70,15 +70,23 @@ public:
 
     boost::optional<User::CredentialData> generate(const std::string& password) {
         if (password.size() < kMinKeyLength || password.size() > kMaxKeyLength) {
-            error() << " security key in " << _filename << " has length " << password.size()
-                    << ", must be between 6 and 1024 chars";
+            LOGV2_ERROR(
+                20255,
+                "Security key in {filename} has length {size}, must be between {minimumLength} "
+                "and {maximumLength} characters",
+                "Security key size is out range",
+                "filename"_attr = _filename,
+                "size"_attr = password.size(),
+                "minimumLength"_attr = kMinKeyLength,
+                "maximumLength"_attr = kMaxKeyLength);
             return boost::none;
         }
 
         auto swSaslPassword = icuSaslPrep(password);
         if (!swSaslPassword.isOK()) {
-            error() << "Could not prep security key file for SCRAM-SHA-256: "
-                    << swSaslPassword.getStatus();
+            LOGV2_ERROR(20256,
+                        "Could not prep security key file for SCRAM-SHA-256",
+                        "error"_attr = swSaslPassword.getStatus());
             return boost::none;
         }
         const auto passwordDigest = mongo::createPasswordDigest(
@@ -109,7 +117,9 @@ private:
         target.storedKey = source[scram::kStoredKeyFieldName].String();
         target.serverKey = source[scram::kServerKeyFieldName].String();
         if (!target.isValid()) {
-            error() << "Could not generate valid credentials from key in " << _filename;
+            LOGV2_ERROR(20257,
+                        "Could not generate valid credentials from key",
+                        "filename"_attr = _filename);
             return false;
         }
 
@@ -128,15 +138,19 @@ using std::string;
 bool setUpSecurityKey(const string& filename) {
     auto swKeyStrings = mongo::readSecurityFile(filename);
     if (!swKeyStrings.isOK()) {
-        log() << swKeyStrings.getStatus().reason();
+        LOGV2(20254, "Read security file failed", "error"_attr = swKeyStrings.getStatus());
         return false;
     }
 
     auto keyStrings = std::move(swKeyStrings.getValue());
 
     if (keyStrings.size() > 2) {
-        error() << "Only two keys are supported in the security key file, " << keyStrings.size()
-                << " are specified in " << filename;
+        LOGV2_ERROR(20258,
+                    "Only two keys are supported in the security key file, {numKeys} are "
+                    "specified in {filename}",
+                    "Only two keys are supported in the security key file",
+                    "numKeys"_attr = keyStrings.size(),
+                    "filename"_attr = filename);
         return false;
     }
 
