@@ -38,16 +38,14 @@ namespace mongo {
 class IndexConsistency;
 class CollectionCatalog;
 class CollectionImpl final : public Collection, public CappedCallback {
-private:
-    static const int kMagicNumber = 1357924;
-
 public:
     enum ValidationAction { WARN, ERROR_V };
     enum ValidationLevel { OFF, MODERATE, STRICT_V };
 
     explicit CollectionImpl(OperationContext* opCtx,
                             const NamespaceString& nss,
-                            OptionalCollectionUUID uuid,
+                            RecordId catalogId,
+                            UUID uuid,
                             std::unique_ptr<RecordStore> recordStore);
 
     ~CollectionImpl();
@@ -56,29 +54,22 @@ public:
     public:
         std::unique_ptr<Collection> make(OperationContext* opCtx,
                                          const NamespaceString& nss,
+                                         RecordId catalogId,
                                          CollectionUUID uuid,
                                          std::unique_ptr<RecordStore> rs) const final;
     };
 
-    bool ok() const final {
-        return _magic == kMagicNumber;
-    }
-
-    CollectionInfoCache* infoCache() final {
-        return _infoCache.get();
-    }
-
-    const CollectionInfoCache* infoCache() const final {
-        return _infoCache.get();
-    }
-
     const NamespaceString& ns() const final {
         return _ns;
     }
 
     void setNs(NamespaceString nss) final;
 
-    OptionalCollectionUUID uuid() const {
+    RecordId getCatalogId() const {
+        return _catalogId;
+    }
+
+    UUID uuid() const {
         return _uuid;
     }
 
@@ -98,6 +89,10 @@ public:
         return _recordStore.get();
     }
 
+    const BSONObj getValidatorDoc() const final {
+        return _validator.validatorDoc.getOwned();
+    }
+
     bool requiresIdIndex() const final;
 
     Snapshotted<BSONObj> docFor(OperationContext* opCtx, RecordId loc) const final {
@@ -167,9 +162,8 @@ public:
      * this method.
      */
     Status insertDocumentsForOplog(OperationContext* opCtx,
-                                   const DocWriter* const* docs,
-                                   Timestamp* timestamps,
-                                   size_t nDocs) final;
+                                   std::vector<Record>* records,
+                                   const std::vector<Timestamp>& timestamps) final;
 
     /**
      * Inserts a document into the record store for a bulk loader that manages the index building
@@ -227,25 +221,6 @@ public:
      */
     Status truncate(OperationContext* opCtx) final;
 
-    /**
-     * @return OK if the validate run successfully
-     *         OK will be returned even if corruption is found
-     *         deatils will be in result
-     */
-    Status validate(OperationContext* opCtx,
-                    ValidateCmdLevel level,
-                    bool background,
-                    ValidateResults* results,
-                    BSONObjBuilder* output) final;
-
-    /**
-     * forces data into cache
-     */
-    Status touch(OperationContext* opCtx,
-                 bool touchData,
-                 bool touchIndexes,
-                 BSONObjBuilder* output) const final;
-
     /**
      * Truncate documents newer than the document at 'end' from the capped
      * collection.  The collection cannot be completely emptied using this
@@ -260,12 +235,11 @@ public:
     /**
      * Returns a non-ok Status if validator is not legal for this collection.
      */
-    StatusWithMatchExpression parseValidator(
-        OperationContext* opCtx,
-        const BSONObj& validator,
-        MatchExpressionParser::AllowedFeatureSet allowedFeatures,
-        boost::optional<ServerGlobalParams::FeatureCompatibility::Version>
-            maxFeatureCompatibilityVersion = boost::none) const final;
+    Validator parseValidator(OperationContext* opCtx,
+                             const BSONObj& validator,
+                             MatchExpressionParser::AllowedFeatureSet allowedFeatures,
+                             boost::optional<ServerGlobalParams::FeatureCompatibility::Version>
+                                 maxFeatureCompatibilityVersion = boost::none) const final;
 
     /**
      * Sets the validator for this collection.
@@ -273,7 +247,7 @@ public:
      * An empty validator removes all validation.
      * Requires an exclusive lock on the collection.
      */
-    Status setValidator(OperationContext* opCtx, BSONObj validatorDoc) final;
+    void setValidator(OperationContext* opCtx, Validator validator) final;
 
     Status setValidationLevel(OperationContext* opCtx, StringData newLevel) final;
     Status setValidationAction(OperationContext* opCtx, StringData newAction) final;
@@ -291,6 +265,9 @@ public:
                            StringData newLevel,
                            StringData newAction) final;
 
+    bool getRecordPreImages() const final;
+    void setRecordPreImages(OperationContext* opCtx, bool val) final;
+
     bool isTemporary(OperationContext* opCtx) const final;
 
     //
@@ -313,6 +290,16 @@ public:
 
     uint64_t dataSize(OperationContext* opCtx) const final;
 
+    /**
+     * Currently fast counts are prone to false negative as it is not tolerant to unclean shutdowns.
+     * So, verify that the collection is really empty by opening the collection cursor and reading
+     * the first document.
+     * Expects to hold at least collection lock in mode IS.
+     * TODO SERVER-24266: After making fast counts tolerant to unclean shutdowns, we can make use of
+     * fast count to determine whether the collection is empty and remove cursor checking logic.
+     */
+    bool isEmpty(OperationContext* opCtx) const final;
+
     inline int averageObjectSize(OperationContext* opCtx) const {
         uint64_t n = numRecords(opCtx);
 
@@ -322,7 +309,7 @@ public:
     }
 
     uint64_t getIndexSize(OperationContext* opCtx,
-                          BSONObjBuilder* details = NULL,
+                          BSONObjBuilder* details = nullptr,
                           int scale = 1) const final;
 
     /**
@@ -366,6 +353,8 @@ public:
 
     void init(OperationContext* opCtx) final;
     bool isInitialized() const final;
+    bool isCommitted() const final;
+    void setCommitted(bool val) final;
 
 private:
     /**
@@ -387,15 +376,14 @@ private:
                             std::vector<InsertStatement>::const_iterator end,
                             OpDebug* opDebug);
 
-    int _magic;
-
     NamespaceString _ns;
-    OptionalCollectionUUID _uuid;
+    RecordId _catalogId;
+    UUID _uuid;
+    bool _committed = true;
 
     // The RecordStore may be null during a repair operation.
     std::unique_ptr<RecordStore> _recordStore;  // owned
     const bool _needCappedLock;
-    std::unique_ptr<CollectionInfoCache> _infoCache;
     std::unique_ptr<IndexCatalog> _indexCatalog;
 
 
@@ -405,21 +393,14 @@ private:
     // If null, the default collation is simple binary compare.
     std::unique_ptr<CollatorInterface> _collator;
 
-    // Empty means no validator.
-    BSONObj _validatorDoc;
 
-    // The collection validator MatchExpression. This is stored as a StatusWith, as we lazily
-    // enforce that collection validators are well formed.
-    // -A non-OK Status indicates that the validator is not well formed, and any attempts to enforce
-    // the validator (inserts) should error.
-    // -A value of {nullptr} indicates that there is no validator.
-    // -Anything else indicates a well formed validator. The MatchExpression will maintain
-    // pointers into _validatorDoc.
-    StatusWithMatchExpression _swValidator;
+    Validator _validator;
 
     ValidationAction _validationAction;
     ValidationLevel _validationLevel;
 
+    bool _recordPreImages = false;
+
     // Notifier object for awaitData. Threads polling a capped collection for new data can wait
     // on this object until notified of the arrival of new data.
     //
