@@ -71,9 +71,16 @@ struct CollectionOptions {
     Status validateForStorage() const;
 
     /**
-     * Parses the "options" subfield of the collection info object.
+     * Parses the collection 'options' into the appropriate struct fields.
+     *
+     * When 'kind' is set to ParseKind::parseForStorage, the 'uuid' field is parsed,
+     * otherwise the 'uuid' field is not parsed.
+     *
+     * When 'kind' is set to ParseKind::parseForCommand, the 'idIndex' field is parsed,
+     * otherwise the 'idIndex' field is not parsed.
      */
-    Status parse(const BSONObj& obj, ParseKind kind = parseForCommand);
+    static StatusWith<CollectionOptions> parse(const BSONObj& options,
+                                               ParseKind kind = parseForCommand);
 
     void appendBSON(BSONObjBuilder* builder) const;
     BSONObj toBSON() const;
@@ -94,8 +101,6 @@ struct CollectionOptions {
     bool matchesStorageOptions(const CollectionOptions& other,
                                CollatorFactoryInterface* collatorFactory) const;
 
-    // ----
-
     // Collection UUID. Present for all CollectionOptions parsed for storage.
     OptionalCollectionUUID uuid;
 
@@ -103,10 +108,6 @@ struct CollectionOptions {
     long long cappedSize = 0;
     long long cappedMaxDocs = 0;
 
-    // (MMAPv1) The following 2 are mutually exclusive, can only have one set.
-    long long initialNumExtents = 0;
-    std::vector<long long> initialExtentSizes;
-
     // The behavior of _id index creation when collection created
     void setNoIdIndex() {
         autoIndexId = NO;
@@ -118,6 +119,7 @@ struct CollectionOptions {
     } autoIndexId = DEFAULT;
 
     bool temp = false;
+    bool recordPreImages = false;
 
     // Storage engine collection options. Always owned or empty.
     BSONObj storageEngine;
