@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kIndex
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
 
 #include "mongo/db/catalog/index_timestamp_helper.h"
 
@@ -35,7 +35,7 @@
 #include "mongo/db/logical_clock.h"
 #include "mongo/db/repl/member_state.h"
 #include "mongo/db/repl/replication_coordinator.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 
@@ -47,8 +47,10 @@ Status setGhostTimestamp(OperationContext* opCtx, Timestamp timestamp) {
     opCtx->recoveryUnit()->setOrderedCommit(false);
 
     auto replCoord = repl::ReplicationCoordinator::get(opCtx);
-    opCtx->recoveryUnit()->onCommit(
-        [replCoord](auto commitTime) { replCoord->attemptToAdvanceStableTimestamp(); });
+    if (replCoord->isReplEnabled()) {
+        opCtx->recoveryUnit()->onCommit(
+            [replCoord](auto commitTime) { replCoord->attemptToAdvanceStableTimestamp(); });
+    }
 
     return Status::OK();
 }
@@ -107,21 +109,25 @@ void IndexTimestampHelper::setGhostCommitTimestampForWrite(OperationContext* opC
 
     auto status = setGhostTimestamp(opCtx, commitTimestamp);
     if (status.code() == ErrorCodes::BadValue) {
-        log() << "Temporarily could not apply ghost commit timestamp. " << status.reason();
+        LOGV2(20379,
+              "Temporarily could not apply ghost commit timestamp.",
+              "reason"_attr = status.reason());
         throw WriteConflictException();
     }
-    LOG(1) << "assigning ghost commit timestamp: " << commitTimestamp.toString();
+    LOGV2_DEBUG(20380,
+                1,
+                "assigning ghost commit timestamp: {commitTimestamp}",
+                "commitTimestamp"_attr = commitTimestamp.toString());
 
     fassert(51053, status);
 }
 
-/**
- * Returns true if writes to the catalog entry for the input namespace require being
- * timestamped. A ghost write is when the operation is not committed with an oplog entry and
- * implies the caller will look at the logical clock to choose a time to use.
- */
-namespace {
-bool requiresGhostCommitTimestampForCatalogWrite(OperationContext* opCtx, NamespaceString nss) {
+bool IndexTimestampHelper::requiresGhostCommitTimestampForCatalogWrite(OperationContext* opCtx,
+                                                                       NamespaceString nss) {
+    if (opCtx->writesAreReplicated()) {
+        return false;
+    }
+
     if (!nss.isReplicated() || nss.coll().startsWith("tmp.mr.")) {
         return false;
     }
@@ -157,7 +163,6 @@ bool requiresGhostCommitTimestampForCatalogWrite(OperationContext* opCtx, Namesp
 
     return true;
 }
-}  // namespace
 
 bool IndexTimestampHelper::setGhostCommitTimestampForCatalogWrite(OperationContext* opCtx,
                                                                   const NamespaceString& nss) {
@@ -168,8 +173,9 @@ bool IndexTimestampHelper::setGhostCommitTimestampForCatalogWrite(OperationConte
     auto status =
         setGhostTimestamp(opCtx, LogicalClock::get(opCtx)->getClusterTime().asTimestamp());
     if (status.code() == ErrorCodes::BadValue) {
-        log() << "Temporarily could not timestamp the index build commit, retrying. "
-              << status.reason();
+        LOGV2(20381,
+              "Temporarily could not timestamp the index build commit, retrying",
+              "reason"_attr = status.reason());
         throw WriteConflictException();
     }
     fassert(50701, status);
