@@ -35,7 +35,7 @@
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/s/database_sharding_state.h"
 #include "mongo/db/views/view_catalog.h"
-#include "mongo/util/fail_point_service.h"
+#include "mongo/util/fail_point.h"
 
 namespace mongo {
 namespace {
@@ -45,44 +45,51 @@ MONGO_FAIL_POINT_DEFINE(setAutoGetCollectionWait);
 }  // namespace
 
 AutoGetDb::AutoGetDb(OperationContext* opCtx, StringData dbName, LockMode mode, Date_t deadline)
-    : _dbLock(opCtx, dbName, mode, deadline), _db([&] {
+    : _opCtx(opCtx), _dbName(dbName), _dbLock(opCtx, dbName, mode, deadline), _db([&] {
           auto databaseHolder = DatabaseHolder::get(opCtx);
           return databaseHolder->getDb(opCtx, dbName);
       }()) {
+    auto dss = DatabaseShardingState::get(opCtx, dbName);
+    auto dssLock = DatabaseShardingState::DSSLock::lockShared(opCtx, dss);
+    dss->checkDbVersion(opCtx, dssLock);
+}
+
+Database* AutoGetDb::ensureDbExists() {
     if (_db) {
-        auto& dss = DatabaseShardingState::get(_db);
-        auto dssLock = DatabaseShardingState::DSSLock::lockShared(opCtx, &dss);
-        dss.checkDbVersion(opCtx, dssLock);
+        return _db;
     }
+
+    auto databaseHolder = DatabaseHolder::get(_opCtx);
+    _db = databaseHolder->openDb(_opCtx, _dbName, nullptr);
+
+    auto dss = DatabaseShardingState::get(_opCtx, _dbName);
+    auto dssLock = DatabaseShardingState::DSSLock::lockShared(_opCtx, dss);
+    dss->checkDbVersion(_opCtx, dssLock);
+
+    return _db;
 }
 
 AutoGetCollection::AutoGetCollection(OperationContext* opCtx,
                                      const NamespaceStringOrUUID& nsOrUUID,
-                                     LockMode modeDB,
                                      LockMode modeColl,
                                      ViewMode viewMode,
                                      Date_t deadline)
     : _autoDb(opCtx,
               !nsOrUUID.dbname().empty() ? nsOrUUID.dbname() : nsOrUUID.nss()->db(),
-              modeDB,
-              deadline),
-      _resolvedNss(resolveNamespaceStringOrUUID(opCtx, nsOrUUID)) {
+              isSharedLockMode(modeColl) ? MODE_IS : MODE_IX,
+              deadline) {
+    if (auto& nss = nsOrUUID.nss()) {
+        uassert(ErrorCodes::InvalidNamespace,
+                str::stream() << "Namespace " << *nss << " is not a valid collection name",
+                nss->isValid());
+    }
 
-    NamespaceString prevResolvedNss;
-    do {
-        _collLock.emplace(opCtx, _resolvedNss, modeColl, deadline);
-
-        // We looked up nsOrUUID without a collection lock so it's possible that the
-        // collection is dropped now. Look it up again.
-        prevResolvedNss = _resolvedNss;
-        _resolvedNss = resolveNamespaceStringOrUUID(opCtx, nsOrUUID);
-    } while (_resolvedNss != prevResolvedNss);
+    _collLock.emplace(opCtx, nsOrUUID, modeColl, deadline);
+    _resolvedNss = CollectionCatalog::get(opCtx).resolveNamespaceStringOrUUID(opCtx, nsOrUUID);
 
     // Wait for a configured amount of time after acquiring locks if the failpoint is enabled
-    MONGO_FAIL_POINT_BLOCK(setAutoGetCollectionWait, customWait) {
-        const BSONObj& data = customWait.getData();
-        sleepFor(Milliseconds(data["waitForMillis"].numberInt()));
-    }
+    setAutoGetCollectionWait.execute(
+        [&](const BSONObj& data) { sleepFor(Milliseconds(data["waitForMillis"].numberInt())); });
 
     Database* const db = _autoDb.getDb();
     invariant(!nsOrUUID.uuid() || db,
@@ -103,7 +110,7 @@ AutoGetCollection::AutoGetCollection(OperationContext* opCtx,
     if (!db)
         return;
 
-    _coll = db->getCollection(opCtx, _resolvedNss);
+    _coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, _resolvedNss);
     invariant(!nsOrUUID.uuid() || _coll,
               str::stream() << "Collection for " << _resolvedNss.ns()
                             << " disappeared after successufully resolving "
@@ -140,52 +147,16 @@ AutoGetCollection::AutoGetCollection(OperationContext* opCtx,
             !_view || viewMode == kViewsPermitted);
 }
 
-NamespaceString AutoGetCollection::resolveNamespaceStringOrUUID(OperationContext* opCtx,
-                                                                NamespaceStringOrUUID nsOrUUID) {
-    if (auto& nss = nsOrUUID.nss()) {
-        uassert(ErrorCodes::InvalidNamespace,
-                str::stream() << "Namespace " << *nss << " is not a valid collection name",
-                nss->isValid());
-        return *nss;
-    }
-
-    CollectionCatalog& catalog = CollectionCatalog::get(opCtx);
-    auto resolvedNss = catalog.lookupNSSByUUID(*nsOrUUID.uuid());
-
-    uassert(ErrorCodes::NamespaceNotFound,
-            str::stream() << "Unable to resolve " << nsOrUUID.toString(),
-            resolvedNss && resolvedNss->isValid());
-
-    uassert(ErrorCodes::NamespaceNotFound,
-            str::stream() << "UUID " << nsOrUUID.toString() << " specified in " << nsOrUUID.dbname()
-                          << " resolved to a collection in a different database: " << *resolvedNss,
-            resolvedNss->db() == nsOrUUID.dbname());
-
-    return *resolvedNss;
+LockMode fixLockModeForSystemDotViewsChanges(const NamespaceString& nss, LockMode mode) {
+    return nss.isSystemDotViews() ? MODE_X : mode;
 }
 
 AutoGetOrCreateDb::AutoGetOrCreateDb(OperationContext* opCtx,
                                      StringData dbName,
                                      LockMode mode,
-                                     Date_t deadline) {
+                                     Date_t deadline)
+    : _autoDb(opCtx, dbName, mode, deadline), _db(_autoDb.ensureDbExists()) {
     invariant(mode == MODE_IX || mode == MODE_X);
-
-    _autoDb.emplace(opCtx, dbName, mode, deadline);
-    _db = _autoDb->getDb();
-
-    // If the database didn't exist, relock in MODE_X
-    if (!_db) {
-        if (mode != MODE_X) {
-            _autoDb.emplace(opCtx, dbName, MODE_X, deadline);
-        }
-
-        auto databaseHolder = DatabaseHolder::get(opCtx);
-        _db = databaseHolder->openDb(opCtx, dbName, &_justCreated);
-    }
-
-    auto& dss = DatabaseShardingState::get(_db);
-    auto dssLock = DatabaseShardingState::DSSLock::lockShared(opCtx, &dss);
-    dss.checkDbVersion(opCtx, dssLock);
 }
 
 ConcealCollectionCatalogChangesBlock::ConcealCollectionCatalogChangesBlock(OperationContext* opCtx)
@@ -220,4 +191,23 @@ ReadSourceScope::~ReadSourceScope() {
     }
 }
 
+AutoGetOplog::AutoGetOplog(OperationContext* opCtx, OplogAccessMode mode, Date_t deadline)
+    : _shouldNotConflictWithSecondaryBatchApplicationBlock(opCtx->lockState()) {
+    auto lockMode = (mode == OplogAccessMode::kRead) ? MODE_IS : MODE_IX;
+    if (mode == OplogAccessMode::kLogOp) {
+        // Invariant that global lock is already held for kLogOp mode.
+        invariant(opCtx->lockState()->isWriteLocked());
+    } else {
+        _globalLock.emplace(opCtx, lockMode, deadline, Lock::InterruptBehavior::kThrow);
+    }
+
+    // Obtain database and collection intent locks for non-document-locking storage engines.
+    if (!opCtx->getServiceContext()->getStorageEngine()->supportsDocLocking()) {
+        _dbWriteLock.emplace(opCtx, NamespaceString::kLocalDb, lockMode, deadline);
+        _collWriteLock.emplace(opCtx, NamespaceString::kRsOplogNamespace, lockMode, deadline);
+    }
+    _oplogInfo = repl::LocalOplogInfo::get(opCtx);
+    _oplog = _oplogInfo->getCollection();
+}
+
 }  // namespace mongo
