@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -31,8 +30,9 @@
 #pragma once
 
 #include <boost/optional.hpp>
+#include <functional>
 
-#include "mongo/client/dbclientinterface.h"
+#include "mongo/db/auth/privilege.h"
 #include "mongo/db/auth/user_name.h"
 #include "mongo/db/cursor_id.h"
 #include "mongo/db/jsobj.h"
@@ -40,7 +40,6 @@
 #include "mongo/db/query/plan_executor.h"
 #include "mongo/db/record_id.h"
 #include "mongo/db/repl/read_concern_level.h"
-#include "mongo/stdx/functional.h"
 
 namespace mongo {
 
@@ -52,22 +51,50 @@ class RecoveryUnit;
  * Parameters used for constructing a ClientCursor. Makes an owned copy of 'originatingCommandObj'
  * to be used across getMores.
  *
- * ClientCursors cannot be constructed in isolation, but rather must be
- * constructed and managed using a CursorManager. See cursor_manager.h for more details.
+ * ClientCursors cannot be constructed in isolation, but rather must be constructed and managed
+ * using a CursorManager. See cursor_manager.h for more details.
  */
 struct ClientCursorParams {
+    // Describes whether callers should acquire locks when using a ClientCursor. Not all cursors
+    // have the same locking behavior. In particular, find cursors require the caller to lock the
+    // collection in MODE_IS before calling methods on the underlying plan executor. Aggregate
+    // cursors, on the other hand, may access multiple collections and acquire their own locks on
+    // any involved collections while producing query results. Therefore, the caller need not
+    // explicitly acquire any locks when using a ClientCursor which houses execution machinery for
+    // an aggregate.
+    //
+    // The policy is consulted on getMore in order to determine locking behavior, since during
+    // getMore we otherwise could not easily know what flavor of cursor we're using.
+    enum class LockPolicy {
+        // The caller is responsible for locking the collection over which this ClientCursor
+        // executes.
+        kLockExternally,
+
+        // The caller need not hold no locks; this ClientCursor's plan executor acquires any
+        // necessary locks itself.
+        kLocksInternally,
+    };
+
     ClientCursorParams(std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> planExecutor,
                        NamespaceString nss,
                        UserNameIterator authenticatedUsersIter,
-                       repl::ReadConcernLevel readConcernLevel,
-                       BSONObj originatingCommandObj)
+                       WriteConcernOptions writeConcernOptions,
+                       repl::ReadConcernArgs readConcernArgs,
+                       BSONObj originatingCommandObj,
+                       LockPolicy lockPolicy,
+                       PrivilegeVector originatingPrivileges,
+                       bool needsMerge)
         : exec(std::move(planExecutor)),
           nss(std::move(nss)),
-          readConcernLevel(readConcernLevel),
+          writeConcernOptions(std::move(writeConcernOptions)),
+          readConcernArgs(std::move(readConcernArgs)),
           queryOptions(exec->getCanonicalQuery()
                            ? exec->getCanonicalQuery()->getQueryRequest().getOptions()
                            : 0),
-          originatingCommandObj(originatingCommandObj.getOwned()) {
+          originatingCommandObj(originatingCommandObj.getOwned()),
+          lockPolicy(lockPolicy),
+          originatingPrivileges(std::move(originatingPrivileges)),
+          needsMerge(needsMerge) {
         while (authenticatedUsersIter.more()) {
             authenticatedUsers.emplace_back(authenticatedUsersIter.next());
         }
@@ -90,9 +117,13 @@ struct ClientCursorParams {
     std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> exec;
     const NamespaceString nss;
     std::vector<UserName> authenticatedUsers;
-    const repl::ReadConcernLevel readConcernLevel;
+    const WriteConcernOptions writeConcernOptions;
+    const repl::ReadConcernArgs readConcernArgs;
     int queryOptions = 0;
     BSONObj originatingCommandObj;
+    const LockPolicy lockPolicy;
+    PrivilegeVector originatingPrivileges;
+    const bool needsMerge;
 };
 
 /**
@@ -112,7 +143,8 @@ struct ClientCursorParams {
  * of inactivity.
  */
 class ClientCursor {
-    MONGO_DISALLOW_COPYING(ClientCursor);
+    ClientCursor(const ClientCursor&) = delete;
+    ClientCursor& operator=(const ClientCursor&) = delete;
 
 public:
     CursorId cursorid() const {
@@ -135,8 +167,16 @@ public:
         return _txnNumber;
     }
 
-    repl::ReadConcernLevel getReadConcernLevel() const {
-        return _readConcernLevel;
+    repl::ReadConcernArgs getReadConcernArgs() const {
+        return _readConcernArgs;
+    }
+
+    WriteConcernOptions getWriteConcernOptions() const {
+        return _writeConcernOptions;
+    }
+
+    bool needsMerge() const {
+        return _needsMerge;
     }
 
     /**
@@ -164,34 +204,83 @@ public:
         return _queryOptions & QueryOption_AwaitData;
     }
 
+    /**
+     * Returns the original command object which created this cursor.
+     */
     const BSONObj& getOriginatingCommandObj() const {
         return _originatingCommand;
     }
 
+    /**
+     * Returns the privileges required to run a getMore against this cursor. This is the same as the
+     * set of privileges which would have been required to create the cursor in the first place.
+     */
+    const PrivilegeVector& getOriginatingPrivileges() const& {
+        return _originatingPrivileges;
+    }
+    void getOriginatingPrivileges() && = delete;
+
     /**
      * Returns the total number of query results returned by the cursor so far.
      */
-    long long pos() const {
-        return _pos;
+    std::uint64_t nReturnedSoFar() const {
+        return _nReturnedSoFar;
     }
 
     /**
      * Increments the cursor's tracked number of query results returned so far by 'n'.
      */
-    void incPos(long long n) {
-        _pos += n;
+    void incNReturnedSoFar(std::uint64_t n) {
+        _nReturnedSoFar += n;
     }
 
     /**
      * Sets the cursor's tracked number of query results returned so far to 'n'.
      */
-    void setPos(long long n) {
-        _pos = n;
+    void setNReturnedSoFar(std::uint64_t n) {
+        invariant(n >= _nReturnedSoFar);
+        _nReturnedSoFar = n;
     }
 
-    //
-    // Timing.
-    //
+    /**
+     * Returns the number of batches returned by this cursor so far.
+     */
+    std::uint64_t getNBatches() const {
+        return _nBatchesReturned;
+    }
+
+    /**
+     * Increments the number of batches returned so far by one.
+     */
+    void incNBatches() {
+        ++_nBatchesReturned;
+    }
+
+    Date_t getLastUseDate() const {
+        return _lastUseDate;
+    }
+
+    Date_t getCreatedDate() const {
+        return _createdDate;
+    }
+
+    void setPlanSummary(std::string ps) {
+        _planSummary = std::move(ps);
+    }
+
+    StringData getPlanSummary() const {
+        return StringData(_planSummary);
+    }
+
+    ClientCursorParams::LockPolicy lockPolicy() const {
+        return _lockPolicy;
+    }
+
+    /**
+     * Returns a generic cursor containing diagnostics about this cursor.
+     * The caller must either have this cursor pinned or hold a mutex from the cursor manager.
+     */
+    GenericCursor toGenericCursor() const;
 
     /**
      * Returns the amount of time execution time available to this cursor. Only valid at the
@@ -214,15 +303,27 @@ public:
     }
 
     /**
-     * Returns the server-wide the count of living cursors. Such a cursor is called an "open
-     * cursor".
+     * Returns the commit point at the time the last batch was returned.
      */
-    static long long totalOpen();
+    boost::optional<repl::OpTime> getLastKnownCommittedOpTime() const {
+        return _lastKnownCommittedOpTime;
+    }
+
+    /**
+     * Sets the commit point at the time the last batch was returned.
+     */
+    void setLastKnownCommittedOpTime(boost::optional<repl::OpTime> lastCommittedOpTime) {
+        _lastKnownCommittedOpTime = std::move(lastCommittedOpTime);
+    }
 
     friend std::size_t partitionOf(const ClientCursor* cursor) {
         return cursor->cursorid();
     }
 
+    boost::optional<OperationKey> getOperationKey() const {
+        return _opKey;
+    }
+
 private:
     friend class CursorManager;
     friend class ClientCursorPin;
@@ -242,7 +343,6 @@ private:
      * private. See cursor_manager.h for more details.
      */
     ClientCursor(ClientCursorParams params,
-                 CursorManager* cursorManager,
                  CursorId cursorId,
                  OperationContext* operationUsingCursor,
                  Date_t now);
@@ -274,7 +374,7 @@ private:
     }
 
     // The ID of the ClientCursor. A value of 0 is used to mean that no cursor id has been assigned.
-    CursorId _cursorid = 0;
+    const CursorId _cursorid = 0;
 
     // Threads may read from this field even if they don't have the cursor pinned, as long as they
     // have the correct partition of the CursorManager locked (just like _authenticatedUsers).
@@ -292,51 +392,54 @@ private:
     // A transaction number for this cursor, if it was provided in the originating command.
     const boost::optional<TxnNumber> _txnNumber;
 
-    const repl::ReadConcernLevel _readConcernLevel;
-
-    CursorManager* _cursorManager;
+    const WriteConcernOptions _writeConcernOptions;
+    const repl::ReadConcernArgs _readConcernArgs;
 
     // Tracks whether dispose() has been called, to make sure it happens before destruction. It is
     // an error to use a ClientCursor once it has been disposed.
     bool _disposed = false;
 
     // Tracks the number of results returned by this cursor so far.
-    long long _pos = 0;
+    std::uint64_t _nReturnedSoFar = 0;
+
+    // Tracks the number of batches returned by this cursor so far.
+    std::uint64_t _nBatchesReturned = 0;
 
     // Holds an owned copy of the command specification received from the client.
     const BSONObj _originatingCommand;
 
+    // The privileges required for the _originatingCommand.
+    const PrivilegeVector _originatingPrivileges;
+
     // See the QueryOptions enum in dbclientinterface.h.
     const int _queryOptions = 0;
 
+    const ClientCursorParams::LockPolicy _lockPolicy;
+
+    // The value of a flag specified on the originating command which indicates whether the result
+    // of this cursor will be consumed by a merging node (mongos or a mongod selected to perform a
+    // merge). Note that this flag is only set for aggregate() commands, and not for find()
+    // commands. It is therefore possible that 'needsMerge' is false when in fact there will be a
+    // merge performed.
+    const bool _needsMerge;
+
     // Unused maxTime budget for this cursor.
     Microseconds _leftoverMaxTimeMicros = Microseconds::max();
 
     // The underlying query execution machinery. Must be non-null.
     std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> _exec;
 
-    //
-    // The following fields are used by the CursorManager and the ClientCursorPin. In most
-    // conditions, they can only be used while holding the CursorManager's mutex. Exceptions
-    // include:
-    //   - If the ClientCursor is pinned, the CursorManager will never change '_isPinned' until
-    //     asked to by the ClientCursorPin.
-    //   - It is safe to read '_killed' while holding a collection lock, which must be held when
-    //     interacting with a ClientCursorPin.
-    //   - A ClientCursorPin can access these members after deregistering the cursor from the
-    //     CursorManager, at which point it has sole ownership of the ClientCursor.
-    //
-
     // While a cursor is being used by a client, it is marked as "pinned" by setting
     // _operationUsingCursor to the current OperationContext.
     //
-    // Cursors always come into existence in a pinned state (this must be non-null at construction).
+    // Cursors always come into existence in a pinned state ('_operationUsingCursor' must be
+    // non-null at construction).
     //
     // To write to this field one of the following must be true:
     // 1) You have a lock on the appropriate partition in CursorManager and the cursor is unpinned
     // (the field is null).
-    // 2) You own the cursor and the cursor manager it was associated with is gone (this can only
-    // happen in ClientCursorPin). In this case, nobody else will try to pin the cursor.
+    // 2) The cursor has already been deregistered from the CursorManager. In this case, nobody else
+    // will try to pin the cursor.
     //
     // To read this field one of the following must be true:
     // 1) You have a lock on the appropriate partition in CursorManager.
@@ -344,6 +447,17 @@ private:
     OperationContext* _operationUsingCursor;
 
     Date_t _lastUseDate;
+    Date_t _createdDate;
+
+    // A string with the plan summary of the cursor's query.
+    std::string _planSummary;
+
+    // Commit point at the time the last batch was returned. This is only used by internal exhaust
+    // oplog fetching. Also see lastKnownCommittedOpTime in GetMoreRequest.
+    boost::optional<repl::OpTime> _lastKnownCommittedOpTime;
+
+    // The client OperationKey associated with this cursor.
+    boost::optional<OperationKey> _opKey;
 };
 
 /**
@@ -359,9 +473,7 @@ private:
  *
  * A pin extends the lifetime of a ClientCursor object until the pin's release. Pinned ClientCursor
  * objects cannot not be killed due to inactivity, and cannot be immediately erased by user kill
- * requests (though they can be marked as interrupted). When a CursorManager is destroyed (e.g. by
- * a collection drop), ownership of any still-pinned ClientCursor objects is transferred to their
- * managing ClientCursorPin objects.
+ * requests (though they can be marked as interrupted).
  *
  * Example usage:
  * {
@@ -376,17 +488,14 @@ private:
  *     // Use cursor. Pin automatically released on block exit.
  * }
  *
- * Clients that wish to access ClientCursor objects owned by collection cursor managers must hold
- * the collection lock while calling any pin method, including pin acquisition by the RAII
- * constructor and pin release by the RAII destructor.  This guards from a collection drop (which
- * requires an exclusive lock on the collection) occurring concurrently with the pin request or
- * unpin request.
- *
- * Clients that wish to access ClientCursor objects owned by the global cursor manager need not
- * hold any locks; the global cursor manager can only be destroyed by a process exit.
+ * Callers need not hold any lock manager locks in order to obtain or release a client cursor pin.
+ * However, in order to use the ClientCursor itself, locks may need to be acquired. Whether locks
+ * are needed to use the ClientCursor can be determined by consulting the ClientCursor's lock
+ * policy.
  */
 class ClientCursorPin {
-    MONGO_DISALLOW_COPYING(ClientCursorPin);
+    ClientCursorPin(const ClientCursorPin&) = delete;
+    ClientCursorPin& operator=(const ClientCursorPin&) = delete;
 
 public:
     /**
@@ -407,9 +516,8 @@ public:
     ~ClientCursorPin();
 
     /**
-     * Releases the pin.  It does not delete the underlying cursor unless ownership has passed
-     * to us after kill.  Turns into a no-op if release() or deleteUnderlying() have already
-     * been called on this pin.
+     * Releases the pin without deleting the underlying cursor. Turns into a no-op if release() or
+     * deleteUnderlying() have already been called on this pin.
      */
     void release();
 
@@ -424,13 +532,18 @@ public:
      */
     ClientCursor* getCursor() const;
 
+    ClientCursor* operator->() {
+        return _cursor;
+    }
+
 private:
     friend class CursorManager;
 
-    ClientCursorPin(OperationContext* opCtx, ClientCursor* cursor);
+    ClientCursorPin(OperationContext* opCtx, ClientCursor* cursor, CursorManager* cursorManager);
 
     OperationContext* _opCtx = nullptr;
     ClientCursor* _cursor = nullptr;
+    CursorManager* _cursorManager = nullptr;
 };
 
 void startClientCursorMonitor();
