@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
@@ -49,8 +49,9 @@
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/matcher/extensions_callback_real.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/query/collection_query_info.h"
+#include "mongo/logv2/log.h"
 #include "mongo/stdx/unordered_set.h"
-#include "mongo/util/log.h"
 
 
 namespace {
@@ -67,21 +68,18 @@ static Status getQuerySettingsAndPlanCache(OperationContext* opCtx,
                                            const string& ns,
                                            QuerySettings** querySettingsOut,
                                            PlanCache** planCacheOut) {
-    *querySettingsOut = NULL;
-    *planCacheOut = NULL;
-    if (NULL == collection) {
+    *querySettingsOut = nullptr;
+    *planCacheOut = nullptr;
+    if (nullptr == collection) {
         return Status(ErrorCodes::BadValue, "no such collection");
     }
 
-    CollectionInfoCache* infoCache = collection->infoCache();
-    invariant(infoCache);
-
-    QuerySettings* querySettings = infoCache->getQuerySettings();
+    QuerySettings* querySettings = CollectionQueryInfo::get(collection).getQuerySettings();
     invariant(querySettings);
 
     *querySettingsOut = querySettings;
 
-    PlanCache* planCache = infoCache->getPlanCache();
+    PlanCache* planCache = CollectionQueryInfo::get(collection).getPlanCache();
     invariant(planCache);
 
     *planCacheOut = planCache;
@@ -255,7 +253,7 @@ Status ClearFilters::clear(OperationContext* opCtx,
     // - clear hints for single query shape when a query shape is described in the
     //   command arguments.
     if (cmdObj.hasField("query")) {
-        auto statusWithCQ = PlanCacheCommand::canonicalize(opCtx, ns, cmdObj);
+        auto statusWithCQ = plan_cache_commands::canonicalize(opCtx, ns, cmdObj);
         if (!statusWithCQ.isOK()) {
             return statusWithCQ.getStatus();
         }
@@ -266,7 +264,10 @@ Status ClearFilters::clear(OperationContext* opCtx,
         // Remove entry from plan cache
         planCache->remove(*cq).transitional_ignore();
 
-        LOG(0) << "Removed index filter on " << redact(cq->toStringShort());
+        LOGV2(20479,
+              "Removed index filter on {query}",
+              "Removed index filter on query",
+              "query"_attr = redact(cq->toStringShort()));
 
         return Status::OK();
     }
@@ -303,7 +304,7 @@ Status ClearFilters::clear(OperationContext* opCtx,
         AllowedIndexEntry entry = *i;
 
         // Create canonical query.
-        auto qr = stdx::make_unique<QueryRequest>(nss);
+        auto qr = std::make_unique<QueryRequest>(nss);
         qr->setFilter(entry.query);
         qr->setSort(entry.sort);
         qr->setProj(entry.projection);
@@ -322,7 +323,10 @@ Status ClearFilters::clear(OperationContext* opCtx,
         planCache->remove(*cq).transitional_ignore();
     }
 
-    LOG(0) << "Removed all index filters for collection: " << ns;
+    LOGV2(20480,
+          "Removed all index filters for collection: {namespace}",
+          "Removed all index filters for collection",
+          "namespace"_attr = ns);
 
     return Status::OK();
 }
@@ -387,7 +391,7 @@ Status SetFilter::set(OperationContext* opCtx,
         }
     }
 
-    auto statusWithCQ = PlanCacheCommand::canonicalize(opCtx, ns, cmdObj);
+    auto statusWithCQ = plan_cache_commands::canonicalize(opCtx, ns, cmdObj);
     if (!statusWithCQ.isOK()) {
         return statusWithCQ.getStatus();
     }
@@ -399,7 +403,11 @@ Status SetFilter::set(OperationContext* opCtx,
     // Remove entry from plan cache.
     planCache->remove(*cq).transitional_ignore();
 
-    LOG(0) << "Index filter set on " << redact(cq->toStringShort()) << " " << indexesElt;
+    LOGV2(20481,
+          "Index filter set on {query} {indexes}",
+          "Index filter set on query",
+          "query"_attr = redact(cq->toStringShort()),
+          "indexes"_attr = indexesElt);
 
     return Status::OK();
 }
