@@ -29,6 +29,7 @@
 
 #include "mongo/platform/basic.h"
 
+#include <memory>
 #include <vector>
 
 #include "mongo/base/checked_cast.h"
@@ -59,7 +60,6 @@
 #include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/db/views/view_catalog.h"
-#include "mongo/stdx/memory.h"
 
 namespace mongo {
 
@@ -67,7 +67,6 @@ using std::string;
 using std::stringstream;
 using std::unique_ptr;
 using std::vector;
-using stdx::make_unique;
 
 namespace {
 
@@ -132,7 +131,7 @@ void _addWorkingSetMember(OperationContext* opCtx,
     WorkingSetMember* member = ws->get(id);
     member->keyData.clear();
     member->recordId = RecordId();
-    member->obj = Snapshotted<BSONObj>(SnapshotId(), maybe);
+    member->resetDocument(SnapshotId(), maybe);
     member->transitionToOwnedObj();
     root->pushBack(id);
 }
@@ -187,7 +186,8 @@ BSONObj buildCollectionBson(OperationContext* opCtx,
         return b.obj();
     }
 
-    CollectionOptions options = DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, nss);
+    CollectionOptions options =
+        DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, collection->getCatalogId());
 
     // While the UUID is stored as a collection option, from the user's perspective it is an
     // unsettable read-only property, so put it in the 'info' section.
@@ -214,6 +214,9 @@ public:
     AllowedOnSecondary secondaryAllowed(ServiceContext*) const final {
         return AllowedOnSecondary::kOptIn;
     }
+    bool maintenanceOk() const override {
+        return false;
+    }
     bool adminOnly() const final {
         return false;
     }
@@ -252,15 +255,17 @@ public:
         const bool nameOnly = jsobj["nameOnly"].trueValue();
         const bool authorizedCollections = jsobj["authorizedCollections"].trueValue();
 
+        // The collator is null because collection objects are compared using binary comparison.
+        auto expCtx = make_intrusive<ExpressionContext>(
+            opCtx, std::unique_ptr<CollatorInterface>(nullptr), NamespaceString(dbname));
+
         // Check for 'filter' argument.
         BSONElement filterElt = jsobj["filter"];
         if (!filterElt.eoo()) {
             if (filterElt.type() != mongo::Object) {
                 uasserted(ErrorCodes::BadValue, "\"filter\" must be an object");
             }
-            // The collator is null because collection objects are compared using binary comparison.
-            const CollatorInterface* collator = nullptr;
-            boost::intrusive_ptr<ExpressionContext> expCtx(new ExpressionContext(opCtx, collator));
+
             StatusWithMatchExpression statusWithMatcher =
                 MatchExpressionParser::parse(filterElt.Obj(), std::move(expCtx));
             uassertStatusOK(statusWithMatcher.getStatus());
@@ -294,8 +299,8 @@ public:
                                                              false,
                                                              cursorNss);
 
-            auto ws = make_unique<WorkingSet>();
-            auto root = make_unique<QueuedDataStage>(opCtx, ws.get());
+            auto ws = std::make_unique<WorkingSet>();
+            auto root = std::make_unique<QueuedDataStage>(expCtx.get(), ws.get());
 
             if (db) {
                 if (auto collNames = _getExactNameMatches(matcher.get())) {
@@ -311,7 +316,8 @@ public:
                         }
 
                         Lock::CollectionLock clk(opCtx, nss, MODE_IS);
-                        Collection* collection = db->getCollection(opCtx, nss);
+                        Collection* collection =
+                            CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
                         BSONObj collBson =
                             buildCollectionBson(opCtx, collection, includePendingDrops, nameOnly);
                         if (!collBson.isEmpty()) {
@@ -359,17 +365,18 @@ public:
             }
 
             exec = uassertStatusOK(PlanExecutor::make(
-                opCtx, std::move(ws), std::move(root), cursorNss, PlanExecutor::NO_YIELD));
+                opCtx, std::move(ws), std::move(root), nullptr, PlanExecutor::NO_YIELD, cursorNss));
 
             for (long long objCount = 0; objCount < batchSize; objCount++) {
-                BSONObj next;
-                PlanExecutor::ExecState state = exec->getNext(&next, NULL);
+                Document nextDoc;
+                PlanExecutor::ExecState state = exec->getNext(&nextDoc, nullptr);
                 if (state == PlanExecutor::IS_EOF) {
                     break;
                 }
                 invariant(state == PlanExecutor::ADVANCED);
 
                 // If we can't fit this result inside the current batch, then we stash it for later.
+                BSONObj next = nextDoc.toBson();
                 if (!FindCommon::haveSpaceForNext(next, objCount, firstBatch.len())) {
                     exec->enqueue(next);
                     break;
@@ -387,15 +394,18 @@ public:
 
         auto pinnedCursor = CursorManager::get(opCtx)->registerCursor(
             opCtx,
-            {std::move(exec),
-             cursorNss,
-             AuthorizationSession::get(opCtx->getClient())->getAuthenticatedUserNames(),
-             opCtx->getWriteConcern(),
-             repl::ReadConcernArgs::get(opCtx),
-             jsobj,
-             ClientCursorParams::LockPolicy::kLocksInternally,
-             uassertStatusOK(AuthorizationSession::get(opCtx->getClient())
-                                 ->checkAuthorizedToListCollections(dbname, jsobj))});
+            {
+                std::move(exec),
+                cursorNss,
+                AuthorizationSession::get(opCtx->getClient())->getAuthenticatedUserNames(),
+                opCtx->getWriteConcern(),
+                repl::ReadConcernArgs::get(opCtx),
+                jsobj,
+                ClientCursorParams::LockPolicy::kLocksInternally,
+                uassertStatusOK(AuthorizationSession::get(opCtx->getClient())
+                                    ->checkAuthorizedToListCollections(dbname, jsobj)),
+                false  // needsMerge always 'false' for listCollections.
+            });
 
         appendCursorResponseObject(
             pinnedCursor.getCursor()->cursorid(), cursorNss.ns(), firstBatch.arr(), &result);
