@@ -58,6 +58,8 @@ struct SpecificStats {
 
 // Every stage has CommonStats.
 struct CommonStats {
+    CommonStats() = delete;
+
     CommonStats(const char* type)
         : stageTypeStr(type),
           works(0),
@@ -67,6 +69,7 @@ struct CommonStats {
           needTime(0),
           needYield(0),
           executionTimeMillis(0),
+          failed(false),
           isEOF(false) {}
 
     uint64_t estimateObjectSizeInBytes() const {
@@ -100,11 +103,8 @@ struct CommonStats {
 
     // TODO: keep track of the total yield time / fetch time done for a plan.
 
+    bool failed;
     bool isEOF;
-
-private:
-    // Default constructor is illegal.
-    CommonStats();
 };
 
 // The universal container for a stage's stats.
@@ -234,9 +234,14 @@ struct CollectionScanStats : public SpecificStats {
     // backwards.
     int direction;
 
-    // If present, indicates that the collection scan will stop and return EOF the first time it
-    // sees a document that does not pass the filter and has a "ts" Timestamp field greater than
-    // 'maxTs'.
+    bool tailable{false};
+
+    // The start location of the scan. Must only be set on forward oplog scans.
+    boost::optional<Timestamp> minTs;
+
+    // Indicates that the collection scan will stop and return EOF the first time it sees a
+    // document that does not pass the filter and has a "ts" Timestamp field greater than 'maxTs'.
+    // Must only be set on forward oplog scans.
     boost::optional<Timestamp> maxTs;
 };
 
@@ -440,6 +445,16 @@ struct IDHackStats : public SpecificStats {
     size_t docsExamined;
 };
 
+struct ReturnKeyStats : public SpecificStats {
+    SpecificStats* clone() const final {
+        return new ReturnKeyStats(*this);
+    }
+
+    uint64_t estimateObjectSizeInBytes() const {
+        return sizeof(*this);
+    }
+};
+
 struct IndexScanStats : public SpecificStats {
     IndexScanStats()
         : indexVersion(0),
@@ -598,17 +613,24 @@ struct SortStats : public SpecificStats {
         return sortPattern.objsize() + sizeof(*this);
     }
 
-    // What's our current memory usage?
-    size_t memUsage = 0u;
-
-    // What's our memory limit?
-    size_t memLimit = 0u;
-
-    // The number of results to return from the sort.
-    size_t limit = 0u;
-
     // The pattern according to which we are sorting.
     BSONObj sortPattern;
+
+    // The number of results to return from the sort.
+    uint64_t limit = 0u;
+
+    // The maximum number of bytes of memory we're willing to use during execution of the sort. If
+    // this limit is exceeded and 'allowDiskUse' is false, the query will fail at execution time. If
+    // 'allowDiskUse' is true, the data will be spilled to disk.
+    uint64_t maxMemoryUsageBytes = 0u;
+
+    // The amount of data we've sorted in bytes. At various times this data may be buffered in
+    // memory or disk-resident, depending on the configuration of 'maxMemoryUsageBytes' and whether
+    // disk use is allowed.
+    uint64_t totalDataSizeBytes = 0u;
+
+    // Whether we spilled data to disk during the execution of this query.
+    bool wasDiskUsed = false;
 };
 
 struct MergeSortStats : public SpecificStats {
