@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -119,12 +118,11 @@ TEST_F(FreeMonQueueTest, TestDeadlinePriority) {
 TEST_F(FreeMonQueueTest, TestFIFO) {
     FreeMonMessageQueue queue;
 
+    queue.enqueue(FreeMonMessage::createWithDeadline(FreeMonMessageType::RegisterServer, Date_t()));
     queue.enqueue(
-        FreeMonMessage::createWithDeadline(FreeMonMessageType::RegisterServer, Date_t::min()));
-    queue.enqueue(FreeMonMessage::createWithDeadline(FreeMonMessageType::AsyncRegisterComplete,
-                                                     Date_t::min()));
+        FreeMonMessage::createWithDeadline(FreeMonMessageType::AsyncRegisterComplete, Date_t()));
     queue.enqueue(
-        FreeMonMessage::createWithDeadline(FreeMonMessageType::RegisterCommand, Date_t::min()));
+        FreeMonMessage::createWithDeadline(FreeMonMessageType::RegisterCommand, Date_t()));
 
     auto item = queue.dequeue(_opCtx.get()->getServiceContext()->getPreciseClockSource()).get();
     ASSERT(item->getType() == FreeMonMessageType::RegisterServer);
@@ -148,13 +146,11 @@ TEST_F(FreeMonQueueTest, TestQueueStop) {
 
     auto swSchedule =
         _mockThreadPool->scheduleWork([&](const executor::TaskExecutor::CallbackArgs& cbArgs) {
-
             barrier.countDownAndWait();
 
             // Try to dequeue from a stopped task queue
             auto item = queue.dequeue(_opCtx.get()->getServiceContext()->getPreciseClockSource());
             ASSERT_FALSE(item.is_initialized());
-
         });
 
     ASSERT_OK(swSchedule.getStatus());
