@@ -27,13 +27,14 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kFTDC
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kFTDC
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/ftdc/file_manager.h"
 
 #include <boost/filesystem.hpp>
+#include <memory>
 #include <string>
 
 #include "mongo/base/string_data.h"
@@ -42,9 +43,8 @@
 #include "mongo/db/ftdc/constants.h"
 #include "mongo/db/ftdc/file_reader.h"
 #include "mongo/db/jsobj.h"
-#include "mongo/stdx/memory.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/log.h"
 #include "mongo/util/str.h"
 #include "mongo/util/time_support.h"
 
@@ -120,7 +120,7 @@ std::vector<boost::filesystem::path> FTDCFileManager::scanDirectory() {
 
         std::string str = filename.generic_string();
         if (str.compare(0, strlen(kFTDCArchiveFile), kFTDCArchiveFile) == 0 &&
-            str != kFTDCInterimFile) {
+            str != kFTDCInterimTempFile && str != kFTDCInterimFile) {
             files.emplace_back(_path / filename);
         }
     }
@@ -227,8 +227,11 @@ Status FTDCFileManager::trimDirectory(std::vector<boost::filesystem::path>& file
         size += fileSize;
 
         if (size >= maxSize) {
-            LOG(1) << "Cleaning file over full-time diagnostic data capture quota, file: "
-                   << (*it).generic_string() << " with size " << fileSize;
+            LOGV2_DEBUG(20628,
+                        1,
+                        "Cleaning file over full-time diagnostic data capture quota",
+                        "fileName"_attr = (*it).generic_string(),
+                        "fileSize"_attr = fileSize);
 
             boost::filesystem::remove(*it, ec);
             if (ec) {
@@ -257,8 +260,9 @@ FTDCFileManager::recoverInterimFile() {
     boost::system::error_code ec;
     size_t size = boost::filesystem::file_size(interimFile, ec);
     if (ec) {
-        log() << "Recover interim file failed as the file size could not be checked: "
-              << ec.message();
+        LOGV2(20629,
+              "Recover interim file failed as the file size could not be checked",
+              "errorMessage"_attr = ec.message());
         return docs;
     }
 
@@ -269,11 +273,10 @@ FTDCFileManager::recoverInterimFile() {
     FTDCFileReader read;
     auto s = read.open(interimFile);
     if (!s.isOK()) {
-        log() << "Unclean full-time diagnostic data capture shutdown detected, found interim file, "
-                 "but failed "
-                 "to open it, some "
-                 "metrics may have been lost. "
-              << s;
+        LOGV2(20630,
+              "Unclean full-time diagnostic data capture shutdown detected, found interim file,  "
+              "but failed to open it, some metrics may have been lost",
+              "error"_attr = s);
 
         // Note: We ignore any actual errors as reading from the interim files is a best-effort
         return docs;
@@ -288,10 +291,10 @@ FTDCFileManager::recoverInterimFile() {
 
     // Warn if the interim file was corrupt or we had an unclean shutdown
     if (!m.isOK() || !docs.empty()) {
-        log() << "Unclean full-time diagnostic data capture shutdown detected, found interim file, "
-                 "some "
-                 "metrics may have been lost. "
-              << m.getStatus();
+        LOGV2(20631,
+              "Unclean full-time diagnostic data capture shutdown detected, found interim file, "
+              "some metrics may have been lost",
+              "error"_attr = m.getStatus());
     }
 
     // Note: We ignore any actual errors as reading from the interim files is a best-effort
