@@ -34,6 +34,7 @@
 #include "mongo/db/index/duplicate_key_tracker.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/multikey_paths.h"
+#include "mongo/db/index/skipped_record_tracker.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/storage/temporary_record_store.h"
 #include "mongo/platform/atomic_word.h"
@@ -46,8 +47,20 @@ class OperationContext;
 
 class IndexBuildInterceptor {
 public:
+    /**
+     * Determines if we will yield locks while draining the side tables.
+     */
+    enum class DrainYieldPolicy { kNoYield, kYield };
+
     enum class Op { kInsert, kDelete };
 
+    /**
+     * Indicates whether to record duplicate keys that have been inserted into the index. When set
+     * to 'kNoTrack', inserted duplicate keys will be ignored. When set to 'kTrack', a subsequent
+     * call to checkDuplicateKeyConstraints is required.
+     */
+    enum class TrackDuplicates { kNoTrack, kTrack };
+
     static bool typeCanFastpathMultikeyUpdates(IndexType type);
 
     /**
@@ -73,18 +86,18 @@ public:
      * On success, `numKeysOut` if non-null will contain the number of keys added or removed.
      */
     Status sideWrite(OperationContext* opCtx,
-                     const std::vector<BSONObj>& keys,
-                     const BSONObjSet& multikeyMetadataKeys,
+                     const KeyStringSet& keys,
+                     const KeyStringSet& multikeyMetadataKeys,
                      const MultikeyPaths& multikeyPaths,
                      RecordId loc,
                      Op op,
                      int64_t* const numKeysOut);
 
     /**
-     * Given a set of duplicate keys, record the keys for later verification by a call to
+     * Given a duplicate key, record the key for later verification by a call to
      * checkDuplicateKeyConstraints();
      */
-    Status recordDuplicateKeys(OperationContext* opCtx, const std::vector<BSONObj>& keys);
+    Status recordDuplicateKey(OperationContext* opCtx, const KeyString::Value& key);
 
     /**
      * Returns Status::OK if all previously recorded duplicate key constraint violations have been
@@ -112,7 +125,24 @@ public:
      */
     Status drainWritesIntoIndex(OperationContext* opCtx,
                                 const InsertDeleteOptions& options,
-                                RecoveryUnit::ReadSource readSource);
+                                TrackDuplicates trackDups,
+                                RecoveryUnit::ReadSource readSource,
+                                DrainYieldPolicy drainYieldPolicy);
+
+    SkippedRecordTracker* getSkippedRecordTracker() {
+        return &_skippedRecordTracker;
+    }
+
+    const SkippedRecordTracker* getSkippedRecordTracker() const {
+        return &_skippedRecordTracker;
+    }
+
+    /**
+     * Tries to index previously skipped records. For each record, if the new indexing attempt is
+     * successful, keys are written directly to the index. Unsuccessful key generation or writes
+     * will return errors.
+     */
+    Status retrySkippedRecords(OperationContext* opCtx, const Collection* collection);
 
     /**
      * Returns 'true' if there are no visible records remaining to be applied from the side writes
@@ -120,42 +150,37 @@ public:
      */
     bool areAllWritesApplied(OperationContext* opCtx) const;
 
-    /**
-     * Returns true if all recorded duplicate key constraint violations have been checked.
-     */
-    bool areAllConstraintsChecked(OperationContext* opCtx) const;
-
     /**
      * When an index builder wants to commit, use this to retrieve any recorded multikey paths
      * that were tracked during the build.
      */
     boost::optional<MultikeyPaths> getMultikeyPaths() const;
 
-    const std::string& getSideWritesTableIdent() const;
-
-    const std::string& getConstraintViolationsTableIdent() const;
-
 private:
     using SideWriteRecord = std::pair<RecordId, BSONObj>;
 
     Status _applyWrite(OperationContext* opCtx,
                        const BSONObj& doc,
                        const InsertDeleteOptions& options,
+                       TrackDuplicates trackDups,
                        int64_t* const keysInserted,
                        int64_t* const keysDeleted);
 
     /**
-     * Yield lock manager locks, but only when holding intent locks. Does nothing otherwise. If this
-     * yields locks, it will also abandon the current storage engine snapshot.
+     * Yield lock manager locks and abandon the current storage engine snapshot.
      */
-    void _tryYield(OperationContext*);
+    void _yield(OperationContext* opCtx);
 
     // The entry for the index that is being built.
     IndexCatalogEntry* _indexCatalogEntry;
 
-    // This temporary record store is owned by the interceptor and dropped along with it.
+    // This temporary record store records intercepted keys that will be written into the index by
+    // calling drainWritesIntoIndex(). It is owned by the interceptor and dropped along with it.
     std::unique_ptr<TemporaryRecordStore> _sideWritesTable;
 
+    // Records RecordIds that have been skipped due to indexing errors.
+    SkippedRecordTracker _skippedRecordTracker;
+
     std::unique_ptr<DuplicateKeyTracker> _duplicateKeyTracker;
 
     int64_t _numApplied{0};
