@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -37,7 +36,9 @@ namespace mongo {
 /**
  * Parses the session information from the body of a request and stores the sessionId and txnNumber
  * on the current operation context. Must only be called once per operation and should be done right
- * in the beginning.
+ * in the beginning. Note that the session info will be stored in the operation context and returned
+ * only if the current request supports it. For example, if attachToOpCtx is false or this is called
+ * within the context of DBDirectClient.
  *
  * Throws if the sessionId/txnNumber combination is not properly formatted.
  *
@@ -47,13 +48,11 @@ namespace mongo {
  *
  * Both isReplSetMemberOrMongos and supportsDocLocking need to be true if the command contains a
  * transaction number, otherwise this function will throw.
- *
- * On success, returns the parsed request information. Returning boost::none implies that the
- * proper command or session requirements were not met.
  */
 OperationSessionInfoFromClient initializeOperationSessionInfo(OperationContext* opCtx,
                                                               const BSONObj& requestBody,
                                                               bool requiresAuth,
+                                                              bool attachToOpCtx,
                                                               bool isReplSetMemberOrMongos,
                                                               bool supportsDocLocking);
 
