@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kControl
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kControl
 
 #include "mongo/platform/basic.h"
 
@@ -36,11 +36,13 @@
 #include "mongo/db/logical_session_id.h"
 #include "mongo/db/logical_session_id_helpers.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/service_context.h"
+#include "mongo/logv2/log.h"
+#include "mongo/logv2/log_severity_suppressor.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/util/duration.h"
-#include "mongo/util/log.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
@@ -87,14 +89,14 @@ void LogicalSessionCacheImpl::joinOnShutDown() {
 Status LogicalSessionCacheImpl::startSession(OperationContext* opCtx,
                                              const LogicalSessionRecord& record) {
     stdx::lock_guard lg(_mutex);
-    return _addToCache(lg, record);
+    return _addToCacheIfNotFull(lg, record);
 }
 
 Status LogicalSessionCacheImpl::vivify(OperationContext* opCtx, const LogicalSessionId& lsid) {
     stdx::lock_guard lg(_mutex);
     auto it = _activeSessions.find(lsid);
     if (it == _activeSessions.end())
-        return _addToCache(lg, makeLogicalSessionRecord(opCtx, lsid, _service->now()));
+        return _addToCacheIfNotFull(lg, makeLogicalSessionRecord(opCtx, lsid, _service->now()));
 
     auto& cacheEntry = it->second;
     cacheEntry.setLastUse(_service->now());
@@ -102,17 +104,17 @@ Status LogicalSessionCacheImpl::vivify(OperationContext* opCtx, const LogicalSes
     return Status::OK();
 }
 
-Status LogicalSessionCacheImpl::refreshNow(Client* client) {
+Status LogicalSessionCacheImpl::refreshNow(OperationContext* opCtx) {
     try {
-        _refresh(client);
+        _refresh(opCtx->getClient());
     } catch (...) {
         return exceptionToStatus();
     }
     return Status::OK();
 }
 
-Status LogicalSessionCacheImpl::reapNow(Client* client) {
-    return _reap(client);
+void LogicalSessionCacheImpl::reapNow(OperationContext* opCtx) {
+    uassertStatusOK(_reap(opCtx->getClient()));
 }
 
 size_t LogicalSessionCacheImpl::size() {
@@ -123,21 +125,43 @@ size_t LogicalSessionCacheImpl::size() {
 void LogicalSessionCacheImpl::_periodicRefresh(Client* client) {
     try {
         _refresh(client);
-    } catch (...) {
-        log() << "Failed to refresh session cache: " << exceptionToStatus();
+    } catch (const DBException& ex) {
+        LOGV2(
+            20710,
+            "Failed to refresh session cache: {error}, will try again at the next refresh interval",
+            "Failed to refresh session cache, will try again at the next refresh interval",
+            "error"_attr = redact(ex));
     }
 }
 
 void LogicalSessionCacheImpl::_periodicReap(Client* client) {
     auto res = _reap(client);
     if (!res.isOK()) {
-        log() << "Failed to reap transaction table: " << res;
+        LOGV2(20711,
+              "Failed to reap transaction table: {error}",
+              "Failed to reap transaction table",
+              "error"_attr = redact(res));
     }
 
     return;
 }
 
 Status LogicalSessionCacheImpl::_reap(Client* client) {
+    boost::optional<ServiceContext::UniqueOperationContext> uniqueCtx;
+    auto* const opCtx = [&] {
+        if (client->getOperationContext()) {
+            return client->getOperationContext();
+        }
+
+        uniqueCtx.emplace(client->makeOperationContext());
+        return uniqueCtx->get();
+    }();
+
+    const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
+    if (replCoord && replCoord->isReplEnabled() && replCoord->getMemberState().arbiter()) {
+        return Status::OK();
+    }
+
     // Take the lock to update some stats.
     {
         stdx::lock_guard<Latch> lk(_mutex);
@@ -151,36 +175,28 @@ Status LogicalSessionCacheImpl::_reap(Client* client) {
         _stats.setTransactionReaperJobCount(_stats.getTransactionReaperJobCount() + 1);
     }
 
-    boost::optional<ServiceContext::UniqueOperationContext> uniqueCtx;
-    auto* const opCtx = [&] {
-        if (client->getOperationContext()) {
-            return client->getOperationContext();
-        }
-
-        uniqueCtx.emplace(client->makeOperationContext());
-        return uniqueCtx->get();
-    }();
-
     int numReaped = 0;
 
     try {
         ON_BLOCK_EXIT([&opCtx] { clearShardingOperationFailedStatus(opCtx); });
-
-        auto existsStatus = _sessionsColl->checkSessionsCollectionExists(opCtx);
-        if (!existsStatus.isOK()) {
-            StringData notSetUpWarning =
-                "Sessions collection is not set up; "
-                "waiting until next sessions reap interval";
-            if (existsStatus.code() != ErrorCodes::NamespaceNotFound ||
-                existsStatus.code() != ErrorCodes::NamespaceNotSharded) {
-                log() << notSetUpWarning << ": " << existsStatus.reason();
+        try {
+            _sessionsColl->checkSessionsCollectionExists(opCtx);
+        } catch (const DBException& ex) {
+            if (ex.code() != ErrorCodes::NamespaceNotFound ||
+                ex.code() != ErrorCodes::NamespaceNotSharded) {
+                LOGV2(
+                    20712,
+                    "Sessions collection is not set up: {error}; waiting until next sessions reap "
+                    "interval",
+                    "Sessions collection is not set up; waiting until next sessions reap interval",
+                    "error"_attr = redact(ex));
             } else {
-                log() << notSetUpWarning;
+                LOGV2(20713,
+                      "Sessions collection is not set up because the namespace is either not found"
+                      "or not sharded; waiting until next sessions reap interval");
             }
-
             return Status::OK();
         }
-
         numReaped = _reapSessionsOlderThanFn(opCtx,
                                              *_sessionsColl,
                                              _service->now() -
@@ -206,6 +222,22 @@ Status LogicalSessionCacheImpl::_reap(Client* client) {
 }
 
 void LogicalSessionCacheImpl::_refresh(Client* client) {
+    // get or make an opCtx
+    boost::optional<ServiceContext::UniqueOperationContext> uniqueCtx;
+    auto* const opCtx = [&client, &uniqueCtx] {
+        if (client->getOperationContext()) {
+            return client->getOperationContext();
+        }
+
+        uniqueCtx.emplace(client->makeOperationContext());
+        return uniqueCtx->get();
+    }();
+
+    const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
+    if (replCoord && replCoord->isReplEnabled() && replCoord->getMemberState().arbiter()) {
+        return;
+    }
+
     // Stats for serverStatus:
     {
         stdx::lock_guard<Latch> lk(_mutex);
@@ -228,24 +260,16 @@ void LogicalSessionCacheImpl::_refresh(Client* client) {
         _stats.setLastSessionsCollectionJobDurationMillis(millis.count());
     });
 
-    // get or make an opCtx
-    boost::optional<ServiceContext::UniqueOperationContext> uniqueCtx;
-    auto* const opCtx = [&client, &uniqueCtx] {
-        if (client->getOperationContext()) {
-            return client->getOperationContext();
-        }
-
-        uniqueCtx.emplace(client->makeOperationContext());
-        return uniqueCtx->get();
-    }();
-
     ON_BLOCK_EXIT([&opCtx] { clearShardingOperationFailedStatus(opCtx); });
 
-    auto setupStatus = _sessionsColl->setupSessionsCollection(opCtx);
-
-    if (!setupStatus.isOK()) {
-        log() << "Sessions collection is not set up; "
-              << "waiting until next sessions refresh interval: " << setupStatus.reason();
+    try {
+        _sessionsColl->setupSessionsCollection(opCtx);
+    } catch (const DBException& ex) {
+        LOGV2(
+            20714,
+            "Failed to refresh session cache, will try again at the next refresh interval {error}",
+            "Failed to refresh session cache, will try again at the next refresh interval",
+            "error"_attr = redact(ex));
         return;
     }
 
@@ -297,7 +321,7 @@ void LogicalSessionCacheImpl::_refresh(Client* client) {
     }
 
     // Refresh the active sessions in the sessions collection.
-    uassertStatusOK(_sessionsColl->refreshSessions(opCtx, activeSessionRecords));
+    _sessionsColl->refreshSessions(opCtx, activeSessionRecords);
     activeSessionsBackSwapper.dismiss();
     {
         stdx::lock_guard<Latch> lk(_mutex);
@@ -305,7 +329,7 @@ void LogicalSessionCacheImpl::_refresh(Client* client) {
     }
 
     // Remove the ending sessions from the sessions collection.
-    uassertStatusOK(_sessionsColl->removeRecords(opCtx, explicitlyEndingSessions));
+    _sessionsColl->removeRecords(opCtx, explicitlyEndingSessions);
     explicitlyEndingBackSwaper.dismiss();
     {
         stdx::lock_guard<Latch> lk(_mutex);
@@ -332,14 +356,13 @@ void LogicalSessionCacheImpl::_refresh(Client* client) {
     }
 
     // think about pruning ending and active out of openCursorSessions
-    auto statusAndRemovedSessions = _sessionsColl->findRemovedSessions(opCtx, openCursorSessions);
+    try {
+        auto removedSessions = _sessionsColl->findRemovedSessions(opCtx, openCursorSessions);
 
-    if (statusAndRemovedSessions.isOK()) {
-        auto removedSessions = statusAndRemovedSessions.getValue();
         for (const auto& lsid : removedSessions) {
             patterns.emplace(makeKillAllSessionsByPattern(opCtx, lsid));
         }
-    } else {
+    } catch (...) {
         // Ignore errors.
     }
 
@@ -367,11 +390,26 @@ LogicalSessionCacheStats LogicalSessionCacheImpl::getStats() {
     return _stats;
 }
 
-Status LogicalSessionCacheImpl::_addToCache(WithLock, LogicalSessionRecord record) {
+Status LogicalSessionCacheImpl::_addToCacheIfNotFull(WithLock, LogicalSessionRecord record) {
     if (_activeSessions.size() >= size_t(maxSessions)) {
-        return {ErrorCodes::TooManyLogicalSessions,
-                "Unable to add session into the cache because the number of active sessions is too "
-                "high"};
+        Status status = {ErrorCodes::TooManyLogicalSessions,
+                         str::stream()
+                             << "Unable to add session ID " << record.getId()
+                             << " into the cache because the number of active sessions is too "
+                                "high"};
+        // Returns Info() unless it was called in the past second.
+        // In that case it will return the quieter Debug(2) */
+        static auto& bumpedSeverity = *new logv2::SeveritySuppressor{
+            Seconds{1}, logv2::LogSeverity::Info(), logv2::LogSeverity::Debug(2)};
+        LOGV2_DEBUG(20715,
+                    bumpedSeverity().toInt(),
+                    "Unable to add session {sessionId} into the cache, too many active sessions: "
+                    "{sessionCount}, maximum: {maxSessions}",
+                    "Unable to add session into the cache, too many active sessions",
+                    "sessionId"_attr = record.getId(),
+                    "sessionCount"_attr = _activeSessions.size(),
+                    "maxSessions"_attr = maxSessions);
+        return status;
     }
 
     _activeSessions.insert(std::make_pair(record.getId(), std::move(record)));
