@@ -39,6 +39,7 @@ namespace mongo {
 
 class DocumentSourceChangeStreamTransform : public DocumentSource {
 public:
+    static constexpr StringData kStageName = "$_internalChangeStreamTransform"_sd;
     /**
      * Creates a new transformation stage from the given specification.
      */
@@ -58,11 +59,13 @@ public:
         return boost::none;
     }
 
-    DocumentSource::GetNextResult getNext();
     const char* getSourceName() const {
         return DocumentSourceChangeStream::kStageName.rawData();
     }
 
+protected:
+    DocumentSource::GetNextResult doGetNext() override;
+
 private:
     // This constructor is private, callers should use the 'create()' method above.
     DocumentSourceChangeStreamTransform(const boost::intrusive_ptr<ExpressionContext>&,
@@ -213,6 +216,9 @@ private:
     // Set to true if this transformation stage can be run on the collectionless namespace.
     bool _isIndependentOfAnyCollection;
 
+    // Set to true if the pre-image optime should be included in output documents.
+    bool _includePreImageOptime = false;
+
     // '_fcv' is used to determine which version of the resume token to generate for each change.
     // This is a snapshot of what the feature compatibility version was at the time the stream was
     // opened or resumed.
