@@ -39,11 +39,13 @@ namespace mongo {
  */
 class DocumentSourceCollStats : public DocumentSource {
 public:
+    static constexpr StringData kStageName = "$collStats"_sd;
+
     class LiteParsed final : public LiteParsedDocumentSource {
     public:
-        static std::unique_ptr<LiteParsed> parse(const AggregationRequest& request,
+        static std::unique_ptr<LiteParsed> parse(const NamespaceString& nss,
                                                  const BSONElement& spec) {
-            return std::make_unique<LiteParsed>(spec.fieldName(), request.getNamespaceString());
+            return std::make_unique<LiteParsed>(spec.fieldName(), nss);
         }
 
         explicit LiteParsed(std::string parseTimeName, NamespaceString nss)
@@ -53,7 +55,8 @@ public:
             return true;
         }
 
-        PrivilegeVector requiredPrivileges(bool isMongos) const final {
+        PrivilegeVector requiredPrivileges(bool isMongos,
+                                           bool bypassDocumentValidation) const final {
             return {Privilege(ResourcePattern::forExactNamespace(_nss), ActionType::collStats)};
         }
 
@@ -70,9 +73,7 @@ public:
     };
 
     DocumentSourceCollStats(const boost::intrusive_ptr<ExpressionContext>& pExpCtx)
-        : DocumentSource(pExpCtx) {}
-
-    GetNextResult getNext() final;
+        : DocumentSource(kStageName, pExpCtx) {}
 
     const char* getSourceName() const final;
 
@@ -83,7 +84,8 @@ public:
                                      DiskUseRequirement::kNoDiskUse,
                                      FacetRequirement::kNotAllowed,
                                      TransactionRequirement::kNotAllowed,
-                                     LookupRequirement::kAllowed);
+                                     LookupRequirement::kAllowed,
+                                     UnionRequirement::kAllowed);
 
         constraints.requiresInputDocSource = false;
         return constraints;
@@ -99,6 +101,8 @@ public:
         BSONElement elem, const boost::intrusive_ptr<ExpressionContext>& pExpCtx);
 
 private:
+    GetNextResult doGetNext() final;
+
     // The raw object given to $collStats containing user specified options.
     BSONObj _collStatsSpec;
     bool _finished = false;
