@@ -59,7 +59,7 @@ REGISTER_DOCUMENT_SOURCE(currentOp,
 constexpr StringData DocumentSourceCurrentOp::kStageName;
 
 std::unique_ptr<DocumentSourceCurrentOp::LiteParsed> DocumentSourceCurrentOp::LiteParsed::parse(
-    const AggregationRequest& request, const BSONElement& spec) {
+    const NamespaceString& nss, const BSONElement& spec) {
     // Need to check the value of allUsers; if true then inprog privilege is required.
     if (spec.type() != BSONType::Object) {
         uasserted(ErrorCodes::TypeMismatch,
@@ -100,17 +100,15 @@ std::unique_ptr<DocumentSourceCurrentOp::LiteParsed> DocumentSourceCurrentOp::Li
         }
     }
 
-    return stdx::make_unique<DocumentSourceCurrentOp::LiteParsed>(allUsers, localOps);
+    return std::make_unique<DocumentSourceCurrentOp::LiteParsed>(
+        spec.fieldName(), allUsers, localOps);
 }
 
-
 const char* DocumentSourceCurrentOp::getSourceName() const {
     return kStageName.rawData();
 }
 
-DocumentSource::GetNextResult DocumentSourceCurrentOp::getNext() {
-    pExpCtx->checkForInterrupt();
-
+DocumentSource::GetNextResult DocumentSourceCurrentOp::doGetNext() {
     if (_ops.empty()) {
         _ops = pExpCtx->mongoProcessInterface->getCurrentOps(pExpCtx,
                                                              _includeIdleConnections,
