@@ -31,28 +31,29 @@
 
 #include "mongo/db/pipeline/document_source_lookup.h"
 
+#include <memory>
+
 #include "mongo/base/init.h"
+#include "mongo/db/exec/document_value/document.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/matcher/expression_algo.h"
-#include "mongo/db/pipeline/document.h"
 #include "mongo/db/pipeline/document_path_support.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_context.h"
-#include "mongo/db/pipeline/value.h"
 #include "mongo/db/query/query_knobs_gen.h"
-#include "mongo/stdx/memory.h"
+#include "mongo/platform/overflow_arithmetic.h"
+#include "mongo/util/fail_point.h"
 
 namespace mongo {
 
 using boost::intrusive_ptr;
 using std::vector;
 
-constexpr size_t DocumentSourceLookUp::kMaxSubPipelineDepth;
-
 DocumentSourceLookUp::DocumentSourceLookUp(NamespaceString fromNs,
                                            std::string as,
                                            const boost::intrusive_ptr<ExpressionContext>& expCtx)
-    : DocumentSource(expCtx),
+    : DocumentSource(kStageName, expCtx),
       _fromNs(std::move(fromNs)),
       _as(std::move(as)),
       _variables(expCtx->variables),
@@ -60,20 +61,7 @@ DocumentSourceLookUp::DocumentSourceLookUp(NamespaceString fromNs,
     const auto& resolvedNamespace = expCtx->getResolvedNamespace(_fromNs);
     _resolvedNs = resolvedNamespace.ns;
     _resolvedPipeline = resolvedNamespace.pipeline;
-
-    // We always set an explicit collator on the copied expression context, even if the collator is
-    // null (i.e. the simple collation). Otherwise, in the situation where the user did not specify
-    // a collation and the simple collation was inherited from the local collection, the execution
-    // machinery will incorrectly interpret the null collator and empty user collation as an
-    // indication that it should inherit the foreign collection's collation.
-    _fromExpCtx = expCtx->copyWith(
-        _resolvedNs, boost::none, expCtx->getCollator() ? expCtx->getCollator()->clone() : nullptr);
-
-    _fromExpCtx->subPipelineDepth += 1;
-    uassert(ErrorCodes::MaxSubPipelineDepthExceeded,
-            str::stream() << "Maximum number of nested $lookup sub-pipelines exceeded. Limit is "
-                          << kMaxSubPipelineDepth,
-            _fromExpCtx->subPipelineDepth <= kMaxSubPipelineDepth);
+    _fromExpCtx = expCtx->copyForSubPipeline(resolvedNamespace.ns);
 }
 
 DocumentSourceLookUp::DocumentSourceLookUp(NamespaceString fromNs,
@@ -121,7 +109,7 @@ DocumentSourceLookUp::DocumentSourceLookUp(NamespaceString fromNs,
 }
 
 std::unique_ptr<DocumentSourceLookUp::LiteParsed> DocumentSourceLookUp::LiteParsed::parse(
-    const AggregationRequest& request, const BSONElement& spec) {
+    const NamespaceString& nss, const BSONElement& spec) {
     uassert(ErrorCodes::FailedToParse,
             str::stream() << "the $lookup stage specification must be an object, but found "
                           << typeName(spec.type()),
@@ -137,29 +125,46 @@ std::unique_ptr<DocumentSourceLookUp::LiteParsed> DocumentSourceLookUp::LitePars
                           << typeName(specObj["from"].type()),
             fromElement.type() == BSONType::String);
 
-    NamespaceString fromNss(request.getNamespaceString().db(), fromElement.valueStringData());
+    NamespaceString fromNss(nss.db(), fromElement.valueStringData());
     uassert(ErrorCodes::InvalidNamespace,
             str::stream() << "invalid $lookup namespace: " << fromNss.ns(),
             fromNss.isValid());
 
-    stdx::unordered_set<NamespaceString> foreignNssSet;
-
     // Recursively lite parse the nested pipeline, if one exists.
     auto pipelineElem = specObj["pipeline"];
-    std::vector<LiteParsedPipeline> liteParsedPipelineVector;
+    boost::optional<LiteParsedPipeline> liteParsedPipeline;
     if (pipelineElem) {
         auto pipeline = uassertStatusOK(AggregationRequest::parsePipelineFromBSON(pipelineElem));
-        AggregationRequest foreignAggReq(fromNss, std::move(pipeline));
-        LiteParsedPipeline liteParsedPipeline(foreignAggReq);
-        auto pipelineInvolvedNamespaces = liteParsedPipeline.getInvolvedNamespaces();
-        foreignNssSet.insert(pipelineInvolvedNamespaces.begin(), pipelineInvolvedNamespaces.end());
-        liteParsedPipelineVector.push_back(std::move(liteParsedPipeline));
+        liteParsedPipeline = LiteParsedPipeline(fromNss, pipeline);
     }
-    foreignNssSet.insert(fromNss);
-    return stdx::make_unique<DocumentSourceLookUp::LiteParsed>(spec.fieldName(),
-                                                               std::move(fromNss),
-                                                               std::move(foreignNssSet),
-                                                               std::move(liteParsedPipelineVector));
+
+    return std::make_unique<DocumentSourceLookUp::LiteParsed>(
+        spec.fieldName(), std::move(fromNss), std::move(liteParsedPipeline));
+}
+
+PrivilegeVector DocumentSourceLookUp::LiteParsed::requiredPrivileges(
+    bool isMongos, bool bypassDocumentValidation) const {
+    PrivilegeVector requiredPrivileges;
+    invariant(_pipelines.size() <= 1);
+    invariant(_foreignNss);
+
+    // If no pipeline is specified or the local/foreignField syntax was used, then assume that we're
+    // reading directly from the collection.
+    if (_pipelines.empty() || !_pipelines[0].startsWithInitialSource()) {
+        Privilege::addPrivilegeToPrivilegeVector(
+            &requiredPrivileges,
+            Privilege(ResourcePattern::forExactNamespace(*_foreignNss), ActionType::find));
+    }
+
+    // Add the sub-pipeline privileges, if one was specified.
+    if (!_pipelines.empty()) {
+        const LiteParsedPipeline& pipeline = _pipelines[0];
+        Privilege::addPrivilegesToPrivilegeVector(
+            &requiredPrivileges,
+            std::move(pipeline.requiredPrivileges(isMongos, bypassDocumentValidation)));
+    }
+
+    return requiredPrivileges;
 }
 
 REGISTER_DOCUMENT_SOURCE(lookup,
@@ -167,23 +172,10 @@ REGISTER_DOCUMENT_SOURCE(lookup,
                          DocumentSourceLookUp::createFromBson);
 
 const char* DocumentSourceLookUp::getSourceName() const {
-    return "$lookup";
+    return kStageName.rawData();
 }
 
 StageConstraints DocumentSourceLookUp::constraints(Pipeline::SplitState) const {
-    // By default, $lookup is allowed in a transaction and does not use disk.
-    auto diskRequirement = DiskUseRequirement::kNoDiskUse;
-    auto txnRequirement = TransactionRequirement::kAllowed;
-
-    // However, if $lookup is specified with a pipeline, it inherits the strictest disk use and
-    // transaction requirement from the children in its pipeline.
-    if (wasConstructedWithPipelineSyntax()) {
-        const auto resolvedRequirements = StageConstraints::resolveDiskUseAndTransactionRequirement(
-            _resolvedIntrospectionPipeline->getSources());
-        diskRequirement = resolvedRequirements.first;
-        txnRequirement = resolvedRequirements.second;
-    }
-
     // If executing on mongos and the foreign collection is sharded, then this stage can run on
     // mongos or any shard.
     HostTypeRequirement hostRequirement =
@@ -198,15 +190,26 @@ StageConstraints DocumentSourceLookUp::constraints(Pipeline::SplitState) const {
         hostRequirement = HostTypeRequirement::kPrimaryShard;
     }
 
+    // By default, $lookup is allowed in a transaction and does not use disk.
     StageConstraints constraints(StreamType::kStreaming,
                                  PositionRequirement::kNone,
                                  hostRequirement,
-                                 diskRequirement,
+                                 DiskUseRequirement::kNoDiskUse,
                                  FacetRequirement::kAllowed,
-                                 txnRequirement,
-                                 LookupRequirement::kAllowed);
+                                 TransactionRequirement::kAllowed,
+                                 LookupRequirement::kAllowed,
+                                 UnionRequirement::kAllowed);
+
+    // However, if $lookup is specified with a pipeline, it inherits the strictest disk use, facet,
+    // transaction, and lookup requirements from the children in its pipeline.
+    if (wasConstructedWithPipelineSyntax()) {
+        constraints = StageConstraints::getStrictestConstraints(
+            _resolvedIntrospectionPipeline->getSources(), constraints);
+    }
 
     constraints.canSwapWithMatch = true;
+    constraints.canSwapWithLimitAndSample = !_unwindSrc;
+
     return constraints;
 }
 
@@ -231,11 +234,28 @@ BSONObj buildEqualityOrQuery(const std::string& fieldName, const BSONArray& valu
     return orBuilder.obj();
 }
 
+void assertIsValidCollectionState(const boost::intrusive_ptr<ExpressionContext>& expCtx) {
+    if (expCtx->mongoProcessInterface->isSharded(expCtx->opCtx, expCtx->ns)) {
+        const bool foreignShardedAllowed =
+            getTestCommandsEnabled() && internalQueryAllowShardedLookup.load();
+        if (!foreignShardedAllowed) {
+            uasserted(51069, "Cannot run $lookup with sharded foreign collection");
+        }
+    }
+}
+
+void lookupPipeValidator(const Pipeline& pipeline) {
+    const auto& sources = pipeline.getSources();
+    std::for_each(sources.begin(), sources.end(), [](auto& src) {
+        uassert(51047,
+                str::stream() << src->getSourceName()
+                              << " is not allowed within a $lookup's sub-pipeline",
+                src->constraints().isAllowedInLookupPipeline());
+    });
+}
 }  // namespace
 
-DocumentSource::GetNextResult DocumentSourceLookUp::getNext() {
-    pExpCtx->checkForInterrupt();
-
+DocumentSource::GetNextResult DocumentSourceLookUp::doGetNext() {
     if (_unwindSrc) {
         return unwindResult();
     }
@@ -261,22 +281,22 @@ DocumentSource::GetNextResult DocumentSourceLookUp::getNext() {
     auto pipeline = buildPipeline(inputDoc);
 
     std::vector<Value> results;
-    int objsize = 0;
+    long long objsize = 0;
     const auto maxBytes = internalLookupStageIntermediateDocumentMaxSizeBytes.load();
+
     while (auto result = pipeline->getNext()) {
-        objsize += result->getApproximateSize();
+        long long safeSum = 0;
+        bool hasOverflowed = overflow::add(objsize, result->getApproximateSize(), &safeSum);
         uassert(4568,
                 str::stream() << "Total size of documents in " << _fromNs.coll()
                               << " matching pipeline's $lookup stage exceeds " << maxBytes
                               << " bytes",
 
-                objsize <= maxBytes);
+                !hasOverflowed && objsize <= maxBytes);
+        objsize = safeSum;
         results.emplace_back(std::move(*result));
     }
-    for (auto&& source : pipeline->getSources()) {
-        if (source->usedDisk())
-            _usedDisk = true;
-    }
+    _usedDisk = _usedDisk || pipeline->usedDisk();
 
     MutableDocument output(std::move(inputDoc));
     output.setNestedField(_as, Value(std::move(results)));
@@ -288,36 +308,49 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
     // Copy all 'let' variables into the foreign pipeline's expression context.
     copyVariablesToExpCtx(_variables, _variablesParseState, _fromExpCtx.get());
 
+    assertIsValidCollectionState(_fromExpCtx);
+
     // Resolve the 'let' variables to values per the given input document.
     resolveLetVariables(inputDoc, &_fromExpCtx->variables);
 
     // If we don't have a cache, build and return the pipeline immediately.
     if (!_cache || _cache->isAbandoned()) {
-        return pExpCtx->mongoProcessInterface->makePipeline(_resolvedPipeline, _fromExpCtx);
+        MakePipelineOptions pipelineOpts;
+        pipelineOpts.optimize = true;
+        pipelineOpts.attachCursorSource = true;
+        pipelineOpts.validator = lookupPipeValidator;
+        // By default, $lookup doesnt support sharded 'from' collections.
+        pipelineOpts.allowTargetingShards = internalQueryAllowShardedLookup.load();
+        return Pipeline::makePipeline(_resolvedPipeline, _fromExpCtx, pipelineOpts);
     }
 
-    // Tailor the pipeline construction for our needs. We want a non-optimized pipeline without a
-    // cursor source.
-    MongoProcessInterface::MakePipelineOptions pipelineOpts;
+    // Construct the basic pipeline without a cache stage. Avoid optimizing here since we need to
+    // add the cache first, as detailed below.
+    MakePipelineOptions pipelineOpts;
     pipelineOpts.optimize = false;
     pipelineOpts.attachCursorSource = false;
-
-    // Construct the basic pipeline without a cache stage.
-    auto pipeline =
-        pExpCtx->mongoProcessInterface->makePipeline(_resolvedPipeline, _fromExpCtx, pipelineOpts);
+    pipelineOpts.validator = lookupPipeValidator;
+    auto pipeline = Pipeline::makePipeline(_resolvedPipeline, _fromExpCtx, pipelineOpts);
 
     // Add the cache stage at the end and optimize. During the optimization process, the cache will
     // either move itself to the correct position in the pipeline, or will abandon itself if no
-    // suitable cache position exists.
-    pipeline->addFinalSource(
-        DocumentSourceSequentialDocumentCache::create(_fromExpCtx, _cache.get_ptr()));
+    // suitable cache position exists. Do it only if pipeline optimization is enabled, otherwise
+    // Pipeline::optimizePipeline() will exit early and correct placement of the cache will not
+    // occur.
+    if (auto fp = globalFailPointRegistry().find("disablePipelineOptimization");
+        fp && fp->shouldFail()) {
+        _cache->abandon();
+    } else {
+        pipeline->addFinalSource(
+            DocumentSourceSequentialDocumentCache::create(_fromExpCtx, _cache.get_ptr()));
+    }
 
     pipeline->optimizePipeline();
 
     if (!_cache->isServing()) {
         // The cache has either been abandoned or has not yet been built. Attach a cursor.
-        pipeline = pExpCtx->mongoProcessInterface->attachCursorSourceToPipeline(_fromExpCtx,
-                                                                                pipeline.release());
+        pipeline = pExpCtx->mongoProcessInterface->attachCursorSourceToPipeline(
+            pipeline.release(), internalQueryAllowShardedLookup.load() /* allowTargetingShards*/);
     }
 
     // If the cache has been abandoned, release it.
@@ -642,20 +675,7 @@ void DocumentSourceLookUp::resolveLetVariables(const Document& localDoc, Variabl
 void DocumentSourceLookUp::initializeResolvedIntrospectionPipeline() {
     copyVariablesToExpCtx(_variables, _variablesParseState, _fromExpCtx.get());
     _resolvedIntrospectionPipeline =
-        uassertStatusOK(Pipeline::parse(_resolvedPipeline, _fromExpCtx));
-
-    auto& sources = _resolvedIntrospectionPipeline->getSources();
-
-    auto it = std::find_if(
-        sources.begin(), sources.end(), [](const boost::intrusive_ptr<DocumentSource>& src) {
-            return !src->constraints().isAllowedInLookupPipeline();
-        });
-
-    // For other stages, use a generic error.
-    uassert(51047,
-            str::stream() << (*it)->getSourceName()
-                          << " is not allowed within a $lookup's sub-pipeline",
-            it == sources.end());
+        Pipeline::parse(_resolvedPipeline, _fromExpCtx, lookupPipeValidator);
 }
 
 void DocumentSourceLookUp::serializeToArray(
@@ -727,12 +747,12 @@ DepsTracker::State DocumentSourceLookUp::getDependencies(DepsTracker* deps) cons
         // We will use the introspection pipeline which we prebuilt during construction.
         invariant(_resolvedIntrospectionPipeline);
 
-        // We are not attempting to enforce that any referenced metadata are in fact available,
+        // We are not attempting to enforce that any referenced metadata are in fact unavailable,
         // this is done elsewhere. We only need to know what variable dependencies exist in the
         // subpipeline for the top-level pipeline. So without knowledge of what metadata is in fact
-        // available, we "lie" and say that all metadata is available to avoid tripping any
+        // unavailable, we "lie" and say that all metadata is available to avoid tripping any
         // assertions.
-        DepsTracker subDeps(DepsTracker::kAllMetadataAvailable);
+        DepsTracker subDeps(DepsTracker::kNoMetadata);
 
         // Get the subpipeline dependencies. Subpipeline stages may reference both 'let' variables
         // declared by this $lookup and variables declared externally.
