@@ -35,13 +35,16 @@
 #include <string>
 #include <vector>
 
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/db/exec/document_value/document_comparator.h"
+#include "mongo/db/exec/document_value/value_comparator.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/pipeline/aggregation_request.h"
-#include "mongo/db/pipeline/document_comparator.h"
-#include "mongo/db/pipeline/mongo_process_interface.h"
-#include "mongo/db/pipeline/value_comparator.h"
+#include "mongo/db/pipeline/javascript_execution.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
+#include "mongo/db/pipeline/runtime_constants_gen.h"
 #include "mongo/db/pipeline/variables.h"
 #include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/datetime/date_time_support.h"
@@ -56,6 +59,7 @@ namespace mongo {
 
 class ExpressionContext : public RefCountable {
 public:
+    static constexpr size_t kMaxSubPipelineViewDepth = 20;
     struct ResolvedNamespace {
         ResolvedNamespace() = default;
         ResolvedNamespace(NamespaceString ns, std::vector<BSONObj> pipeline);
@@ -91,9 +95,7 @@ public:
 
         boost::intrusive_ptr<ExpressionContext> _expCtx;
 
-        BSONObj _originalCollation;
-        std::unique_ptr<CollatorInterface> _originalCollatorOwned;
-        const CollatorInterface* _originalCollatorUnowned{nullptr};
+        std::unique_ptr<CollatorInterface> _originalCollator;
     };
 
     /**
@@ -108,11 +110,33 @@ public:
                       boost::optional<UUID> collUUID);
 
     /**
-     * Constructs an ExpressionContext suitable for use outside of the aggregation system, including
-     * for MatchExpression parsing and executing pipeline-style operations in the Update system.
+     * Constructs an ExpressionContext to be used for Pipeline parsing and evaluation. This version
+     * requires finer-grained parameters but does not require an AggregationRequest.
+     * 'resolvedNamespaces' maps collection names (not full namespaces) to ResolvedNamespaces.
      */
     ExpressionContext(OperationContext* opCtx,
-                      const CollatorInterface* collator,
+                      const boost::optional<ExplainOptions::Verbosity>& explain,
+                      bool fromMongos,
+                      bool needsMerge,
+                      bool allowDiskUse,
+                      bool bypassDocumentValidation,
+                      bool isMapReduceCommand,
+                      const NamespaceString& ns,
+                      const boost::optional<RuntimeConstants>& runtimeConstants,
+                      std::unique_ptr<CollatorInterface> collator,
+                      const std::shared_ptr<MongoProcessInterface>& mongoProcessInterface,
+                      StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces,
+                      boost::optional<UUID> collUUID);
+
+    /**
+     * Constructs an ExpressionContext suitable for use outside of the aggregation system, including
+     * for MatchExpression parsing and executing pipeline-style operations in the Update system.
+     *
+     * If 'collator' is null, the simple collator will be used.
+     */
+    ExpressionContext(OperationContext* opCtx,
+                      std::unique_ptr<CollatorInterface> collator,
+                      const NamespaceString& ns,
                       const boost::optional<RuntimeConstants>& runtimeConstants = boost::none);
 
     /**
@@ -143,10 +167,39 @@ public:
     }
 
     const CollatorInterface* getCollator() const {
-        return _collator;
+        return _collator.get();
     }
 
-    void setCollator(const CollatorInterface* collator);
+    bool shouldCollectExecStats() const {
+        return static_cast<bool>(explain);
+    }
+
+    /**
+     * Returns the BSON spec for the ExpressionContext's collator, or the simple collator spec if
+     * the collator is null.
+     *
+     * The ExpressionContext is always set up with the fully-resolved collation. So even though
+     * SERVER-24433 describes an ambiguity between a null collator, here we can say confidently that
+     * null must mean simple since we have already handled "absence of a collator" before creating
+     * the ExpressionContext.
+     */
+    BSONObj getCollatorBSON() const {
+        return _collator ? _collator->getSpec().toBSON() : CollationSpec::kSimpleSpec;
+    }
+
+    /**
+     * Sets '_collator' and resets 'documentComparator' and 'valueComparator'.
+     *
+     * Use with caution - '_collator' is used in the context of a Pipeline, and it is illegal
+     * to change the collation once a Pipeline has been parsed with this ExpressionContext.
+     */
+    void setCollator(std::unique_ptr<CollatorInterface> collator) {
+        _collator = std::move(collator);
+
+        // Document/Value comparisons must be aware of the collation.
+        _documentComparator = DocumentComparator(_collator.get());
+        _valueComparator = ValueComparator(_collator.get());
+    }
 
     const DocumentComparator& getDocumentComparator() const {
         return _documentComparator;
@@ -165,12 +218,22 @@ public:
 
     /**
      * Returns an ExpressionContext that is identical to 'this' that can be used to execute a
-     * separate aggregation pipeline on 'ns' with the optional 'uuid'.
+     * separate aggregation pipeline on 'ns' with the optional 'uuid' and an updated collator.
      */
     boost::intrusive_ptr<ExpressionContext> copyWith(
         NamespaceString ns,
         boost::optional<UUID> uuid = boost::none,
-        boost::optional<std::unique_ptr<CollatorInterface>> collator = boost::none) const;
+        boost::optional<std::unique_ptr<CollatorInterface>> updatedCollator = boost::none) const;
+
+    boost::intrusive_ptr<ExpressionContext> copyForSubPipeline(NamespaceString nss) const {
+        uassert(ErrorCodes::MaxSubPipelineDepthExceeded,
+                str::stream() << "Maximum number of nested sub-pipelines exceeded. Limit is "
+                              << ExpressionContext::kMaxSubPipelineViewDepth,
+                subPipelineDepth < kMaxSubPipelineViewDepth);
+        auto newCopy = copyWith(std::move(nss));
+        newCopy->subPipelineDepth += 1;
+        return newCopy;
+    }
 
     /**
      * Returns the ResolvedNamespace corresponding to 'nss'. It is an error to call this method on a
@@ -194,22 +257,53 @@ public:
         _resolvedNamespaces = std::move(resolvedNamespaces);
     }
 
-    auto getRuntimeConstants() const {
+    const RuntimeConstants& getRuntimeConstants() const {
         return variables.getRuntimeConstants();
     }
+
+    /**
+     * Retrieves the Javascript Scope for the current thread or creates a new one if it has not been
+     * created yet. Initializes the Scope with the 'jsScope' variables from the runtimeConstants.
+     * Loads the Scope with the functions stored in system.js if the expression isn't executed on
+     * mongos and is called from a MapReduce command or `forceLoadOfStoredProcedures` is true.
+     *
+     * Returns a JsExec and a boolean indicating whether the Scope was created as part of this call.
+     */
+    auto getJsExecWithScope(bool forceLoadOfStoredProcedures = false) const {
+        uassert(31264,
+                "Cannot run server-side javascript without the javascript engine enabled",
+                getGlobalScriptEngine());
+        const auto& runtimeConstants = getRuntimeConstants();
+        const boost::optional<bool> isMapReduceCommand = runtimeConstants.getIsMapReduce();
+        if (inMongos) {
+            invariant(!forceLoadOfStoredProcedures);
+            invariant(!isMapReduceCommand);
+        }
+
+        // Stored procedures are only loaded for the $where expression and MapReduce command.
+        const bool loadStoredProcedures = forceLoadOfStoredProcedures || isMapReduceCommand;
+
+        if (hasWhereClause && !loadStoredProcedures) {
+            uasserted(4649200,
+                      "A single operation cannot use both JavaScript aggregation expressions and "
+                      "$where.");
+        }
+
+        const boost::optional<mongo::BSONObj>& scope = runtimeConstants.getJsScope();
+        return JsExecution::get(
+            opCtx, scope.get_value_or(BSONObj()), ns.db(), loadStoredProcedures, jsHeapLimitMB);
+    }
+
     // The explain verbosity requested by the user, or boost::none if no explain was requested.
     boost::optional<ExplainOptions::Verbosity> explain;
 
-    // The comment provided by the user, or the empty string if no comment was provided.
-    std::string comment;
-
     bool fromMongos = false;
     bool needsMerge = false;
-    bool mergeByPBRT = false;
     bool inMongos = false;
     bool allowDiskUse = false;
     bool bypassDocumentValidation = false;
     bool inMultiDocumentTransaction = false;
+    bool hasWhereClause = false;
 
     NamespaceString ns;
 
@@ -220,6 +314,15 @@ public:
 
     OperationContext* opCtx;
 
+    // When set restricts the global JavaScript heap size limit for any Scope returned by
+    // getJsExecWithScope(). This limit is ignored if larger than the global limit dictated by the
+    // 'jsHeapLimitMB' server parameter.
+    boost::optional<int> jsHeapLimitMB;
+
+    // When set this timeout limits the allowed execution time for a JavaScript function invocation
+    // under any Scope returned by getJsExecWithScope().
+    int jsFnTimeoutMillis;
+
     // An interface for accessing information or performing operations that have different
     // implementations on mongod and mongos, or that only make sense on one of the two.
     // Additionally, putting some of this functionality behind an interface prevents aggregation
@@ -228,10 +331,6 @@ public:
 
     const TimeZoneDatabase* timeZoneDatabase;
 
-    // Collation requested by the user for this pipeline. Empty if the user did not request a
-    // collation.
-    BSONObj collation;
-
     Variables variables;
     VariablesParseState variablesParseState;
 
@@ -247,37 +346,30 @@ public:
     boost::optional<ServerGlobalParams::FeatureCompatibility::Version>
         maxFeatureCompatibilityVersion;
 
+    // In a shard mongoD, we use this to determine the format for "sortKey" data. This format has to
+    // match the format that the mongoS expects.
+    // TODO (SERVER-43361): After branching for 4.5, there will be no need to support any format
+    // other than the "k44SortKey" format, so this will be removed.
+    SortKeyFormat sortKeyFormat = SortKeyFormat::k44SortKey;
+
     // True if this context is associated with a pipeline which is permitted to use the new
     // upsertSupplied mechanism for applicable $merge modes.
+    // TODO SERVER-44884: remove this when we branch for 4.5.
     bool useNewUpsert = false;
 
+    // True if this ExpressionContext is used to parse a view definition pipeline.
+    bool isParsingViewDefinition = false;
+
+    // True if this ExpressionContext is used to parse a collection validator expression.
+    bool isParsingCollectionValidator = false;
+
 protected:
     static const int kInterruptCheckPeriod = 128;
 
-    ExpressionContext(NamespaceString nss,
-                      std::shared_ptr<MongoProcessInterface>,
-                      const TimeZoneDatabase* tzDb);
-
-    /**
-     * Sets '_ownedCollator' and resets '_collator', 'documentComparator' and 'valueComparator'.
-     *
-     * Use with caution - '_ownedCollator' is used in the context of a Pipeline, and it is illegal
-     * to change the collation once a Pipeline has been parsed with this ExpressionContext.
-     */
-    void setCollator(std::unique_ptr<CollatorInterface> collator) {
-        _ownedCollator = std::move(collator);
-        setCollator(_ownedCollator.get());
-    }
-
     friend class CollatorStash;
 
-    // Collator used for comparisons. This is owned in the context of a Pipeline.
-    // TODO SERVER-31294: Move ownership of an aggregation's collator elsewhere.
-    std::unique_ptr<CollatorInterface> _ownedCollator;
-
-    // Collator used for comparisons. If '_ownedCollator' is non-null, then this must point to the
-    // same collator object.
-    const CollatorInterface* _collator = nullptr;
+    // Collator used for comparisons.
+    std::unique_ptr<CollatorInterface> _collator;
 
     // Used for all comparisons of Document/Value during execution of the aggregation operation.
     // Must not be changed after parsing a Pipeline with this ExpressionContext.
