@@ -32,7 +32,6 @@
 #include <set>
 #include <string>
 
-#include "mongo/db/exec/projection_exec_agg.h"
 #include "mongo/db/field_ref.h"
 #include "mongo/db/index/multikey_paths.h"
 #include "mongo/db/index_names.h"
@@ -41,9 +40,9 @@
 #include "mongo/util/str.h"
 
 namespace mongo {
-
 class CollatorInterface;
 class MatchExpression;
+class WildcardProjection;
 
 /**
  * A CoreIndexInfo is a representation of an index in the catalog with parsed information which is
@@ -60,16 +59,16 @@ struct CoreIndexInfo {
                   Identifier ident,
                   const MatchExpression* fe = nullptr,
                   const CollatorInterface* ci = nullptr,
-                  const ProjectionExecAgg* projExec = nullptr)
+                  const WildcardProjection* wildcardProj = nullptr)
         : identifier(std::move(ident)),
           keyPattern(kp),
           filterExpr(fe),
           type(type),
           sparse(sp),
           collator(ci),
-          wildcardProjection(projExec) {
+          wildcardProjection(wildcardProj) {
         // We always expect a projection executor for $** indexes, and none otherwise.
-        invariant((type == IndexType::INDEX_WILDCARD) == (projExec != nullptr));
+        invariant((type == IndexType::INDEX_WILDCARD) == (wildcardProjection != nullptr));
     }
 
     virtual ~CoreIndexInfo() = default;
@@ -137,7 +136,7 @@ struct CoreIndexInfo {
 
     // For $** indexes, a pointer to the projection executor owned by the index access method. Null
     // unless this IndexEntry represents a wildcard index, in which case this is always non-null.
-    const ProjectionExecAgg* wildcardProjection = nullptr;
+    const WildcardProjection* wildcardProjection = nullptr;
 };
 
 /**
@@ -159,8 +158,8 @@ struct IndexEntry : CoreIndexInfo {
                const MatchExpression* fe,
                const BSONObj& io,
                const CollatorInterface* ci,
-               const ProjectionExecAgg* projExec)
-        : CoreIndexInfo(kp, type, sp, std::move(ident), fe, ci, projExec),
+               const WildcardProjection* wildcardProjection)
+        : CoreIndexInfo(kp, type, sp, std::move(ident), fe, ci, wildcardProjection),
           multikey(mk),
           multikeyPaths(mkp),
           multikeyPathSet(std::move(multikeyPathSet)),
