@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -165,6 +164,9 @@ bool Interval::precedes(const Interval& other) const {
     return false;
 }
 
+bool Interval::isMinToMax() const {
+    return (start.type() == BSONType::MinKey && end.type() == BSONType::MaxKey);
+}
 
 Interval::IntervalComparison Interval::compare(const Interval& other) const {
     //
