@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/db/query/plan_enumerator.h"
 
@@ -35,7 +35,7 @@
 
 #include "mongo/db/query/index_tag.h"
 #include "mongo/db/query/indexability.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/string_map.h"
 
 namespace {
@@ -259,6 +259,7 @@ PlanEnumerator::PlanEnumerator(const PlanEnumeratorParams& params)
     : _root(params.root),
       _indices(params.indices),
       _ixisect(params.intersect),
+      _enumerateOrChildrenLockstep(params.enumerateOrChildrenLockstep),
       _orLimit(params.maxSolutionsPerOr),
       _intersectLimit(params.maxIntersectPerAnd) {}
 
@@ -290,7 +291,7 @@ std::string PlanEnumerator::dumpMemo() {
 }
 
 string PlanEnumerator::NodeAssignment::toString() const {
-    if (NULL != andAssignment) {
+    if (nullptr != andAssignment) {
         str::stream ss;
         ss << "AND enumstate counter " << andAssignment->counter;
         for (size_t i = 0; i < andAssignment->choices.size(); ++i) {
@@ -316,7 +317,7 @@ string PlanEnumerator::NodeAssignment::toString() const {
             }
         }
         return ss;
-    } else if (NULL != arrayAssignment) {
+    } else if (nullptr != arrayAssignment) {
         str::stream ss;
         ss << "ARRAY SUBNODES enumstate " << arrayAssignment->counter << "/ ONE OF: [ ";
         for (size_t i = 0; i < arrayAssignment->subnodes.size(); ++i) {
@@ -324,8 +325,7 @@ string PlanEnumerator::NodeAssignment::toString() const {
         }
         ss << "]";
         return ss;
-    } else {
-        verify(NULL != orAssignment);
+    } else if (nullptr != orAssignment) {
         str::stream ss;
         ss << "ALL OF: [ ";
         for (size_t i = 0; i < orAssignment->subnodes.size(); ++i) {
@@ -333,14 +333,33 @@ string PlanEnumerator::NodeAssignment::toString() const {
         }
         ss << "]";
         return ss;
+    } else if (nullptr != lockstepOrAssignment) {
+        str::stream ss;
+        ss << "ALL OF (lockstep): {";
+        ss << "\n\ttotalEnumerated: " << lockstepOrAssignment->totalEnumerated;
+        ss << "\n\tsubnodes: [ ";
+        for (auto&& node : lockstepOrAssignment->subnodes) {
+            ss << "\n\t\t{";
+            ss << "memoId: " << node.memoId << ", ";
+            ss << "iterationCount: " << node.iterationCount << ", ";
+            if (node.maxIterCount) {
+                ss << "maxIterCount: " << node.maxIterCount;
+            } else {
+                ss << "maxIterCount: none";
+            }
+            ss << "},";
+        }
+        ss << "\n]";
+        return ss;
     }
+    MONGO_UNREACHABLE;
 }
 
 PlanEnumerator::MemoID PlanEnumerator::memoIDForNode(MatchExpression* node) {
     stdx::unordered_map<MatchExpression*, MemoID>::iterator it = _nodeToId.find(node);
 
     if (_nodeToId.end() == it) {
-        error() << "Trying to look up memo entry for node, none found.";
+        LOGV2_ERROR(20945, "Trying to look up memo entry for node, none found");
         MONGO_UNREACHABLE;
     }
 
@@ -359,7 +378,11 @@ unique_ptr<MatchExpression> PlanEnumerator::getNext() {
     tagForSort(tree.get());
 
     _root->resetTag();
-    LOG(5) << "Enumerator: memo just before moving:" << endl << dumpMemo();
+    LOGV2_DEBUG(20943,
+                5,
+                "Enumerator: memo just before moving:\n{memo}",
+                "Enumerator: memo just before moving",
+                "memo"_attr = dumpMemo());
     _done = nextMemo(memoIDForNode(_root));
     return tree;
 }
@@ -426,11 +449,19 @@ bool PlanEnumerator::prepMemo(MatchExpression* node, PrepMemoContext context) {
         NodeAssignment* assign;
         allocateAssignment(node, &assign, &myMemoID);
 
-        OrAssignment* orAssignment = new OrAssignment();
-        for (size_t i = 0; i < node->numChildren(); ++i) {
-            orAssignment->subnodes.push_back(memoIDForNode(node->getChild(i)));
+        if (_enumerateOrChildrenLockstep) {
+            LockstepOrAssignment* newOrAssign = new LockstepOrAssignment();
+            for (size_t i = 0; i < node->numChildren(); ++i) {
+                newOrAssign->subnodes.push_back({memoIDForNode(node->getChild(i)), 0, boost::none});
+            }
+            assign->lockstepOrAssignment.reset(newOrAssign);
+        } else {
+            OrAssignment* orAssignment = new OrAssignment();
+            for (size_t i = 0; i < node->numChildren(); ++i) {
+                orAssignment->subnodes.push_back(memoIDForNode(node->getChild(i)));
+            }
+            assign->orAssignment.reset(orAssignment);
         }
-        assign->orAssignment.reset(orAssignment);
         return true;
     } else if (Indexability::arrayUsesIndexOnChildren(node)) {
         // Add each of our children as a subnode.  We enumerate through each subnode one at a
@@ -506,7 +537,7 @@ bool PlanEnumerator::prepMemo(MatchExpression* node, PrepMemoContext context) {
 
         // There can only be one mandatory predicate (at most one $text, at most one
         // $geoNear, can't combine $text/$geoNear).
-        MatchExpression* mandatoryPred = NULL;
+        MatchExpression* mandatoryPred = nullptr;
 
         // There could be multiple indices which we could use to satisfy the mandatory
         // predicate. Keep the set of such indices. Currently only one text index is
@@ -528,7 +559,7 @@ bool PlanEnumerator::prepMemo(MatchExpression* node, PrepMemoContext context) {
                 // This should include only TEXT and GEO_NEAR preds.
 
                 // We expect either 0 or 1 mandatory predicates.
-                invariant(NULL == mandatoryPred);
+                invariant(nullptr == mandatoryPred);
 
                 // Mandatory predicates are TEXT or GEO_NEAR.
                 invariant(MatchExpression::TEXT == child->matchType() ||
@@ -577,7 +608,7 @@ bool PlanEnumerator::prepMemo(MatchExpression* node, PrepMemoContext context) {
             return true;
         }
 
-        if (NULL != mandatoryPred) {
+        if (nullptr != mandatoryPred) {
             // We must have at least one index which can be used to answer 'mandatoryPred'.
             invariant(!mandatoryIndices.empty());
             return enumerateMandatoryIndex(
@@ -805,7 +836,7 @@ void PlanEnumerator::assignPredicate(
         // This method should only be called if we can combine bounds.
         const bool canCombineBounds = true;
         dest.tagData =
-            stdx::make_unique<IndexTag>(indexAssignment->index, position, canCombineBounds);
+            std::make_unique<IndexTag>(indexAssignment->index, position, canCombineBounds);
         indexAssignment->orPushdowns.emplace_back(pred, std::move(dest));
     } else {
         indexAssignment->preds.push_back(pred);
@@ -1325,7 +1356,7 @@ void PlanEnumerator::getMultikeyCompoundablePreds(const vector<MatchExpression*>
     // initializing the top-level scope with the prefix of the full path.
     for (size_t i = 0; i < assigned.size(); i++) {
         const MatchExpression* assignedPred = assigned[i];
-        invariant(NULL != assignedPred->getTag());
+        invariant(nullptr != assignedPred->getTag());
         RelevantTag* usedRt = static_cast<RelevantTag*>(assignedPred->getTag());
         set<string> usedPrefixes;
         usedPrefixes.insert(getPathPrefix(usedRt->path));
@@ -1336,7 +1367,7 @@ void PlanEnumerator::getMultikeyCompoundablePreds(const vector<MatchExpression*>
         // the $elemMatch context. For example, if 'assigned' is {a: {$elemMatch: {b: 1}}},
         // then we will have already added "a" to the set for NULL. We now
         // also need to add "b" to the set for the $elemMatch.
-        if (NULL != usedRt->elemMatchExpr) {
+        if (nullptr != usedRt->elemMatchExpr) {
             set<string> elemMatchUsed;
             // Whereas getPathPrefix(usedRt->path) is the prefix of the full path,
             // usedRt->pathPrefix contains the prefix of the portion of the
@@ -1568,19 +1599,24 @@ void PlanEnumerator::compound(const vector<MatchExpression*>& tryCompound,
 //
 
 void PlanEnumerator::tagMemo(size_t id) {
-    LOG(5) << "Tagging memoID " << id;
+    LOGV2_DEBUG(20944, 5, "Tagging memoID {id}", "Tagging memoID", "id"_attr = id);
     NodeAssignment* assign = _memo[id];
-    verify(NULL != assign);
+    verify(nullptr != assign);
 
-    if (NULL != assign->orAssignment) {
+    if (nullptr != assign->orAssignment) {
         OrAssignment* oa = assign->orAssignment.get();
         for (size_t i = 0; i < oa->subnodes.size(); ++i) {
             tagMemo(oa->subnodes[i]);
         }
-    } else if (NULL != assign->arrayAssignment) {
+    } else if (nullptr != assign->lockstepOrAssignment) {
+        LockstepOrAssignment* oa = assign->lockstepOrAssignment.get();
+        for (auto&& node : oa->subnodes) {
+            tagMemo(node.memoId);
+        }
+    } else if (nullptr != assign->arrayAssignment) {
         ArrayAssignment* aa = assign->arrayAssignment.get();
         tagMemo(aa->subnodes[aa->counter]);
-    } else if (NULL != assign->andAssignment) {
+    } else if (nullptr != assign->andAssignment) {
         AndAssignment* aa = assign->andAssignment.get();
         verify(aa->counter < aa->choices.size());
 
@@ -1620,23 +1656,137 @@ void PlanEnumerator::tagMemo(size_t id) {
     }
 }
 
+bool PlanEnumerator::LockstepOrAssignment::allIdentical() const {
+    const auto firstCounter = subnodes[0].iterationCount;
+    for (auto&& subnode : subnodes) {
+        if (subnode.iterationCount != firstCounter) {
+            return false;
+        }
+    }
+    return true;
+}
+
+bool PlanEnumerator::LockstepOrAssignment::shouldResetBeforeProceeding(
+    size_t totalEnumerated) const {
+    if (totalEnumerated == 0 || !exhaustedLockstepIteration) {
+        return false;
+    }
+
+    size_t totalPossibleEnumerations = 1;
+    for (auto&& subnode : subnodes) {
+        if (!subnode.maxIterCount) {
+            return false;  // Haven't yet looped over this child entirely, not ready yet.
+        }
+        totalPossibleEnumerations *= subnode.maxIterCount.get();
+    }
+
+    // If we're able to compute a total number expected enumerations, we must have already cycled
+    // through each of the subnodes at least once. So if we've done that and then iterated all
+    // possible enumerations, we're about to repeat ourselves.
+    return totalEnumerated % totalPossibleEnumerations == 0;
+}
+
+bool PlanEnumerator::_nextMemoForLockstepOrAssignment(
+    PlanEnumerator::LockstepOrAssignment* assignment) {
+
+    if (!assignment->exhaustedLockstepIteration) {
+        // We have not yet finished advancing all children simultaneously, so we'll loop over
+        // each child and advance it.
+
+        // Because we're doing things in a special order, we have to be careful to not duplicate
+        // ourselves. If each child has the same number of alternatives, we will eventually
+        // "carry" or roll over each child back to the beginning. When this happens, we should
+        // not return that plan again.
+        bool everyoneRolledOver = true;
+
+        for (auto&& node : assignment->subnodes) {
+            ++node.iterationCount;
+            const bool wrappedAround = nextMemo(node.memoId);
+            if (wrappedAround) {
+                node.maxIterCount = node.iterationCount;
+                node.iterationCount = 0;
+                // We ran out of "lockstep runway" of sorts. At least one of the subnodes was
+                // exhausted, so this will be our last time advancing all children in lockstep.
+                assignment->exhaustedLockstepIteration = true;
+            } else {
+                everyoneRolledOver = false;
+            }
+        }
+        // Edge case: if every child has only one option available, we are already finished
+        // enumerating.
+        if (assignment->shouldResetBeforeProceeding(assignment->totalEnumerated)) {
+            assignment->exhaustedLockstepIteration = false;
+            return true;  // We're back at the beginning, no need to reset.
+        }
+        if (!everyoneRolledOver) {
+            // Either there's more lockstep iteration to come, or the subnodes have different
+            // amounts of options. In either case, we are now in a new enumeration state so just
+            // return.
+            return false;
+        }
+        // Otherwise we just rolled over and went back to the first enumeration state, so we need to
+        // keep going to avoid duplicating that state. Fall through to below to start "normal", not
+        // lockstep iteration.
+    }
+
+    auto advanceOnce = [this, assignment]() {
+        for (auto&& node : assignment->subnodes) {
+            ++node.iterationCount;
+            const bool wrappedAround = nextMemo(node.memoId);
+            if (!wrappedAround) {
+                return;
+            }
+            node.maxIterCount = node.iterationCount;
+            node.iterationCount = 0;
+        }
+    };
+    advanceOnce();
+    while (assignment->allIdentical()) {
+        // All sub-nodes have the same enumeration state, skip this one since we already did
+        // it above. This is expected to happen pretty often. For example, if we have two subnodes
+        // each enumerating two states, we'd expect the order to be: 00, 11 (these two iterated
+        // above), then 00 (skipped by falling through above after finishing lockstep iteration),
+        // then 10, 11 (skipped here), 00 (skipped here), then finally 01.
+        advanceOnce();
+    }
+
+    // This special ordering is tricky to reset. Because it iterates the sub nodes in such a
+    // unique order, it can be difficult to know when it has actually finished iterating. Our
+    // strategy is just to compute a total and go back to the beginning once we hit that total.
+    if (!assignment->shouldResetBeforeProceeding(assignment->totalEnumerated)) {
+        return false;
+    }
+    // Reset!
+    for (auto&& subnode : assignment->subnodes) {
+        while (!nextMemo(subnode.memoId)) {
+            // Keep advancing till it rolls over.
+        }
+        subnode.iterationCount = 0;
+    }
+    assignment->exhaustedLockstepIteration = false;
+    return true;
+}
+
 bool PlanEnumerator::nextMemo(size_t id) {
     NodeAssignment* assign = _memo[id];
-    verify(NULL != assign);
+    verify(nullptr != assign);
 
-    if (NULL != assign->orAssignment) {
+    if (nullptr != assign->orAssignment) {
         OrAssignment* oa = assign->orAssignment.get();
 
-        // Limit the number of OR enumerations
+        // Limit the number of OR enumerations.
         oa->counter++;
         if (oa->counter >= _orLimit) {
+            LOGV2_DEBUG(3639300,
+                        1,
+                        "plan enumerator exceeded threshold for OR enumerations",
+                        "orEnumerationLimit"_attr = _orLimit);
             return true;
         }
 
-        // OR just walks through telling its children to
-        // move forward.
+        // OR just walks through telling its children to move forward.
         for (size_t i = 0; i < oa->subnodes.size(); ++i) {
-            // If there's no carry, we just stop.  If there's a carry, we move the next child
+            // If there's no carry, we just stop. If there's a carry, we move the next child
             // forward.
             if (!nextMemo(oa->subnodes[i])) {
                 return false;
@@ -1644,7 +1794,20 @@ bool PlanEnumerator::nextMemo(size_t id) {
         }
         // If we're here, the last subnode had a carry, therefore the OR has a carry.
         return true;
-    } else if (NULL != assign->arrayAssignment) {
+    } else if (nullptr != assign->lockstepOrAssignment) {
+        LockstepOrAssignment* assignment = assign->lockstepOrAssignment.get();
+
+        // Limit the number of OR enumerations.
+        ++assignment->totalEnumerated;
+        if (assignment->totalEnumerated >= _orLimit) {
+            LOGV2_DEBUG(3639301,
+                        1,
+                        "plan enumerator exceeded threshold for OR enumerations",
+                        "orEnumerationLimit"_attr = _orLimit);
+            return true;
+        }
+        return _nextMemoForLockstepOrAssignment(assignment);
+    } else if (nullptr != assign->arrayAssignment) {
         ArrayAssignment* aa = assign->arrayAssignment.get();
         // moving to next on current subnode is OK
         if (!nextMemo(aa->subnodes[aa->counter])) {
@@ -1657,7 +1820,7 @@ bool PlanEnumerator::nextMemo(size_t id) {
         }
         aa->counter = 0;
         return true;
-    } else if (NULL != assign->andAssignment) {
+    } else if (nullptr != assign->andAssignment) {
         AndAssignment* aa = assign->andAssignment.get();
 
         // One of our subnodes might have to move on to its next enumeration state.
@@ -1676,11 +1839,9 @@ bool PlanEnumerator::nextMemo(size_t id) {
         }
         aa->counter = 0;
         return true;
+    } else {
+        MONGO_UNREACHABLE;
     }
-
-    // This shouldn't happen.
-    verify(0);
-    return false;
 }
 
 }  // namespace mongo
