@@ -36,6 +36,8 @@
 namespace mongo {
 namespace {
 
+const static NamespaceString kNs("db.dummyNs");
+
 TEST_F(QueryPlannerTest, PartialIndexEq) {
     params.options = QueryPlannerParams::NO_TABLE_SCAN;
     BSONObj filterObj(fromjson("{a: {$gt: 0}}"));
@@ -49,8 +51,8 @@ TEST_F(QueryPlannerTest, PartialIndexEq) {
         "{filter: null, pattern: {a: 1}, "
         "bounds: {a: [[1, 1, true, true]]}}}}}");
 
-    runQuery(fromjson("{a: -1}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: -1}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexNot) {
@@ -59,14 +61,14 @@ TEST_F(QueryPlannerTest, PartialIndexNot) {
     std::unique_ptr<MatchExpression> filterExpr = parseMatchExpression(filterObj);
     addIndex(fromjson("{a: 1}"), filterExpr.get());
 
-    runQuery(fromjson("{a: {$ne: 1}}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: {$ne: 1}}"));
+    assertNoSolutions();
 
-    runQuery(fromjson("{a: {$ne: -1}}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: {$ne: -1}}"));
+    assertNoSolutions();
 
-    runQuery(fromjson("{a: {$not: {$lte: 0}}}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: {$not: {$lte: 0}}}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexElemMatchObj) {
@@ -76,11 +78,11 @@ TEST_F(QueryPlannerTest, PartialIndexElemMatchObj) {
     addIndex(fromjson("{'a.b': 1}"), filterExpr.get());
 
     // The following query could in theory use the partial index, but we don't support it right now.
-    runQuery(fromjson("{a: {$elemMatch: {b: 1}}}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: {$elemMatch: {b: 1}}}"));
+    assertNoSolutions();
 
-    runQuery(fromjson("{a: {$elemMatch: {b: -1}}}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: {$elemMatch: {b: -1}}}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexElemMatchObjContainingOr) {
@@ -90,8 +92,8 @@ TEST_F(QueryPlannerTest, PartialIndexElemMatchObjContainingOr) {
     addIndex(fromjson("{'a.b': 1}"), filterExpr.get());
 
     // The following query could in theory use the partial index, but we don't support it right now.
-    runQuery(fromjson("{a: {$elemMatch: {$or: [{b: 1}, {b: 2}]}}}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: {$elemMatch: {$or: [{b: 1}, {b: 2}]}}}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexElemMatchObjWithBadSubobjectFilter) {
@@ -100,11 +102,11 @@ TEST_F(QueryPlannerTest, PartialIndexElemMatchObjWithBadSubobjectFilter) {
     std::unique_ptr<MatchExpression> filterExpr = parseMatchExpression(filterObj);
     addIndex(fromjson("{'a.b': 1}"), filterExpr.get());
 
-    runQuery(fromjson("{a: {$elemMatch: {$or: [{b: 1}, {b: 2}]}}}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: {$elemMatch: {$or: [{b: 1}, {b: 2}]}}}"));
+    assertNoSolutions();
 
-    runQuery(fromjson("{a: {$elemMatch: {b: {$in: [1, 2]}}}}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: {$elemMatch: {b: {$in: [1, 2]}}}}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexAndSingleAssignment) {
@@ -120,8 +122,8 @@ TEST_F(QueryPlannerTest, PartialIndexAndSingleAssignment) {
         "{filter: null, pattern: {a: 1}, "
         "bounds: {a: [[1, 1, true, true]]}}}}}");
 
-    runQuery(fromjson("{a: 1, f: -1}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: 1, f: -1}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexAndMultipleAssignments) {
@@ -137,8 +139,8 @@ TEST_F(QueryPlannerTest, PartialIndexAndMultipleAssignments) {
         "{filter: null, pattern: {a: 1}, "
         "bounds: {a: [[0, 10, true, true]]}}}}}");
 
-    runQuery(fromjson("{a: {$gte: 0, $lte: 10}, f: -1}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: {$gte: 0, $lte: 10}, f: -1}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexOr) {
@@ -156,8 +158,8 @@ TEST_F(QueryPlannerTest, PartialIndexOr) {
         "{ixscan: {filter: null, pattern: {_id: 1}, "
         "bounds: {_id: [[0, 0, true, true]]}}}]}}}}");
 
-    runQuery(fromjson("{$or: [{a: -1}, {_id: 0}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$or: [{a: -1}, {_id: 0}]}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexOrContainingAnd) {
@@ -176,8 +178,8 @@ TEST_F(QueryPlannerTest, PartialIndexOrContainingAnd) {
         "{ixscan: {filter: null, pattern: {_id: 1}, "
         "bounds: {_id: [[0, 0, true, true]]}}}]}}}}");
 
-    runQuery(fromjson("{$or: [{a: 1, f: -1}, {_id: 0}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$or: [{a: 1, f: -1}, {_id: 0}]}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexOrContainingAndMultipleAssignments) {
@@ -213,8 +215,8 @@ TEST_F(QueryPlannerTest, PartialIndexOrContainingAndMultipleAssignments) {
         "{ixscan: {filter: null, pattern: {a: 1}, "
         "bounds: {a: [[2, 2, true, true]]}}}]}}}}");
 
-    runQuery(fromjson("{$or: [{_id: 0, a: -1}, {a: -2}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$or: [{_id: 0, a: -1}, {a: -2}]}"));
+    assertNoSolutions();
 }
 
 
@@ -234,8 +236,8 @@ TEST_F(QueryPlannerTest, PartialIndexAndContainingOrContainingAnd) {
         "{ixscan: {filter: null, pattern: {_id: 1}, "
         "bounds: {_id: [[0, 0, true, true]]}}}]}}}}");
 
-    runQuery(fromjson("{x: 1, $or: [{a: 1, f: -1}, {_id: 0}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{x: 1, $or: [{a: 1, f: -1}, {_id: 0}]}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexAndContainingOrContainingAndSatisfyingPredOutside) {
@@ -255,8 +257,8 @@ TEST_F(QueryPlannerTest, PartialIndexAndContainingOrContainingAndSatisfyingPredO
         "bounds: {_id: [[0, 0, true, true]]}}}]}}}}");
 
     // The following query could in theory use the partial index, but we don't support it right now.
-    runQuery(fromjson("{f: 1, x: 1, $or: [{a: 1, g: 1}, {_id: 0}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{f: 1, x: 1, $or: [{a: 1, g: 1}, {_id: 0}]}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexOrContainingAndContainingOr) {
@@ -277,8 +279,8 @@ TEST_F(QueryPlannerTest, PartialIndexOrContainingAndContainingOr) {
         "{ixscan: {filter: null, pattern: {_id: 1}, "
         "bounds: {_id: [[1, 1, true, true]]}}}]}}}}");
 
-    runQuery(fromjson("{$or: [{x: 1, $or: [{a: -1}, {_id: 2}]}, {_id: 1}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$or: [{x: 1, $or: [{a: -1}, {_id: 2}]}, {_id: 1}]}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexProvidingSort) {
@@ -294,8 +296,8 @@ TEST_F(QueryPlannerTest, PartialIndexProvidingSort) {
         "{filter: null, pattern: {a: 1}, "
         "bounds: {a: [['MinKey', 'MaxKey', true, true]]}}}}}");
 
-    runQuerySortProj(fromjson("{f: -1}"), fromjson("{a: 1}"), BSONObj());
-    assertNumSolutions(0U);
+    runInvalidQuerySortProj(fromjson("{f: -1}"), fromjson("{a: 1}"), BSONObj());
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexOrContainingNot) {
@@ -304,11 +306,11 @@ TEST_F(QueryPlannerTest, PartialIndexOrContainingNot) {
     std::unique_ptr<MatchExpression> filterExpr = parseMatchExpression(filterObj);
     addIndex(fromjson("{a: 1}"), filterExpr.get());
 
-    runQuery(fromjson("{$or: [{a: {$ne: 1}}, {_id: 1}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$or: [{a: {$ne: 1}}, {_id: 1}]}"));
+    assertNoSolutions();
 
-    runQuery(fromjson("{$or: [{a: {$ne: -1}}, {_id: 1}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$or: [{a: {$ne: -1}}, {_id: 1}]}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexMultipleSameAnd) {
@@ -338,8 +340,8 @@ TEST_F(QueryPlannerTest, PartialIndexMultipleSameAnd) {
         "{filter: null, pattern: {a: 1}, "
         "bounds: {a: [[1, 1, true, true]]}}}}}");
 
-    runQuery(fromjson("{a: 1, b: 1, f: -1, g: -1}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: 1, b: 1, f: -1, g: -1}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexMultipleSameAndCompoundSharedPrefix) {
@@ -362,8 +364,8 @@ TEST_F(QueryPlannerTest, PartialIndexMultipleSameAndCompoundSharedPrefix) {
         "bounds: {a: [[1, 1, true, true]], "
         "c: [['MinKey', 'MaxKey', true, true]]}}}}}");
 
-    runQuery(fromjson("{a: 1, f: -1}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: 1, f: -1}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexMultipleSameOr) {
@@ -386,11 +388,11 @@ TEST_F(QueryPlannerTest, PartialIndexMultipleSameOr) {
         "{filter: null, pattern: {b: 1}, "
         "bounds: {b: [[1, 1, true, true]]}}}}}]}}");
 
-    runQuery(fromjson("{$or: [{a: 1, f: 1}, {b: 1, g: -1}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$or: [{a: 1, f: 1}, {b: 1, g: -1}]}"));
+    assertNoSolutions();
 
-    runQuery(fromjson("{$or: [{a: 1, f: -1}, {b: 1, g: -1}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$or: [{a: 1, f: -1}, {b: 1, g: -1}]}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexAndCompoundIndex) {
@@ -407,8 +409,8 @@ TEST_F(QueryPlannerTest, PartialIndexAndCompoundIndex) {
         "bounds: {a: [[1, 1, true, true]], "
         "b: [[1, 1, true, true]]}}}}}");
 
-    runQuery(fromjson("{a: 1, b: 1, f: -1}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{a: 1, b: 1, f: -1}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexOrCompoundIndex) {
@@ -427,8 +429,8 @@ TEST_F(QueryPlannerTest, PartialIndexOrCompoundIndex) {
         "{ixscan: {filter: null, pattern: {_id: 1}, "
         "bounds: {_id: [[1, 1, true, true]]}}}]}}}}");
 
-    runQuery(fromjson("{$or: [{a: 1, b: 1, f: -1}, {_id: 1}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$or: [{a: 1, b: 1, f: -1}, {_id: 1}]}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexNor) {
@@ -438,19 +440,24 @@ TEST_F(QueryPlannerTest, PartialIndexNor) {
     addIndex(fromjson("{a: 1}"), filterExpr.get());
 
     // The following query could in theory use the partial index, but we don't support it right now.
-    runQuery(fromjson("{$nor: [{a: 1, f: 1}, {_id: 1}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$nor: [{a: 1, f: 1}, {_id: 1}]}"));
+    assertNoSolutions();
 
-    runQuery(fromjson("{$nor: [{a: 1, f: -1}, {_id: 1}]}"));
-    assertNumSolutions(0U);
+    runInvalidQuery(fromjson("{$nor: [{a: 1, f: -1}, {_id: 1}]}"));
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexStringComparisonMatchingCollators) {
     params.options = QueryPlannerParams::NO_TABLE_SCAN;
     BSONObj filterObj(fromjson("{a: {$gt: 'cba'}}"));
-    CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
-    std::unique_ptr<MatchExpression> filterExpr = parseMatchExpression(filterObj, &collator);
-    addIndex(fromjson("{a: 1}"), filterExpr.get(), &collator);
+
+    auto expCtxForPartialFilter = make_intrusive<ExpressionContext>(
+        opCtx.get(),
+        std::make_unique<CollatorInterfaceMock>(CollatorInterfaceMock::MockType::kReverseString),
+        kNs);
+    std::unique_ptr<MatchExpression> filterExpr =
+        parseMatchExpression(filterObj, expCtxForPartialFilter);
+    addIndex(fromjson("{a: 1}"), filterExpr.get(), expCtxForPartialFilter->getCollator());
 
     runQueryAsCommand(
         fromjson("{find: 'testns', filter: {a: 'abc'}, collation: {locale: 'reverse'}}"));
@@ -460,17 +467,22 @@ TEST_F(QueryPlannerTest, PartialIndexStringComparisonMatchingCollators) {
         "{filter: null, pattern: {a: 1}, "
         "bounds: {a: [['cba', 'cba', true, true]]}}}}}");
 
-    runQueryAsCommand(
+    runInvalidQueryAsCommand(
         fromjson("{find: 'testns', filter: {a: 'zaa'}, collation: {locale: 'reverse'}}"));
-    assertNumSolutions(0U);
+    assertNoSolutions();
 }
 
 TEST_F(QueryPlannerTest, PartialIndexNoStringComparisonNonMatchingCollators) {
     params.options = QueryPlannerParams::NO_TABLE_SCAN;
     BSONObj filterObj(fromjson("{a: {$gt: 0}}"));
-    CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kAlwaysEqual);
-    std::unique_ptr<MatchExpression> filterExpr = parseMatchExpression(filterObj, &collator);
-    addIndex(fromjson("{a: 1}"), filterExpr.get(), &collator);
+
+    auto expCtxForPartialFilter = make_intrusive<ExpressionContext>(
+        opCtx.get(),
+        std::make_unique<CollatorInterfaceMock>(CollatorInterfaceMock::MockType::kReverseString),
+        kNs);
+    std::unique_ptr<MatchExpression> filterExpr =
+        parseMatchExpression(filterObj, expCtxForPartialFilter);
+    addIndex(fromjson("{a: 1}"), filterExpr.get(), expCtxForPartialFilter->getCollator());
 
     runQueryAsCommand(fromjson("{find: 'testns', filter: {a: 1}, collation: {locale: 'reverse'}}"));
     assertNumSolutions(1U);
@@ -486,8 +498,8 @@ TEST_F(QueryPlannerTest, InternalExprEqCannotUsePartialIndex) {
     auto filterExpr = parseMatchExpression(filterObj);
     addIndex(fromjson("{a: 1}"), filterExpr.get());
 
-    runQueryAsCommand(fromjson("{find: 'testns', filter: {a: {$_internalExprEq: 1}}}"));
-    assertNumSolutions(0U);
+    runInvalidQueryAsCommand(fromjson("{find: 'testns', filter: {a: {$_internalExprEq: 1}}}"));
+    assertNoSolutions();
 }
 
 }  // namespace
