@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -34,8 +33,8 @@
 #include <boost/optional.hpp>
 #include <boost/optional/optional_io.hpp>
 
+#include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/collection_mock.h"
-#include "mongo/db/catalog/uuid_catalog.h"
 #include "mongo/db/dbmessage.h"
 #include "mongo/db/json.h"
 #include "mongo/db/namespace_string.h"
@@ -138,24 +137,6 @@ TEST(QueryRequestTest, PositiveNToReturn) {
     ASSERT_OK(qr.validate());
 }
 
-TEST(QueryRequestTest, NegativeMaxScan) {
-    QueryRequest qr(testns);
-    qr.setMaxScan(-1);
-    ASSERT_NOT_OK(qr.validate());
-}
-
-TEST(QueryRequestTest, ZeroMaxScan) {
-    QueryRequest qr(testns);
-    qr.setMaxScan(0);
-    ASSERT_OK(qr.validate());
-}
-
-TEST(QueryRequestTest, PositiveMaxScan) {
-    QueryRequest qr(testns);
-    qr.setMaxScan(1);
-    ASSERT_OK(qr.validate());
-}
-
 TEST(QueryRequestTest, NegativeMaxTimeMS) {
     QueryRequest qr(testns);
     qr.setMaxTimeMS(-1);
@@ -180,10 +161,12 @@ TEST(QueryRequestTest, ValidSortOrder) {
     ASSERT_OK(qr.validate());
 }
 
-TEST(QueryRequestTest, InvalidSortOrderString) {
+TEST(QueryRequestTest, DoesNotErrorOnInvalidSortPattern) {
     QueryRequest qr(testns);
     qr.setSort(fromjson("{a: \"\"}"));
-    ASSERT_NOT_OK(qr.validate());
+    // QueryRequest isn't responsible for validating the sort pattern, so it is considered valid
+    // even though the sort pattern {a: ""} is not well-formed.
+    ASSERT_OK(qr.validate());
 }
 
 TEST(QueryRequestTest, MinFieldsNotPrefixOfMax) {
@@ -258,28 +241,111 @@ TEST(QueryRequestTest, ValidSortProj) {
     ASSERT_OK(metaQR.validate());
 }
 
-TEST(QueryRequestTest, ForbidNonMetaSortOnFieldWithMetaProject) {
-    QueryRequest badQR(testns);
-    badQR.setProj(fromjson("{a: {$meta: \"textScore\"}}"));
-    badQR.setSort(fromjson("{a: 1}"));
-    ASSERT_NOT_OK(badQR.validate());
-
-    QueryRequest goodQR(testns);
-    goodQR.setProj(fromjson("{a: {$meta: \"textScore\"}}"));
-    goodQR.setSort(fromjson("{b: 1}"));
-    ASSERT_OK(goodQR.validate());
+TEST(QueryRequestTest, TextScoreMetaSortOnFieldDoesNotRequireMetaProjection) {
+    QueryRequest qr(testns);
+    qr.setProj(fromjson("{b: 1}"));
+    qr.setSort(fromjson("{a: {$meta: 'textScore'}}"));
+    ASSERT_OK(qr.validate());
 }
 
-TEST(QueryRequestTest, ForbidMetaSortOnFieldWithoutMetaProject) {
-    QueryRequest qrMatching(testns);
-    qrMatching.setProj(fromjson("{a: 1}"));
-    qrMatching.setSort(fromjson("{a: {$meta: \"textScore\"}}"));
-    ASSERT_NOT_OK(qrMatching.validate());
+TEST(QueryRequestTest, TextScoreMetaProjectionDoesNotRequireTextScoreMetaSort) {
+    QueryRequest qr(testns);
+    qr.setProj(fromjson("{a: {$meta: \"textScore\"}}"));
+    qr.setSort(fromjson("{b: 1}"));
+    ASSERT_OK(qr.validate());
+}
 
-    QueryRequest qrNonMatching(testns);
-    qrNonMatching.setProj(fromjson("{b: 1}"));
-    qrNonMatching.setSort(fromjson("{a: {$meta: \"textScore\"}}"));
-    ASSERT_NOT_OK(qrNonMatching.validate());
+TEST(QueryRequestTest, RequestResumeTokenWithHint) {
+    QueryRequest qr(testns);
+    qr.setRequestResumeToken(true);
+    ASSERT_NOT_OK(qr.validate());
+    qr.setHint(fromjson("{a: 1}"));
+    ASSERT_NOT_OK(qr.validate());
+    qr.setHint(fromjson("{$natural: 1}"));
+    ASSERT_OK(qr.validate());
+}
+
+TEST(QueryRequestTest, RequestResumeTokenWithSort) {
+    QueryRequest qr(testns);
+    qr.setRequestResumeToken(true);
+    // Hint must be explicitly set for the query request to validate.
+    qr.setHint(fromjson("{$natural: 1}"));
+    ASSERT_OK(qr.validate());
+    qr.setSort(fromjson("{a: 1}"));
+    ASSERT_NOT_OK(qr.validate());
+    qr.setSort(fromjson("{$natural: 1}"));
+    ASSERT_OK(qr.validate());
+}
+
+TEST(QueryRequestTest, InvalidResumeAfterWrongRecordIdType) {
+    QueryRequest qr(testns);
+    BSONObj resumeAfter = BSON("$recordId" << 1);
+    qr.setResumeAfter(resumeAfter);
+    qr.setRequestResumeToken(true);
+    // Hint must be explicitly set for the query request to validate.
+    qr.setHint(fromjson("{$natural: 1}"));
+    ASSERT_NOT_OK(qr.validate());
+    resumeAfter = BSON("$recordId" << 1LL);
+    qr.setResumeAfter(resumeAfter);
+    ASSERT_OK(qr.validate());
+}
+
+TEST(QueryRequestTest, InvalidResumeAfterExtraField) {
+    QueryRequest qr(testns);
+    BSONObj resumeAfter = BSON("$recordId" << 1LL << "$extra" << 1);
+    qr.setResumeAfter(resumeAfter);
+    qr.setRequestResumeToken(true);
+    // Hint must be explicitly set for the query request to validate.
+    qr.setHint(fromjson("{$natural: 1}"));
+    ASSERT_NOT_OK(qr.validate());
+}
+
+TEST(QueryRequestTest, ResumeAfterWithHint) {
+    QueryRequest qr(testns);
+    BSONObj resumeAfter = BSON("$recordId" << 1LL);
+    qr.setResumeAfter(resumeAfter);
+    qr.setRequestResumeToken(true);
+    ASSERT_NOT_OK(qr.validate());
+    qr.setHint(fromjson("{a: 1}"));
+    ASSERT_NOT_OK(qr.validate());
+    qr.setHint(fromjson("{$natural: 1}"));
+    ASSERT_OK(qr.validate());
+}
+
+TEST(QueryRequestTest, ResumeAfterWithSort) {
+    QueryRequest qr(testns);
+    BSONObj resumeAfter = BSON("$recordId" << 1LL);
+    qr.setResumeAfter(resumeAfter);
+    qr.setRequestResumeToken(true);
+    // Hint must be explicitly set for the query request to validate.
+    qr.setHint(fromjson("{$natural: 1}"));
+    ASSERT_OK(qr.validate());
+    qr.setSort(fromjson("{a: 1}"));
+    ASSERT_NOT_OK(qr.validate());
+    qr.setSort(fromjson("{$natural: 1}"));
+    ASSERT_OK(qr.validate());
+}
+
+TEST(QueryRequestTest, ResumeNoSpecifiedRequestResumeToken) {
+    QueryRequest qr(testns);
+    BSONObj resumeAfter = BSON("$recordId" << 1LL);
+    qr.setResumeAfter(resumeAfter);
+    // Hint must be explicitly set for the query request to validate.
+    qr.setHint(fromjson("{$natural: 1}"));
+    ASSERT_NOT_OK(qr.validate());
+    qr.setRequestResumeToken(true);
+    ASSERT_OK(qr.validate());
+}
+
+TEST(QueryRequestTest, ExplicitEmptyResumeAfter) {
+    QueryRequest qr(NamespaceString::kRsOplogNamespace);
+    BSONObj resumeAfter = fromjson("{}");
+    // Hint must be explicitly set for the query request to validate.
+    qr.setHint(fromjson("{$natural: 1}"));
+    qr.setResumeAfter(resumeAfter);
+    ASSERT_OK(qr.validate());
+    qr.setRequestResumeToken(true);
+    ASSERT_OK(qr.validate());
 }
 
 //
@@ -305,46 +371,6 @@ TEST(QueryRequestTest, IsTextScoreMeta) {
     ASSERT_FALSE(isFirstElementTextScoreMeta("{a: {$meta: \"textScore\", b: 1}}"));
 }
 
-//
-// Sort order validation
-// In a valid sort order, each element satisfies one of:
-// 1. a number with value 1
-// 2. a number with value -1
-// 3. isTextScoreMeta
-//
-
-TEST(QueryRequestTest, ValidateSortOrder) {
-    // Valid sorts
-    ASSERT(QueryRequest::isValidSortOrder(fromjson("{}")));
-    ASSERT(QueryRequest::isValidSortOrder(fromjson("{a: 1}")));
-    ASSERT(QueryRequest::isValidSortOrder(fromjson("{a: -1}")));
-    ASSERT(QueryRequest::isValidSortOrder(fromjson("{a: {$meta: \"textScore\"}}")));
-
-    // Invalid sorts
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: 100}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: 0}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: -100}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: Infinity}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: -Infinity}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: true}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: false}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: null}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: {}}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: {b: 1}}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: []}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: [1, 2, 3]}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: \"\"}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: \"bb\"}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: {$meta: 1}}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: {$meta: \"image\"}}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{a: {$world: \"textScore\"}}")));
-    ASSERT_FALSE(
-        QueryRequest::isValidSortOrder(fromjson("{a: {$meta: \"textScore\","
-                                                " b: 1}}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{'': 1}")));
-    ASSERT_FALSE(QueryRequest::isValidSortOrder(fromjson("{'': -1}")));
-}
-
 //
 // Tests for parsing a query request from a command BSON object.
 //
@@ -367,8 +393,7 @@ TEST(QueryRequestTest, ParseFromCommandWithOptions) {
         "filter: {a: 3},"
         "sort: {a: 1},"
         "projection: {_id: 0, a: 1},"
-        "showRecordId: true,"
-        "maxScan: 1000}}");
+        "showRecordId: true}}");
     const NamespaceString nss("test.testns");
     bool isExplain = false;
     unique_ptr<QueryRequest> qr(
@@ -376,7 +401,6 @@ TEST(QueryRequestTest, ParseFromCommandWithOptions) {
 
     // Make sure the values from the command BSON are reflected in the QR.
     ASSERT(qr->showRecordId());
-    ASSERT_EQUALS(1000, qr->getMaxScan());
 }
 
 TEST(QueryRequestTest, ParseFromCommandHintAsString) {
@@ -422,7 +446,10 @@ TEST(QueryRequestTest, ParseFromCommandAllFlagsTrue) {
         "oplogReplay: true,"
         "noCursorTimeout: true,"
         "awaitData: true,"
-        "allowPartialResults: true}");
+        "allowPartialResults: true,"
+        "readOnce: true,"
+        "_use44SortKeys: true,"
+        "allowSpeculativeMajorityRead: true}");
     const NamespaceString nss("test.testns");
     bool isExplain = false;
     unique_ptr<QueryRequest> qr(
@@ -435,9 +462,30 @@ TEST(QueryRequestTest, ParseFromCommandAllFlagsTrue) {
     ASSERT(qr->isNoCursorTimeout());
     ASSERT(qr->isTailableAndAwaitData());
     ASSERT(qr->isAllowPartialResults());
+    ASSERT(qr->isReadOnce());
+    ASSERT(qr->use44SortKeys());
+    ASSERT(qr->allowSpeculativeMajorityRead());
 }
 
-TEST(QueryRequestTest, ParseFromCommandCommentWithValidMinMax) {
+TEST(QueryRequestTest, ParseFromCommandReadOnceDefaultsToFalse) {
+    BSONObj cmdObj = fromjson("{find: 'testns'}");
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
+    ASSERT(!qr->isReadOnce());
+}
+
+TEST(QueryRequestTest, ParseFromCommandUse44SortKeysDefaultsToFalse) {
+    BSONObj cmdObj = fromjson("{find: 'testns'}");
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
+    ASSERT(!qr->use44SortKeys());
+}
+
+TEST(QueryRequestTest, ParseFromCommandValidMinMax) {
     BSONObj cmdObj = fromjson(
         "{find: 'testns',"
         "comment: 'the comment',"
@@ -448,7 +496,6 @@ TEST(QueryRequestTest, ParseFromCommandCommentWithValidMinMax) {
     unique_ptr<QueryRequest> qr(
         assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
 
-    ASSERT_EQUALS("the comment", qr->getComment());
     BSONObj expectedMin = BSON("a" << 1);
     ASSERT_EQUALS(0, expectedMin.woCompare(qr->getMin()));
     BSONObj expectedMax = BSON("a" << 2);
@@ -456,19 +503,22 @@ TEST(QueryRequestTest, ParseFromCommandCommentWithValidMinMax) {
 }
 
 TEST(QueryRequestTest, ParseFromCommandAllNonOptionFields) {
+    RuntimeConstants rtc{Date_t::now(), Timestamp(1, 1)};
+    BSONObj rtcObj = BSON("runtimeConstants" << rtc.toBSON());
     BSONObj cmdObj = fromjson(
-        "{find: 'testns',"
-        "filter: {a: 1},"
-        "sort: {b: 1},"
-        "projection: {c: 1},"
-        "hint: {d: 1},"
-        "readConcern: {e: 1},"
-        "$queryOptions: {$readPreference: 'secondary'},"
-        "collation: {f: 1},"
-        "limit: 3,"
-        "skip: 5,"
-        "batchSize: 90,"
-        "singleBatch: false}");
+                         "{find: 'testns',"
+                         "filter: {a: 1},"
+                         "sort: {b: 1},"
+                         "projection: {c: 1},"
+                         "hint: {d: 1},"
+                         "readConcern: {e: 1},"
+                         "$queryOptions: {$readPreference: 'secondary'},"
+                         "collation: {f: 1},"
+                         "limit: 3,"
+                         "skip: 5,"
+                         "batchSize: 90,"
+                         "singleBatch: false}")
+                         .addField(rtcObj["runtimeConstants"]);
     const NamespaceString nss("test.testns");
     bool isExplain = false;
     unique_ptr<QueryRequest> qr(
@@ -493,6 +543,9 @@ TEST(QueryRequestTest, ParseFromCommandAllNonOptionFields) {
     ASSERT_EQUALS(3, *qr->getLimit());
     ASSERT_EQUALS(5, *qr->getSkip());
     ASSERT_EQUALS(90, *qr->getBatchSize());
+    ASSERT(qr->getRuntimeConstants().has_value());
+    ASSERT_EQUALS(qr->getRuntimeConstants()->getLocalNow(), rtc.getLocalNow());
+    ASSERT_EQUALS(qr->getRuntimeConstants()->getClusterTime(), rtc.getClusterTime());
     ASSERT(qr->wantMore());
 }
 
@@ -608,17 +661,6 @@ TEST(QueryRequestTest, ParseFromCommandSingleBatchWrongType) {
     ASSERT_NOT_OK(result.getStatus());
 }
 
-TEST(QueryRequestTest, ParseFromCommandCommentWrongType) {
-    BSONObj cmdObj = fromjson(
-        "{find: 'testns',"
-        "filter:  {a: 1},"
-        "comment: 1}");
-    const NamespaceString nss("test.testns");
-    bool isExplain = false;
-    auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
-    ASSERT_NOT_OK(result.getStatus());
-}
-
 TEST(QueryRequestTest, ParseFromCommandUnwrappedReadPrefWrongType) {
     BSONObj cmdObj = fromjson(
         "{find: 'testns',"
@@ -630,18 +672,6 @@ TEST(QueryRequestTest, ParseFromCommandUnwrappedReadPrefWrongType) {
     ASSERT_NOT_OK(result.getStatus());
 }
 
-TEST(QueryRequestTest, ParseFromCommandMaxScanWrongType) {
-    BSONObj cmdObj = fromjson(
-        "{find: 'testns',"
-        "filter:  {a: 1},"
-        "maxScan: true,"
-        "comment: 'foo'}");
-    const NamespaceString nss("test.testns");
-    bool isExplain = false;
-    auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
-    ASSERT_NOT_OK(result.getStatus());
-}
-
 TEST(QueryRequestTest, ParseFromCommandMaxTimeMSWrongType) {
     BSONObj cmdObj = fromjson(
         "{find: 'testns',"
@@ -797,6 +827,54 @@ TEST(QueryRequestTest, ParseFromCommandCollationWrongType) {
     auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
     ASSERT_NOT_OK(result.getStatus());
 }
+
+TEST(QueryRequestTest, ParseFromCommandReadOnceWrongType) {
+    BSONObj cmdObj = fromjson(
+        "{find: 'testns',"
+        "readOnce: 1}");
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
+    ASSERT_EQ(ErrorCodes::FailedToParse, result.getStatus());
+}
+
+TEST(QueryRequestTest, ParseFromCommandRuntimeConstantsWrongType) {
+    BSONObj cmdObj = BSON("find"
+                          << "testns"
+                          << "runtimeConstants"
+                          << "shouldNotBeString");
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
+    ASSERT_EQ(ErrorCodes::FailedToParse, result.getStatus());
+}
+
+TEST(QueryRequestTest, ParseFromCommandRuntimeConstantsSubfieldsWrongType) {
+    BSONObj cmdObj = BSON("find"
+                          << "testns"
+                          << "runtimeConstants"
+                          << BSON("localNow"
+                                  << "shouldBeDate"
+                                  << "clusterTime"
+                                  << "shouldBeTimestamp"));
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    ASSERT_THROWS_CODE(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain),
+                       AssertionException,
+                       ErrorCodes::TypeMismatch);
+}
+
+TEST(QueryRequestTest, ParseFromCommandUse44SortKeysWrongType) {
+    BSONObj cmdObj = BSON("find"
+                          << "testns"
+                          << "_use44SortKeys"
+                          << "shouldBeBool");
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
+    ASSERT_NOT_OK(result.getStatus());
+}
+
 //
 // Parsing errors where a field has the right type but a bad value.
 //
@@ -884,6 +962,133 @@ TEST(QueryRequestTest, ParseFromCommandDefaultBatchSize) {
     ASSERT(!qr->getLimit());
 }
 
+TEST(QueryRequestTest, ParseFromCommandRequestResumeToken) {
+    BSONObj cmdObj = BSON("find"
+                          << "testns"
+                          << "hint" << BSON("$natural" << 1) << "sort" << BSON("$natural" << 1)
+                          << "$_requestResumeToken" << true);
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
+    ASSERT(qr->getRequestResumeToken());
+}
+
+TEST(QueryRequestTest, ParseFromCommandResumeToken) {
+    BSONObj cmdObj =
+        BSON("find"
+             << "testns"
+             << "hint" << BSON("$natural" << 1) << "sort" << BSON("$natural" << 1)
+             << "$_requestResumeToken" << true << "$_resumeAfter" << BSON("$recordId" << 1LL));
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
+    ASSERT(!qr->getResumeAfter().isEmpty());
+    ASSERT(qr->getRequestResumeToken());
+}
+
+TEST(QueryRequestTest, ParseFromCommandEmptyResumeToken) {
+    BSONObj resumeAfter = fromjson("{}");
+    BSONObj cmdObj = BSON("find"
+                          << "testns"
+                          << "hint" << BSON("$natural" << 1) << "sort" << BSON("$natural" << 1)
+                          << "$_requestResumeToken" << true << "$_resumeAfter" << resumeAfter);
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
+    ASSERT(qr->getRequestResumeToken());
+    ASSERT(qr->getResumeAfter().isEmpty());
+}
+
+//
+// Test asFindCommand ns and uuid variants.
+//
+
+TEST(QueryRequestTest, AsFindCommandAllNonOptionFields) {
+    BSONObj rtcObj =
+        BSON("runtimeConstants" << (RuntimeConstants{Date_t::now(), Timestamp(1, 1)}.toBSON()));
+    BSONObj cmdObj = fromjson(
+                         "{find: 'testns',"
+                         "filter: {a: 1},"
+                         "projection: {c: 1},"
+                         "sort: {b: 1},"
+                         "hint: {d: 1},"
+                         "readConcern: {e: 1},"
+                         "collation: {f: 1},"
+                         "skip: 5,"
+                         "limit: 3,"
+                         "batchSize: 90,"
+                         "singleBatch: true}")
+                         .addField(rtcObj["runtimeConstants"]);
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
+    ASSERT_BSONOBJ_EQ(cmdObj, qr->asFindCommand());
+}
+
+TEST(QueryRequestTest, AsFindCommandWithUuidAllNonOptionFields) {
+    BSONObj rtcObj =
+        BSON("runtimeConstants" << (RuntimeConstants{Date_t::now(), Timestamp(1, 1)}.toBSON()));
+    BSONObj cmdObj =
+        fromjson(
+            // This binary value is UUID("01234567-89ab-cdef-edcb-a98765432101")
+            "{find: { \"$binary\" : \"ASNFZ4mrze/ty6mHZUMhAQ==\", \"$type\" : \"04\" },"
+            "filter: {a: 1},"
+            "projection: {c: 1},"
+            "sort: {b: 1},"
+            "hint: {d: 1},"
+            "readConcern: {e: 1},"
+            "collation: {f: 1},"
+            "skip: 5,"
+            "limit: 3,"
+            "batchSize: 90,"
+            "singleBatch: true}")
+            .addField(rtcObj["runtimeConstants"]);
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
+    ASSERT_BSONOBJ_EQ(cmdObj, qr->asFindCommandWithUuid());
+}
+
+TEST(QueryRequestTest, AsFindCommandWithUuidNoAvailableNamespace) {
+    BSONObj cmdObj =
+        fromjson("{find: { \"$binary\" : \"ASNFZ4mrze/ty6mHZUMhAQ==\", \"$type\" : \"04\" }}");
+    QueryRequest qr(NamespaceStringOrUUID(
+        "test", UUID::parse("01234567-89ab-cdef-edcb-a98765432101").getValue()));
+    ASSERT_BSONOBJ_EQ(cmdObj, qr.asFindCommandWithUuid());
+}
+
+TEST(QueryRequestTest, AsFindCommandWithResumeToken) {
+    BSONObj cmdObj =
+        BSON("find"
+             << "testns"
+             << "sort" << BSON("$natural" << 1) << "hint" << BSON("$natural" << 1)
+             << "$_requestResumeToken" << true << "$_resumeAfter" << BSON("$recordId" << 1LL));
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
+    ASSERT_BSONOBJ_EQ(cmdObj, qr->asFindCommand());
+}
+
+TEST(QueryRequestTest, AsFindCommandWithEmptyResumeToken) {
+    BSONObj resumeAfter = fromjson("{}");
+    BSONObj cmdObj = BSON("find"
+                          << "testns"
+                          << "hint" << BSON("$natural" << 1) << "sort" << BSON("$natural" << 1)
+                          << "$_requestResumeToken" << true << "$_resumeAfter" << resumeAfter);
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
+    ASSERT(qr->asFindCommand().getField("$_resumeAfter").eoo());
+}
+
+//
 //
 // Errors checked in QueryRequest::validate().
 //
@@ -899,7 +1104,7 @@ TEST(QueryRequestTest, ParseFromCommandMinMaxDifferentFieldsError) {
     ASSERT_NOT_OK(result.getStatus());
 }
 
-TEST(QueryRequestTest, ParseCommandForbidNonMetaSortOnFieldWithMetaProject) {
+TEST(QueryRequestTest, ParseCommandAllowNonMetaSortOnFieldWithMetaProject) {
     BSONObj cmdObj;
 
     cmdObj = fromjson(
@@ -909,7 +1114,7 @@ TEST(QueryRequestTest, ParseCommandForbidNonMetaSortOnFieldWithMetaProject) {
     const NamespaceString nss("test.testns");
     bool isExplain = false;
     auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
-    ASSERT_NOT_OK(result.getStatus());
+    ASSERT_OK(result.getStatus());
 
     cmdObj = fromjson(
         "{find: 'testns',"
@@ -918,7 +1123,7 @@ TEST(QueryRequestTest, ParseCommandForbidNonMetaSortOnFieldWithMetaProject) {
     ASSERT_OK(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain).getStatus());
 }
 
-TEST(QueryRequestTest, ParseCommandForbidMetaSortOnFieldWithoutMetaProject) {
+TEST(QueryRequestTest, ParseCommandAllowMetaSortOnFieldWithoutMetaProject) {
     BSONObj cmdObj;
 
     cmdObj = fromjson(
@@ -928,14 +1133,14 @@ TEST(QueryRequestTest, ParseCommandForbidMetaSortOnFieldWithoutMetaProject) {
     const NamespaceString nss("test.testns");
     bool isExplain = false;
     auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
-    ASSERT_NOT_OK(result.getStatus());
+    ASSERT_OK(result.getStatus());
 
     cmdObj = fromjson(
         "{find: 'testns',"
         "projection: {b: 1},"
         "sort: {a: {$meta: 'textScore'}}}");
     result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
-    ASSERT_NOT_OK(result.getStatus());
+    ASSERT_OK(result.getStatus());
 }
 
 TEST(QueryRequestTest, ParseCommandForbidExhaust) {
@@ -993,7 +1198,6 @@ TEST(QueryRequestTest, DefaultQueryParametersCorrect) {
     ASSERT_EQUALS(true, qr->wantMore());
     ASSERT_FALSE(qr->getNToReturn());
     ASSERT_EQUALS(false, qr->isExplain());
-    ASSERT_EQUALS(0, qr->getMaxScan());
     ASSERT_EQUALS(0, qr->getMaxTimeMS());
     ASSERT_EQUALS(false, qr->returnKey());
     ASSERT_EQUALS(false, qr->showRecordId());
@@ -1005,6 +1209,28 @@ TEST(QueryRequestTest, DefaultQueryParametersCorrect) {
     ASSERT_EQUALS(false, qr->isTailableAndAwaitData());
     ASSERT_EQUALS(false, qr->isExhaust());
     ASSERT_EQUALS(false, qr->isAllowPartialResults());
+    ASSERT_EQUALS(false, qr->getRuntimeConstants().has_value());
+    ASSERT_EQUALS(false, qr->allowDiskUse());
+}
+
+TEST(QueryRequestTest, ParseCommandAllowDiskUseTrue) {
+    BSONObj cmdObj = fromjson("{find: 'testns', allowDiskUse: true}");
+    const NamespaceString nss("test.testns");
+    const bool isExplain = false;
+    auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
+
+    ASSERT_OK(result.getStatus());
+    ASSERT_EQ(true, result.getValue()->allowDiskUse());
+}
+
+TEST(QueryRequestTest, ParseCommandAllowDiskUseFalse) {
+    BSONObj cmdObj = fromjson("{find: 'testns', allowDiskUse: false}");
+    const NamespaceString nss("test.testns");
+    const bool isExplain = false;
+    auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
+
+    ASSERT_OK(result.getStatus());
+    ASSERT_EQ(false, result.getValue()->allowDiskUse());
 }
 
 //
@@ -1136,29 +1362,12 @@ TEST(QueryRequestTest, ConvertToAggregationWithNoWantMoreLimitOneSucceeds) {
     ASSERT_OK(qr.asAggregationCommand());
 }
 
-TEST(QueryRequestTest, ConvertToAggregationWithMaxScanFails) {
-    QueryRequest qr(testns);
-    qr.setMaxScan(7);
-    ASSERT_NOT_OK(qr.asAggregationCommand());
-}
-
 TEST(QueryRequestTest, ConvertToAggregationWithReturnKeyFails) {
     QueryRequest qr(testns);
     qr.setReturnKey(true);
     ASSERT_NOT_OK(qr.asAggregationCommand());
 }
 
-TEST(QueryRequestTest, ConvertToAggregationWithCommentSucceeds) {
-    QueryRequest qr(testns);
-    qr.setComment("test");
-    const auto aggCmd = qr.asAggregationCommand();
-    ASSERT_OK(aggCmd);
-
-    auto ar = AggregationRequest::parseFromBSON(testns, aggCmd.getValue());
-    ASSERT_OK(ar.getStatus());
-    ASSERT_EQ(qr.getComment(), ar.getValue().getComment());
-}
-
 TEST(QueryRequestTest, ConvertToAggregationWithShowRecordIdFails) {
     QueryRequest qr(testns);
     qr.setShowRecordId(true);
@@ -1201,6 +1410,19 @@ TEST(QueryRequestTest, ConvertToAggregationWithNToReturnFails) {
     ASSERT_NOT_OK(qr.asAggregationCommand());
 }
 
+TEST(QueryRequestTest, ConvertToAggregationWithRequestResumeTokenFails) {
+    QueryRequest qr(testns);
+    qr.setRequestResumeToken(true);
+    ASSERT_NOT_OK(qr.asAggregationCommand());
+}
+
+TEST(QueryRequestTest, ConvertToAggregationWithResumeAfterFails) {
+    QueryRequest qr(testns);
+    BSONObj resumeAfter = BSON("$recordId" << 1LL);
+    qr.setResumeAfter(resumeAfter);
+    ASSERT_NOT_OK(qr.asAggregationCommand());
+}
+
 TEST(QueryRequestTest, ConvertToAggregationWithPipeline) {
     QueryRequest qr(testns);
     qr.setFilter(BSON("x" << 1));
@@ -1278,29 +1500,72 @@ TEST(QueryRequestTest, ConvertToAggregationWithCollationSucceeds) {
     ASSERT_BSONOBJ_EQ(ar.getValue().getCollation(), BSON("f" << 1));
 }
 
-TEST(QueryRequestTest, ParseFromLegacyObjMetaOpComment) {
-    BSONObj queryObj = fromjson(
-        "{$query: {a: 1},"
-        "$comment: {b: 2, c: {d: 'ParseFromLegacyObjMetaOpComment'}}}");
-    const NamespaceString nss("test.testns");
-    unique_ptr<QueryRequest> qr(
-        assertGet(QueryRequest::fromLegacyQuery(nss, queryObj, BSONObj(), 0, 0, 0)));
-
-    // Ensure that legacy comment meta-operator is parsed to a string comment
-    ASSERT_EQ(qr->getComment(), "{ b: 2, c: { d: \"ParseFromLegacyObjMetaOpComment\" } }");
-    ASSERT_BSONOBJ_EQ(qr->getFilter(), fromjson("{a: 1}"));
+TEST(QueryRequestTest, ConvertToAggregationWithReadOnceFails) {
+    QueryRequest qr(testns);
+    qr.setReadOnce(true);
+    const auto aggCmd = qr.asAggregationCommand();
+    ASSERT_EQ(ErrorCodes::InvalidPipelineOperator, aggCmd.getStatus().code());
 }
 
-TEST(QueryRequestTest, ParseFromLegacyStringMetaOpComment) {
-    BSONObj queryObj = fromjson(
-        "{$query: {a: 1},"
-        "$comment: 'ParseFromLegacyStringMetaOpComment'}");
-    const NamespaceString nss("test.testns");
-    unique_ptr<QueryRequest> qr(
-        assertGet(QueryRequest::fromLegacyQuery(nss, queryObj, BSONObj(), 0, 0, 0)));
+TEST(QueryRequestTest, ConvertToAggregationWithAllowSpeculativeMajorityReadFails) {
+    QueryRequest qr(testns);
+    qr.setAllowSpeculativeMajorityRead(true);
+    const auto aggCmd = qr.asAggregationCommand();
+    ASSERT_EQ(ErrorCodes::InvalidPipelineOperator, aggCmd.getStatus().code());
+}
 
-    ASSERT_EQ(qr->getComment(), "ParseFromLegacyStringMetaOpComment");
-    ASSERT_BSONOBJ_EQ(qr->getFilter(), fromjson("{a: 1}"));
+TEST(QueryRequestTest, ConvertToAggregationWithRuntimeConstantsSucceeds) {
+    RuntimeConstants rtc{Date_t::now(), Timestamp(1, 1)};
+    QueryRequest qr(testns);
+    qr.setRuntimeConstants(rtc);
+    auto agg = qr.asAggregationCommand();
+    ASSERT_OK(agg);
+
+    auto ar = AggregationRequest::parseFromBSON(testns, agg.getValue());
+    ASSERT_OK(ar.getStatus());
+    ASSERT(ar.getValue().getRuntimeConstants().has_value());
+    ASSERT_EQ(ar.getValue().getRuntimeConstants()->getLocalNow(), rtc.getLocalNow());
+    ASSERT_EQ(ar.getValue().getRuntimeConstants()->getClusterTime(), rtc.getClusterTime());
+}
+
+TEST(QueryRequestTest, ConvertToAggregationWithAllowDiskUseTrueSucceeds) {
+    QueryRequest qr(testns);
+    qr.setAllowDiskUse(true);
+    const auto aggCmd = qr.asAggregationCommand();
+    ASSERT_OK(aggCmd.getStatus());
+
+    auto ar = AggregationRequest::parseFromBSON(testns, aggCmd.getValue());
+    ASSERT_OK(ar.getStatus());
+    ASSERT_EQ(true, ar.getValue().shouldAllowDiskUse());
+}
+
+TEST(QueryRequestTest, ConvertToAggregationWithAllowDiskUseFalseSucceeds) {
+    QueryRequest qr(testns);
+    qr.setAllowDiskUse(false);
+    const auto aggCmd = qr.asAggregationCommand();
+    ASSERT_OK(aggCmd.getStatus());
+
+    auto ar = AggregationRequest::parseFromBSON(testns, aggCmd.getValue());
+    ASSERT_OK(ar.getStatus());
+    ASSERT_EQ(false, ar.getValue().shouldAllowDiskUse());
+}
+
+TEST(QueryRequestTest, ConvertToFindWithAllowDiskUseTrueSucceeds) {
+    QueryRequest qr(testns);
+    qr.setAllowDiskUse(true);
+    const auto findCmd = qr.asFindCommand();
+
+    BSONElement elem = findCmd[QueryRequest::kAllowDiskUseField];
+    ASSERT_EQ(true, elem.isBoolean());
+    ASSERT_EQ(true, elem.Bool());
+}
+
+TEST(QueryRequestTest, ConvertToFindWithAllowDiskUseFalseSucceeds) {
+    QueryRequest qr(testns);
+    qr.setAllowDiskUse(false);
+    const auto findCmd = qr.asFindCommand();
+
+    ASSERT_FALSE(findCmd[QueryRequest::kAllowDiskUseField]);
 }
 
 TEST(QueryRequestTest, ParseFromLegacyQuery) {
@@ -1313,8 +1578,7 @@ TEST(QueryRequestTest, ParseFromLegacyQuery) {
             $hint: {hint: 1},
             $explain: false,
             $min: {x: 'min'},
-            $max: {x: 'max'},
-            $maxScan: 7
+            $max: {x: 'max'}
          })");
     const NamespaceString nss("test.testns");
     unique_ptr<QueryRequest> qr(assertGet(QueryRequest::fromLegacyQuery(
@@ -1331,7 +1595,6 @@ TEST(QueryRequestTest, ParseFromLegacyQuery) {
     ASSERT_EQ(qr->getNToReturn(), boost::optional<long long>(kNToReturn));
     ASSERT_EQ(qr->wantMore(), true);
     ASSERT_EQ(qr->isExplain(), false);
-    ASSERT_EQ(qr->getMaxScan(), 7);
     ASSERT_EQ(qr->isSlaveOk(), false);
     ASSERT_EQ(qr->isOplogReplay(), false);
     ASSERT_EQ(qr->isNoCursorTimeout(), false);
@@ -1369,17 +1632,17 @@ class QueryRequestTest : public ServiceContextTest {};
 
 TEST_F(QueryRequestTest, ParseFromUUID) {
     auto opCtx = makeOperationContext();
-    // Register a UUID/Collection pair in the UUIDCatalog.
+    // Register a UUID/Collection pair in the CollectionCatalog.
     const CollectionUUID uuid = UUID::gen();
     const NamespaceString nss("test.testns");
-    Collection coll(stdx::make_unique<CollectionMock>(nss));
-    UUIDCatalog& catalog = UUIDCatalog::get(opCtx.get());
-    catalog.onCreateCollection(opCtx.get(), &coll, uuid);
-    QueryRequest qr(uuid);
+    std::unique_ptr<Collection> collection = std::make_unique<CollectionMock>(nss);
+    CollectionCatalog& catalog = CollectionCatalog::get(opCtx.get());
+    catalog.registerCollection(uuid, &collection);
+    QueryRequest qr(NamespaceStringOrUUID("test", uuid));
     // Ensure a call to refreshNSS succeeds.
     qr.refreshNSS(opCtx.get());
     ASSERT_EQ(nss, qr.nss());
 }
 
-}  // namespace mongo
 }  // namespace
+}  // namespace mongo
