@@ -109,6 +109,14 @@ protected:
         return indexSpecs;
     }
 
+    void setMockServerReplies(const StatusWith<mongo::BSONObj>& collStatsSwBson,
+                              const StatusWith<mongo::BSONObj>& countSwBson,
+                              const StatusWith<mongo::BSONObj>& listIndexesSwBson) {
+        _mockServer->setCommandReply("collStats", collStatsSwBson);
+        _mockServer->setCommandReply("count", countSwBson);
+        _mockServer->setCommandReply("listIndexes", listIndexesSwBson);
+    }
+
     BSONObj& getIdIndexSpec(CollectionCloner* cloner) {
         return cloner->_idIndexSpec;
     }
@@ -147,9 +155,23 @@ class CollectionClonerTestNonResumable : public CollectionClonerTest {
     }
 };
 
+TEST_F(CollectionClonerTestResumable, CollectionClonerPassesThroughErrorFromCollStatsCommand) {
+    auto cloner = makeCollectionCloner();
+    cloner->setStopAfterStage_forTest("count");
+    // The collection cloner pre-stage makes a remote call to collStats to store in-progress
+    // metrics.
+    _mockServer->setCommandReply("collStats", Status(ErrorCodes::OperationFailed, ""));
+    _mockServer->setCommandReply("count", createCountResponse(100));
+    ASSERT_OK(cloner->run());
+    ASSERT_EQ(100, getProgressMeter(cloner.get()).total());
+}
+
 TEST_F(CollectionClonerTestResumable, CountStage) {
     auto cloner = makeCollectionCloner();
     cloner->setStopAfterStage_forTest("count");
+    // The collection cloner pre-stage makes a remote call to collStats to store in-progress
+    // metrics.
+    _mockServer->setCommandReply("collStats", BSON("size" << 10000));
     _mockServer->setCommandReply("count", createCountResponse(100));
     ASSERT_OK(cloner->run());
     ASSERT_EQ(100, getProgressMeter(cloner.get()).total());
@@ -159,6 +181,9 @@ TEST_F(CollectionClonerTestResumable, CountStage) {
 TEST_F(CollectionClonerTestResumable, CountStageNegativeCount) {
     auto cloner = makeCollectionCloner();
     cloner->setStopAfterStage_forTest("count");
+    // The collection cloner pre-stage makes a remote call to collStats to store in-progress
+    // metrics.
+    _mockServer->setCommandReply("collStats", BSON("size" << 10000));
     _mockServer->setCommandReply("count", createCountResponse(-100));
     ASSERT_OK(cloner->run());
     ASSERT_EQ(0, getProgressMeter(cloner.get()).total());
@@ -167,6 +192,9 @@ TEST_F(CollectionClonerTestResumable, CountStageNegativeCount) {
 // On NamespaceNotFound, the CollectionCloner should exit without doing anything.
 TEST_F(CollectionClonerTestResumable, CountStageNamespaceNotFound) {
     auto cloner = makeCollectionCloner();
+    // The collection cloner pre-stage makes a remote call to collStats to store in-progress
+    // metrics.
+    _mockServer->setCommandReply("collStats", BSON("size" << 10000));
     _mockServer->setCommandReply("count", Status(ErrorCodes::NamespaceNotFound, "NoSuchUuid"));
     ASSERT_OK(cloner->run());
 }
@@ -174,6 +202,9 @@ TEST_F(CollectionClonerTestResumable, CountStageNamespaceNotFound) {
 TEST_F(CollectionClonerTestResumable,
        CollectionClonerPassesThroughNonRetriableErrorFromCountCommand) {
     auto cloner = makeCollectionCloner();
+    // The collection cloner pre-stage makes a remote call to collStats to store in-progress
+    // metrics.
+    _mockServer->setCommandReply("collStats", BSON("size" << 10000));
     _mockServer->setCommandReply("count", Status(ErrorCodes::OperationFailed, ""));
     ASSERT_EQUALS(ErrorCodes::OperationFailed, cloner->run());
 }
@@ -181,6 +212,9 @@ TEST_F(CollectionClonerTestResumable,
 TEST_F(CollectionClonerTestResumable,
        CollectionClonerPassesThroughCommandStatusErrorFromCountCommand) {
     auto cloner = makeCollectionCloner();
+    // The collection cloner pre-stage makes a remote call to collStats to store in-progress
+    // metrics.
+    _mockServer->setCommandReply("collStats", BSON("size" << 10000));
     _mockServer->setCommandReply("count", Status(ErrorCodes::OperationFailed, ""));
     _mockServer->setCommandReply("count",
                                  BSON("ok" << 0 << "errmsg"
@@ -194,6 +228,9 @@ TEST_F(CollectionClonerTestResumable,
 TEST_F(CollectionClonerTestResumable,
        CollectionClonerReturnsNoSuchKeyOnMissingDocumentCountFieldName) {
     auto cloner = makeCollectionCloner();
+    // The collection cloner pre-stage makes a remote call to collStats to store in-progress
+    // metrics.
+    _mockServer->setCommandReply("collStats", BSON("size" << 10000));
     cloner->setStopAfterStage_forTest("count");
     _mockServer->setCommandReply("count", BSON("ok" << 1));
     auto status = cloner->run();
@@ -203,8 +240,8 @@ TEST_F(CollectionClonerTestResumable,
 TEST_F(CollectionClonerTestResumable, ListIndexesReturnedNoIndexes) {
     auto cloner = makeCollectionCloner();
     cloner->setStopAfterStage_forTest("listIndexes");
-    _mockServer->setCommandReply("count", createCountResponse(1));
-    _mockServer->setCommandReply("listIndexes", createCursorResponse(_nss.ns(), BSONArray()));
+    setMockServerReplies(
+        BSON("size" << 10), createCountResponse(1), createCursorResponse(_nss.ns(), BSONArray()));
     ASSERT_OK(cloner->run());
     ASSERT(getIdIndexSpec(cloner.get()).isEmpty());
     ASSERT(getIndexSpecs(cloner.get()).empty());
@@ -214,9 +251,9 @@ TEST_F(CollectionClonerTestResumable, ListIndexesReturnedNoIndexes) {
 // NamespaceNotFound is treated the same as no index.
 TEST_F(CollectionClonerTestResumable, ListIndexesReturnedNamespaceNotFound) {
     auto cloner = makeCollectionCloner();
-    _mockServer->setCommandReply("count", createCountResponse(1));
-    _mockServer->setCommandReply("listIndexes",
-                                 Status(ErrorCodes::NamespaceNotFound, "No indexes here."));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(1),
+                         Status(ErrorCodes::NamespaceNotFound, "No indexes here."));
     ASSERT_OK(cloner->run());
     ASSERT(!_loader);  // We expect not to have run the create collection.
     ASSERT(getIdIndexSpec(cloner.get()).isEmpty());
@@ -227,9 +264,9 @@ TEST_F(CollectionClonerTestResumable, ListIndexesReturnedNamespaceNotFound) {
 TEST_F(CollectionClonerTestResumable, ListIndexesHasResults) {
     auto cloner = makeCollectionCloner();
     cloner->setStopAfterStage_forTest("listIndexes");
-    _mockServer->setCommandReply("count", createCountResponse(1));
-    _mockServer->setCommandReply(
-        "listIndexes",
+    setMockServerReplies(
+        BSON("size" << 10),
+        createCountResponse(1),
         createCursorResponse(
             _nss.ns(),
             BSON_ARRAY(_secondaryIndexSpecs[0] << _idIndexSpec << _secondaryIndexSpecs[1])));
@@ -244,13 +281,15 @@ TEST_F(CollectionClonerTestResumable, ListIndexesHasResults) {
 TEST_F(CollectionClonerTestResumable, CollectionClonerResendsListIndexesCommandOnRetriableError) {
     auto cloner = makeCollectionCloner();
     cloner->setStopAfterStage_forTest("listIndexes");
-    _mockServer->setCommandReply("count", createCountResponse(1));
 
-    // Respond once with failure, once with success.
+    // Respond to listIndexes once with failure, once with success.
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(1),
+                         Status(ErrorCodes::HostNotFound, "HostNotFound"));
     _mockServer->setCommandReply(
         "listIndexes",
-        {Status(ErrorCodes::HostNotFound, "HostNotFound"),
-         createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec << _secondaryIndexSpecs[0]))});
+        createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec << _secondaryIndexSpecs[0])));
+
     ASSERT_OK(cloner->run());
     ASSERT_BSONOBJ_EQ(_idIndexSpec, getIdIndexSpec(cloner.get()));
     ASSERT_EQ(1, getIndexSpecs(cloner.get()).size());
@@ -277,13 +316,15 @@ TEST_F(CollectionClonerTestResumable, BeginCollection) {
 
     auto cloner = makeCollectionCloner();
     cloner->setStopAfterStage_forTest("createCollection");
-    _mockServer->setCommandReply("count", createCountResponse(1));
+
     BSONArrayBuilder indexSpecs;
     indexSpecs.append(_idIndexSpec);
     for (const auto& secondaryIndexSpec : _secondaryIndexSpecs) {
         indexSpecs.append(secondaryIndexSpec);
     }
-    _mockServer->setCommandReply("listIndexes", createCursorResponse(_nss.ns(), indexSpecs.arr()));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(1),
+                         createCursorResponse(_nss.ns(), indexSpecs.arr()));
 
     ASSERT_EQUALS(Status::OK(), cloner->run());
 
@@ -305,16 +346,16 @@ TEST_F(CollectionClonerTestResumable, BeginCollectionFailed) {
 
     auto cloner = makeCollectionCloner();
     cloner->setStopAfterStage_forTest("createCollection");
-    _mockServer->setCommandReply("count", createCountResponse(1));
-    _mockServer->setCommandReply("listIndexes", createCursorResponse(_nss.ns(), BSONArray()));
+    setMockServerReplies(
+        BSON("size" << 10), createCountResponse(1), createCursorResponse(_nss.ns(), BSONArray()));
     ASSERT_EQUALS(ErrorCodes::OperationFailed, cloner->run());
 }
 
 TEST_F(CollectionClonerTestResumable, InsertDocumentsSingleBatch) {
     // Set up data for preliminary stages
-    _mockServer->setCommandReply("count", createCountResponse(2));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(2),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -332,9 +373,9 @@ TEST_F(CollectionClonerTestResumable, InsertDocumentsSingleBatch) {
 
 TEST_F(CollectionClonerTestResumable, InsertDocumentsMultipleBatches) {
     // Set up data for preliminary stages
-    _mockServer->setCommandReply("count", createCountResponse(2));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(2),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -354,9 +395,9 @@ TEST_F(CollectionClonerTestResumable, InsertDocumentsMultipleBatches) {
 
 TEST_F(CollectionClonerTestResumable, InsertDocumentsScheduleDBWorkFailed) {
     // Set up data for preliminary stages
-    _mockServer->setCommandReply("count", createCountResponse(2));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(2),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -391,9 +432,9 @@ TEST_F(CollectionClonerTestResumable, InsertDocumentsScheduleDBWorkFailed) {
 
 TEST_F(CollectionClonerTestResumable, InsertDocumentsCallbackCanceled) {
     // Set up data for preliminary stages
-    _mockServer->setCommandReply("count", createCountResponse(2));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(2),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -434,9 +475,9 @@ TEST_F(CollectionClonerTestResumable, InsertDocumentsCallbackCanceled) {
 
 TEST_F(CollectionClonerTestResumable, InsertDocumentsFailed) {
     // Set up data for preliminary stages
-    _mockServer->setCommandReply("count", createCountResponse(2));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(2),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -493,8 +534,9 @@ TEST_F(CollectionClonerTestResumable, DoNotCreateIDIndexIfAutoIndexIdUsed) {
     const BSONObj doc = BSON("_id" << 1);
     _mockServer->insert(_nss.ns(), doc);
 
-    _mockServer->setCommandReply("count", createCountResponse(1));
-    _mockServer->setCommandReply("listIndexes", createCursorResponse(_nss.ns(), BSONArray()));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(1),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
 
     CollectionOptions options;
     options.autoIndexId = CollectionOptions::NO;
@@ -512,9 +554,11 @@ TEST_F(CollectionClonerTestNonResumable, NonResumableQuerySuccess) {
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(3));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    // The collection cloner pre-stage makes a remote call to collStats to store in-progress
+    // metrics.
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(3),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -535,9 +579,11 @@ TEST_F(CollectionClonerTestNonResumable, NonResumableQueryFailure) {
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(3));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    // The collection cloner pre-stage makes a remote call to collStats to store in-progress
+    // metrics.
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(3),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(_idIndexSpec)));
 
     auto beforeStageFailPoint = globalFailPointRegistry().find("hangBeforeClonerStage");
     auto timesEnteredBeforeStage = beforeStageFailPoint->setMode(
@@ -577,6 +623,9 @@ TEST_F(CollectionClonerTestResumable, ResumableQueryFailTransientlyBeforeFirstBa
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
+    // The collection cloner pre-stage makes a remote call to collStats to store in-progress
+    // metrics.
+    _mockServer->setCommandReply("collStats", BSON("size" << 10));
     _mockServer->setCommandReply("count", createCountResponse(3));
     _mockServer->setCommandReply("listIndexes",
                                  createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
@@ -640,9 +689,9 @@ TEST_F(CollectionClonerTestResumable, ResumableQueryFailTransientlyAfterFirstBat
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(5));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(5),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -697,9 +746,9 @@ TEST_F(CollectionClonerTestResumable, ResumableQueryNonRetriableError) {
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(3));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(3),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -743,9 +792,9 @@ TEST_F(CollectionClonerTestResumable,
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(3));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(3),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -787,9 +836,9 @@ TEST_F(CollectionClonerTestResumable, ResumableQueryKillCursorsNetworkError) {
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(3));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(3),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -842,9 +891,9 @@ TEST_F(CollectionClonerTestResumable, ResumableQueryKillCursorsOtherError) {
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(3));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(3),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -901,9 +950,9 @@ TEST_F(CollectionClonerTestResumable, ResumableQueryNonTransientErrorAtRetry) {
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(5));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(5),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -963,9 +1012,9 @@ TEST_F(CollectionClonerTestResumable, ResumableQueryNonTransientErrorAfterPastRe
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(5));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(5),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -1041,9 +1090,9 @@ TEST_F(CollectionClonerTestResumable, ResumableQueryTwoResumes) {
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(5));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(5),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     // Set up documents to be returned from upstream node.
     _mockServer->insert(_nss.ns(), BSON("_id" << 1));
@@ -1130,9 +1179,9 @@ TEST_F(CollectionClonerTestNonResumable, NonResumableCursorErrorDropOK) {
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(3));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(3),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     auto beforeStageFailPoint = globalFailPointRegistry().find("hangBeforeClonerStage");
     auto timesEnteredBeforeStage = beforeStageFailPoint->setMode(
@@ -1179,9 +1228,9 @@ TEST_F(CollectionClonerTestNonResumable, NonResumableCursorErrorThenOtherError)
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(3));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(3),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     auto beforeStageFailPoint = globalFailPointRegistry().find("hangBeforeClonerStage");
     auto timesEnteredBeforeStage = beforeStageFailPoint->setMode(
@@ -1230,9 +1279,9 @@ TEST_F(CollectionClonerTestNonResumable, NonResumableCursorErrorThenSuccessEqual
     // Set up data for preliminary stages
     auto idIndexSpec = BSON("v" << 1 << "key" << BSON("_id" << 1) << "name"
                                 << "_id_");
-    _mockServer->setCommandReply("count", createCountResponse(3));
-    _mockServer->setCommandReply("listIndexes",
-                                 createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
+    setMockServerReplies(BSON("size" << 10),
+                         createCountResponse(3),
+                         createCursorResponse(_nss.ns(), BSON_ARRAY(idIndexSpec)));
 
     auto beforeStageFailPoint = globalFailPointRegistry().find("hangBeforeClonerStage");
     auto timesEnteredBeforeStage = beforeStageFailPoint->setMode(
