@@ -66,6 +66,16 @@ public:
     static const std::string kInternalVoterTagName;
     static const std::string kInternalElectableTagName;
     static const std::string kInternalAllTagName;
+    static const std::string kConfigAllTagName;
+    static const std::string kConfigVoterTagName;
+
+    /**
+     * Inline `kMaxUserMemberId` to allow others use
+     * the constant without linking to `member_config.cpp`.
+     *
+     * Maximum allowed member id for new nodes added by user.
+     */
+    inline static const int kMaxUserMemberId = 255;
 
     /**
      * Construct a MemberConfig from the contents of "mcfg".
