@@ -29,6 +29,8 @@
 
 #include "mongo/platform/basic.h"
 
+#include <memory>
+
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/client.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
@@ -36,14 +38,13 @@
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/json.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/repl/oplog_applier_impl_test_fixture.h"
 #include "mongo/db/repl/oplog_buffer_collection.h"
 #include "mongo/db/repl/oplog_interface_local.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/storage_interface_impl.h"
-#include "mongo/db/repl/sync_tail_test_fixture.h"
 #include "mongo/db/service_context_d_test_fixture.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/unittest/barrier.h"
 #include "mongo/unittest/death_test.h"
 #include "mongo/unittest/unittest.h"
@@ -59,6 +60,7 @@ protected:
 
 protected:
     ServiceContext::UniqueOperationContext makeOperationContext() const;
+    void pushSentinel(OplogBuffer& oplogBuffer);
 
     StorageInterface* _storageInterface = nullptr;
     ServiceContext::UniqueOperationContext _opCtx;
@@ -68,15 +70,20 @@ private:
     void tearDown() override;
 };
 
+void OplogBufferCollectionTest::pushSentinel(OplogBuffer& oplogBuffer) {
+    const std::vector<BSONObj> oplog = {BSONObj()};
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
+}
+
 void OplogBufferCollectionTest::setUp() {
     ServiceContextMongoDTest::setUp();
     auto service = getServiceContext();
 
     // AutoGetCollectionForReadCommand requires a valid replication coordinator in order to check
     // the shard version.
-    ReplicationCoordinator::set(service, stdx::make_unique<ReplicationCoordinatorMock>(service));
+    ReplicationCoordinator::set(service, std::make_unique<ReplicationCoordinatorMock>(service));
 
-    auto storageInterface = stdx::make_unique<StorageInterfaceImpl>();
+    auto storageInterface = std::make_unique<StorageInterfaceImpl>();
     _storageInterface = storageInterface.get();
     StorageInterface::set(service, std::move(storageInterface));
 
@@ -148,9 +155,10 @@ TEST_F(OplogBufferCollectionTest, StartupWithUserProvidedNamespaceCreatesCollect
     testStartupCreatesCollection(_opCtx.get(), _storageInterface, makeNamespace(_agent));
 }
 
-DEATH_TEST_F(OplogBufferCollectionTest,
-             StartupWithOplogNamespaceTriggersFatalAssertion,
-             "Fatal assertion 40154 Location28838: cannot create a non-capped oplog collection") {
+DEATH_TEST_REGEX_F(
+    OplogBufferCollectionTest,
+    StartupWithOplogNamespaceTriggersFatalAssertion,
+    "Fatal assertion.*40154.*Location28838: cannot create a non-capped oplog collection") {
     testStartupCreatesCollection(_opCtx.get(), _storageInterface, NamespaceString("local.oplog.Z"));
 }
 
@@ -228,9 +236,9 @@ TEST_F(OplogBufferCollectionTest, addIdToDocumentGeneratesIdForSentinelFromLastP
         ts2, 3U, OplogBufferCollection::addIdToDocument({}, ts2, 2U));
 }
 
-DEATH_TEST_F(OplogBufferCollectionTest,
-             addIdToDocumentWithMissingTimestampFieldTriggersInvariantFailure,
-             "Invariant failure !ts.isNull()") {
+DEATH_TEST_REGEX_F(OplogBufferCollectionTest,
+                   addIdToDocumentWithMissingTimestampFieldTriggersInvariantFailure,
+                   R"#(Invariant failure.*!ts.isNull\(\))#") {
     OplogBufferCollection::addIdToDocument(BSON("x" << 1), {}, 0);
 }
 
@@ -339,7 +347,7 @@ TEST_F(OplogBufferCollectionTest, ShutdownWithDropCollectionAtShutdownFalseDoesN
 
         oplogBuffer.startup(_opCtx.get());
         ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-        oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog[0]);
+        oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
         ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
 
         oplogBuffer.shutdown(_opCtx.get());
@@ -354,11 +362,11 @@ TEST_F(OplogBufferCollectionTest, ShutdownWithDropCollectionAtShutdownFalseDoesN
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
 }
 
-DEATH_TEST_F(OplogBufferCollectionTest,
-             StartupWithExistingCollectionFailsWhenEntryHasNoId,
-             "Fatal assertion 40348 IndexNotFound: Index not found, "
-             "ns:local.OplogBufferCollectionTest_"
-             "StartupWithExistingCollectionFailsWhenEntryHasNoId, index: _id_") {
+DEATH_TEST_REGEX_F(OplogBufferCollectionTest,
+                   StartupWithExistingCollectionFailsWhenEntryHasNoId,
+                   "Fatal assertion.*40348.*IndexNotFound: Index not found, "
+                   "ns:local.OplogBufferCollectionTest_"
+                   "StartupWithExistingCollectionFailsWhenEntryHasNoId, index: _id_") {
     auto nss = makeNamespace(_agent);
     CollectionOptions collOpts;
     collOpts.setNoIdIndex();
@@ -374,9 +382,9 @@ DEATH_TEST_F(OplogBufferCollectionTest,
     oplogBuffer.startup(_opCtx.get());
 }
 
-DEATH_TEST_F(OplogBufferCollectionTest,
-             StartupWithExistingCollectionFailsWhenEntryHasNoTimestamp,
-             "Fatal assertion 40405 NoSuchKey: Missing expected field \"ts\"") {
+DEATH_TEST_REGEX_F(OplogBufferCollectionTest,
+                   StartupWithExistingCollectionFailsWhenEntryHasNoTimestamp,
+                   R"#(Fatal assertion.*40405.*NoSuchKey: Missing expected field \\"ts\\")#") {
     auto nss = makeNamespace(_agent);
     ASSERT_OK(_storageInterface->createCollection(_opCtx.get(), nss, CollectionOptions()));
     ASSERT_OK(_storageInterface->insertDocument(_opCtx.get(),
@@ -390,9 +398,9 @@ DEATH_TEST_F(OplogBufferCollectionTest,
     oplogBuffer.startup(_opCtx.get());
 }
 
-DEATH_TEST_F(OplogBufferCollectionTest,
-             StartupWithExistingCollectionFailsWhenEntryHasNoSentinelCount,
-             "Fatal assertion 40406 NoSuchKey: Missing expected field \"s\"") {
+DEATH_TEST_REGEX_F(OplogBufferCollectionTest,
+                   StartupWithExistingCollectionFailsWhenEntryHasNoSentinelCount,
+                   R"#(Fatal assertion.*40406.*NoSuchKey: Missing expected field \\"s\\")#") {
     auto nss = makeNamespace(_agent);
     ASSERT_OK(_storageInterface->createCollection(_opCtx.get(), nss, CollectionOptions()));
     ASSERT_OK(_storageInterface->insertDocument(
@@ -456,7 +464,7 @@ TEST_F(OplogBufferCollectionTest, PushOneDocumentWithPushAllNonBlockingAddsDocum
     oplogBuffer.startup(_opCtx.get());
     const std::vector<BSONObj> oplog = {makeOplogEntry(1)};
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+    oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
 
     _assertDocumentsInCollectionEquals(_opCtx.get(), nss, oplog);
@@ -470,75 +478,47 @@ TEST_F(OplogBufferCollectionTest,
     oplogBuffer.startup(_opCtx.get());
     const std::vector<BSONObj> emptyOplogEntries;
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.pushAllNonBlocking(nullptr, emptyOplogEntries.begin(), emptyOplogEntries.end());
+    oplogBuffer.push(nullptr, emptyOplogEntries.begin(), emptyOplogEntries.end());
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
 }
 
-TEST_F(OplogBufferCollectionTest, PushOneDocumentWithPushAddsDocument) {
-    auto nss = makeNamespace(_agent);
-    OplogBufferCollection oplogBuffer(_storageInterface, nss);
-
-    oplogBuffer.startup(_opCtx.get());
-    BSONObj oplog = makeOplogEntry(1);
-    ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.push(_opCtx.get(), oplog);
-    ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
-
-    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog});
-}
-
-TEST_F(OplogBufferCollectionTest, PushOneDocumentWithPushEvenIfFullAddsDocument) {
-    auto nss = makeNamespace(_agent);
-    OplogBufferCollection oplogBuffer(_storageInterface, nss);
-
-    oplogBuffer.startup(_opCtx.get());
-    BSONObj oplog = makeOplogEntry(1);
-    ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog);
-    ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
-
-    ASSERT_EQUALS(0UL, oplogBuffer.getSentinelCount_forTest());
-
-    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog});
-}
-
 TEST_F(OplogBufferCollectionTest, PeekDoesNotRemoveDocument) {
     auto nss = makeNamespace(_agent);
     OplogBufferCollection oplogBuffer(_storageInterface, nss);
 
     oplogBuffer.startup(_opCtx.get());
-    BSONObj oplog1 = makeOplogEntry(1);
+    const std::vector<BSONObj> oplog1 = {makeOplogEntry(1)};
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.push(_opCtx.get(), oplog1);
+    oplogBuffer.push(_opCtx.get(), oplog1.cbegin(), oplog1.cend());
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
 
     // _peekOneSide should provide correct bound inclusion to storage engine when collection has one
     // document.
     BSONObj doc;
     ASSERT_TRUE(oplogBuffer.peek(_opCtx.get(), &doc));
-    ASSERT_BSONOBJ_EQ(doc, oplog1);
+    ASSERT_BSONOBJ_EQ(doc, oplog1[0]);
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
 
-    BSONObj oplog2 = makeOplogEntry(2);
-    oplogBuffer.push(_opCtx.get(), oplog2);
+    const std::vector<BSONObj> oplog2 = {makeOplogEntry(2)};
+    oplogBuffer.push(_opCtx.get(), oplog2.cbegin(), oplog2.cend());
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
 
     // _peekOneSide should return same result after adding new oplog entry.
     ASSERT_TRUE(oplogBuffer.peek(_opCtx.get(), &doc));
-    ASSERT_BSONOBJ_EQ(doc, oplog1);
+    ASSERT_BSONOBJ_EQ(doc, oplog1[0]);
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
 
-    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog1, oplog2});
+    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog1[0], oplog2[0]});
 }
 
 TEST_F(OplogBufferCollectionTest, PeekingFromExistingCollectionReturnsDocument) {
     auto nss = makeNamespace(_agent);
-    const std::vector<BSONObj> oplog = {makeOplogEntry(1), makeOplogEntry(2)};
+    const auto entry1 = makeOplogEntry(1);
     ASSERT_OK(_storageInterface->createCollection(_opCtx.get(), nss, CollectionOptions()));
     ASSERT_OK(_storageInterface->insertDocument(
         _opCtx.get(),
         nss,
-        TimestampedBSONObj{std::get<0>(OplogBufferCollection::addIdToDocument(oplog[0], {}, 0)),
+        TimestampedBSONObj{std::get<0>(OplogBufferCollection::addIdToDocument(entry1, {}, 0)),
                            Timestamp(0)},
         OpTime::kUninitializedTerm));
 
@@ -549,21 +529,22 @@ TEST_F(OplogBufferCollectionTest, PeekingFromExistingCollectionReturnsDocument)
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
     ASSERT_NOT_EQUALS(oplogBuffer.getCount(), 0UL);
     ASSERT_FALSE(oplogBuffer.isEmpty());
-    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog[0]});
+    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {entry1});
 
     BSONObj doc;
     ASSERT_TRUE(oplogBuffer.peek(_opCtx.get(), &doc));
-    ASSERT_BSONOBJ_EQ(doc, oplog[0]);
+    ASSERT_BSONOBJ_EQ(doc, entry1);
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
 
-    oplogBuffer.push(_opCtx.get(), oplog[1]);
+    const std::vector<BSONObj> oplog = {makeOplogEntry(2)};
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
 
     ASSERT_TRUE(oplogBuffer.peek(_opCtx.get(), &doc));
-    ASSERT_BSONOBJ_EQ(doc, oplog[0]);
+    ASSERT_BSONOBJ_EQ(doc, entry1);
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
 
-    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, oplog);
+    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {makeOplogEntry(1), makeOplogEntry(2)});
 }
 
 TEST_F(OplogBufferCollectionTest, PeekWithNoDocumentsReturnsFalse) {
@@ -586,14 +567,14 @@ TEST_F(OplogBufferCollectionTest, PopDoesNotRemoveDocumentFromCollection) {
     OplogBufferCollection oplogBuffer(_storageInterface, nss);
 
     oplogBuffer.startup(_opCtx.get());
-    BSONObj oplog = makeOplogEntry(1);
+    const std::vector<BSONObj> oplog = {makeOplogEntry(1)};
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.push(_opCtx.get(), oplog);
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
 
     BSONObj doc;
     ASSERT_TRUE(oplogBuffer.tryPop(_opCtx.get(), &doc));
-    ASSERT_BSONOBJ_EQ(doc, oplog);
+    ASSERT_BSONOBJ_EQ(doc, oplog[0]);
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
 
     _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog});
@@ -625,7 +606,7 @@ TEST_F(OplogBufferCollectionTest, PopAndPeekReturnDocumentsInOrder) {
         makeOplogEntry(3),
     };
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+    oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
     ASSERT_EQUALS(oplogBuffer.getCount(), 3UL);
 
     _assertDocumentsInCollectionEquals(_opCtx.get(), nss, oplog);
@@ -670,7 +651,7 @@ TEST_F(OplogBufferCollectionTest, LastObjectPushedReturnsNewestOplogEntry) {
         makeOplogEntry(3),
     };
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+    oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
     ASSERT_EQUALS(oplogBuffer.getCount(), 3UL);
 
     auto doc = oplogBuffer.lastObjectPushed(_opCtx.get());
@@ -710,7 +691,7 @@ TEST_F(OplogBufferCollectionTest,
     ASSERT_BSONOBJ_EQ(*oplogBuffer.lastObjectPushed(_opCtx.get()), secondDoc);
 
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
-    oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+    oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
     ASSERT_EQUALS(oplogBuffer.getCount(), 5UL);
 
     ASSERT_BSONOBJ_EQ(*oplogBuffer.lastObjectPushed(_opCtx.get()), oplog[2]);
@@ -732,9 +713,9 @@ TEST_F(OplogBufferCollectionTest, IsEmptyReturnsTrueWhenEmptyAndFalseWhenNot) {
     OplogBufferCollection oplogBuffer(_storageInterface, nss);
 
     oplogBuffer.startup(_opCtx.get());
-    BSONObj oplog = makeOplogEntry(1);
+    const std::vector<BSONObj> oplog = {makeOplogEntry(1)};
     ASSERT_TRUE(oplogBuffer.isEmpty());
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog);
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
     ASSERT_FALSE(oplogBuffer.isEmpty());
 }
 
@@ -749,47 +730,46 @@ TEST_F(OplogBufferCollectionTest, ClearClearsCollection) {
     ASSERT_EQUALS(Timestamp(), oplogBuffer.getLastPushedTimestamp_forTest());
     ASSERT_EQUALS(Timestamp(), oplogBuffer.getLastPoppedTimestamp_forTest());
 
-    BSONObj oplog = makeOplogEntry(1);
-    oplogBuffer.push(_opCtx.get(), oplog);
+    const std::vector<BSONObj> oplog = {makeOplogEntry(1)};
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
-    ASSERT_EQUALS(oplogBuffer.getSize(), std::size_t(oplog.objsize()));
+    ASSERT_EQUALS(oplogBuffer.getSize(), std::size_t(oplog[0].objsize()));
     ASSERT_EQUALS(0U, oplogBuffer.getSentinelCount_forTest());
-    ASSERT_EQUALS(oplog["ts"].timestamp(), oplogBuffer.getLastPushedTimestamp_forTest());
+    ASSERT_EQUALS(oplog[0]["ts"].timestamp(), oplogBuffer.getLastPushedTimestamp_forTest());
     ASSERT_EQUALS(Timestamp(), oplogBuffer.getLastPoppedTimestamp_forTest());
 
     _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog});
 
-    BSONObj sentinel;
-    oplogBuffer.push(_opCtx.get(), sentinel);
+    pushSentinel(oplogBuffer);
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
-    ASSERT_EQUALS(oplogBuffer.getSize(), std::size_t(oplog.objsize() + BSONObj().objsize()));
+    ASSERT_EQUALS(oplogBuffer.getSize(), std::size_t(oplog[0].objsize() + BSONObj().objsize()));
     ASSERT_EQUALS(1U, oplogBuffer.getSentinelCount_forTest());
-    ASSERT_EQUALS(oplog["ts"].timestamp(), oplogBuffer.getLastPushedTimestamp_forTest());
+    ASSERT_EQUALS(oplog[0]["ts"].timestamp(), oplogBuffer.getLastPushedTimestamp_forTest());
     ASSERT_EQUALS(Timestamp(), oplogBuffer.getLastPoppedTimestamp_forTest());
 
-    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog, sentinel});
+    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog[0], BSONObj()});
 
-    BSONObj oplog2 = makeOplogEntry(2);
-    oplogBuffer.push(_opCtx.get(), oplog2);
+    const std::vector<BSONObj> oplog2 = {makeOplogEntry(2)};
+    oplogBuffer.push(_opCtx.get(), oplog2.cbegin(), oplog2.cend());
     ASSERT_EQUALS(oplogBuffer.getCount(), 3UL);
     ASSERT_EQUALS(oplogBuffer.getSize(),
-                  std::size_t(oplog.objsize() + BSONObj().objsize() + oplog2.objsize()));
+                  std::size_t(oplog[0].objsize() + BSONObj().objsize() + oplog2[0].objsize()));
     ASSERT_EQUALS(0U, oplogBuffer.getSentinelCount_forTest());
-    ASSERT_EQUALS(oplog2["ts"].timestamp(), oplogBuffer.getLastPushedTimestamp_forTest());
+    ASSERT_EQUALS(oplog2[0]["ts"].timestamp(), oplogBuffer.getLastPushedTimestamp_forTest());
     ASSERT_EQUALS(Timestamp(), oplogBuffer.getLastPoppedTimestamp_forTest());
 
-    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog, sentinel, oplog2});
+    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog[0], BSONObj(), oplog2[0]});
 
     BSONObj poppedDoc;
     ASSERT_TRUE(oplogBuffer.tryPop(_opCtx.get(), &poppedDoc));
-    ASSERT_BSONOBJ_EQ(oplog, poppedDoc);
+    ASSERT_BSONOBJ_EQ(oplog[0], poppedDoc);
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
-    ASSERT_EQUALS(oplogBuffer.getSize(), std::size_t(BSONObj().objsize() + oplog2.objsize()));
+    ASSERT_EQUALS(oplogBuffer.getSize(), std::size_t(BSONObj().objsize() + oplog2[0].objsize()));
     ASSERT_EQUALS(0U, oplogBuffer.getSentinelCount_forTest());
-    ASSERT_EQUALS(oplog2["ts"].timestamp(), oplogBuffer.getLastPushedTimestamp_forTest());
-    ASSERT_EQUALS(oplog["ts"].timestamp(), oplogBuffer.getLastPoppedTimestamp_forTest());
+    ASSERT_EQUALS(oplog2[0]["ts"].timestamp(), oplogBuffer.getLastPushedTimestamp_forTest());
+    ASSERT_EQUALS(oplog[0]["ts"].timestamp(), oplogBuffer.getLastPoppedTimestamp_forTest());
 
-    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog, sentinel, oplog2});
+    _assertDocumentsInCollectionEquals(_opCtx.get(), nss, {oplog[0], BSONObj(), oplog2[0]});
 
     oplogBuffer.clear(_opCtx.get());
     ASSERT_TRUE(AutoGetCollectionForReadCommand(_opCtx.get(), nss).getCollection());
@@ -833,7 +813,7 @@ TEST_F(OplogBufferCollectionTest, WaitForDataBlocksAndFindsDocument) {
     oplogBuffer.startup(_opCtx.get());
 
     unittest::Barrier barrier(2U);
-    BSONObj oplog = makeOplogEntry(1);
+    const std::vector<BSONObj> oplog = {makeOplogEntry(1)};
     BSONObj doc;
     bool success = false;
     std::size_t count = 0;
@@ -847,12 +827,12 @@ TEST_F(OplogBufferCollectionTest, WaitForDataBlocksAndFindsDocument) {
 
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
     barrier.countDownAndWait();
-    oplogBuffer.push(_opCtx.get(), oplog);
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
     peekingThread.join();
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
     ASSERT_TRUE(success);
     ASSERT_TRUE(oplogBuffer.peek(_opCtx.get(), &doc));
-    ASSERT_BSONOBJ_EQ(doc, oplog);
+    ASSERT_BSONOBJ_EQ(doc, oplog[0]);
     ASSERT_EQUALS(count, 1UL);
 }
 
@@ -862,7 +842,7 @@ TEST_F(OplogBufferCollectionTest, TwoWaitForDataInvocationsBlockAndFindSameDocum
     oplogBuffer.startup(_opCtx.get());
 
     unittest::Barrier barrier(3U);
-    BSONObj oplog = makeOplogEntry(1);
+    const std::vector<BSONObj> oplog = {makeOplogEntry(1)};
     bool success1 = false;
     std::size_t count1 = 0;
 
@@ -885,14 +865,14 @@ TEST_F(OplogBufferCollectionTest, TwoWaitForDataInvocationsBlockAndFindSameDocum
 
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
     barrier.countDownAndWait();
-    oplogBuffer.push(_opCtx.get(), oplog);
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
     peekingThread1.join();
     peekingThread2.join();
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
     ASSERT_TRUE(success1);
     BSONObj doc;
     ASSERT_TRUE(oplogBuffer.peek(_opCtx.get(), &doc));
-    ASSERT_BSONOBJ_EQ(doc, oplog);
+    ASSERT_BSONOBJ_EQ(doc, oplog[0]);
     ASSERT_EQUALS(count1, 1UL);
     ASSERT_TRUE(success2);
     ASSERT_EQUALS(count2, 1UL);
@@ -922,13 +902,12 @@ TEST_F(OplogBufferCollectionTest, WaitForDataBlocksAndTimesOutWhenItDoesNotFindD
     ASSERT_EQUALS(count, 0UL);
 }
 
-void _testPushSentinelsProperly(
-    OperationContext* opCtx,
-    const NamespaceString& nss,
-    StorageInterface* storageInterface,
-    stdx::function<void(OperationContext* opCtx,
-                        OplogBufferCollection* oplogBuffer,
-                        const std::vector<BSONObj>& oplog)> pushDocsFn) {
+void _testPushSentinelsProperly(OperationContext* opCtx,
+                                const NamespaceString& nss,
+                                StorageInterface* storageInterface,
+                                std::function<void(OperationContext* opCtx,
+                                                   OplogBufferCollection* oplogBuffer,
+                                                   const std::vector<BSONObj>& oplog)> pushDocsFn) {
     OplogBufferCollection oplogBuffer(storageInterface, nss);
     oplogBuffer.startup(opCtx);
     const std::vector<BSONObj> oplog = {
@@ -945,62 +924,6 @@ void _testPushSentinelsProperly(
     _assertDocumentsInCollectionEquals(opCtx, nss, oplog);
 }
 
-TEST_F(OplogBufferCollectionTest, PushPushesOnSentinelsProperly) {
-    auto nss = makeNamespace(_agent);
-    _testPushSentinelsProperly(_opCtx.get(),
-                               nss,
-                               _storageInterface,
-                               [](OperationContext* opCtx,
-                                  OplogBufferCollection* oplogBuffer,
-                                  const std::vector<BSONObj>& oplog) {
-                                   oplogBuffer->push(opCtx, oplog[0]);
-                                   ASSERT_EQUALS(1U, oplogBuffer->getSentinelCount_forTest());
-
-                                   oplogBuffer->push(opCtx, oplog[1]);
-                                   ASSERT_EQUALS(0U, oplogBuffer->getSentinelCount_forTest());
-
-                                   oplogBuffer->push(opCtx, oplog[2]);
-                                   ASSERT_EQUALS(1U, oplogBuffer->getSentinelCount_forTest());
-
-                                   oplogBuffer->push(opCtx, oplog[3]);
-                                   ASSERT_EQUALS(2U, oplogBuffer->getSentinelCount_forTest());
-
-                                   oplogBuffer->push(opCtx, oplog[4]);
-                                   ASSERT_EQUALS(0U, oplogBuffer->getSentinelCount_forTest());
-
-                                   oplogBuffer->push(opCtx, oplog[5]);
-                                   ASSERT_EQUALS(1U, oplogBuffer->getSentinelCount_forTest());
-                               });
-}
-
-TEST_F(OplogBufferCollectionTest, PushEvenIfFullPushesOnSentinelsProperly) {
-    auto nss = makeNamespace(_agent);
-    _testPushSentinelsProperly(_opCtx.get(),
-                               nss,
-                               _storageInterface,
-                               [](OperationContext* opCtx,
-                                  OplogBufferCollection* oplogBuffer,
-                                  const std::vector<BSONObj>& oplog) {
-                                   oplogBuffer->pushEvenIfFull(opCtx, oplog[0]);
-                                   ASSERT_EQUALS(1U, oplogBuffer->getSentinelCount_forTest());
-
-                                   oplogBuffer->pushEvenIfFull(opCtx, oplog[1]);
-                                   ASSERT_EQUALS(0U, oplogBuffer->getSentinelCount_forTest());
-
-                                   oplogBuffer->pushEvenIfFull(opCtx, oplog[2]);
-                                   ASSERT_EQUALS(1U, oplogBuffer->getSentinelCount_forTest());
-
-                                   oplogBuffer->pushEvenIfFull(opCtx, oplog[3]);
-                                   ASSERT_EQUALS(2U, oplogBuffer->getSentinelCount_forTest());
-
-                                   oplogBuffer->pushEvenIfFull(opCtx, oplog[4]);
-                                   ASSERT_EQUALS(0U, oplogBuffer->getSentinelCount_forTest());
-
-                                   oplogBuffer->pushEvenIfFull(opCtx, oplog[5]);
-                                   ASSERT_EQUALS(1U, oplogBuffer->getSentinelCount_forTest());
-                               });
-}
-
 TEST_F(OplogBufferCollectionTest, PushAllNonBlockingPushesOnSentinelsProperly) {
     auto nss = makeNamespace(_agent);
     _testPushSentinelsProperly(_opCtx.get(),
@@ -1009,16 +932,15 @@ TEST_F(OplogBufferCollectionTest, PushAllNonBlockingPushesOnSentinelsProperly) {
                                [](OperationContext* opCtx,
                                   OplogBufferCollection* oplogBuffer,
                                   const std::vector<BSONObj>& oplog) {
-                                   oplogBuffer->pushAllNonBlocking(
-                                       opCtx, oplog.cbegin(), oplog.cend());
+                                   oplogBuffer->push(opCtx, oplog.cbegin(), oplog.cend());
                                    ASSERT_EQUALS(1U, oplogBuffer->getSentinelCount_forTest());
                                });
 }
 
-DEATH_TEST_F(
+DEATH_TEST_REGEX_F(
     OplogBufferCollectionTest,
     PushAllNonBlockingWithOutOfOrderDocumentsTriggersInvariantFailure,
-    "Invariant failure value.isEmpty() ? ts == previousTimestamp : ts > previousTimestamp") {
+    R"#(Invariant failure.*value.isEmpty\(\) \? ts == previousTimestamp : ts > previousTimestamp)#") {
     auto nss = makeNamespace(_agent);
     OplogBufferCollection oplogBuffer(_storageInterface, nss);
 
@@ -1028,7 +950,7 @@ DEATH_TEST_F(
         makeOplogEntry(1),
     };
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+    oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
 }
 
 TEST_F(OplogBufferCollectionTest, SentinelInMiddleIsReturnedInOrder) {
@@ -1043,10 +965,7 @@ TEST_F(OplogBufferCollectionTest, SentinelInMiddleIsReturnedInOrder) {
         makeOplogEntry(3),
     };
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog[0]);
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog[1]);
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog[2]);
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog[3]);
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
     ASSERT_EQUALS(oplogBuffer.getCount(), 4UL);
 
     _assertDocumentsInCollectionEquals(_opCtx.get(), nss, oplog);
@@ -1095,8 +1014,7 @@ TEST_F(OplogBufferCollectionTest, SentinelAtBeginningIsReturnedAtBeginning) {
     oplogBuffer.startup(_opCtx.get());
     const std::vector<BSONObj> oplog = {BSONObj(), makeOplogEntry(1)};
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog[0]);
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog[1]);
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
 
     _assertDocumentsInCollectionEquals(_opCtx.get(), nss, oplog);
@@ -1129,7 +1047,7 @@ TEST_F(OplogBufferCollectionTest, SentinelAtEndIsReturnedAtEnd) {
     oplogBuffer.startup(_opCtx.get());
     const std::vector<BSONObj> oplog = {makeOplogEntry(1), BSONObj()};
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+    oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
 
     _assertDocumentsInCollectionEquals(_opCtx.get(), nss, oplog);
@@ -1169,7 +1087,7 @@ TEST_F(OplogBufferCollectionTest, MultipleSentinelsAreReturnedInOrder) {
         BSONObj(),
     };
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-    oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.cbegin(), oplog.cend());
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
     ASSERT_EQUALS(oplogBuffer.getCount(), 6UL);
 
     _assertDocumentsInCollectionEquals(_opCtx.get(), nss, oplog);
@@ -1237,7 +1155,7 @@ TEST_F(OplogBufferCollectionTest, SentinelAtFrontOfExistingCollectionIsReturnedP
 
         oplogBuffer.startup(_opCtx.get());
         ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-        oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+        oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
         ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
         oplogBuffer.shutdown(_opCtx.get());
     }
@@ -1260,7 +1178,7 @@ TEST_F(OplogBufferCollectionTest, SentinelAtFrontOfExistingCollectionIsReturnedP
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
 
     // Push and pop another sentinel to make sure that they're counted correctly.
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), BSONObj());
+    pushSentinel(oplogBuffer);
     ASSERT_TRUE((*oplogBuffer.lastObjectPushed(_opCtx.get())).isEmpty());
     ASSERT_EQUALS(oplogBuffer.getCount(), 3UL);
 
@@ -1287,7 +1205,7 @@ TEST_F(OplogBufferCollectionTest, TwoSentinelsAtFrontOfExistingCollectionAreRetu
 
         oplogBuffer.startup(_opCtx.get());
         ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-        oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+        oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
         ASSERT_EQUALS(oplogBuffer.getCount(), 4UL);
         oplogBuffer.shutdown(_opCtx.get());
     }
@@ -1310,7 +1228,7 @@ TEST_F(OplogBufferCollectionTest, TwoSentinelsAtFrontOfExistingCollectionAreRetu
     ASSERT_EQUALS(oplogBuffer.getCount(), 4UL);
 
     // Push and pop another sentinel to make sure that they're counted correctly.
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), BSONObj());
+    pushSentinel(oplogBuffer);
     ASSERT_EQUALS(oplogBuffer.getCount(), 5UL);
     ASSERT_EQUALS(2UL, oplogBuffer.getSentinelCount_forTest());
 
@@ -1345,7 +1263,7 @@ TEST_F(OplogBufferCollectionTest, SentinelAtBackOfExistingCollectionIsReturnedPr
 
         oplogBuffer.startup(_opCtx.get());
         ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-        oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+        oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
         ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
         oplogBuffer.shutdown(_opCtx.get());
     }
@@ -1368,7 +1286,7 @@ TEST_F(OplogBufferCollectionTest, SentinelAtBackOfExistingCollectionIsReturnedPr
     ASSERT_EQUALS(oplogBuffer.getCount(), 2UL);
 
     // Push and pop another sentinel to make sure that they're counted correctly.
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), BSONObj());
+    pushSentinel(oplogBuffer);
     ASSERT_EQUALS(oplogBuffer.getCount(), 3UL);
     ASSERT_EQUALS(2UL, oplogBuffer.getSentinelCount_forTest());
 
@@ -1395,7 +1313,7 @@ TEST_F(OplogBufferCollectionTest, TwoSentinelsAtBackOfExistingCollectionAreRetur
 
         oplogBuffer.startup(_opCtx.get());
         ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-        oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+        oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
         ASSERT_EQUALS(oplogBuffer.getCount(), 4UL);
         oplogBuffer.shutdown(_opCtx.get());
     }
@@ -1418,7 +1336,7 @@ TEST_F(OplogBufferCollectionTest, TwoSentinelsAtBackOfExistingCollectionAreRetur
     ASSERT_EQUALS(oplogBuffer.getCount(), 4UL);
 
     // Push and pop another sentinel to make sure that they're counted correctly.
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), BSONObj());
+    pushSentinel(oplogBuffer);
     ASSERT_TRUE((*oplogBuffer.lastObjectPushed(_opCtx.get())).isEmpty());
     ASSERT_EQUALS(oplogBuffer.getCount(), 5UL);
     ASSERT_EQUALS(3UL, oplogBuffer.getSentinelCount_forTest());
@@ -1454,7 +1372,7 @@ TEST_F(OplogBufferCollectionTest, SentinelInMiddleOfExistingCollectionIsReturned
 
         oplogBuffer.startup(_opCtx.get());
         ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
-        oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.begin(), oplog.end());
+        oplogBuffer.push(_opCtx.get(), oplog.begin(), oplog.end());
         ASSERT_EQUALS(oplogBuffer.getCount(), 3UL);
         oplogBuffer.shutdown(_opCtx.get());
     }
@@ -1477,7 +1395,7 @@ TEST_F(OplogBufferCollectionTest, SentinelInMiddleOfExistingCollectionIsReturned
     ASSERT_EQUALS(oplogBuffer.getCount(), 3UL);
 
     // Push and pop another sentinel to make sure that they're counted correctly.
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), BSONObj());
+    pushSentinel(oplogBuffer);
     ASSERT_TRUE((*oplogBuffer.lastObjectPushed(_opCtx.get())).isEmpty());
     ASSERT_EQUALS(oplogBuffer.getCount(), 4UL);
     ASSERT_EQUALS(1UL, oplogBuffer.getSentinelCount_forTest());
@@ -1506,7 +1424,6 @@ TEST_F(OplogBufferCollectionTest, WaitForDataBlocksAndFindsSentinel) {
     oplogBuffer.startup(_opCtx.get());
 
     unittest::Barrier barrier(2U);
-    BSONObj oplog;
     BSONObj doc;
     bool success = false;
     std::size_t count = 0;
@@ -1520,7 +1437,7 @@ TEST_F(OplogBufferCollectionTest, WaitForDataBlocksAndFindsSentinel) {
 
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
     barrier.countDownAndWait();
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog);
+    pushSentinel(oplogBuffer);
     peekingThread.join();
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
     ASSERT_TRUE(success);
@@ -1535,7 +1452,6 @@ TEST_F(OplogBufferCollectionTest, TwoWaitForDataInvocationsBlockAndFindSameSenti
     oplogBuffer.startup(_opCtx.get());
 
     unittest::Barrier barrier(3U);
-    BSONObj oplog;
     bool success1 = false;
     std::size_t count1 = 0;
 
@@ -1558,7 +1474,7 @@ TEST_F(OplogBufferCollectionTest, TwoWaitForDataInvocationsBlockAndFindSameSenti
 
     ASSERT_EQUALS(oplogBuffer.getCount(), 0UL);
     barrier.countDownAndWait();
-    oplogBuffer.pushEvenIfFull(_opCtx.get(), oplog);
+    pushSentinel(oplogBuffer);
     peekingThread1.join();
     peekingThread2.join();
     ASSERT_EQUALS(oplogBuffer.getCount(), 1UL);
@@ -1602,7 +1518,7 @@ TEST_F(OplogBufferCollectionTest, PeekFillsCacheWithDocumentsFromCollection) {
     for (int i = 0; i < 5; ++i) {
         oplog.push_back(makeOplogEntry(i + 1));
     };
-    oplogBuffer.pushAllNonBlocking(_opCtx.get(), oplog.cbegin(), oplog.cend());
+    oplogBuffer.push(_opCtx.get(), oplog.cbegin(), oplog.cend());
     _assertDocumentsInCollectionEquals(_opCtx.get(), nss, oplog);
 
     // Before any peek operations, peek cache should be empty.
