@@ -47,7 +47,6 @@
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/server_status_metric.h"
-#include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/lasterror.h"
@@ -134,20 +133,6 @@ public:
 
             status = replCoord->waitForMemberState(expectedState, timeout);
 
-            uassertStatusOK(status);
-            return true;
-        } else if (cmdObj.hasElement("waitForDrainFinish")) {
-            long long timeoutMillis;
-            auto status = bsonExtractIntegerField(cmdObj, "waitForDrainFinish", &timeoutMillis);
-            uassertStatusOK(status);
-            Milliseconds timeout(timeoutMillis);
-            LOGV2(21575,
-                  "replSetTest: waiting {timeout} for applier buffer to finish draining",
-                  "replSetTest: waiting for applier buffer to finish draining",
-                  "timeout"_attr = timeout);
-
-            status = replCoord->waitForDrainFinish(timeout);
-
             uassertStatusOK(status);
             return true;
         } else if (cmdObj.hasElement("getLastStableRecoveryTimestamp")) {
@@ -430,8 +415,8 @@ public:
                 // Convert the error code to be more specific.
                 uasserted(ErrorCodes::CurrentConfigNotCommittedYet, status.reason());
             } else if (status == ErrorCodes::PrimarySteppedDown) {
-                // Return NotMaster since the command has no side effect yet.
-                status = {ErrorCodes::NotMaster, status.reason()};
+                // Return NotWritablePrimary since the command has no side effect yet.
+                status = {ErrorCodes::NotWritablePrimary, status.reason()};
             }
             uassertStatusOK(status);
         }
