@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kReplication
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 #include "mongo/platform/basic.h"
 
@@ -42,7 +42,6 @@
 #include "mongo/db/server_options.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/log.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -51,6 +50,7 @@ namespace {
 
 const std::string kConfigFieldName = "config";
 const std::string kConfigVersionFieldName = "v";
+const std::string kConfigTermFieldName = "configTerm";
 const std::string kElectionTimeFieldName = "electionTime";
 const std::string kMemberStateFieldName = "state";
 const std::string kOkFieldName = "ok";
@@ -81,6 +81,9 @@ void ReplSetHeartbeatResponse::addToBSON(BSONObjBuilder* builder) const {
     if (_configVersion != -1) {
         *builder << kConfigVersionFieldName << _configVersion;
     }
+    if (_configTerm != OpTime::kUninitializedTerm) {
+        *builder << kConfigTermFieldName << _configTerm;
+    }
     if (!_setName.empty()) {
         *builder << kReplSetFieldName << _setName;
     }
@@ -109,9 +112,7 @@ BSONObj ReplSetHeartbeatResponse::toBSON() const {
     return builder.obj();
 }
 
-Status ReplSetHeartbeatResponse::initialize(const BSONObj& doc,
-                                            long long term,
-                                            bool requireWallTime) {
+Status ReplSetHeartbeatResponse::initialize(const BSONObj& doc, long long term) {
     auto status = getStatusFromCommandResult(doc);
     if (!status.isOK()) {
         return status;
@@ -158,14 +159,10 @@ Status ReplSetHeartbeatResponse::initialize(const BSONObj& doc,
     _durableWallTime = Date_t();
     status = bsonExtractTypedField(
         doc, kDurableWallTimeFieldName, BSONType::Date, &durableWallTimeElement);
-    if (!status.isOK() && (status != ErrorCodes::NoSuchKey || requireWallTime)) {
-        // We ignore NoSuchKey errors if the FeatureCompatibilityVersion is less than 4.2, since
-        // older version nodes may not report wall clock times.
+    if (!status.isOK()) {
         return status;
     }
-    if (status.isOK()) {
-        _durableWallTime = durableWallTimeElement.Date();
-    }
+    _durableWallTime = durableWallTimeElement.Date();
     _durableOpTimeSet = true;
 
 
@@ -179,14 +176,10 @@ Status ReplSetHeartbeatResponse::initialize(const BSONObj& doc,
     _appliedWallTime = Date_t();
     status = bsonExtractTypedField(
         doc, kAppliedWallTimeFieldName, BSONType::Date, &appliedWallTimeElement);
-    if (!status.isOK() && (status != ErrorCodes::NoSuchKey || requireWallTime)) {
-        // We ignore NoSuchKey errors if the FeatureCompatibilityVersion is less than 4.2, since
-        // older version nodes may not report wall clock times.
+    if (!status.isOK()) {
         return status;
     }
-    if (status.isOK()) {
-        _appliedWallTime = appliedWallTimeElement.Date();
-    }
+    _appliedWallTime = appliedWallTimeElement.Date();
     _appliedOpTimeSet = true;
 
     const BSONElement memberStateElement = doc[kMemberStateFieldName];
@@ -228,6 +221,12 @@ Status ReplSetHeartbeatResponse::initialize(const BSONObj& doc,
     }
     _configVersion = configVersionElement.numberInt();
 
+    // Allow a missing term field for backward compatibility.
+    const BSONElement configTermElement = doc[kConfigTermFieldName];
+    if (!configTermElement.eoo() && configVersionElement.type() == NumberInt) {
+        _configTerm = configTermElement.numberInt();
+    }
+
     const BSONElement syncingToElement = doc[kSyncSourceFieldName];
     if (syncingToElement.eoo()) {
         _syncingTo = HostAndPort();
