@@ -29,13 +29,14 @@
 
 #pragma once
 
+#include <functional>
+
 #include "mongo/base/status_with.h"
 #include "mongo/db/op_observer.h"
 #include "mongo/db/repl/oplog_entry.h"
 #include "mongo/db/repl/roll_back_local_operations.h"
 #include "mongo/db/repl/rollback.h"
 #include "mongo/db/repl/storage_interface.h"
-#include "mongo/stdx/functional.h"
 
 namespace mongo {
 
@@ -340,13 +341,6 @@ private:
      */
     Status _checkAgainstTimeLimit(RollBackLocalOperations::RollbackCommonPoint commonPoint);
 
-    /**
-     * Finds the timestamp of the record after the common point to put into the oplog truncate
-     * after point.
-     */
-    Timestamp _findTruncateTimestamp(
-        OperationContext* opCtx, RollBackLocalOperations::RollbackCommonPoint commonPoint) const;
-
     /**
      * Kills all user operations currently being performed. Since this node is a secondary, these
      * operations are all reads.
@@ -363,14 +357,14 @@ private:
     Status _transitionToRollback(OperationContext* opCtx);
 
     /**
-     * Waits for any in-progress background index builds to complete. We do this before beginning
+     * Stops any active index builds and waits for them to complete. We do this before beginning
      * the rollback process to prevent any issues surrounding index builds pausing/resuming around a
      * call to 'recoverToStableTimestamp'. It's not clear that an index build, resumed in this way,
      * that continues until completion, would be consistent with the collection data. Waiting for
      * all background index builds to complete is a conservative approach, to avoid any of these
      * potential issues.
      */
-    Status _awaitBgIndexCompletion(OperationContext* opCtx);
+    void _stopAndWaitForIndexBuilds(OperationContext* opCtx);
 
     /**
      * Recovers to the stable timestamp while holding the global exclusive lock.
@@ -398,10 +392,10 @@ private:
     Status _findRecordStoreCounts(OperationContext* opCtx);
 
     /**
-     * Executes the critical section in rollback, defined as the window between aborting and
-     * reconstructing prepared transactions.
+     * Executes the phase of rollback between aborting and reconstructing prepared transactions. We
+     * cannot safely recover if we fail during this phase.
      */
-    Status _runRollbackCriticalSection(
+    void _runPhaseFromAbortToReconstructPreparedTxns(
         OperationContext* opCtx, RollBackLocalOperations::RollbackCommonPoint commonPoint) noexcept;
 
     /**
@@ -491,7 +485,7 @@ private:
     stdx::unordered_map<UUID, long long, UUID::Hash> _countDiffs;  // (N)
 
     // Maintains counts and namespaces of drop-pending collections.
-    using PendingDropInfo = struct {
+    struct PendingDropInfo {
         long long count = 0;
         NamespaceString nss;
     };
