@@ -27,18 +27,18 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/s/collection_metadata.h"
 
+#include <memory>
+
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/bson/util/builder.h"
 #include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/s/catalog/type_chunk.h"
-#include "mongo/stdx/memory.h"
-#include "mongo/util/log.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -69,7 +69,7 @@ BSONObj CollectionMetadata::extractDocumentKey(const BSONObj& doc) const {
 void CollectionMetadata::toBSONBasic(BSONObjBuilder& bb) const {
     if (isSharded()) {
         _cm->getVersion().appendLegacyWithField(&bb, "collVersion");
-        getShardVersion().appendLegacyWithField(&bb, "shardVersion");
+        getShardVersionForLogging().appendLegacyWithField(&bb, "shardVersion");
         bb.append("keyPattern", _cm->getShardKeyPattern().toBSON());
     } else {
         ChunkVersion::UNSHARDED().appendLegacyWithField(&bb, "collVersion");
@@ -80,13 +80,13 @@ void CollectionMetadata::toBSONBasic(BSONObjBuilder& bb) const {
 std::string CollectionMetadata::toStringBasic() const {
     if (isSharded()) {
         return str::stream() << "collection version: " << _cm->getVersion().toString()
-                             << ", shard version: " << getShardVersion().toString();
+                             << ", shard version: " << getShardVersionForLogging().toString();
     } else {
         return "collection version: <unsharded>";
     }
 }
 
-RangeMap CollectionMetadata::getChunks() const {
+RangeMap CollectionMetadata::getOwnedChunks() const {
     invariant(isSharded());
 
     RangeMap chunksMap(SimpleBSONObjComparator::kInstance.makeBSONObjIndexedMap<BSONObj>());
@@ -136,16 +136,15 @@ Status CollectionMetadata::checkChunkIsValid(const ChunkType& chunk) const {
 
     return Status::OK();
 }
-
 boost::optional<ChunkRange> CollectionMetadata::getNextOrphanRange(
-    const RangeMap& receivingChunks, const BSONObj& origLookupKey) const {
+    const RangeMap& chunksMap,
+    const RangeMap& receivingChunks,
+    const BSONObj& origLookupKey) const {
     invariant(isSharded());
 
     const BSONObj maxKey = getMaxKey();
     BSONObj lookupKey = origLookupKey;
 
-    auto chunksMap = getChunks();
-
     while (lookupKey.woCompare(maxKey) < 0) {
         using Its = std::pair<RangeMap::const_iterator, RangeMap::const_iterator>;
 
@@ -206,6 +205,12 @@ boost::optional<ChunkRange> CollectionMetadata::getNextOrphanRange(
     return boost::none;
 }
 
+boost::optional<ChunkRange> CollectionMetadata::getNextOrphanRange(
+    const RangeMap& receivingChunks, const BSONObj& origLookupKey) const {
+    invariant(isSharded());
+    return getNextOrphanRange(getOwnedChunks(), receivingChunks, origLookupKey);
+}
+
 void CollectionMetadata::toBSONChunks(BSONArrayBuilder* builder) const {
     if (!isSharded())
         return;
