@@ -90,6 +90,63 @@ const WriteConcernOptions kMajorityWriteConcern(WriteConcernOptions::kMajority,
                                                 WriteConcernOptions::SyncMode::UNSET,
                                                 -1);
 
+void checkOutSessionAndVerifyTxnState(OperationContext* opCtx) {
+    MongoDOperationContextSession::checkOut(opCtx);
+    TransactionParticipant::get(opCtx).beginOrContinue(opCtx,
+                                                       *opCtx->getTxnNumber(),
+                                                       boost::none /* autocommit */,
+                                                       boost::none /* startTransaction */);
+}
+
+template <typename Callable>
+constexpr bool returnsVoid() {
+    return std::is_void_v<std::invoke_result_t<Callable>>;
+}
+
+// Yields the checked out session before running the given function, if the migration is using the
+// 4.4 range deleter protocol. If the function runs without throwing, will reacquire the session and
+// verify it is still valid to proceed with the migration.
+template <typename Callable, std::enable_if_t<!returnsVoid<Callable>(), int> = 0>
+auto runWithoutSession(OperationContext* opCtx,
+                       Callable&& callable,
+                       bool useFCV44RangeDeleterProtocol) {
+    // A session is only checked out in the 4.4 protocol.
+    if (useFCV44RangeDeleterProtocol) {
+        MongoDOperationContextSession::checkIn(opCtx);
+    }
+
+    auto retVal = callable();
+
+    // The below code can throw, so it cannot run in a scope guard.
+    opCtx->checkForInterrupt();
+
+    if (useFCV44RangeDeleterProtocol) {
+        checkOutSessionAndVerifyTxnState(opCtx);
+    }
+
+    return retVal;
+}
+
+// Same as runWithoutSession above but takes a void function.
+template <typename Callable, std::enable_if_t<returnsVoid<Callable>(), int> = 0>
+void runWithoutSession(OperationContext* opCtx,
+                       Callable&& callable,
+                       bool useFCV44RangeDeleterProtocol) {
+    // A session is only checked out in the 4.4 protocol.
+    if (useFCV44RangeDeleterProtocol) {
+        MongoDOperationContextSession::checkIn(opCtx);
+    }
+
+    callable();
+
+    // The below code can throw, so it cannot run in a scope guard.
+    opCtx->checkForInterrupt();
+
+    if (useFCV44RangeDeleterProtocol) {
+        checkOutSessionAndVerifyTxnState(opCtx);
+    }
+}
+
 /**
  * Returns a human-readabale name of the migration manager's state.
  */
@@ -690,7 +747,7 @@ void MigrationDestinationManager::cloneCollectionIndexesAndOptions(
 
         // Checks that the collection's UUID matches the donor's.
         auto checkUUIDsMatch = [&](const Collection* collection) {
-            uassert(ErrorCodes::NotMaster,
+            uassert(ErrorCodes::NotWritablePrimary,
                     str::stream() << "Unable to create collection " << nss.ns()
                                   << " because the node is not primary",
                     repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, nss));
@@ -788,8 +845,10 @@ void MigrationDestinationManager::_migrateThread() {
         // duration of the recipient's side of the migration. This guarantees that if the
         // donor shard has failed over, then the new donor primary cannot bump the
         // txnNumber on this session while this node is still executing the recipient side
-        //(which is important because otherwise, this node may create orphans after the
-        // range deletion task on this node has been processed).
+        // (which is important because otherwise, this node may create orphans after the
+        // range deletion task on this node has been processed). The recipient will periodically
+        // yield this session, but will verify the txnNumber has not changed before continuing,
+        // preserving the guarantee that orphans cannot be created after the txnNumber is advanced.
         if (_useFCV44RangeDeleterProtocol) {
             opCtx->setLogicalSessionId(_lsid);
             opCtx->setTxnNumber(_txnNumber);
@@ -890,7 +949,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
                     outerOpCtx, _nss, donorCollectionOptionsAndIndexes.uuid, range);
 
                 if (!status.isOK()) {
-                    _setStateFail(redact(status.reason()));
+                    _setStateFail(redact(status.toString()));
                     return;
                 }
 
@@ -905,7 +964,24 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
                                                     CleanWhenEnum::kNow);
             recipientDeletionTask.setPending(true);
 
-            migrationutil::persistRangeDeletionTaskLocally(outerOpCtx, recipientDeletionTask);
+            // It is illegal to wait for write concern with a session checked out, so persist the
+            // range deletion task with an immediately satsifiable write concern and then wait for
+            // majority after yielding the session.
+            migrationutil::persistRangeDeletionTaskLocally(
+                outerOpCtx, recipientDeletionTask, WriteConcernOptions());
+
+            runWithoutSession(
+                outerOpCtx,
+                [&] {
+                    WriteConcernResult ignoreResult;
+                    auto latestOpTime =
+                        repl::ReplClientInfo::forClient(outerOpCtx->getClient()).getLastOp();
+                    uassertStatusOK(waitForWriteConcern(outerOpCtx,
+                                                        latestOpTime,
+                                                        WriteConcerns::kMajorityWriteConcern,
+                                                        &ignoreResult));
+                },
+                _useFCV44RangeDeleterProtocol);
         } else {
             // Synchronously delete any data which might have been left orphaned in the range
             // being moved, and wait for completion
@@ -922,7 +998,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
             if (!cleanupStatus.isOK() &&
                 cleanupStatus !=
                     ErrorCodes::RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist) {
-                _setStateFail(redact(cleanupStatus.reason()));
+                _setStateFail(redact(cleanupStatus.toString()));
                 return;
             }
 
@@ -976,6 +1052,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
 
         auto assertNotAborted = [&](OperationContext* opCtx) {
             opCtx->checkForInterrupt();
+            outerOpCtx->checkForInterrupt();
             uassert(50748, "Migration aborted while copying documents", getState() != ABORT);
         };
 
@@ -1020,20 +1097,26 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
                     _clonedBytes += batchClonedBytes;
                 }
                 if (_writeConcern.needToWaitForOtherNodes()) {
-                    repl::ReplicationCoordinator::StatusAndDuration replStatus =
-                        repl::ReplicationCoordinator::get(opCtx)->awaitReplication(
-                            opCtx,
-                            repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp(),
-                            _writeConcern);
-                    if (replStatus.status.code() == ErrorCodes::WriteConcernFailed) {
-                        LOGV2_WARNING(22011,
-                                      "secondaryThrottle on, but doc insert timed out; continuing",
-                                      "migrationId"_attr = _useFCV44RangeDeleterProtocol
-                                          ? _migrationId->toBSON()
-                                          : BSONObj());
-                    } else {
-                        uassertStatusOK(replStatus.status);
-                    }
+                    runWithoutSession(
+                        outerOpCtx,
+                        [&] {
+                            repl::ReplicationCoordinator::StatusAndDuration replStatus =
+                                repl::ReplicationCoordinator::get(opCtx)->awaitReplication(
+                                    opCtx,
+                                    repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp(),
+                                    _writeConcern);
+                            if (replStatus.status.code() == ErrorCodes::WriteConcernFailed) {
+                                LOGV2_WARNING(
+                                    22011,
+                                    "secondaryThrottle on, but doc insert timed out; continuing",
+                                    "migrationId"_attr = _useFCV44RangeDeleterProtocol
+                                        ? _migrationId->toBSON()
+                                        : BSONObj());
+                            } else {
+                                uassertStatusOK(replStatus.status);
+                            }
+                        },
+                        _useFCV44RangeDeleterProtocol);
                 }
 
                 sleepmillis(migrateCloneInsertionBatchDelayMS.load());
@@ -1102,6 +1185,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
             int i;
             for (i = 0; i < maxIterations; i++) {
                 opCtx->checkForInterrupt();
+                outerOpCtx->checkForInterrupt();
 
                 if (getState() == ABORT) {
                     LOGV2(22002,
@@ -1111,8 +1195,12 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
                     return;
                 }
 
-                if (opReplicatedEnough(opCtx, lastOpApplied, _writeConcern))
+                if (runWithoutSession(
+                        outerOpCtx,
+                        [&] { return opReplicatedEnough(opCtx, lastOpApplied, _writeConcern); },
+                        _useFCV44RangeDeleterProtocol)) {
                     break;
+                }
 
                 if (i > 100) {
                     LOGV2(22003,
@@ -1143,10 +1231,16 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
               "migrationId"_attr =
                   _useFCV44RangeDeleterProtocol ? _migrationId->toBSON() : BSONObj());
 
-        auto awaitReplicationResult = repl::ReplicationCoordinator::get(opCtx)->awaitReplication(
-            opCtx, lastOpApplied, _writeConcern);
-        uassertStatusOKWithContext(awaitReplicationResult.status,
-                                   awaitReplicationResult.status.codeString());
+        runWithoutSession(outerOpCtx,
+                          [&] {
+                              auto awaitReplicationResult =
+                                  repl::ReplicationCoordinator::get(opCtx)->awaitReplication(
+                                      opCtx, lastOpApplied, _writeConcern);
+                              uassertStatusOKWithContext(
+                                  awaitReplicationResult.status,
+                                  awaitReplicationResult.status.codeString());
+                          },
+                          _useFCV44RangeDeleterProtocol);
 
         LOGV2(22005,
               "Chunk data replicated successfully.",
@@ -1161,6 +1255,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
         bool transferAfterCommit = false;
         while (getState() == STEADY || getState() == COMMIT_START) {
             opCtx->checkForInterrupt();
+            outerOpCtx->checkForInterrupt();
 
             // Make sure we do at least one transfer after recv'ing the commit message. If we
             // aren't sure that at least one transfer happens *after* our state changes to
@@ -1199,7 +1294,9 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
             // 1) The from side has told us that it has locked writes (COMMIT_START)
             // 2) We've checked at least one more time for un-transmitted mods
             if (getState() == COMMIT_START && transferAfterCommit == true) {
-                if (_flushPendingWrites(opCtx, lastOpApplied)) {
+                if (runWithoutSession(outerOpCtx,
+                                      [&] { return _flushPendingWrites(opCtx, lastOpApplied); },
+                                      _useFCV44RangeDeleterProtocol)) {
                     break;
                 }
             }
@@ -1218,7 +1315,8 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
         migrateThreadHangAtStep5.pauseWhileSet();
     }
 
-    _sessionMigration->join();
+    runWithoutSession(
+        outerOpCtx, [&] { _sessionMigration->join(); }, _useFCV44RangeDeleterProtocol);
     if (_sessionMigration->getState() == SessionCatalogMigrationDestination::State::ErrorOccurred) {
         _setStateFail(redact(_sessionMigration->getErrMsg()));
         return;
