@@ -29,8 +29,10 @@
 
 #pragma once
 
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/repl/optime.h"
 #include "mongo/db/s/collection_metadata.h"
+#include "mongo/db/s/collection_sharding_runtime.h"
 #include "mongo/db/s/migration_coordinator_document_gen.h"
 #include "mongo/db/s/persistent_task_store.h"
 #include "mongo/db/s/range_deletion_task_gen.h"
@@ -138,7 +140,8 @@ void persistMigrationCoordinatorLocally(OperationContext* opCtx,
  * concern.
  */
 void persistRangeDeletionTaskLocally(OperationContext* opCtx,
-                                     const RangeDeletionTask& deletionTask);
+                                     const RangeDeletionTask& deletionTask,
+                                     const WriteConcernOptions& writeConcern);
 
 /**
  * Updates the migration coordinator document to set the decision field to "committed" and waits for
@@ -221,5 +224,34 @@ void refreshFilteringMetadataUntilSuccess(OperationContext* opCtx, const Namespa
  */
 void resumeMigrationCoordinationsOnStepUp(OperationContext* opCtx);
 
+template <typename Callable>
+void forEachOrphanRange(OperationContext* opCtx, const NamespaceString& nss, Callable&& handler) {
+    AutoGetCollection autoColl(opCtx, nss, MODE_IX);
+
+    const auto css = CollectionShardingRuntime::get(opCtx, nss);
+    const auto collDesc = css->getCollectionDescription();
+    if (!collDesc.isSharded()) {
+        return;
+    }
+
+    auto startingKey = collDesc.getMinKey();
+    const auto ownedChunks = collDesc->getOwnedChunks();
+    const auto emptyReceivingChunks =
+        RangeMap{SimpleBSONObjComparator::kInstance.makeBSONObjIndexedMap<BSONObj>()};
+
+    while (true) {
+        opCtx->checkForInterrupt();
+        auto range = collDesc->getNextOrphanRange(ownedChunks, emptyReceivingChunks, startingKey);
+
+        if (!range) {
+            return;
+        }
+
+        handler(*range);
+
+        startingKey = range->getMax();
+    }
+}
+
 }  // namespace migrationutil
 }  // namespace mongo
