@@ -27,12 +27,14 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/s/session_catalog_migration_destination.h"
 
+#include <functional>
+
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/client/connection_string.h"
 #include "mongo/db/concurrency/d_concurrency.h"
@@ -44,11 +46,10 @@
 #include "mongo/db/session_catalog_mongod.h"
 #include "mongo/db/transaction_participant.h"
 #include "mongo/db/write_concern.h"
+#include "mongo/logv2/redaction.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/shard_id.h"
-#include "mongo/stdx/functional.h"
-#include "mongo/util/log.h"
 
 namespace mongo {
 namespace {
@@ -78,66 +79,52 @@ BSONObj buildMigrateSessionCmd(const MigrationSessionId& migrationSessionId) {
 }
 
 /**
- * Determines whether the oplog entry has a link to either preImage/postImage and return a new
- * oplogLink that contains the same link, but pointing to lastResult.oplogTime. For example, if
- * entry has link to preImageTs, this returns an oplogLink with preImageTs pointing to
+ * Determines whether the oplog entry has a link to either preImage/postImage and sets a new link
+ * to lastResult.oplogTime. For example, if entry has link to preImageTs, this sets preImageTs to
  * lastResult.oplogTime.
  *
  * It is an error to have both preImage and postImage as well as not having them at all.
  */
-repl::OplogLink extractPrePostImageTs(const ProcessOplogResult& lastResult,
-                                      const repl::OplogEntry& entry) {
-    repl::OplogLink oplogLink;
-
+void setPrePostImageTs(const ProcessOplogResult& lastResult, repl::MutableOplogEntry* entry) {
     if (!lastResult.isPrePostImage) {
         uassert(40628,
-                str::stream() << "expected oplog with ts: " << entry.getTimestamp().toString()
+                str::stream() << "expected oplog with ts: " << entry->getTimestamp().toString()
                               << " to not have " << repl::OplogEntryBase::kPreImageOpTimeFieldName
                               << " or " << repl::OplogEntryBase::kPostImageOpTimeFieldName,
-                !entry.getPreImageOpTime() && !entry.getPostImageOpTime());
-
-        return oplogLink;
+                !entry->getPreImageOpTime() && !entry->getPostImageOpTime());
+        return;
     }
 
     invariant(!lastResult.oplogTime.isNull());
 
-    const auto& sessionInfo = entry.getOperationSessionInfo();
-    const auto sessionId = *sessionInfo.getSessionId();
-    const auto txnNum = *sessionInfo.getTxnNumber();
-
     uassert(40629,
-            str::stream() << "expected oplog with ts: " << entry.getTimestamp().toString() << ": "
-                          << redact(entry.toBSON()) << " to have session: " << lastResult.sessionId,
-            lastResult.sessionId == sessionId);
+            str::stream() << "expected oplog with ts: " << entry->getTimestamp().toString() << ": "
+                          << redact(entry->toBSON())
+                          << " to have session: " << lastResult.sessionId,
+            lastResult.sessionId == entry->getSessionId());
     uassert(40630,
-            str::stream() << "expected oplog with ts: " << entry.getTimestamp().toString() << ": "
-                          << redact(entry.toBSON()) << " to have txnNumber: " << lastResult.txnNum,
-            lastResult.txnNum == txnNum);
+            str::stream() << "expected oplog with ts: " << entry->getTimestamp().toString() << ": "
+                          << redact(entry->toBSON()) << " to have txnNumber: " << lastResult.txnNum,
+            lastResult.txnNum == entry->getTxnNumber());
 
-    if (entry.getPreImageOpTime()) {
-        oplogLink.preImageOpTime = lastResult.oplogTime;
-    } else if (entry.getPostImageOpTime()) {
-        oplogLink.postImageOpTime = lastResult.oplogTime;
+    if (entry->getPreImageOpTime()) {
+        entry->setPreImageOpTime(lastResult.oplogTime);
+    } else if (entry->getPostImageOpTime()) {
+        entry->setPostImageOpTime(lastResult.oplogTime);
     } else {
         uasserted(40631,
-                  str::stream() << "expected oplog with opTime: " << entry.getOpTime().toString()
-                                << ": " << redact(entry.toBSON()) << " to have either "
+                  str::stream() << "expected oplog with opTime: " << entry->getOpTime().toString()
+                                << ": " << redact(entry->toBSON()) << " to have either "
                                 << repl::OplogEntryBase::kPreImageOpTimeFieldName << " or "
                                 << repl::OplogEntryBase::kPostImageOpTimeFieldName);
     }
-
-    return oplogLink;
 }
 
 /**
  * Parses the oplog into an oplog entry and makes sure that it contains the expected fields.
  */
-repl::OplogEntry parseOplog(const BSONObj& oplogBSON) {
-    auto oplogEntry = uassertStatusOK(repl::OplogEntry::parse(oplogBSON));
-
-    // Session oplog entries must always contain wall clock time, because we will not be
-    // transferring anything from a previous version of the server
-    invariant(oplogEntry.getWallClockTime());
+repl::MutableOplogEntry parseOplog(const BSONObj& oplogBSON) {
+    auto oplogEntry = uassertStatusOK(repl::MutableOplogEntry::parse(oplogBSON));
 
     const auto& sessionInfo = oplogEntry.getOperationSessionInfo();
 
@@ -195,13 +182,11 @@ BSONObj getNextSessionOplogBatch(OperationContext* opCtx,
 ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
                                        const ProcessOplogResult& lastResult) {
     auto oplogEntry = parseOplog(oplogBSON);
-    const auto& sessionInfo = oplogEntry.getOperationSessionInfo();
 
     ProcessOplogResult result;
-    result.sessionId = *sessionInfo.getSessionId();
-    result.txnNum = *sessionInfo.getTxnNumber();
+    result.sessionId = *oplogEntry.getSessionId();
+    result.txnNum = *oplogEntry.getTxnNumber();
 
-    BSONObj object2;
     if (oplogEntry.getOpType() == repl::OpTypeEnum::kNoop) {
         // Note: Oplog is already no-op type, no need to nest.
         // There are two types of type 'n' oplog format expected here:
@@ -212,8 +197,11 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
         //     findAndModify operation. In this case, o field contains the relevant info
         //     and o2 will be empty.
 
+        BSONObj object2;
         if (oplogEntry.getObject2()) {
             object2 = *oplogEntry.getObject2();
+        } else {
+            oplogEntry.setObject2(object2);
         }
 
         if (object2.isEmpty()) {
@@ -227,7 +215,7 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
                     !lastResult.isPrePostImage);
         }
     } else {
-        object2 = oplogBSON;  // TODO: strip redundant info?
+        oplogEntry.setObject2(oplogBSON);  // TODO: strip redundant info?
     }
 
     const auto stmtId = *oplogEntry.getStatementId();
@@ -260,11 +248,16 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
         throw;
     }
 
-    BSONObj object(result.isPrePostImage
-                       ? oplogEntry.getObject()
-                       : BSON(SessionCatalogMigrationDestination::kSessionMigrateOplogTag << 1));
-    auto oplogLink = extractPrePostImageTs(lastResult, oplogEntry);
-    oplogLink.prevOpTime = txnParticipant.getLastWriteOpTime();
+    if (!result.isPrePostImage)
+        oplogEntry.setObject(
+            BSON(SessionCatalogMigrationDestination::kSessionMigrateOplogTag << 1));
+    setPrePostImageTs(lastResult, &oplogEntry);
+    oplogEntry.setPrevWriteOpTimeInTransaction(txnParticipant.getLastWriteOpTime());
+
+    oplogEntry.setOpType(repl::OpTypeEnum::kNoop);
+    oplogEntry.setFromMigrate(true);
+    // Reset OpTime so logOp() can assign a new one.
+    oplogEntry.setOpTime(OplogSlot());
 
     writeConflictRetry(
         opCtx,
@@ -280,18 +273,7 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
                 opCtx, NamespaceString::kSessionTransactionsTableNamespace.db(), MODE_IX);
             WriteUnitOfWork wunit(opCtx);
 
-            result.oplogTime = repl::logOp(opCtx,
-                                           "n",
-                                           oplogEntry.getNss(),
-                                           oplogEntry.getUuid(),
-                                           object,
-                                           &object2,
-                                           true,
-                                           *oplogEntry.getWallClockTime(),
-                                           sessionInfo,
-                                           stmtId,
-                                           oplogLink,
-                                           OplogSlot());
+            result.oplogTime = repl::logOp(opCtx, &oplogEntry);
 
             const auto& oplogOpTime = result.oplogTime;
             uassert(40633,
@@ -306,7 +288,7 @@ ProcessOplogResult processSessionOplog(const BSONObj& oplogBSON,
                 sessionTxnRecord.setSessionId(result.sessionId);
                 sessionTxnRecord.setTxnNum(result.txnNum);
                 sessionTxnRecord.setLastWriteOpTime(oplogOpTime);
-                sessionTxnRecord.setLastWriteDate(*oplogEntry.getWallClockTime());
+                sessionTxnRecord.setLastWriteDate(oplogEntry.getWallClockTime());
                 // We do not migrate transaction oplog entries so don't set the txn state.
                 txnParticipant.onMigrateCompletedOnPrimary(opCtx, {stmtId}, sessionTxnRecord);
             }
