@@ -69,6 +69,8 @@ public:
      */
     void onStepUp() override;
 
+    void shutDown() override;
+
     /**
      * Sets any notifications waiting for this version to arrive and invalidates the catalog cache's
      * chunk metadata for collection 'nss' so that the next caller provokes a refresh.
@@ -82,7 +84,7 @@ public:
 
     void getDatabase(
         StringData dbName,
-        stdx::function<void(OperationContext*, StatusWith<DatabaseType>)> callbackFn) override;
+        std::function<void(OperationContext*, StatusWith<DatabaseType>)> callbackFn) override;
 
     void waitForCollectionFlush(OperationContext* opCtx, const NamespaceString& nss) override;
 
@@ -342,7 +344,7 @@ private:
         OperationContext* opCtx,
         const NamespaceString& nss,
         const ChunkVersion& catalogCacheSinceVersion,
-        stdx::function<void(OperationContext*, StatusWith<CollectionAndChangedChunks>)> callbackFn,
+        std::function<void(OperationContext*, StatusWith<CollectionAndChangedChunks>)> callbackFn,
         std::shared_ptr<Notification<void>> notify);
 
     /**
@@ -361,7 +363,7 @@ private:
         const NamespaceString& nss,
         const ChunkVersion& catalogCacheSinceVersion,
         long long currentTerm,
-        stdx::function<void(OperationContext*, StatusWith<CollectionAndChangedChunks>)> callbackFn,
+        std::function<void(OperationContext*, StatusWith<CollectionAndChangedChunks>)> callbackFn,
         std::shared_ptr<Notification<void>> notify);
 
     /**
@@ -373,7 +375,7 @@ private:
     void _runSecondaryGetDatabase(
         OperationContext* opCtx,
         StringData dbName,
-        stdx::function<void(OperationContext*, StatusWith<DatabaseType>)> callbackFn);
+        std::function<void(OperationContext*, StatusWith<DatabaseType>)> callbackFn);
 
     /**
      * Refreshes db version from the config server's metadata store, and schedules maintenance
@@ -389,7 +391,7 @@ private:
         OperationContext* opCtx,
         StringData dbName,
         long long termScheduled,
-        stdx::function<void(OperationContext*, StatusWith<DatabaseType>)> callbackFn);
+        std::function<void(OperationContext*, StatusWith<DatabaseType>)> callbackFn);
 
     /**
      * Loads chunk metadata from the shard persisted metadata store and any in-memory tasks with
@@ -473,7 +475,7 @@ private:
 
     // Loader used by the shard primary to retrieve the authoritative routing metadata from the
     // config server
-    const std::unique_ptr<CatalogCacheLoader> _configServerLoader;
+    std::unique_ptr<CatalogCacheLoader> _configServerLoader;
 
     // Thread pool used to run blocking tasks which perform disk reads and writes
     ThreadPool _threadPool;
@@ -484,6 +486,9 @@ private:
     // Protects the class state below
     Mutex _mutex = MONGO_MAKE_LATCH("ShardServerCatalogCacheLoader::_mutex");
 
+    // True if shutDown was called.
+    bool _inShutdown{false};
+
     // This value is bumped every time the set of currently scheduled tasks should no longer be
     // running. This includes, replica set state transitions and shutdown.
     long long _term{0};
