@@ -32,6 +32,7 @@
 #include <string>
 
 #include "mongo/client/connection_string.h"
+#include "mongo/db/logical_session_id.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/s/migration_session_id.h"
 #include "mongo/s/request_types/migration_secondary_throttle_options.h"
@@ -59,6 +60,21 @@ public:
      * builder, without closing the builder. The builder must be empty, but callers are free to
      * append more fields once the command has been constructed.
      */
+    static void appendAsCommand(BSONObjBuilder* builder,
+                                const NamespaceString& nss,
+                                const UUID& migrationId,
+                                const LogicalSessionId& lsid,
+                                TxnNumber txnNumber,
+                                const MigrationSessionId& sessionId,
+                                const ConnectionString& fromShardConnectionString,
+                                const ShardId& fromShardId,
+                                const ShardId& toShardId,
+                                const BSONObj& chunkMinKey,
+                                const BSONObj& chunkMaxKey,
+                                const BSONObj& shardKeyPattern,
+                                const MigrationSecondaryThrottleOptions& secondaryThrottle);
+
+    // TODO (SERVER-44787): Remove this overload after 4.4 is released.
     static void appendAsCommand(BSONObjBuilder* builder,
                                 const NamespaceString& nss,
                                 const MigrationSessionId& sessionId,
@@ -82,6 +98,28 @@ public:
         return _fromShardCS;
     }
 
+    // TODO (SERVER-44787): Remove this function after 4.4 is released.
+    // Use this check so that getMigrationId() is never called in a cluster that's not fully
+    // upgraded to 4.4.
+    bool hasMigrationId() const {
+        return _migrationId.is_initialized();
+    }
+
+    const UUID& getMigrationId() const {
+        // TODO (SERVER-44787): change _migrationId to non-optional and remove invariant after 4.4
+        // is released.
+        invariant(_migrationId);
+        return *_migrationId;
+    }
+
+    const LogicalSessionId& getLsid() const {
+        return _lsid;
+    }
+
+    const TxnNumber getTxnNumber() const {
+        return _txnNumber;
+    }
+
     const ShardId& getFromShardId() const {
         return _fromShardId;
     }
@@ -114,6 +152,16 @@ private:
     // The collection for which this request applies
     NamespaceString _nss;
 
+    /**
+     *  Migration UUID generated by the donor. If the donor is running a 4.2 binary, this will not
+     *  be included in the request, so it's optional.
+     *
+     *  TODO (SERVER-44787): Make non-optional after 4.4 is released.
+     */
+    boost::optional<UUID> _migrationId;
+    LogicalSessionId _lsid;
+    TxnNumber _txnNumber;
+
     // The session id of this migration
     MigrationSessionId _sessionId;
 
