@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
@@ -54,8 +54,9 @@
 #include "mongo/db/server_options_nongeneral_gen.h"
 #include "mongo/db/server_options_server_helpers.h"
 #include "mongo/logger/logger.h"
+#include "mongo/unittest/log_test.h"
 #include "mongo/unittest/unittest.h"
-#include "mongo/util/log.h"
+#include "mongo/util/errno_util.h"
 #include "mongo/util/options_parser/environment.h"
 #include "mongo/util/options_parser/option_section.h"
 #include "mongo/util/options_parser/options_parser.h"
@@ -66,6 +67,10 @@ namespace {
 
 using mongo::ErrorCodes;
 using mongo::Status;
+
+using mongo::unittest::getMinimumLogSeverity;
+using mongo::unittest::hasMinimumLogSeverity;
+
 namespace moe = mongo::optionenvironment;
 
 MONGO_INITIALIZER(ServerLogRedirection)(mongo::InitializerContext*) {
@@ -96,15 +101,17 @@ private:
     std::string _config;
 };
 
-TEST(Verbosity, Default) {
+class Verbosity : public mongo::unittest::Test {
+    /** Reset the log level before we test */
+    mongo::unittest::MinimumLoggedSeverityGuard _severityGuard{mongo::logv2::LogComponent::kDefault,
+                                                               mongo::logv2::LogSeverity::Info()};
+};
+
+TEST_F(Verbosity, Default) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -119,19 +126,14 @@ TEST(Verbosity, Default) {
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
     // Make sure the log level didn't change since we didn't specify any verbose options
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(),
-                  ::mongo::logger::LogSeverity::Info());
+    ASSERT_EQUALS(getMinimumLogSeverity(), ::mongo::logv2::LogSeverity::Info());
 }
 
-TEST(Verbosity, CommandLineImplicit) {
+TEST_F(Verbosity, CommandLineImplicit) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -147,19 +149,14 @@ TEST(Verbosity, CommandLineImplicit) {
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
     int verbosity = 1;
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
+    ASSERT_EQUALS(getMinimumLogSeverity(), ::mongo::logv2::LogSeverity::Debug(verbosity));
 }
 
-TEST(Verbosity, CommandLineString) {
+TEST_F(Verbosity, CommandLineString) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -176,19 +173,14 @@ TEST(Verbosity, CommandLineString) {
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
     int verbosity = 4;
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
+    ASSERT_EQUALS(getMinimumLogSeverity(), ::mongo::logv2::LogSeverity::Debug(verbosity));
 }
 
-TEST(Verbosity, CommandLineStringDisguisedLongForm) {
+TEST_F(Verbosity, CommandLineStringDisguisedLongForm) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -205,19 +197,14 @@ TEST(Verbosity, CommandLineStringDisguisedLongForm) {
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
     int verbosity = 4;
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
+    ASSERT_EQUALS(getMinimumLogSeverity(), ::mongo::logv2::LogSeverity::Debug(verbosity));
 }
 
-TEST(Verbosity, CommandLineEmptyString) {
+TEST_F(Verbosity, CommandLineEmptyString) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -229,15 +216,11 @@ TEST(Verbosity, CommandLineEmptyString) {
     ASSERT_NOT_OK(parser.run(options, argv, env_map, &environment));
 }
 
-TEST(Verbosity, CommandLineBadString) {
+TEST_F(Verbosity, CommandLineBadString) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -251,15 +234,11 @@ TEST(Verbosity, CommandLineBadString) {
     ASSERT_NOT_OK(::mongo::validateServerOptions(environment));
 }
 
-TEST(Verbosity, CommandLineBadStringOnlyDash) {
+TEST_F(Verbosity, CommandLineBadStringOnlyDash) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -270,15 +249,11 @@ TEST(Verbosity, CommandLineBadStringOnlyDash) {
     ASSERT_NOT_OK(parser.run(options, argv, env_map, &environment));
 }
 
-TEST(Verbosity, CommandLineBadStringOnlyTwoDashes) {
+TEST_F(Verbosity, CommandLineBadStringOnlyTwoDashes) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -289,15 +264,11 @@ TEST(Verbosity, CommandLineBadStringOnlyTwoDashes) {
     ASSERT_OK(parser.run(options, argv, env_map, &environment));
 }
 
-TEST(Verbosity, INIConfigString) {
+TEST_F(Verbosity, INIConfigString) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -316,19 +287,14 @@ TEST(Verbosity, INIConfigString) {
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
     int verbosity = 4;
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
+    ASSERT_EQUALS(getMinimumLogSeverity(), ::mongo::logv2::LogSeverity::Debug(verbosity));
 }
 
-TEST(Verbosity, INIConfigBadString) {
+TEST_F(Verbosity, INIConfigBadString) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -344,15 +310,11 @@ TEST(Verbosity, INIConfigBadString) {
     ASSERT_NOT_OK(::mongo::validateServerOptions(environment));
 }
 
-TEST(Verbosity, INIConfigEmptyString) {
+TEST_F(Verbosity, INIConfigEmptyString) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -371,19 +333,14 @@ TEST(Verbosity, INIConfigEmptyString) {
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
     int verbosity = 0;
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
+    ASSERT_EQUALS(getMinimumLogSeverity(), ::mongo::logv2::LogSeverity::Debug(verbosity));
 }
 
-TEST(Verbosity, JSONConfigString) {
+TEST_F(Verbosity, JSONConfigString) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -402,19 +359,14 @@ TEST(Verbosity, JSONConfigString) {
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
     int verbosity = 4;
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
+    ASSERT_EQUALS(getMinimumLogSeverity(), ::mongo::logv2::LogSeverity::Debug(verbosity));
 }
 
-TEST(Verbosity, MultipleSourcesMultipleOptions) {
+TEST_F(Verbosity, MultipleSourcesMultipleOptions) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
-
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
     std::vector<std::string> argv;
@@ -435,21 +387,17 @@ TEST(Verbosity, MultipleSourcesMultipleOptions) {
     ASSERT_OK(::mongo::storeServerOptions(environment));
 
     int verbosity = 3;
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
+    ASSERT_EQUALS(getMinimumLogSeverity(), ::mongo::logv2::LogSeverity::Debug(verbosity));
 }
 
-TEST(Verbosity, YAMLConfigStringLogComponent) {
+TEST_F(Verbosity, YAMLConfigStringLogComponent) {
     OptionsParserTester parser;
     moe::Environment environment;
     moe::OptionSection options;
 
-    // Reset the log level before we test
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogSeverity::Info());
     // Log level for Storage will be cleared by config file value.
-    ::mongo::logger::globalLogDomain()->setMinimumLoggedSeverity(
-        ::mongo::logger::LogComponent::kStorage, ::mongo::logger::LogSeverity::Debug(1));
+    auto storageSeverityGuard = mongo::unittest::MinimumLoggedSeverityGuard{
+        mongo::logv2::LogComponent::kStorage, mongo::logv2::LogSeverity::Debug(1)};
 
     ASSERT_OK(::mongo::addGeneralServerOptions(&options));
 
@@ -481,39 +429,29 @@ TEST(Verbosity, YAMLConfigStringLogComponent) {
     int verbosity = 4;
 
     // Default
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(
-                      ::mongo::logger::LogComponent::kDefault),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
+    ASSERT_EQUALS(getMinimumLogSeverity(), ::mongo::logv2::LogSeverity::Debug(verbosity));
+    ASSERT_EQUALS(getMinimumLogSeverity(::mongo::logv2::LogComponent::kDefault),
+                  ::mongo::logv2::LogSeverity::Debug(verbosity));
 
     // AccessControl
-    ASSERT_TRUE(::mongo::logger::globalLogDomain()->hasMinimumLogSeverity(
-        ::mongo::logger::LogComponent::kAccessControl));
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(
-                      ::mongo::logger::LogComponent::kAccessControl),
-                  ::mongo::logger::LogSeverity::Log());
+    ASSERT_TRUE(hasMinimumLogSeverity(::mongo::logv2::LogComponent::kAccessControl));
+    ASSERT_EQUALS(getMinimumLogSeverity(::mongo::logv2::LogComponent::kAccessControl),
+                  ::mongo::logv2::LogSeverity::Log());
 
     // Query - not mentioned in configuration. should match default.
-    ASSERT_FALSE(::mongo::logger::globalLogDomain()->hasMinimumLogSeverity(
-        ::mongo::logger::LogComponent::kStorage));
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(
-                      ::mongo::logger::LogComponent::kStorage),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
+    ASSERT_FALSE(hasMinimumLogSeverity(::mongo::logv2::LogComponent::kStorage));
+    ASSERT_EQUALS(getMinimumLogSeverity(::mongo::logv2::LogComponent::kStorage),
+                  ::mongo::logv2::LogSeverity::Debug(verbosity));
 
     // Storage - cleared by -1 value in configuration. should match default.
-    ASSERT_FALSE(::mongo::logger::globalLogDomain()->hasMinimumLogSeverity(
-        ::mongo::logger::LogComponent::kStorage));
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(
-                      ::mongo::logger::LogComponent::kStorage),
-                  ::mongo::logger::LogSeverity::Debug(verbosity));
+    ASSERT_FALSE(hasMinimumLogSeverity(::mongo::logv2::LogComponent::kStorage));
+    ASSERT_EQUALS(getMinimumLogSeverity(::mongo::logv2::LogComponent::kStorage),
+                  ::mongo::logv2::LogSeverity::Debug(verbosity));
 
     // Journaling - explicitly set to 2 in configuration.
-    ASSERT_TRUE(::mongo::logger::globalLogDomain()->hasMinimumLogSeverity(
-        ::mongo::logger::LogComponent::kJournal));
-    ASSERT_EQUALS(::mongo::logger::globalLogDomain()->getMinimumLogSeverity(
-                      ::mongo::logger::LogComponent::kJournal),
-                  ::mongo::logger::LogSeverity::Debug(2));
+    ASSERT_TRUE(hasMinimumLogSeverity(::mongo::logv2::LogComponent::kJournal));
+    ASSERT_EQUALS(getMinimumLogSeverity(::mongo::logv2::LogComponent::kJournal),
+                  ::mongo::logv2::LogSeverity::Debug(2));
 }
 
 TEST(SetupOptions, Default) {
