@@ -62,13 +62,26 @@
 #include "mongo/platform/overflow_arithmetic.h"
 #include "mongo/s/is_mongos.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/bufreader.h"
 #include "mongo/util/destructor_guard.h"
 #include "mongo/util/str.h"
 #include "mongo/util/unowned_ptr.h"
 
 namespace mongo {
 
+namespace {
+
+/**
+ * Calculates and returns a new murmur hash value based on the prior murmur hash and a new piece
+ * of data.
+ */
+uint32_t addDataToChecksum(const void* startOfData, size_t sizeOfData, uint32_t checksum) {
+    unsigned newChecksum;
+    MurmurHash3_x86_32(startOfData, sizeOfData, checksum, &newChecksum);
+    return newChecksum;
+}
+
+}  // namespace
+
 namespace sorter {
 
 using std::shared_ptr;
@@ -129,7 +142,7 @@ public:
         return !_data.empty();
     }
     Data next() {
-        Data out = _data.front();
+        Data out = std::move(_data.front());
         _data.pop_front();
         return out;
     }
@@ -158,12 +171,14 @@ public:
     FileIterator(const std::string& fileName,
                  std::streampos fileStartOffset,
                  std::streampos fileEndOffset,
-                 const Settings& settings)
+                 const Settings& settings,
+                 const uint32_t checksum)
         : _settings(settings),
           _done(false),
           _fileName(fileName),
           _fileStartOffset(fileStartOffset),
-          _fileEndOffset(fileEndOffset) {
+          _fileEndOffset(fileEndOffset),
+          _originalChecksum(checksum) {
         uassert(16815,
                 str::stream() << "unexpected empty file: " << _fileName,
                 boost::filesystem::file_size(_fileName) != 0);
@@ -188,6 +203,18 @@ public:
                 str::stream() << "error closing file \"" << _fileName
                               << "\": " << myErrnoWithDescription(),
                 !_file.fail());
+
+        // If the file iterator reads through all data objects, we can ensure non-corrupt data
+        // by comparing the newly calculated checksum with the original checksum from the data
+        // written to disk. Some iterators do not read back all data from the file, which prohibits
+        // the _afterReadChecksum from obtaining all the information needed. Thus, we only fassert
+        // if all data that was written to disk is read back and the checksums are not equivalent.
+        if (_done && _bufferReader->atEof() && (_originalChecksum != _afterReadChecksum)) {
+            fassert(31182,
+                    Status(ErrorCodes::Error::ChecksumMismatch,
+                           "Data read from disk does not match what was written to disk. Possible "
+                           "corruption of data."));
+        }
     }
 
     bool more() {
@@ -200,12 +227,22 @@ public:
         verify(!_done);
         fillBufferIfNeeded();
 
+        const char* startOfNewData = static_cast<const char*>(_bufferReader->pos());
+
         // Note: calling read() on the _bufferReader buffer in the deserialize function advances the
         // buffer. Since Key comes before Value in the _bufferReader, and C++ makes no function
         // parameter evaluation order guarantees, we cannot deserialize Key and Value straight into
         // the Data constructor
         auto first = Key::deserializeForSorter(*_bufferReader, _settings.first);
         auto second = Value::deserializeForSorter(*_bufferReader, _settings.second);
+
+        // The difference of _bufferReader's position before and after reading the data
+        // will provide the length of the data that was just read.
+        const char* endOfNewData = static_cast<const char*>(_bufferReader->pos());
+
+        _afterReadChecksum =
+            addDataToChecksum(startOfNewData, endOfNewData - startOfNewData, _afterReadChecksum);
+
         return Data(std::move(first), std::move(second));
     }
 
@@ -307,12 +344,23 @@ private:
 
     const Settings _settings;
     bool _done;
+
     std::unique_ptr<char[]> _buffer;
     std::unique_ptr<BufReader> _bufferReader;
     std::string _fileName;            // File containing the sorted data range.
     std::streampos _fileStartOffset;  // File offset at which the sorted data range starts.
     std::streampos _fileEndOffset;    // File offset at which the sorted data range ends.
     std::ifstream _file;
+
+    // Checksum value that is updated with each read of a data object from disk. We can compare
+    // this value with _originalChecksum to check for data corruption if and only if the
+    // FileIterator is exhausted.
+    uint32_t _afterReadChecksum = 0;
+
+    // Checksum value retrieved from SortedFileWriter that was calculated as data was spilled
+    // to disk. This is not modified, and is only used for comparison against _afterReadChecksum
+    // when the FileIterator is exhausted to ensure no data corruption.
+    const uint32_t _originalChecksum;
 };
 
 /**
@@ -494,7 +542,7 @@ public:
     void add(const Key& key, const Value& val) {
         invariant(!_done);
 
-        _data.push_back(std::make_pair(key, val));
+        _data.emplace_back(key.getOwned(), val.getOwned());
 
         _memUsed += key.memUsageForSorter();
         _memUsed += val.memUsageForSorter();
@@ -547,15 +595,13 @@ private:
             return;
 
         if (!_opts.extSortAllowed) {
-            // XXX This error message is only correct for aggregation, but it is also the
-            // only way this code could be hit at the moment. If the Sorter is used
-            // elsewhere where extSortAllowed could possibly be false, this message will
-            // need to be revisited.
-            uasserted(16819,
-                      str::stream()
-                          << "Sort exceeded memory limit of " << _opts.maxMemoryUsageBytes
-                          << " bytes, but did not opt in to external sorting. Aborting operation."
-                          << " Pass allowDiskUse:true to opt in.");
+            // This error message only applies to sorts from user queries made through the find or
+            // aggregation commands. Other clients, such as bulk index builds, should suppress this
+            // error, either by allowing external sorting or by catching and throwing a more
+            // appropriate error.
+            uasserted(ErrorCodes::QueryExceededMemoryLimitNoDiskUseAllowed,
+                      str::stream() << "Sort exceeded memory limit of " << _opts.maxMemoryUsageBytes
+                                    << " bytes, but did not opt in to external sorting.");
         }
 
         sort();
@@ -608,7 +654,7 @@ public:
             _haveData = true;
         }
 
-        _best = contender;
+        _best = {contender.first.getOwned(), contender.second.getOwned()};
     }
 
     Iterator* done() {
@@ -678,7 +724,7 @@ public:
             if (_haveCutoff && !less(contender, _cutoff))
                 return;
 
-            _data.push_back(contender);
+            _data.emplace_back(contender.first.getOwned(), contender.second.getOwned());
 
             _memUsed += key.memUsageForSorter();
             _memUsed += val.memUsageForSorter();
@@ -706,7 +752,7 @@ public:
         _memUsed -= _data.front().second.memUsageForSorter();
 
         std::pop_heap(_data.begin(), _data.end(), less);
-        _data.back() = contender;
+        _data.back() = {contender.first.getOwned(), contender.second.getOwned()};
         std::push_heap(_data.begin(), _data.end(), less);
 
         if (_memUsed > _opts.maxMemoryUsageBytes)
@@ -834,11 +880,10 @@ private:
             return;
 
         if (!_opts.extSortAllowed) {
-            // XXX This error message is only correct for aggregation, but it is also the
-            // only way this code could be hit at the moment. If the Sorter is used
-            // elsewhere where extSortAllowed could possibly be false, this message will
-            // need to be revisited.
-            uasserted(16820,
+            // This error message only applies to sorts from user queries made through the find or
+            // aggregation commands. Other clients should suppress this error, either by allowing
+            // external sorting or by catching and throwing a more appropriate error.
+            uasserted(ErrorCodes::QueryExceededMemoryLimitNoDiskUseAllowed,
                       str::stream()
                           << "Sort exceeded memory limit of " << _opts.maxMemoryUsageBytes
                           << " bytes, but did not opt in to external sorting. Aborting operation."
@@ -930,9 +975,19 @@ SortedFileWriter<Key, Value>::SortedFileWriter(const SortOptions& opts,
 
 template <typename Key, typename Value>
 void SortedFileWriter<Key, Value>::addAlreadySorted(const Key& key, const Value& val) {
+
+    // Offset that points to the place in the buffer where a new data object will be stored.
+    int _nextObjPos = _buffer.len();
+
+    // Add serialized key and value to the buffer.
     key.serializeForSorter(_buffer);
     val.serializeForSorter(_buffer);
 
+    // Serializing the key and value grows the buffer, but _buffer.buf() still points to the
+    // beginning. Use _buffer.len() to determine portion of buffer containing new datum.
+    _checksum =
+        addDataToChecksum(_buffer.buf() + _nextObjPos, _buffer.len() - _nextObjPos, _checksum);
+
     if (_buffer.len() > 64 * 1024)
         spill();
 }
@@ -1002,7 +1057,7 @@ SortIteratorInterface<Key, Value>* SortedFileWriter<Key, Value>::done() {
     _file.close();
 
     return new sorter::FileIterator<Key, Value>(
-        _fileName, _fileStartOffset, _fileEndOffset, _settings);
+        _fileName, _fileStartOffset, _fileEndOffset, _settings, _checksum);
 }
 
 //
