@@ -27,6 +27,8 @@
  *    it in the license file.
  */
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/sorter/sorter.h"
@@ -44,6 +46,7 @@
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/str.h"
 
+#include "mongo/logv2/log.h"
 #include <memory>
 
 namespace mongo {
@@ -204,8 +207,10 @@ void _assertIteratorsEquivalent(It1 it1, It2 it2, int line) {
         it1->closeSource();
         it2->closeSource();
     } catch (...) {
-        mongo::unittest::log() << "Failure from line " << line << " on iteration " << iteration
-                               << std::endl;
+        LOGV2(22047,
+              "Failure from line {line} on iteration {iteration}",
+              "line"_attr = line,
+              "iteration"_attr = iteration);
         it1->closeSource();
         it2->closeSource();
         throw;
@@ -516,7 +521,7 @@ class Dupes : public Basic {
 template <bool Random = true>
 class LotsOfDataLittleMemory : public Basic {
 public:
-    LotsOfDataLittleMemory() : _array(new int[NUM_ITEMS]), _random(int64_t(time(0))) {
+    LotsOfDataLittleMemory() : _array(new int[NUM_ITEMS]), _random(int64_t(time(nullptr))) {
         for (int i = 0; i < NUM_ITEMS; i++)
             _array[i] = i;
 
@@ -578,9 +583,9 @@ class LotsOfDataWithLimit : public LotsOfDataLittleMemory<Random> {
 };
 }  // namespace SorterTests
 
-class SorterSuite : public mongo::unittest::Suite {
+class SorterSuite : public mongo::unittest::OldStyleSuiteSpecification {
 public:
-    SorterSuite() : Suite("sorter") {}
+    SorterSuite() : mongo::unittest::OldStyleSuiteSpecification("sorter") {}
 
     template <typename T>
     static constexpr uint64_t kMaxAsU64 = std::numeric_limits<T>::max();
@@ -619,5 +624,5 @@ public:
     }
 };
 
-mongo::unittest::SuiteInstance<SorterSuite> extSortTests;
+mongo::unittest::OldStyleSuiteInitializer<SorterSuite> extSortTests;
 }  // namespace mongo
