@@ -64,6 +64,8 @@ public:
 
         void updateTTLSetting(long long newExpireSeconds);
 
+        void updateHiddenSetting(bool hidden);
+
         std::string name() const {
             return spec["name"].String();
         }
@@ -74,12 +76,8 @@ public:
         KVPrefix prefix = KVPrefix::kNotPrefixed;
         bool isBackgroundSecondaryBuild = false;
 
-        long versionOfBuild = kIndexBuildVersion;
-        // If true, a two-phase index build is in progress, false otherwise.
-        bool runTwoPhaseBuild = false;
-        boost::optional<std::string> buildPhase;
-        boost::optional<std::string> constraintViolationsIdent;
-        boost::optional<std::string> sideWritesIdent;
+        // If initialized, a two-phase index build is in progress.
+        boost::optional<UUID> buildUUID;
 
         // If non-empty, 'multikeyPaths' is a vector with size equal to the number of elements in
         // the index key pattern. Each element in the vector is an ordered set of positions
