@@ -35,6 +35,7 @@
 #include "mongo/db/operation_context.h"
 #include "mongo/db/record_id.h"
 #include "mongo/db/storage/index_entry_comparison.h"
+#include "mongo/db/storage/key_string.h"
 
 #pragma once
 
@@ -45,13 +46,6 @@ class BucketDeletionNotification;
 class SortedDataBuilderInterface;
 struct ValidateResults;
 
-/**
- * This enum is returned by any functions that could potentially insert special format onto disk. It
- * is a way to inform the callers to do something when special format exists on disk.
- * TODO SERVER-36385: Remove this enum in 4.4.
- */
-enum SpecialFormatInserted { NoSpecialFormatInserted = 0, LongTypeBitsInserted = 1 };
-
 /**
  * This is the uniform interface for storing indexes and supporting point queries as well as range
  * queries. The actual implementation is up to the storage engine. All the storage engines must
@@ -59,6 +53,9 @@ enum SpecialFormatInserted { NoSpecialFormatInserted = 0, LongTypeBitsInserted =
  */
 class SortedDataInterface {
 public:
+    SortedDataInterface(KeyString::Version keyStringVersion, Ordering ordering)
+        : _keyStringVersion(keyStringVersion), _ordering(ordering) {}
+
     virtual ~SortedDataInterface() {}
 
     //
@@ -81,7 +78,8 @@ public:
                                                        bool dupsAllowed) = 0;
 
     /**
-     * Insert an entry into the index with the specified key and RecordId.
+     * Insert an entry into the index with the specified KeyString, which must have a RecordId
+     * appended to the end.
      *
      * @param opCtx the transaction under which the insert takes place
      * @param dupsAllowed true if duplicate keys are allowed, and false
@@ -89,37 +87,33 @@ public:
      *
      * @return Status::OK() if the insert succeeded,
      *
-     *         ErrorCodes::DuplicateKey if 'key' already exists in 'this' index
+     *         ErrorCodes::DuplicateKey if 'keyString' already exists in 'this' index
      *         at a RecordId other than 'loc' and duplicates were not allowed
-     *
-     *         SpecialFormatInserted::LongTypeBitsInserted if the key we've
-     *         inserted has long typebits.
      */
-    virtual StatusWith<SpecialFormatInserted> insert(OperationContext* opCtx,
-                                                     const BSONObj& key,
-                                                     const RecordId& loc,
-                                                     bool dupsAllowed) = 0;
+    virtual Status insert(OperationContext* opCtx,
+                          const KeyString::Value& keyString,
+                          bool dupsAllowed) = 0;
 
     /**
-     * Remove the entry from the index with the specified key and RecordId.
+     * Remove the entry from the index with the specified KeyString, which must have a RecordId
+     * appended to the end.
      *
      * @param opCtx the transaction under which the remove takes place
      * @param dupsAllowed true if duplicate keys are allowed, and false
      *        otherwise
      */
     virtual void unindex(OperationContext* opCtx,
-                         const BSONObj& key,
-                         const RecordId& loc,
+                         const KeyString::Value& keyString,
                          bool dupsAllowed) = 0;
 
     /**
-     * Return ErrorCodes::DuplicateKey if there is more than one occurence of 'key' in this index,
-     * and Status::OK() otherwise. This call is only allowed on a unique index, and will invariant
-     * otherwise.
+     * Return ErrorCodes::DuplicateKey if there is more than one occurence of 'KeyString' in this
+     * index, and Status::OK() otherwise. This call is only allowed on a unique index, and will
+     * invariant otherwise.
      *
      * @param opCtx the transaction under which this operation takes place
      */
-    virtual Status dupKeyCheck(OperationContext* opCtx, const BSONObj& key) = 0;
+    virtual Status dupKeyCheck(OperationContext* opCtx, const KeyString::Value& keyString) = 0;
 
     /**
      * Attempt to reduce the storage space used by this index via compaction. Only called if the
@@ -159,19 +153,6 @@ public:
      */
     virtual bool isEmpty(OperationContext* opCtx) = 0;
 
-    /**
-     * Attempt to bring the entirety of 'this' index into memory.
-     *
-     * If the underlying storage engine does not support the operation,
-     * returns ErrorCodes::CommandNotSupported
-     *
-     * @return Status::OK()
-     */
-    virtual Status touch(OperationContext* opCtx) const {
-        return Status(ErrorCodes::CommandNotSupported,
-                      "this storage engine does not support touch");
-    }
-
     /**
      * Return the number of entries in 'this' index.
      *
@@ -180,10 +161,24 @@ public:
      */
     virtual long long numEntries(OperationContext* opCtx) const {
         long long x = -1;
-        fullValidate(opCtx, &x, NULL);
+        fullValidate(opCtx, &x, nullptr);
         return x;
     }
 
+    /*
+     * Return the KeyString version for 'this' index.
+     */
+    KeyString::Version getKeyStringVersion() const {
+        return _keyStringVersion;
+    }
+
+    /*
+     * Return the ordering for 'this' index.
+     */
+    Ordering getOrdering() const {
+        return _ordering;
+    }
+
     /**
      * Navigates over the sorted data.
      *
@@ -257,44 +252,50 @@ public:
          * If not positioned, returns boost::none.
          */
         virtual boost::optional<IndexKeyEntry> next(RequestedInfo parts = kKeyAndLoc) = 0;
+        virtual boost::optional<KeyStringEntry> nextKeyString() = 0;
 
         //
         // Seeking
         //
 
         /**
-         * Seeks to the provided key and returns current position.
-         *
-         * TODO consider removing once IndexSeekPoint has been cleaned up a bit. In particular,
-         * need a way to specify use whole keyPrefix and nothing else and to support the
-         * combination of empty and exclusive. Should also make it easier to construct for the
-         * common cases.
+         * Seeks to the provided keyString and returns the KeyStringEntry.
+         * The provided keyString has discriminator information encoded.
          */
-        virtual boost::optional<IndexKeyEntry> seek(const BSONObj& key,
-                                                    bool inclusive,
-                                                    RequestedInfo parts = kKeyAndLoc) = 0;
+        virtual boost::optional<KeyStringEntry> seekForKeyString(
+            const KeyString::Value& keyString) = 0;
 
         /**
-         * Seeks to the position described by seekPoint and returns the current position.
-         *
-         * NOTE: most implementations should just pass seekPoint to
-         * IndexEntryComparison::makeQueryObject().
+         * Seeks to the provided keyString and returns the IndexKeyEntry.
+         * The provided keyString has discriminator information encoded.
          */
-        virtual boost::optional<IndexKeyEntry> seek(const IndexSeekPoint& seekPoint,
+        virtual boost::optional<IndexKeyEntry> seek(const KeyString::Value& keyString,
                                                     RequestedInfo parts = kKeyAndLoc) = 0;
 
         /**
          * Seeks to a key with a hint to the implementation that you only want exact matches. If
          * an exact match can't be found, boost::none will be returned and the resulting
          * position of the cursor is unspecified.
+         *
+         * This will not accept a KeyString with a Discriminator other than kInclusive. Since
+         * keys are not stored with Discriminators, an exact match would never be found.
          */
-        virtual boost::optional<IndexKeyEntry> seekExact(const BSONObj& key,
-                                                         RequestedInfo parts = kKeyAndLoc) {
-            auto kv = seek(key, true, kKeyAndLoc);
-            if (kv && kv->key.woCompare(key, BSONObj(), /*considerFieldNames*/ false) == 0)
-                return kv;
-            return {};
-        }
+        virtual boost::optional<KeyStringEntry> seekExactForKeyString(
+            const KeyString::Value& keyString) = 0;
+
+        /**
+         * Seeks to a key with a hint to the implementation that you only want exact matches. If
+         * an exact match can't be found, boost::none will be returned and the resulting
+         * position of the cursor is unspecified.
+         *
+         * This will not accept a KeyString with a Discriminator other than kInclusive. Since
+         * keys are not stored with Discriminators, an exact match would never be found.
+         *
+         * Unlike the previous method, this one will return IndexKeyEntry if an exact match is
+         * found.
+         */
+        virtual boost::optional<IndexKeyEntry> seekExact(const KeyString::Value& keyString,
+                                                         RequestedInfo parts = kKeyAndLoc) = 0;
 
         //
         // Saving and restoring state
@@ -364,6 +365,10 @@ public:
     //
 
     virtual Status initAsEmpty(OperationContext* opCtx) = 0;
+
+protected:
+    const KeyString::Version _keyStringVersion;
+    const Ordering _ordering;
 };
 
 /**
@@ -374,17 +379,12 @@ public:
     virtual ~SortedDataBuilderInterface() {}
 
     /**
-     * Adds 'key' to intermediate storage.
+     * Adds 'keyString' to intermediate storage.
      *
-     * 'key' must be > or >= the last key passed to this function (depends on _dupsAllowed).  If
-     * this is violated an error Status (ErrorCodes::InternalError) will be returned.
-     *
-     * @return Status::OK() if addKey succeeded,
-     *
-     *         SpecialFormatInserted::LongTypeBitsInserted if we've inserted any
-     *         key with long typebits.
+     * 'keyString' must be > or >= the last key passed to this function (depends on _dupsAllowed).
+     * If this is violated an error Status (ErrorCodes::InternalError) will be returned.
      */
-    virtual StatusWith<SpecialFormatInserted> addKey(const BSONObj& key, const RecordId& loc) = 0;
+    virtual Status addKey(const KeyString::Value& keyString) = 0;
 
     /**
      * Do any necessary work to finish building the tree.
@@ -394,17 +394,8 @@ public:
      *
      * This is called outside of any WriteUnitOfWork to allow implementations to split this up
      * into multiple units.
-     *
-     * @return SpecialFormatInserted::LongTypeBitsInserted if we've inserted any
-     * key with long typebits.
-     *
-     * TODO SERVER-36385: Change the return type from SpecialFormatInserted back to "void" as that
-     * was a hack introduced in SERVER-36280 for detecting long TypeBits in an edge case in one of
-     * the unique index builder implementations.
      */
-    virtual SpecialFormatInserted commit(bool mayInterrupt) {
-        return SpecialFormatInserted::NoSpecialFormatInserted;
-    }
+    virtual void commit(bool mayInterrupt) {}
 };
 
 }  // namespace mongo
