@@ -53,8 +53,38 @@ TEST(SortedDataInterface, BuilderAddKey) {
         const std::unique_ptr<SortedDataBuilderInterface> builder(
             sorted->getBulkBuilder(opCtx.get(), true));
 
-        ASSERT_OK(builder->addKey(key1, loc1));
-        ASSERT_EQUALS(SpecialFormatInserted::NoSpecialFormatInserted, builder->commit(false));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), key1, loc1)));
+        builder->commit(false);
+    }
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        ASSERT_EQUALS(1, sorted->numEntries(opCtx.get()));
+    }
+}
+
+/*
+ * Add a KeyString using a bulk builder.
+ */
+TEST(SortedDataInterface, BuilderAddKeyString) {
+    const auto harnessHelper(newSortedDataInterfaceHarnessHelper());
+    const std::unique_ptr<SortedDataInterface> sorted(
+        harnessHelper->newSortedDataInterface(/*unique=*/false, /*partial=*/false));
+
+    KeyString::Builder keyString1(sorted->getKeyStringVersion(), key1, sorted->getOrdering(), loc1);
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        ASSERT(sorted->isEmpty(opCtx.get()));
+    }
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        const std::unique_ptr<SortedDataBuilderInterface> builder(
+            sorted->getBulkBuilder(opCtx.get(), true));
+
+        ASSERT_OK(builder->addKey(keyString1.getValueCopy()));
+        builder->commit(false);
     }
 
     {
@@ -81,8 +111,8 @@ TEST(SortedDataInterface, BuilderAddKeyWithReservedRecordId) {
         RecordId reservedLoc(RecordId::ReservedId::kWildcardMultikeyMetadataId);
         ASSERT(reservedLoc.isReserved());
 
-        ASSERT_OK(builder->addKey(key1, reservedLoc));
-        ASSERT_EQUALS(SpecialFormatInserted::NoSpecialFormatInserted, builder->commit(false));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), key1, reservedLoc)));
+        builder->commit(false);
     }
 
     {
@@ -107,8 +137,8 @@ TEST(SortedDataInterface, BuilderAddCompoundKey) {
         const std::unique_ptr<SortedDataBuilderInterface> builder(
             sorted->getBulkBuilder(opCtx.get(), true));
 
-        ASSERT_OK(builder->addKey(compoundKey1a, loc1));
-        ASSERT_EQUALS(SpecialFormatInserted::NoSpecialFormatInserted, builder->commit(false));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), compoundKey1a, loc1)));
+        builder->commit(false);
     }
 
     {
@@ -135,9 +165,45 @@ TEST(SortedDataInterface, BuilderAddSameKey) {
         const std::unique_ptr<SortedDataBuilderInterface> builder(
             sorted->getBulkBuilder(opCtx.get(), false));
 
-        ASSERT_OK(builder->addKey(key1, loc1));
-        ASSERT_EQUALS(ErrorCodes::DuplicateKey, builder->addKey(key1, loc2));
-        ASSERT_EQUALS(SpecialFormatInserted::NoSpecialFormatInserted, builder->commit(false));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), key1, loc1)));
+        ASSERT_EQUALS(ErrorCodes::DuplicateKey,
+                      builder->addKey(makeKeyString(sorted.get(), key1, loc2)));
+        builder->commit(false);
+    }
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        ASSERT_EQUALS(1, sorted->numEntries(opCtx.get()));
+    }
+}
+
+/*
+ * Add the same KeyString multiple times using a bulk builder and verify that the returned status is
+ * ErrorCodes::DuplicateKey when duplicates are not allowed.
+ */
+TEST(SortedDataInterface, BuilderAddSameKeyString) {
+    const auto harnessHelper(newSortedDataInterfaceHarnessHelper());
+    const std::unique_ptr<SortedDataInterface> sorted(
+        harnessHelper->newSortedDataInterface(/*unique=*/true, /*partial=*/false));
+
+    KeyString::Builder keyStringLoc1(
+        sorted->getKeyStringVersion(), key1, sorted->getOrdering(), loc1);
+    KeyString::Builder keyStringLoc2(
+        sorted->getKeyStringVersion(), key1, sorted->getOrdering(), loc2);
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        ASSERT(sorted->isEmpty(opCtx.get()));
+    }
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        const std::unique_ptr<SortedDataBuilderInterface> builder(
+            sorted->getBulkBuilder(opCtx.get(), false));
+
+        ASSERT_OK(builder->addKey(keyStringLoc1.getValueCopy()));
+        ASSERT_EQUALS(ErrorCodes::DuplicateKey, builder->addKey(keyStringLoc2.getValueCopy()));
+        builder->commit(false);
     }
 
     {
@@ -163,9 +229,44 @@ TEST(SortedDataInterface, BuilderAddSameKeyWithDupsAllowed) {
         const std::unique_ptr<SortedDataBuilderInterface> builder(
             sorted->getBulkBuilder(opCtx.get(), true /* allow duplicates */));
 
-        ASSERT_OK(builder->addKey(key1, loc1));
-        ASSERT_OK(builder->addKey(key1, loc2));
-        ASSERT_EQUALS(SpecialFormatInserted::NoSpecialFormatInserted, builder->commit(false));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), key1, loc1)));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), key1, loc2)));
+        builder->commit(false);
+    }
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        ASSERT_EQUALS(2, sorted->numEntries(opCtx.get()));
+    }
+}
+
+/*
+ * Add the same KeyString multiple times using a bulk builder and verify that the returned status is
+ * OK when duplicates are allowed.
+ */
+TEST(SortedDataInterface, BuilderAddSameKeyStringWithDupsAllowed) {
+    const auto harnessHelper(newSortedDataInterfaceHarnessHelper());
+    const std::unique_ptr<SortedDataInterface> sorted(
+        harnessHelper->newSortedDataInterface(/*unique=*/false, /*partial=*/false));
+
+    KeyString::Builder keyStringLoc1(
+        sorted->getKeyStringVersion(), key1, sorted->getOrdering(), loc1);
+    KeyString::Builder keyStringLoc2(
+        sorted->getKeyStringVersion(), key1, sorted->getOrdering(), loc2);
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        ASSERT(sorted->isEmpty(opCtx.get()));
+    }
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        const std::unique_ptr<SortedDataBuilderInterface> builder(
+            sorted->getBulkBuilder(opCtx.get(), true /* allow duplicates */));
+
+        ASSERT_OK(builder->addKey(keyStringLoc1.getValueCopy()));
+        ASSERT_OK(builder->addKey(keyStringLoc2.getValueCopy()));
+        builder->commit(false);
     }
 
     {
@@ -190,10 +291,44 @@ TEST(SortedDataInterface, BuilderAddMultipleKeys) {
         const std::unique_ptr<SortedDataBuilderInterface> builder(
             sorted->getBulkBuilder(opCtx.get(), true));
 
-        ASSERT_OK(builder->addKey(key1, loc1));
-        ASSERT_OK(builder->addKey(key2, loc2));
-        ASSERT_OK(builder->addKey(key3, loc3));
-        ASSERT_EQUALS(SpecialFormatInserted::NoSpecialFormatInserted, builder->commit(false));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), key1, loc1)));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), key2, loc2)));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), key3, loc3)));
+        builder->commit(false);
+    }
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        ASSERT_EQUALS(3, sorted->numEntries(opCtx.get()));
+    }
+}
+
+/*
+ * Add multiple KeyStrings using a bulk builder.
+ */
+TEST(SortedDataInterface, BuilderAddMultipleKeyStrings) {
+    const auto harnessHelper(newSortedDataInterfaceHarnessHelper());
+    const std::unique_ptr<SortedDataInterface> sorted(
+        harnessHelper->newSortedDataInterface(/*unique=*/false, /*partial=*/false));
+
+    KeyString::Builder keyString1(sorted->getKeyStringVersion(), key1, sorted->getOrdering(), loc1);
+    KeyString::Builder keyString2(sorted->getKeyStringVersion(), key2, sorted->getOrdering(), loc2);
+    KeyString::Builder keyString3(sorted->getKeyStringVersion(), key3, sorted->getOrdering(), loc3);
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        ASSERT(sorted->isEmpty(opCtx.get()));
+    }
+
+    {
+        const ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
+        const std::unique_ptr<SortedDataBuilderInterface> builder(
+            sorted->getBulkBuilder(opCtx.get(), true));
+
+        ASSERT_OK(builder->addKey(keyString1.getValueCopy()));
+        ASSERT_OK(builder->addKey(keyString2.getValueCopy()));
+        ASSERT_OK(builder->addKey(keyString3.getValueCopy()));
+        builder->commit(false);
     }
 
     {
@@ -218,12 +353,12 @@ TEST(SortedDataInterface, BuilderAddMultipleCompoundKeys) {
         const std::unique_ptr<SortedDataBuilderInterface> builder(
             sorted->getBulkBuilder(opCtx.get(), true));
 
-        ASSERT_OK(builder->addKey(compoundKey1a, loc1));
-        ASSERT_OK(builder->addKey(compoundKey1b, loc2));
-        ASSERT_OK(builder->addKey(compoundKey1c, loc4));
-        ASSERT_OK(builder->addKey(compoundKey2b, loc3));
-        ASSERT_OK(builder->addKey(compoundKey3a, loc5));
-        ASSERT_EQUALS(SpecialFormatInserted::NoSpecialFormatInserted, builder->commit(false));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), compoundKey1a, loc1)));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), compoundKey1b, loc2)));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), compoundKey1c, loc4)));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), compoundKey2b, loc3)));
+        ASSERT_OK(builder->addKey(makeKeyString(sorted.get(), compoundKey3a, loc5)));
+        builder->commit(false);
     }
 
     {
