@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kStorage
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
@@ -40,8 +40,8 @@
 #include "mongo/db/storage/wiredtiger/wiredtiger_prepare_conflict.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/hex.h"
-#include "mongo/util/log.h"
 
 namespace mongo {
 namespace {
@@ -52,33 +52,9 @@ namespace {
 // because the recovery unit may not ever actually be in a prepared state.
 MONGO_FAIL_POINT_DEFINE(WTAlwaysNotifyPrepareConflictWaiters);
 
-// SnapshotIds need to be globally unique, as they are used in a WorkingSetMember to
-// determine if documents changed, but a different recovery unit may be used across a getMore,
-// so there is a chance the snapshot ID will be reused.
-AtomicWord<unsigned long long> nextSnapshotId{1};
+logv2::LogSeverity kSlowTransactionSeverity = logv2::LogSeverity::Debug(1);
 
-logger::LogSeverity kSlowTransactionSeverity = logger::LogSeverity::Debug(1);
-
-/**
- * Returns a string representation of WiredTigerRecoveryUnit::State for logging.
- */
-std::string toString(WiredTigerRecoveryUnit::State state) {
-    switch (state) {
-        case WiredTigerRecoveryUnit::State::kInactive:
-            return "Inactive";
-        case WiredTigerRecoveryUnit::State::kInactiveInUnitOfWork:
-            return "InactiveInUnitOfWork";
-        case WiredTigerRecoveryUnit::State::kActiveNotInUnitOfWork:
-            return "ActiveNotInUnitOfWork";
-        case WiredTigerRecoveryUnit::State::kActive:
-            return "Active";
-        case WiredTigerRecoveryUnit::State::kCommitting:
-            return "Committing";
-        case WiredTigerRecoveryUnit::State::kAborting:
-            return "Aborting";
-    }
-    MONGO_UNREACHABLE;
-}
+MONGO_FAIL_POINT_DEFINE(doUntimestampedWritesForIdempotencyTests);
 
 }  // namespace
 
@@ -185,12 +161,10 @@ WiredTigerRecoveryUnit::WiredTigerRecoveryUnit(WiredTigerSessionCache* sc)
 
 WiredTigerRecoveryUnit::WiredTigerRecoveryUnit(WiredTigerSessionCache* sc,
                                                WiredTigerOplogManager* oplogManager)
-    : _sessionCache(sc),
-      _oplogManager(oplogManager),
-      _mySnapshotId(nextSnapshotId.fetchAndAdd(1)) {}
+    : _sessionCache(sc), _oplogManager(oplogManager) {}
 
 WiredTigerRecoveryUnit::~WiredTigerRecoveryUnit() {
-    invariant(!_inUnitOfWork(), toString(_state));
+    invariant(!_inUnitOfWork(), toString(_getState()));
     _abort();
 }
 
@@ -200,93 +174,72 @@ void WiredTigerRecoveryUnit::_commit() {
     // be boost::none and we'll set the commit time to that.
     auto commitTime = _commitTimestamp.isNull() ? _lastTimestampSet : _commitTimestamp;
 
-    try {
-        bool notifyDone = !_prepareTimestamp.isNull();
-        if (_session && _isActive()) {
-            _txnClose(true);
-        }
-        _setState(State::kCommitting);
+    bool notifyDone = !_prepareTimestamp.isNull();
+    if (_session && _isActive()) {
+        _txnClose(true);
+    }
+    _setState(State::kCommitting);
 
-        if (MONGO_FAIL_POINT(WTAlwaysNotifyPrepareConflictWaiters)) {
-            notifyDone = true;
-        }
-
-        if (notifyDone) {
-            _sessionCache->notifyPreparedUnitOfWorkHasCommittedOrAborted();
-        }
-
-        for (Changes::const_iterator it = _changes.begin(), end = _changes.end(); it != end; ++it) {
-            (*it)->commit(commitTime);
-        }
-        _changes.clear();
-    } catch (...) {
-        std::terminate();
+    if (MONGO_unlikely(WTAlwaysNotifyPrepareConflictWaiters.shouldFail())) {
+        notifyDone = true;
     }
 
+    if (notifyDone) {
+        _sessionCache->notifyPreparedUnitOfWorkHasCommittedOrAborted();
+    }
+
+    commitRegisteredChanges(commitTime);
     _setState(State::kInactive);
 }
 
 void WiredTigerRecoveryUnit::_abort() {
-    try {
-        bool notifyDone = !_prepareTimestamp.isNull();
-        if (_session && _isActive()) {
-            _txnClose(false);
-        }
-        _setState(State::kAborting);
+    bool notifyDone = !_prepareTimestamp.isNull();
+    if (_session && _isActive()) {
+        _txnClose(false);
+    }
+    _setState(State::kAborting);
 
-        if (MONGO_FAIL_POINT(WTAlwaysNotifyPrepareConflictWaiters)) {
-            notifyDone = true;
-        }
-
-        if (notifyDone) {
-            _sessionCache->notifyPreparedUnitOfWorkHasCommittedOrAborted();
-        }
-
-        for (Changes::const_reverse_iterator it = _changes.rbegin(), end = _changes.rend();
-             it != end;
-             ++it) {
-            Change* change = it->get();
-            LOG(2) << "CUSTOM ROLLBACK " << redact(demangleName(typeid(*change)));
-            change->rollback();
-        }
-        _changes.clear();
-    } catch (...) {
-        std::terminate();
+    if (notifyDone || MONGO_unlikely(WTAlwaysNotifyPrepareConflictWaiters.shouldFail())) {
+        _sessionCache->notifyPreparedUnitOfWorkHasCommittedOrAborted();
     }
 
+    abortRegisteredChanges();
     _setState(State::kInactive);
 }
 
 void WiredTigerRecoveryUnit::beginUnitOfWork(OperationContext* opCtx) {
-    invariant(!_inUnitOfWork(), toString(_state));
+    invariant(!_inUnitOfWork(), toString(_getState()));
     invariant(!_isCommittingOrAborting(),
               str::stream() << "cannot begin unit of work while commit or rollback handlers are "
                                "running: "
-                            << toString(_state));
+                            << toString(_getState()));
     _setState(_isActive() ? State::kActive : State::kInactiveInUnitOfWork);
 }
 
 void WiredTigerRecoveryUnit::prepareUnitOfWork() {
-    invariant(_inUnitOfWork(), toString(_state));
+    invariant(_inUnitOfWork(), toString(_getState()));
     invariant(!_prepareTimestamp.isNull());
 
     auto session = getSession();
     WT_SESSION* s = session->getSession();
 
-    LOG(1) << "preparing transaction at time: " << _prepareTimestamp;
+    LOGV2_DEBUG(22410,
+                1,
+                "preparing transaction at time: {prepareTimestamp}",
+                "prepareTimestamp"_attr = _prepareTimestamp);
 
     const std::string conf = "prepare_timestamp=" + integerToHex(_prepareTimestamp.asULL());
     // Prepare the transaction.
     invariantWTOK(s->prepare_transaction(s, conf.c_str()));
 }
 
-void WiredTigerRecoveryUnit::commitUnitOfWork() {
-    invariant(_inUnitOfWork(), toString(_state));
+void WiredTigerRecoveryUnit::doCommitUnitOfWork() {
+    invariant(_inUnitOfWork(), toString(_getState()));
     _commit();
 }
 
-void WiredTigerRecoveryUnit::abortUnitOfWork() {
-    invariant(_inUnitOfWork(), toString(_state));
+void WiredTigerRecoveryUnit::doAbortUnitOfWork() {
+    invariant(_inUnitOfWork(), toString(_getState()));
     _abort();
 }
 
@@ -296,35 +249,46 @@ void WiredTigerRecoveryUnit::_ensureSession() {
     }
 }
 
-bool WiredTigerRecoveryUnit::waitUntilDurable() {
-    invariant(!_inUnitOfWork(), toString(_state));
-    const bool forceCheckpoint = false;
-    const bool stableCheckpoint = false;
-    _sessionCache->waitUntilDurable(forceCheckpoint, stableCheckpoint);
+bool WiredTigerRecoveryUnit::waitUntilDurable(OperationContext* opCtx) {
+    invariant(!_inUnitOfWork(), toString(_getState()));
+    invariant(!opCtx->lockState()->isLocked() || storageGlobalParams.repair);
+
+    // Flushes the journal log to disk. Checkpoints all data if journaling is disabled.
+    _sessionCache->waitUntilDurable(opCtx,
+                                    WiredTigerSessionCache::Fsync::kJournal,
+                                    WiredTigerSessionCache::UseJournalListener::kUpdate);
+
     return true;
 }
 
-bool WiredTigerRecoveryUnit::waitUntilUnjournaledWritesDurable(bool stableCheckpoint) {
-    invariant(!_inUnitOfWork(), toString(_state));
-    const bool forceCheckpoint = true;
-    // Calling `waitUntilDurable` with `forceCheckpoint` set to false only performs a log
-    // (journal) flush, and thus has no effect on unjournaled writes. Setting `forceCheckpoint` to
-    // true will lock in stable writes to unjournaled tables.
-    _sessionCache->waitUntilDurable(forceCheckpoint, stableCheckpoint);
-    return true;
-}
+bool WiredTigerRecoveryUnit::waitUntilUnjournaledWritesDurable(OperationContext* opCtx,
+                                                               bool stableCheckpoint) {
+    invariant(!_inUnitOfWork(), toString(_getState()));
+    invariant(!opCtx->lockState()->isLocked() || storageGlobalParams.repair);
 
-void WiredTigerRecoveryUnit::registerChange(Change* change) {
-    invariant(_inUnitOfWork(), toString(_state));
-    _changes.push_back(std::unique_ptr<Change>{change});
+    // Take a checkpoint, rather than only flush the (oplog) journal, in order to lock in stable
+    // writes to unjournaled tables.
+    //
+    // If 'stableCheckpoint' is set, then we will only checkpoint data up to and including the
+    // stable_timestamp set on WT at the time of the checkpoint. Otherwise, we will checkpoint all
+    // of the data.
+    WiredTigerSessionCache::Fsync fsyncType = stableCheckpoint
+        ? WiredTigerSessionCache::Fsync::kCheckpointStableTimestamp
+        : WiredTigerSessionCache::Fsync::kCheckpointAll;
+    _sessionCache->waitUntilDurable(
+        opCtx, fsyncType, WiredTigerSessionCache::UseJournalListener::kUpdate);
+
+    return true;
 }
 
 void WiredTigerRecoveryUnit::assertInActiveTxn() const {
     if (_isActive()) {
         return;
     }
-    severe() << "Recovery unit is not active. Current state: " << toString(_state);
-    fassertFailed(28575);
+    LOGV2_FATAL(28575,
+                "Recovery unit is not active. Current state: {currentState}",
+                "Recovery unit is not active.",
+                "currentState"_attr = _getState());
 }
 
 boost::optional<int64_t> WiredTigerRecoveryUnit::getOplogVisibilityTs() {
@@ -354,8 +318,8 @@ WiredTigerSession* WiredTigerRecoveryUnit::getSessionNoTxn() {
     return session;
 }
 
-void WiredTigerRecoveryUnit::abandonSnapshot() {
-    invariant(!_inUnitOfWork(), toString(_state));
+void WiredTigerRecoveryUnit::doAbandonSnapshot() {
+    invariant(!_inUnitOfWork(), toString(_getState()));
     if (_isActive()) {
         // Can't be in a WriteUnitOfWork, so safe to rollback
         _txnClose(false);
@@ -368,16 +332,55 @@ void WiredTigerRecoveryUnit::preallocateSnapshot() {
     getSession();
 }
 
+void WiredTigerRecoveryUnit::refreshSnapshot() {
+    // First, start a new transaction at the same timestamp as the current one.  Then end the
+    // current transaction.  This overlap will prevent WT from cleaning up history required to serve
+    // the read timestamp.
+
+    // Currently, this code only works for kNoOverlap or kNoTimestamp.
+    invariant(_timestampReadSource == ReadSource::kNoOverlap ||
+              _timestampReadSource == ReadSource::kNoTimestamp);
+    invariant(_isActive());
+    invariant(!_inUnitOfWork());
+    invariant(!_noEvictionAfterRollback);
+
+    auto newSession = _sessionCache->getSession();
+    WiredTigerBeginTxnBlock txnOpen(newSession->getSession(),
+                                    _prepareConflictBehavior,
+                                    _roundUpPreparedTimestamps,
+                                    RoundUpReadTimestamp::kNoRoundForce);
+    if (_timestampReadSource != ReadSource::kNoTimestamp) {
+        auto status = txnOpen.setReadSnapshot(_readAtTimestamp);
+        fassert(5035300, status);
+    }
+    txnOpen.done();
+
+    // Now end the previous transaction.
+    auto wtSession = _session->getSession();
+    auto wtRet = wtSession->rollback_transaction(wtSession, nullptr);
+    invariantWTOK(wtRet);
+    LOGV2_DEBUG(5035301,
+                3,
+                "WT begin_transaction & rollback_transaction",
+                "snapshotId"_attr = getSnapshotId().toNumber());
+
+    _session = std::move(newSession);
+}
+
 void WiredTigerRecoveryUnit::_txnClose(bool commit) {
-    invariant(_isActive(), toString(_state));
+    invariant(_isActive(), toString(_getState()));
     WT_SESSION* s = _session->getSession();
     if (_timer) {
         const int transactionTime = _timer->millis();
         // `serverGlobalParams.slowMs` can be set to values <= 0. In those cases, give logging a
         // break.
         if (transactionTime >= std::max(1, serverGlobalParams.slowMS)) {
-            LOG(kSlowTransactionSeverity) << "Slow WT transaction. Lifetime of SnapshotId "
-                                          << _mySnapshotId << " was " << transactionTime << "ms";
+            LOGV2_DEBUG(22411,
+                        kSlowTransactionSeverity.toInt(),
+                        "Slow WT transaction. Lifetime of SnapshotId {getSnapshotId_toNumber} was "
+                        "{transactionTime}ms",
+                        "getSnapshotId_toNumber"_attr = getSnapshotId().toNumber(),
+                        "transactionTime"_attr = transactionTime);
         }
     }
 
@@ -389,7 +392,9 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
             // read timestamp.
             invariant(_readAtTimestamp.isNull() || _commitTimestamp >= _readAtTimestamp);
 
-            conf << "commit_timestamp=" << integerToHex(_commitTimestamp.asULL()) << ",";
+            if (MONGO_likely(!doUntimestampedWritesForIdempotencyTests.shouldFail())) {
+                conf << "commit_timestamp=" << integerToHex(_commitTimestamp.asULL()) << ",";
+            }
             _isTimestamped = true;
         }
 
@@ -397,20 +402,44 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
             conf << "durable_timestamp=" << integerToHex(_durableTimestamp.asULL());
         }
 
+        if (_mustBeTimestamped) {
+            invariant(_isTimestamped);
+        }
+
         wtRet = s->commit_transaction(s, conf.str().c_str());
-        LOG(3) << "WT commit_transaction for snapshot id " << _mySnapshotId;
+
+        LOGV2_DEBUG(22412,
+                    3,
+                    "WT commit_transaction for snapshot id {snapshotId}",
+                    "WT commit_transaction",
+                    "snapshotId"_attr = getSnapshotId().toNumber());
     } else {
-        wtRet = s->rollback_transaction(s, nullptr);
-        invariant(!wtRet);
-        LOG(3) << "WT rollback_transaction for snapshot id " << _mySnapshotId;
+        StringBuilder config;
+        if (_noEvictionAfterRollback) {
+            // The only point at which rollback_transaction() can time out is in the bonus-eviction
+            // phase. If the timeout expires here, the function will stop the eviction and return
+            // success. It cannot return an error due to timeout.
+            config << "operation_timeout_ms=1,";
+        }
+
+        wtRet = s->rollback_transaction(s, config.str().c_str());
+
+        LOGV2_DEBUG(22413,
+                    3,
+                    "WT rollback_transaction for snapshot id {snapshotId}",
+                    "WT rollback_transaction",
+                    "snapshotId"_attr = getSnapshotId().toNumber());
     }
 
     if (_isTimestamped) {
         if (!_orderedCommit) {
             // We only need to update oplog visibility where commits can be out-of-order with
-            // respect to their assigned optime and such commits might otherwise be visible.
+            // respect to their assigned optime. This will ensure the oplog read timestamp gets
+            // updated when oplog 'holes' are filled: the last commit filling the last hole will
+            // prompt the oplog read timestamp to be forwarded.
+            //
             // This should happen only on primary nodes.
-            _oplogManager->triggerJournalFlush();
+            _oplogManager->triggerOplogVisibilityUpdate();
         }
         _isTimestamped = false;
     }
@@ -431,15 +460,10 @@ void WiredTigerRecoveryUnit::_txnClose(bool commit) {
     _durableTimestamp = Timestamp();
     _catalogConflictTimestamp = Timestamp();
     _roundUpPreparedTimestamps = RoundUpPreparedTimestamps::kNoRound;
-    _mySnapshotId = nextSnapshotId.fetchAndAdd(1);
     _isOplogReader = false;
     _oplogVisibleTs = boost::none;
     _orderedCommit = true;  // Default value is true; we assume all writes are ordered.
-}
-
-SnapshotId WiredTigerRecoveryUnit::getSnapshotId() const {
-    // TODO: use actual wiredtiger txn id
-    return SnapshotId(_mySnapshotId);
+    _mustBeTimestamped = false;
 }
 
 Status WiredTigerRecoveryUnit::obtainMajorityCommittedSnapshot() {
@@ -460,7 +484,6 @@ boost::optional<Timestamp> WiredTigerRecoveryUnit::getPointInTimeReadTimestamp()
     // transaction to establish a read timestamp, but only for ReadSources that are expected to have
     // read timestamps.
     switch (_timestampReadSource) {
-        case ReadSource::kUnset:
         case ReadSource::kNoTimestamp:
             return boost::none;
         case ReadSource::kMajorityCommitted:
@@ -486,19 +509,19 @@ boost::optional<Timestamp> WiredTigerRecoveryUnit::getPointInTimeReadTimestamp()
 
     switch (_timestampReadSource) {
         case ReadSource::kLastApplied:
-            // The lastApplied timestamp is not always available, so it is not possible to invariant
-            // that it exists as other ReadSources do.
+        case ReadSource::kNoOverlap:
+            // The lastApplied and allDurable timestamps are not always available if the system has
+            // not accepted writes, so it is not possible to invariant that it exists as other
+            // ReadSources do.
             if (!_readAtTimestamp.isNull()) {
                 return _readAtTimestamp;
             }
             return boost::none;
-        case ReadSource::kNoOverlap:
         case ReadSource::kAllDurableSnapshot:
             invariant(!_readAtTimestamp.isNull());
             return _readAtTimestamp;
 
         // The follow ReadSources returned values in the first switch block.
-        case ReadSource::kUnset:
         case ReadSource::kNoTimestamp:
         case ReadSource::kMajorityCommitted:
         case ReadSource::kProvided:
@@ -508,10 +531,10 @@ boost::optional<Timestamp> WiredTigerRecoveryUnit::getPointInTimeReadTimestamp()
 }
 
 void WiredTigerRecoveryUnit::_txnOpen() {
-    invariant(!_isActive(), toString(_state));
+    invariant(!_isActive(), toString(_getState()));
     invariant(!_isCommittingOrAborting(),
               str::stream() << "commit or rollback handler reopened transaction: "
-                            << toString(_state));
+                            << toString(_getState()));
     _ensureSession();
 
     // Only start a timer for transaction's lifetime if we're going to log it.
@@ -521,7 +544,6 @@ void WiredTigerRecoveryUnit::_txnOpen() {
     WT_SESSION* session = _session->getSession();
 
     switch (_timestampReadSource) {
-        case ReadSource::kUnset:
         case ReadSource::kNoTimestamp: {
             if (_isOplogReader) {
                 _oplogVisibleTs = static_cast<std::int64_t>(_oplogManager->getOplogReadTimestamp());
@@ -539,14 +561,7 @@ void WiredTigerRecoveryUnit::_txnOpen() {
             break;
         }
         case ReadSource::kLastApplied: {
-            if (_sessionCache->snapshotManager().getLocalSnapshot()) {
-                _readAtTimestamp = _sessionCache->snapshotManager().beginTransactionOnLocalSnapshot(
-                    session, _prepareConflictBehavior, _roundUpPreparedTimestamps);
-            } else {
-                WiredTigerBeginTxnBlock(
-                    session, _prepareConflictBehavior, _roundUpPreparedTimestamps)
-                    .done();
-            }
+            _readAtTimestamp = _beginTransactionAtLastAppliedTimestamp(session);
             break;
         }
         case ReadSource::kNoOverlap: {
@@ -576,7 +591,11 @@ void WiredTigerRecoveryUnit::_txnOpen() {
         }
     }
 
-    LOG(3) << "WT begin_transaction for snapshot id " << _mySnapshotId;
+    LOGV2_DEBUG(22414,
+                3,
+                "WT begin_transaction",
+                "snapshotId"_attr = getSnapshotId().toNumber(),
+                "readSource"_attr = toString(_timestampReadSource));
 }
 
 Timestamp WiredTigerRecoveryUnit::_beginTransactionAtAllDurableTimestamp(WT_SESSION* session) {
@@ -584,7 +603,7 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtAllDurableTimestamp(WT_SESS
                                     _prepareConflictBehavior,
                                     _roundUpPreparedTimestamps,
                                     RoundUpReadTimestamp::kRound);
-    Timestamp txnTimestamp = Timestamp(_oplogManager->fetchAllDurableValue(session->connection));
+    Timestamp txnTimestamp = _sessionCache->getKVEngine()->getAllDurableTimestamp();
     auto status = txnOpen.setReadSnapshot(txnTimestamp);
     fassert(50948, status);
 
@@ -596,10 +615,56 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtAllDurableTimestamp(WT_SESS
     return readTimestamp;
 }
 
+namespace {
+/**
+ * This mutex serializes starting read transactions at lastApplied. This throttles new transactions
+ * so they do not overwhelm the WiredTiger spinlock that manages the global read timestamp queue.
+ * Because this queue can grow larger than the number of active transactions, the MongoDB ticketing
+ * system alone cannot restrict its growth and thus bound the amount of time the queue is locked.
+ * TODO: WT-6709
+ */
+Mutex _lastAppliedTxnMutex = MONGO_MAKE_LATCH("lastAppliedTxnMutex");
+}  // namespace
+
+Timestamp WiredTigerRecoveryUnit::_beginTransactionAtLastAppliedTimestamp(WT_SESSION* session) {
+    auto lastApplied = _sessionCache->snapshotManager().getLastApplied();
+    if (!lastApplied) {
+        // When there is not a lastApplied timestamp available, read without a timestamp. Do not
+        // round up the read timestamp to the oldest timestamp.
+
+        // There is a race that allows new transactions to start between the time we check for a
+        // read timestamp and start our transaction, which can temporarily violate the contract of
+        // kLastApplied. That is, writes will be visible that occur after the lastApplied time. This
+        // is only possible for readers that start immediately after an initial sync that did not
+        // replicate any oplog entries. Future transactions will start reading at a timestamp once
+        // timestamped writes have been made.
+        WiredTigerBeginTxnBlock txnOpen(
+            session, _prepareConflictBehavior, _roundUpPreparedTimestamps);
+        LOGV2_DEBUG(4847500, 2, "no read timestamp available for kLastApplied");
+        txnOpen.done();
+        return Timestamp();
+    }
+
+    {
+        stdx::lock_guard<Mutex> lock(_lastAppliedTxnMutex);
+        WiredTigerBeginTxnBlock txnOpen(session,
+                                        _prepareConflictBehavior,
+                                        _roundUpPreparedTimestamps,
+                                        RoundUpReadTimestamp::kRound);
+        auto status = txnOpen.setReadSnapshot(*lastApplied);
+        fassert(4847501, status);
+        txnOpen.done();
+    }
+
+    // We might have rounded to oldest between calling getLastApplied and setReadSnapshot. We
+    // need to get the actual read timestamp we used.
+    return _getTransactionReadTimestamp(session);
+}
+
 Timestamp WiredTigerRecoveryUnit::_beginTransactionAtNoOverlapTimestamp(WT_SESSION* session) {
 
-    auto lastApplied = _sessionCache->snapshotManager().getLocalSnapshot();
-    Timestamp allDurable = Timestamp(_oplogManager->fetchAllDurableValue(session->connection));
+    auto lastApplied = _sessionCache->snapshotManager().getLastApplied();
+    Timestamp allDurable = Timestamp(_sessionCache->getKVEngine()->getAllDurableTimestamp());
 
     // When using timestamps for reads and writes, it's important that readers and writers don't
     // overlap with the timestamps they use. In other words, at any point in the system there should
@@ -627,17 +692,34 @@ Timestamp WiredTigerRecoveryUnit::_beginTransactionAtNoOverlapTimestamp(WT_SESSI
     // should read afterward.
     Timestamp readTimestamp = (lastApplied) ? std::min(*lastApplied, allDurable) : allDurable;
 
+    if (readTimestamp.isNull()) {
+        // When there is not an all_durable or lastApplied timestamp available, read without a
+        // timestamp. Do not round up the read timestamp to the oldest timestamp.
+
+        // There is a race that allows new transactions to start between the time we check for a
+        // read timestamp and start our transaction, which can temporarily violate the contract of
+        // kNoOverlap. That is, writes will be visible that occur after the all_durable time. This
+        // is only possible for readers that start immediately after an initial sync that did not
+        // replicate any oplog entries. Future transactions will start reading at a timestamp once
+        // timestamped writes have been made.
+        WiredTigerBeginTxnBlock txnOpen(
+            session, _prepareConflictBehavior, _roundUpPreparedTimestamps);
+        LOGV2_DEBUG(4452900, 1, "no read timestamp available for kNoOverlap");
+        txnOpen.done();
+        return readTimestamp;
+    }
+
     WiredTigerBeginTxnBlock txnOpen(session,
                                     _prepareConflictBehavior,
                                     _roundUpPreparedTimestamps,
                                     RoundUpReadTimestamp::kRound);
     auto status = txnOpen.setReadSnapshot(readTimestamp);
     fassert(51066, status);
-
-    // We might have rounded to oldest between calling getAllDurable and setReadSnapshot. We need
-    // to get the actual read timestamp we used.
-    readTimestamp = _getTransactionReadTimestamp(session);
     txnOpen.done();
+
+    // We might have rounded to oldest between calling getAllDurable and setReadSnapshot. We
+    // need to get the actual read timestamp we used.
+    readTimestamp = _getTransactionReadTimestamp(session);
     return readTimestamp;
 }
 
@@ -646,15 +728,18 @@ Timestamp WiredTigerRecoveryUnit::_getTransactionReadTimestamp(WT_SESSION* sessi
     auto wtstatus = session->query_timestamp(session, buf, "get=read");
     invariantWTOK(wtstatus);
     uint64_t read_timestamp;
-    fassert(50949, parseNumberFromStringWithBase(buf, 16, &read_timestamp));
+    fassert(50949, NumberParser().base(16)(buf, &read_timestamp));
     return Timestamp(read_timestamp);
 }
 
 Status WiredTigerRecoveryUnit::setTimestamp(Timestamp timestamp) {
     _ensureSession();
-    LOG(3) << "WT set timestamp of future write operations to " << timestamp;
+    LOGV2_DEBUG(22415,
+                3,
+                "WT set timestamp of future write operations to {timestamp}",
+                "timestamp"_attr = timestamp);
     WT_SESSION* session = _session->getSession();
-    invariant(_inUnitOfWork(), toString(_state));
+    invariant(_inUnitOfWork(), toString(_getState()));
     invariant(_prepareTimestamp.isNull());
     invariant(_commitTimestamp.isNull(),
               str::stream() << "Commit timestamp set to " << _commitTimestamp.toString()
@@ -669,6 +754,11 @@ Status WiredTigerRecoveryUnit::setTimestamp(Timestamp timestamp) {
     // Starts the WT transaction associated with this session.
     getSession();
 
+    if (MONGO_unlikely(doUntimestampedWritesForIdempotencyTests.shouldFail())) {
+        _isTimestamped = true;
+        return Status::OK();
+    }
+
     const std::string conf = "commit_timestamp=" + integerToHex(timestamp.asULL());
     auto rc = session->timestamp_transaction(session, conf.c_str());
     if (rc == 0) {
@@ -682,7 +772,7 @@ void WiredTigerRecoveryUnit::setCommitTimestamp(Timestamp timestamp) {
     // setPrepareTimestamp() is called. Prepared transactions ensure the correct timestamping
     // semantics and the set-once commitTimestamp behavior is exactly what prepared transactions
     // want.
-    invariant(!_inUnitOfWork() || !_prepareTimestamp.isNull(), toString(_state));
+    invariant(!_inUnitOfWork() || !_prepareTimestamp.isNull(), toString(_getState()));
     invariant(_commitTimestamp.isNull(),
               str::stream() << "Commit timestamp set to " << _commitTimestamp.toString()
                             << " and trying to set it to " << timestamp.toString());
@@ -712,7 +802,7 @@ Timestamp WiredTigerRecoveryUnit::getDurableTimestamp() const {
 }
 
 void WiredTigerRecoveryUnit::clearCommitTimestamp() {
-    invariant(!_inUnitOfWork(), toString(_state));
+    invariant(!_inUnitOfWork(), toString(_getState()));
     invariant(!_commitTimestamp.isNull());
     invariant(!_lastTimestampSet,
               str::stream() << "Last timestamp set is " << _lastTimestampSet->toString()
@@ -723,7 +813,7 @@ void WiredTigerRecoveryUnit::clearCommitTimestamp() {
 }
 
 void WiredTigerRecoveryUnit::setPrepareTimestamp(Timestamp timestamp) {
-    invariant(_inUnitOfWork(), toString(_state));
+    invariant(_inUnitOfWork(), toString(_getState()));
     invariant(_prepareTimestamp.isNull(),
               str::stream() << "Trying to set prepare timestamp to " << timestamp.toString()
                             << ". It's already set to " << _prepareTimestamp.toString());
@@ -738,7 +828,7 @@ void WiredTigerRecoveryUnit::setPrepareTimestamp(Timestamp timestamp) {
 }
 
 Timestamp WiredTigerRecoveryUnit::getPrepareTimestamp() const {
-    invariant(_inUnitOfWork(), toString(_state));
+    invariant(_inUnitOfWork(), toString(_getState()));
     invariant(!_prepareTimestamp.isNull());
     invariant(_commitTimestamp.isNull(),
               str::stream() << "Commit timestamp is " << _commitTimestamp.toString()
@@ -757,7 +847,7 @@ void WiredTigerRecoveryUnit::setPrepareConflictBehavior(PrepareConflictBehavior
     // ignoring prepare conflicts, since that behavior is applied when the transaction is opened.
     invariant(
         !_isActive(),
-        str::stream() << "Current state: " << toString(_state)
+        str::stream() << "Current state: " << toString(_getState())
                       << ". Invalid internal state while setting prepare conflict behavior to: "
                       << static_cast<int>(behavior));
 
@@ -772,20 +862,22 @@ void WiredTigerRecoveryUnit::setRoundUpPreparedTimestamps(bool value) {
     // This cannot be called after WiredTigerRecoveryUnit::_txnOpen.
     invariant(!_isActive(),
               str::stream() << "Can't change round up prepared timestamps flag "
-                            << "when current state is " << toString(_state));
+                            << "when current state is " << toString(_getState()));
     _roundUpPreparedTimestamps =
         (value) ? RoundUpPreparedTimestamps::kRound : RoundUpPreparedTimestamps::kNoRound;
 }
 
 void WiredTigerRecoveryUnit::setTimestampReadSource(ReadSource readSource,
                                                     boost::optional<Timestamp> provided) {
-    LOG(3) << "setting timestamp read source: " << static_cast<int>(readSource)
-           << ", provided timestamp: " << ((provided) ? provided->toString() : "none");
-
+    LOGV2_DEBUG(22416,
+                3,
+                "setting timestamp read source",
+                "readSource"_attr = toString(readSource),
+                "provided"_attr = ((provided) ? provided->toString() : "none"));
     invariant(!_isActive() || _timestampReadSource == readSource,
-              str::stream() << "Current state: " << toString(_state)
+              str::stream() << "Current state: " << toString(_getState())
                             << ". Invalid internal state while setting timestamp read source: "
-                            << static_cast<int>(readSource) << ", provided timestamp: "
+                            << toString(readSource) << ", provided timestamp: "
                             << (provided ? provided->toString() : "none"));
     invariant(!provided == (readSource != ReadSource::kProvided));
     invariant(!(provided && provided->isNull()));
@@ -820,25 +912,9 @@ std::shared_ptr<StorageStats> WiredTigerRecoveryUnit::getOperationStatistics() c
     return statsPtr;
 }
 
-void WiredTigerRecoveryUnit::_setState(State newState) {
-    _state = newState;
-}
-
-bool WiredTigerRecoveryUnit::_isActive() const {
-    return State::kActiveNotInUnitOfWork == _state || State::kActive == _state;
-}
-
-bool WiredTigerRecoveryUnit::_inUnitOfWork() const {
-    return State::kInactiveInUnitOfWork == _state || State::kActive == _state;
-}
-
-bool WiredTigerRecoveryUnit::_isCommittingOrAborting() const {
-    return State::kCommitting == _state || State::kAborting == _state;
-}
-
 void WiredTigerRecoveryUnit::setCatalogConflictingTimestamp(Timestamp timestamp) {
     // This cannot be called after a storage snapshot is allocated.
-    invariant(!_isActive(), toString(_state));
+    invariant(!_isActive(), toString(_getState()));
     invariant(_timestampReadSource == ReadSource::kNoTimestamp,
               str::stream() << "Illegal to set catalog conflicting timestamp for a read source "
                             << static_cast<int>(_timestampReadSource));
