@@ -104,24 +104,17 @@ public:
 
     void beginUnitOfWork(OperationContext* opCtx) override;
     void prepareUnitOfWork() override;
-    void commitUnitOfWork() override;
-    void abortUnitOfWork() override;
 
-    bool waitUntilDurable() override;
+    bool waitUntilDurable(OperationContext* opCtx) override;
 
-    bool waitUntilUnjournaledWritesDurable(bool stableCheckpoint = true) override;
+    bool waitUntilUnjournaledWritesDurable(OperationContext* opCtx, bool stableCheckpoint) override;
 
-    void registerChange(Change* change) override;
-
-    void abandonSnapshot() override;
     void preallocateSnapshot() override;
 
     Status obtainMajorityCommittedSnapshot() override;
 
     boost::optional<Timestamp> getPointInTimeReadTimestamp() override;
 
-    SnapshotId getSnapshotId() const override;
-
     Status setTimestamp(Timestamp timestamp) override;
 
     void setCommitTimestamp(Timestamp timestamp) override;
@@ -169,6 +162,8 @@ public:
 
     std::shared_ptr<StorageStats> getOperationStatistics() const override;
 
+    void refreshSnapshot() override;
+
     // ---- WT STUFF
 
     WiredTigerSession* getSession();
@@ -176,6 +171,10 @@ public:
         _isOplogReader = true;
     }
 
+    bool getIsOplogReader() const {
+        return _isOplogReader;
+    }
+
     /**
      * Enter a period of wait or computation during which there are no WT calls.
      * Any non-relevant cached handles can be closed.
@@ -205,54 +204,12 @@ public:
 
     static void appendGlobalStats(BSONObjBuilder& b);
 
-    /**
-     * State transitions:
-     *
-     *   /------------------------> Inactive <-----------------------------\
-     *   |                             |                                   |
-     *   |                             |                                   |
-     *   |              /--------------+--------------\                    |
-     *   |              |                             |                    | abandonSnapshot()
-     *   |              |                             |                    |
-     *   |   beginUOW() |                             | _txnOpen()         |
-     *   |              |                             |                    |
-     *   |              V                             V                    |
-     *   |    InactiveInUnitOfWork          ActiveNotInUnitOfWork ---------/
-     *   |              |                             |
-     *   |              |                             |
-     *   |   _txnOpen() |                             | beginUOW()
-     *   |              |                             |
-     *   |              \--------------+--------------/
-     *   |                             |
-     *   |                             |
-     *   |                             V
-     *   |                           Active
-     *   |                             |
-     *   |                             |
-     *   |              /--------------+--------------\
-     *   |              |                             |
-     *   |              |                             |
-     *   |   abortUOW() |                             | commitUOW()
-     *   |              |                             |
-     *   |              V                             V
-     *   |          Aborting                      Committing
-     *   |              |                             |
-     *   |              |                             |
-     *   |              |                             |
-     *   \--------------+-----------------------------/
-     *
-     */
-    enum class State {
-        kInactive,
-        kInactiveInUnitOfWork,
-        kActiveNotInUnitOfWork,
-        kActive,
-        kAborting,
-        kCommitting,
-    };
-    State getState_forTest() const;
-
 private:
+    void doCommitUnitOfWork() override;
+    void doAbortUnitOfWork() override;
+
+    void doAbandonSnapshot() override;
+
     void _abort();
     void _commit();
 
@@ -272,39 +229,24 @@ private:
      */
     Timestamp _beginTransactionAtNoOverlapTimestamp(WT_SESSION* session);
 
+    /**
+     * Starts a transaction at the lastApplied timestamp. Returns the timestamp at which the
+     * transaction was started.
+     */
+    Timestamp _beginTransactionAtLastAppliedTimestamp(WT_SESSION* session);
+
     /**
      * Returns the timestamp at which the current transaction is reading.
      */
     Timestamp _getTransactionReadTimestamp(WT_SESSION* session);
 
-    /**
-     * Transitions to new state.
-     */
-    void _setState(State newState);
-
-    /**
-     * Returns true if active.
-     */
-    bool _isActive() const;
-
-    /**
-     * Returns true if currently managed by a WriteUnitOfWork.
-     */
-    bool _inUnitOfWork() const;
-
-    /**
-     * Returns true if currently running commit or rollback handlers
-     */
-    bool _isCommittingOrAborting() const;
-
     WiredTigerSessionCache* _sessionCache;  // not owned
     WiredTigerOplogManager* _oplogManager;  // not owned
     UniqueWiredTigerSession _session;
-    State _state = State::kInactive;
     bool _isTimestamped = false;
 
     // Specifies which external source to use when setting read timestamps on transactions.
-    ReadSource _timestampReadSource = ReadSource::kUnset;
+    ReadSource _timestampReadSource = ReadSource::kNoTimestamp;
 
     // Commits are assumed ordered.  Unordered commits are assumed to always need to reserve a
     // new optime, and thus always call oplogDiskLocRegister() on the record store.
@@ -321,15 +263,12 @@ private:
     Timestamp _durableTimestamp;
     Timestamp _prepareTimestamp;
     boost::optional<Timestamp> _lastTimestampSet;
-    uint64_t _mySnapshotId;
     Timestamp _majorityCommittedSnapshot;
     Timestamp _readAtTimestamp;
     Timestamp _catalogConflictTimestamp;
     std::unique_ptr<Timer> _timer;
     bool _isOplogReader = false;
     boost::optional<int64_t> _oplogVisibleTs = boost::none;
-    typedef std::vector<std::unique_ptr<Change>> Changes;
-    Changes _changes;
 };
 
 }  // namespace mongo
