@@ -27,6 +27,8 @@
  *    it in the license file.
  */
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
+
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/ttl_collection_cache.h"
@@ -34,9 +36,13 @@
 #include <algorithm>
 
 #include "mongo/db/service_context.h"
+#include "mongo/logv2/log.h"
+#include "mongo/util/fail_point.h"
 
 namespace mongo {
 
+MONGO_FAIL_POINT_DEFINE(hangTTLCollectionCacheAfterRegisteringInfo);
+
 namespace {
 const auto getTTLCollectionCache = ServiceContext::declareDecoration<TTLCollectionCache>();
 }
@@ -45,20 +51,27 @@ TTLCollectionCache& TTLCollectionCache::get(ServiceContext* ctx) {
     return getTTLCollectionCache(ctx);
 }
 
-void TTLCollectionCache::registerCollection(const NamespaceString& collectionNS) {
-    stdx::lock_guard<Latch> lock(_ttlCollectionsLock);
-    _ttlCollections.push_back(collectionNS.ns());
+void TTLCollectionCache::registerTTLInfo(std::pair<UUID, std::string>&& ttlInfo) {
+    {
+        stdx::lock_guard<Latch> lock(_ttlInfosLock);
+        _ttlInfos.push_back(std::move(ttlInfo));
+    }
+
+    if (MONGO_unlikely(hangTTLCollectionCacheAfterRegisteringInfo.shouldFail())) {
+        LOGV2(4664000, "Hanging due to hangTTLCollectionCacheAfterRegisteringInfo fail point");
+        hangTTLCollectionCacheAfterRegisteringInfo.pauseWhileSet();
+    }
 }
 
-void TTLCollectionCache::unregisterCollection(const NamespaceString& collectionNS) {
-    stdx::lock_guard<Latch> lock(_ttlCollectionsLock);
-    auto collIter = std::find(_ttlCollections.begin(), _ttlCollections.end(), collectionNS.ns());
-    fassert(40220, collIter != _ttlCollections.end());
-    _ttlCollections.erase(collIter);
+void TTLCollectionCache::deregisterTTLInfo(const std::pair<UUID, std::string>& ttlInfo) {
+    stdx::lock_guard<Latch> lock(_ttlInfosLock);
+    auto collIter = std::find(_ttlInfos.begin(), _ttlInfos.end(), ttlInfo);
+    fassert(40220, collIter != _ttlInfos.end());
+    _ttlInfos.erase(collIter);
 }
 
-std::vector<std::string> TTLCollectionCache::getCollections() {
-    stdx::lock_guard<Latch> lock(_ttlCollectionsLock);
-    return _ttlCollections;
+std::vector<std::pair<UUID, std::string>> TTLCollectionCache::getTTLInfos() {
+    stdx::lock_guard<Latch> lock(_ttlInfosLock);
+    return _ttlInfos;
 }
 };  // namespace mongo
