@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -60,7 +59,7 @@ public:
     QueryStageSubplanTest() : _client(_opCtx.get()) {}
 
     virtual ~QueryStageSubplanTest() {
-        OldClientWriteContext ctx(opCtx(), nss.ns());
+        dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
         _client.dropCollection(nss.ns());
     }
 
@@ -68,6 +67,10 @@ public:
         ASSERT_OK(dbtests::createIndex(opCtx(), nss.ns(), obj));
     }
 
+    void dropIndex(BSONObj keyPattern) {
+        _client.dropIndex(nss.ns(), std::move(keyPattern));
+    }
+
     void insert(const BSONObj& doc) {
         _client.insert(nss.ns(), doc);
     }
@@ -76,6 +79,10 @@ public:
         return _opCtx.get();
     }
 
+    ExpressionContext* expCtx() {
+        return _expCtx.get();
+    }
+
     ServiceContext* serviceContext() {
         return _opCtx->getServiceContext();
     }
@@ -102,6 +109,8 @@ protected:
 
     const ServiceContext::UniqueOperationContext _opCtx = cc().makeOperationContext();
     ClockSource* _clock = _opCtx->getServiceContext()->getFastClockSource();
+    boost::intrusive_ptr<ExpressionContext> _expCtx =
+        make_intrusive<ExpressionContext>(_opCtx.get(), nullptr, nss);
 
 private:
     DBDirectClient _client;
@@ -114,11 +123,10 @@ private:
  * back to regular planning.
  */
 TEST_F(QueryStageSubplanTest, QueryStageSubplanGeo2dOr) {
-    OldClientWriteContext ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
     addIndex(BSON("a"
                   << "2d"
-                  << "b"
-                  << 1));
+                  << "b" << 1));
     addIndex(BSON("a"
                   << "2d"));
 
@@ -126,7 +134,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanGeo2dOr) {
         "{$or: [{a: {$geoWithin: {$centerSphere: [[0,0],10]}}},"
         "{a: {$geoWithin: {$centerSphere: [[1,1],10]}}}]}");
 
-    auto qr = stdx::make_unique<QueryRequest>(nss);
+    auto qr = std::make_unique<QueryRequest>(nss);
     qr->setFilter(query);
     auto statusWithCQ = CanonicalQuery::canonicalize(opCtx(), std::move(qr));
     ASSERT_OK(statusWithCQ.getStatus());
@@ -140,7 +148,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanGeo2dOr) {
 
     WorkingSet ws;
     std::unique_ptr<SubplanStage> subplan(
-        new SubplanStage(opCtx(), collection, &ws, plannerParams, cq.get()));
+        new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
 
     // Plan selection should succeed due to falling back on regular planning.
     PlanYieldPolicy yieldPolicy(PlanExecutor::NO_YIELD, _clock);
@@ -148,41 +156,41 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanGeo2dOr) {
 }
 
 /**
- * Test the SubplanStage's ability to plan an individual branch using the plan cache.
+ * Helper function to verify that branches of an $or can be subplanned from the cache. Assumes that
+ * the indexes to be tested have already been created for the given WriteContextForTest.
  */
-TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanFromCache) {
-    OldClientWriteContext ctx(opCtx(), nss.ns());
-
-    addIndex(BSON("a" << 1));
-    addIndex(BSON("a" << 1 << "b" << 1));
-    addIndex(BSON("c" << 1));
-
-    for (int i = 0; i < 10; i++) {
-        insert(BSON("a" << 1 << "b" << i << "c" << i));
-    }
-
+void assertSubplanFromCache(QueryStageSubplanTest* test, const dbtests::WriteContextForTests& ctx) {
     // This query should result in a plan cache entry for the first $or branch, because
     // there are two competing indices. The second branch has only one relevant index, so
     // its winning plan should not be cached.
     BSONObj query = fromjson("{$or: [{a: 1, b: 3}, {c: 1}]}");
 
+    for (int i = 0; i < 10; i++) {
+        test->insert(BSON("a" << 1 << "b" << i << "c" << i));
+    }
+
     Collection* collection = ctx.getCollection();
 
-    auto qr = stdx::make_unique<QueryRequest>(nss);
+    auto qr = std::make_unique<QueryRequest>(nss);
     qr->setFilter(query);
-    auto statusWithCQ = CanonicalQuery::canonicalize(opCtx(), std::move(qr));
+    auto statusWithCQ = CanonicalQuery::canonicalize(test->opCtx(), std::move(qr));
     ASSERT_OK(statusWithCQ.getStatus());
     std::unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
 
     // Get planner params.
     QueryPlannerParams plannerParams;
-    fillOutPlannerParams(opCtx(), collection, cq.get(), &plannerParams);
+    fillOutPlannerParams(test->opCtx(), collection, cq.get(), &plannerParams);
+
+    // For the remainder of this test, ensure that cache entries are available immediately, and
+    // don't need go through an 'inactive' state before being usable.
+    internalQueryCacheDisableInactiveEntries.store(true);
 
     WorkingSet ws;
     std::unique_ptr<SubplanStage> subplan(
-        new SubplanStage(opCtx(), collection, &ws, plannerParams, cq.get()));
+        new SubplanStage(test->expCtx(), collection, &ws, plannerParams, cq.get()));
 
-    PlanYieldPolicy yieldPolicy(PlanExecutor::NO_YIELD, _clock);
+    PlanYieldPolicy yieldPolicy(PlanExecutor::NO_YIELD,
+                                test->serviceContext()->getFastClockSource());
     ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
 
     // Nothing is in the cache yet, so neither branch should have been planned from
@@ -193,7 +201,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanFromCache) {
     // If we repeat the same query, the plan for the first branch should have come from
     // the cache.
     ws.clear();
-    subplan.reset(new SubplanStage(opCtx(), collection, &ws, plannerParams, cq.get()));
+    subplan.reset(new SubplanStage(test->expCtx(), collection, &ws, plannerParams, cq.get()));
 
     ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
 
@@ -201,15 +209,41 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanFromCache) {
     ASSERT_FALSE(subplan->branchPlannedFromCache(1));
 }
 
+/**
+ * Test the SubplanStage's ability to plan an individual branch using the plan cache.
+ */
+TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanFromCache) {
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+
+    addIndex(BSON("a" << 1));
+    addIndex(BSON("a" << 1 << "b" << 1));
+    addIndex(BSON("c" << 1));
+
+    assertSubplanFromCache(this, ctx);
+}
+
+/**
+ * Test that the SubplanStage can plan an individual branch from the cache using a $** index.
+ */
+TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanFromCacheWithWildcardIndex) {
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
+
+    addIndex(BSON("$**" << 1));
+    addIndex(BSON("a.$**" << 1));
+
+    assertSubplanFromCache(this, ctx);
+}
+
 /**
  * Ensure that the subplan stage doesn't create a plan cache entry if there are no query results.
  */
 TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheZeroResults) {
-    OldClientWriteContext ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
 
     addIndex(BSON("a" << 1 << "b" << 1));
     addIndex(BSON("a" << 1));
     addIndex(BSON("c" << 1));
+    addIndex(BSON("$**" << 1));
 
     for (int i = 0; i < 10; i++) {
         insert(BSON("a" << 1 << "b" << i << "c" << i));
@@ -222,7 +256,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheZeroResults) {
 
     Collection* collection = ctx.getCollection();
 
-    auto qr = stdx::make_unique<QueryRequest>(nss);
+    auto qr = std::make_unique<QueryRequest>(nss);
     qr->setFilter(query);
     auto statusWithCQ = CanonicalQuery::canonicalize(opCtx(), std::move(qr));
     ASSERT_OK(statusWithCQ.getStatus());
@@ -234,7 +268,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheZeroResults) {
 
     WorkingSet ws;
     std::unique_ptr<SubplanStage> subplan(
-        new SubplanStage(opCtx(), collection, &ws, plannerParams, cq.get()));
+        new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
 
     PlanYieldPolicy yieldPolicy(PlanExecutor::NO_YIELD, _clock);
     ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
@@ -248,7 +282,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheZeroResults) {
     // from the cache (because the first call to pickBestPlan() refrained from creating any
     // cache entries).
     ws.clear();
-    subplan.reset(new SubplanStage(opCtx(), collection, &ws, plannerParams, cq.get()));
+    subplan.reset(new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
 
     ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
 
@@ -257,14 +291,15 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheZeroResults) {
 }
 
 /**
- * Ensure that the subplan stage doesn't create a plan cache entry if there are no query results.
+ * Ensure that the subplan stage doesn't create a plan cache entry if the candidate plans tie.
  */
 TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheTies) {
-    OldClientWriteContext ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
 
     addIndex(BSON("a" << 1 << "b" << 1));
     addIndex(BSON("a" << 1 << "c" << 1));
     addIndex(BSON("d" << 1));
+    addIndex(BSON("$**" << 1));
 
     for (int i = 0; i < 10; i++) {
         insert(BSON("a" << 1 << "e" << 1 << "d" << 1));
@@ -277,7 +312,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheTies) {
 
     Collection* collection = ctx.getCollection();
 
-    auto qr = stdx::make_unique<QueryRequest>(nss);
+    auto qr = std::make_unique<QueryRequest>(nss);
     qr->setFilter(query);
     auto statusWithCQ = CanonicalQuery::canonicalize(opCtx(), std::move(qr));
     ASSERT_OK(statusWithCQ.getStatus());
@@ -289,7 +324,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheTies) {
 
     WorkingSet ws;
     std::unique_ptr<SubplanStage> subplan(
-        new SubplanStage(opCtx(), collection, &ws, plannerParams, cq.get()));
+        new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
 
     PlanYieldPolicy yieldPolicy(PlanExecutor::NO_YIELD, _clock);
     ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
@@ -303,7 +338,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanDontCacheTies) {
     // from the cache (because the first call to pickBestPlan() refrained from creating any
     // cache entries).
     ws.clear();
-    subplan.reset(new SubplanStage(opCtx(), collection, &ws, plannerParams, cq.get()));
+    subplan.reset(new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
 
     ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
 
@@ -439,7 +474,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanCanUseSubplanning) {
  * Regression test for SERVER-19388.
  */
 TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanRootedOrNE) {
-    OldClientWriteContext ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
     addIndex(BSON("a" << 1 << "b" << 1));
     addIndex(BSON("a" << 1 << "c" << 1));
 
@@ -449,7 +484,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanRootedOrNE) {
     insert(BSON("_id" << 3 << "a" << 3));
     insert(BSON("_id" << 4));
 
-    auto qr = stdx::make_unique<QueryRequest>(nss);
+    auto qr = std::make_unique<QueryRequest>(nss);
     qr->setFilter(fromjson("{$or: [{a: 1}, {a: {$ne:1}}]}"));
     qr->setSort(BSON("d" << 1));
     auto cq = unittest::assertGet(CanonicalQuery::canonicalize(opCtx(), std::move(qr)));
@@ -461,7 +496,7 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanRootedOrNE) {
 
     WorkingSet ws;
     std::unique_ptr<SubplanStage> subplan(
-        new SubplanStage(opCtx(), collection, &ws, plannerParams, cq.get()));
+        new SubplanStage(_expCtx.get(), collection, &ws, plannerParams, cq.get()));
 
     PlanYieldPolicy yieldPolicy(PlanExecutor::NO_YIELD, _clock);
     ASSERT_OK(subplan->pickBestPlan(&yieldPolicy));
@@ -471,7 +506,6 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanRootedOrNE) {
     while (stageState != PlanStage::IS_EOF) {
         WorkingSetID id = WorkingSet::INVALID_ID;
         stageState = subplan->work(&id);
-        ASSERT_NE(stageState, PlanStage::DEAD);
         ASSERT_NE(stageState, PlanStage::FAILURE);
         if (stageState == PlanStage::ADVANCED) {
             ++numResults;
@@ -482,9 +516,9 @@ TEST_F(QueryStageSubplanTest, QueryStageSubplanPlanRootedOrNE) {
 }
 
 TEST_F(QueryStageSubplanTest, ShouldReportErrorIfExceedsTimeLimitDuringPlanning) {
-    OldClientWriteContext ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
     // Build a query with a rooted $or.
-    auto queryRequest = stdx::make_unique<QueryRequest>(nss);
+    auto queryRequest = std::make_unique<QueryRequest>(nss);
     queryRequest->setFilter(BSON("$or" << BSON_ARRAY(BSON("p1" << 1) << BSON("p2" << 2))));
     auto canonicalQuery =
         uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(queryRequest)));
@@ -508,16 +542,16 @@ TEST_F(QueryStageSubplanTest, ShouldReportErrorIfExceedsTimeLimitDuringPlanning)
     // Create the SubplanStage.
     WorkingSet workingSet;
     SubplanStage subplanStage(
-        opCtx(), ctx.getCollection(), &workingSet, params, canonicalQuery.get());
+        _expCtx.get(), ctx.getCollection(), &workingSet, params, canonicalQuery.get());
 
     AlwaysTimeOutYieldPolicy alwaysTimeOutPolicy(serviceContext()->getFastClockSource());
     ASSERT_EQ(ErrorCodes::ExceededTimeLimit, subplanStage.pickBestPlan(&alwaysTimeOutPolicy));
 }
 
 TEST_F(QueryStageSubplanTest, ShouldReportErrorIfKilledDuringPlanning) {
-    OldClientWriteContext ctx(opCtx(), nss.ns());
+    dbtests::WriteContextForTests ctx(opCtx(), nss.ns());
     // Build a query with a rooted $or.
-    auto queryRequest = stdx::make_unique<QueryRequest>(nss);
+    auto queryRequest = std::make_unique<QueryRequest>(nss);
     queryRequest->setFilter(BSON("$or" << BSON_ARRAY(BSON("p1" << 1) << BSON("p2" << 2))));
     auto canonicalQuery =
         uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(queryRequest)));
@@ -533,11 +567,102 @@ TEST_F(QueryStageSubplanTest, ShouldReportErrorIfKilledDuringPlanning) {
     // Create the SubplanStage.
     WorkingSet workingSet;
     SubplanStage subplanStage(
-        opCtx(), ctx.getCollection(), &workingSet, params, canonicalQuery.get());
+        _expCtx.get(), ctx.getCollection(), &workingSet, params, canonicalQuery.get());
 
     AlwaysPlanKilledYieldPolicy alwaysPlanKilledYieldPolicy(serviceContext()->getFastClockSource());
     ASSERT_EQ(ErrorCodes::QueryPlanKilled, subplanStage.pickBestPlan(&alwaysPlanKilledYieldPolicy));
 }
 
+TEST_F(QueryStageSubplanTest, ShouldThrowOnRestoreIfIndexDroppedBeforePlanSelection) {
+    Collection* collection = nullptr;
+    {
+        dbtests::WriteContextForTests ctx{opCtx(), nss.ns()};
+        addIndex(BSON("p1" << 1 << "opt1" << 1));
+        addIndex(BSON("p1" << 1 << "opt2" << 1));
+        addIndex(BSON("p2" << 1 << "opt1" << 1));
+        addIndex(BSON("p2" << 1 << "opt2" << 1));
+        addIndex(BSON("irrelevant" << 1));
+
+        collection = ctx.getCollection();
+        ASSERT(collection);
+    }
+
+    // Build a query with a rooted $or.
+    auto queryRequest = std::make_unique<QueryRequest>(nss);
+    queryRequest->setFilter(BSON("$or" << BSON_ARRAY(BSON("p1" << 1) << BSON("p2" << 2))));
+    auto canonicalQuery =
+        uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(queryRequest)));
+
+    // Add 4 indices: 2 for each predicate to choose from, and one index which is not relevant to
+    // the query.
+    QueryPlannerParams params;
+    fillOutPlannerParams(opCtx(), collection, canonicalQuery.get(), &params);
+
+    boost::optional<AutoGetCollectionForReadCommand> collLock;
+    collLock.emplace(opCtx(), nss);
+
+    // Create the SubplanStage.
+    WorkingSet workingSet;
+    SubplanStage subplanStage(_expCtx.get(), collection, &workingSet, params, canonicalQuery.get());
+
+    // Mimic a yield by saving the state of the subplan stage. Then, drop an index not being used
+    // while yielded.
+    subplanStage.saveState();
+    collLock.reset();
+    dropIndex(BSON("irrelevant" << 1));
+
+    // Attempt to restore state. This should throw due the index drop. As a future improvement, we
+    // may wish to make the subplan stage tolerate drops of indices it is not using.
+    collLock.emplace(opCtx(), nss);
+    ASSERT_THROWS_CODE(subplanStage.restoreState(), DBException, ErrorCodes::QueryPlanKilled);
+}
+
+TEST_F(QueryStageSubplanTest, ShouldNotThrowOnRestoreIfIndexDroppedAfterPlanSelection) {
+    Collection* collection = nullptr;
+    {
+        dbtests::WriteContextForTests ctx{opCtx(), nss.ns()};
+        addIndex(BSON("p1" << 1 << "opt1" << 1));
+        addIndex(BSON("p1" << 1 << "opt2" << 1));
+        addIndex(BSON("p2" << 1 << "opt1" << 1));
+        addIndex(BSON("p2" << 1 << "opt2" << 1));
+        addIndex(BSON("irrelevant" << 1));
+
+        collection = ctx.getCollection();
+        ASSERT(collection);
+    }
+
+    // Build a query with a rooted $or.
+    auto queryRequest = std::make_unique<QueryRequest>(nss);
+    queryRequest->setFilter(BSON("$or" << BSON_ARRAY(BSON("p1" << 1) << BSON("p2" << 2))));
+    auto canonicalQuery =
+        uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(queryRequest)));
+
+    // Add 4 indices: 2 for each predicate to choose from, and one index which is not relevant to
+    // the query.
+    QueryPlannerParams params;
+    fillOutPlannerParams(opCtx(), collection, canonicalQuery.get(), &params);
+
+    boost::optional<AutoGetCollectionForReadCommand> collLock;
+    collLock.emplace(opCtx(), nss);
+
+    // Create the SubplanStage.
+    WorkingSet workingSet;
+    SubplanStage subplanStage(_expCtx.get(), collection, &workingSet, params, canonicalQuery.get());
+
+    PlanYieldPolicy yieldPolicy(PlanExecutor::YIELD_MANUAL, serviceContext()->getFastClockSource());
+    ASSERT_OK(subplanStage.pickBestPlan(&yieldPolicy));
+
+    // Mimic a yield by saving the state of the subplan stage and dropping our lock. Then drop an
+    // index not being used while yielded.
+    subplanStage.saveState();
+    collLock.reset();
+    dropIndex(BSON("irrelevant" << 1));
+
+    // Restoring state should succeed, since the plan selected by pickBestPlan() does not use the
+    // index {irrelevant: 1}.
+    collLock.emplace(opCtx(), nss);
+    subplanStage.restoreState();
+}
+
 }  // namespace
 }  // namespace mongo
