@@ -237,7 +237,9 @@ ServiceContext* initialize(const char* yaml_config) {
     serviceContext->setPeriodicRunner(std::move(periodicRunner));
 
     setUpCatalog(serviceContext);
-    initializeStorageEngine(serviceContext, StorageEngineInitFlags::kAllowNoLockFile);
+    auto lastStorageEngineShutdownState =
+        initializeStorageEngine(serviceContext, StorageEngineInitFlags::kAllowNoLockFile);
+    invariant(LastStorageEngineShutdownState::kClean == lastStorageEngineShutdownState);
 
     // Warn if we detect configurations for multiple registered storage engines in the same
     // configuration file/environment.
@@ -307,6 +309,10 @@ ServiceContext* initialize(const char* yaml_config) {
         invariant(serverGlobalParams.featureCompatibility.isVersionInitialized());
     }
 
+    // Notify the storage engine that startup is completed before repair exits below, as repair sets
+    // the upgrade flag to true.
+    serviceContext->getStorageEngine()->notifyStartupComplete();
+
     if (storageGlobalParams.upgrade) {
         LOGV2(22553, "finished checking dbs");
         exitCleanly(EXIT_CLEAN);
