@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -28,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kExecutor
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
@@ -36,26 +35,24 @@
 
 #include <memory>
 
-#include "mongo/base/disallow_copying.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/executor/network_interface.h"
 #include "mongo/executor/network_interface_mock.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/executor/task_executor_test_fixture.h"
-#include "mongo/stdx/memory.h"
+#include "mongo/logv2/log.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/clock_source_mock.h"
-#include "mongo/util/log.h"
-#include "mongo/util/mongoutils/str.h"
+#include "mongo/util/str.h"
 
 namespace mongo {
 namespace executor {
 namespace {
 
 using ExecutorFactory =
-    stdx::function<std::unique_ptr<TaskExecutor>(std::unique_ptr<NetworkInterfaceMock>)>;
+    std::function<std::unique_ptr<TaskExecutor>(std::unique_ptr<NetworkInterfaceMock>)>;
 
 class CommonTaskExecutorTestFixture : public TaskExecutorTest {
 public:
@@ -72,7 +69,7 @@ private:
 };
 
 using ExecutorTestCaseFactory =
-    stdx::function<std::unique_ptr<CommonTaskExecutorTestFixture>(ExecutorFactory)>;
+    std::function<std::unique_ptr<CommonTaskExecutorTestFixture>(ExecutorFactory)>;
 using ExecutorTestCaseMap = stdx::unordered_map<std::string, ExecutorTestCaseFactory>;
 
 static ExecutorTestCaseMap& executorTestCaseRegistry() {
@@ -81,33 +78,36 @@ static ExecutorTestCaseMap& executorTestCaseRegistry() {
 }
 
 class CetRegistrationAgent {
-    MONGO_DISALLOW_COPYING(CetRegistrationAgent);
+    CetRegistrationAgent(const CetRegistrationAgent&) = delete;
+    CetRegistrationAgent& operator=(const CetRegistrationAgent&) = delete;
 
 public:
     CetRegistrationAgent(const std::string& name, ExecutorTestCaseFactory makeTest) {
         auto& entry = executorTestCaseRegistry()[name];
         if (entry) {
-            severe() << "Multiple attempts to register ExecutorTest named " << name;
-            fassertFailed(28713);
+            LOGV2_FATAL(28713,
+                        "Multiple attempts to register ExecutorTest named {executor}",
+                        "Multiple attempts to register ExecutorTest",
+                        "executor"_attr = name);
         }
         entry = std::move(makeTest);
     }
 };
 
-#define COMMON_EXECUTOR_TEST(TEST_NAME)                                         \
-    class CET_##TEST_NAME : public CommonTaskExecutorTestFixture {              \
-    public:                                                                     \
-        CET_##TEST_NAME(ExecutorFactory makeExecutor)                           \
-            : CommonTaskExecutorTestFixture(std::move(makeExecutor)) {}         \
-                                                                                \
-    private:                                                                    \
-        void _doTest() override;                                                \
-        static const CetRegistrationAgent _agent;                               \
-    };                                                                          \
-    const CetRegistrationAgent CET_##TEST_NAME::_agent(                         \
-        #TEST_NAME, [](ExecutorFactory makeExecutor) {                          \
-            return stdx::make_unique<CET_##TEST_NAME>(std::move(makeExecutor)); \
-        });                                                                     \
+#define COMMON_EXECUTOR_TEST(TEST_NAME)                                        \
+    class CET_##TEST_NAME : public CommonTaskExecutorTestFixture {             \
+    public:                                                                    \
+        CET_##TEST_NAME(ExecutorFactory makeExecutor)                          \
+            : CommonTaskExecutorTestFixture(std::move(makeExecutor)) {}        \
+                                                                               \
+    private:                                                                   \
+        void _doTest() override;                                               \
+        static const CetRegistrationAgent _agent;                              \
+    };                                                                         \
+    const CetRegistrationAgent CET_##TEST_NAME::_agent(                        \
+        #TEST_NAME, [](ExecutorFactory makeExecutor) {                         \
+            return std::make_unique<CET_##TEST_NAME>(std::move(makeExecutor)); \
+        });                                                                    \
     void CET_##TEST_NAME::_doTest()
 
 auto makeSetStatusClosure(Status* target) {
@@ -148,14 +148,12 @@ auto makeSetStatusOnRemoteCommandCompletionClosure(const RemoteCommandRequest* e
     return [=](const TaskExecutor::RemoteCommandCallbackArgs& cbData) {
         if (cbData.request != *expectedRequest) {
             auto desc = [](const RemoteCommandRequest& request) -> std::string {
-                return mongoutils::str::stream() << "Request(" << request.target.toString() << ", "
-                                                 << request.dbname << ", " << request.cmdObj << ')';
+                return str::stream() << "Request(" << request.target.toString() << ", "
+                                     << request.dbname << ", " << request.cmdObj << ')';
             };
-            *outStatus =
-                Status(ErrorCodes::BadValue,
-                       mongoutils::str::stream() << "Actual request: " << desc(cbData.request)
-                                                 << "; expected: "
-                                                 << desc(*expectedRequest));
+            *outStatus = Status(ErrorCodes::BadValue,
+                                str::stream() << "Actual request: " << desc(cbData.request)
+                                              << "; expected: " << desc(*expectedRequest));
             return;
         }
         *outStatus = cbData.response.status;
@@ -171,16 +169,6 @@ COMMON_EXECUTOR_TEST(RunOne) {
     ASSERT_OK(status);
 }
 
-COMMON_EXECUTOR_TEST(Schedule1ButShutdown) {
-    TaskExecutor& executor = getExecutor();
-    Status status = getDetectableErrorStatus();
-    ASSERT_OK(executor.scheduleWork(makeSetStatusAndShutdownClosure(&status)).getStatus());
-    executor.shutdown();
-    launchExecutorThread();
-    joinExecutorThread();
-    ASSERT_EQUALS(status, ErrorCodes::CallbackCanceled);
-}
-
 COMMON_EXECUTOR_TEST(Schedule2Cancel1) {
     TaskExecutor& executor = getExecutor();
     Status status1 = getDetectableErrorStatus();
@@ -208,7 +196,8 @@ COMMON_EXECUTOR_TEST(OneSchedulesAnother) {
 }
 
 class EventChainAndWaitingTest {
-    MONGO_DISALLOW_COPYING(EventChainAndWaitingTest);
+    EventChainAndWaitingTest(const EventChainAndWaitingTest&) = delete;
+    EventChainAndWaitingTest& operator=(const EventChainAndWaitingTest&) = delete;
 
 public:
     EventChainAndWaitingTest(TaskExecutor* exec, NetworkInterfaceMock* network);
@@ -307,13 +296,14 @@ void EventChainAndWaitingTest::onGo(const TaskExecutor::CallbackArgs& cbData) {
         return;
     }
     triggerEvent = errorOrTriggerEvent.getValue();
-    StatusWith<TaskExecutor::CallbackHandle> cbHandle = executor->onEvent(triggerEvent, triggered2);
+    StatusWith<TaskExecutor::CallbackHandle> cbHandle =
+        executor->onEvent(triggerEvent, std::move(triggered2));
     if (!cbHandle.isOK()) {
         status1 = cbHandle.getStatus();
         executor->shutdown();
         return;
     }
-    cbHandle = executor->onEvent(triggerEvent, triggered3);
+    cbHandle = executor->onEvent(triggerEvent, std::move(triggered3));
     if (!cbHandle.isOK()) {
         status1 = cbHandle.getStatus();
         executor->shutdown();
@@ -346,7 +336,7 @@ COMMON_EXECUTOR_TEST(EventWaitingWithTimeoutTest) {
 
     auto serviceContext = ServiceContext::make();
 
-    serviceContext->setFastClockSource(stdx::make_unique<ClockSourceMock>());
+    serviceContext->setFastClockSource(std::make_unique<ClockSourceMock>());
     auto mockClock = static_cast<ClockSourceMock*>(serviceContext->getFastClockSource());
 
     auto client = serviceContext->makeClient("for testing");
@@ -368,7 +358,7 @@ COMMON_EXECUTOR_TEST(EventSignalWithTimeoutTest) {
 
     auto serviceContext = ServiceContext::make();
 
-    serviceContext->setFastClockSource(stdx::make_unique<ClockSourceMock>());
+    serviceContext->setFastClockSource(std::make_unique<ClockSourceMock>());
     auto mockClock = static_cast<ClockSourceMock*>(serviceContext->getFastClockSource());
 
     auto client = serviceContext->makeClient("for testing");
@@ -486,7 +476,7 @@ COMMON_EXECUTOR_TEST(RemoteCommandWithTimeout) {
     ASSERT_EQUALS(startTime + Milliseconds(2), net->now());
     net->exitNetwork();
     executor.wait(cbHandle);
-    ASSERT_EQUALS(ErrorCodes::NetworkTimeout, status);
+    ASSERT_EQUALS(ErrorCodes::NetworkInterfaceExceededTimeLimit, status);
 }
 
 COMMON_EXECUTOR_TEST(CallbackHandleComparison) {
@@ -534,10 +524,11 @@ COMMON_EXECUTOR_TEST(CallbackHandleComparison) {
 }  // namespace
 
 void addTestsForExecutor(const std::string& suiteName, ExecutorFactory makeExecutor) {
-    auto suite = unittest::Suite::getSuite(suiteName);
+    auto& suite = unittest::Suite::getSuite(suiteName);
     for (auto testCase : executorTestCaseRegistry()) {
-        suite->add(str::stream() << suiteName << "::" << testCase.first,
-                   [testCase, makeExecutor] { testCase.second(makeExecutor)->run(); });
+        suite.add(str::stream() << suiteName << "::" << testCase.first,
+                  __FILE__,
+                  [testCase, makeExecutor] { testCase.second(makeExecutor)->run(); });
     }
 }
 
