@@ -29,15 +29,15 @@
 
 #pragma once
 
+#include <list>
 #include <memory>
 
 #include "mongo/executor/task_executor.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
-#include "mongo/stdx/list.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/transport/baton.h"
-#include "mongo/util/fail_point_service.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/hierarchical_acquisition.h"
 
 namespace mongo {
@@ -45,8 +45,6 @@ namespace mongo {
 class ThreadPoolInterface;
 
 namespace executor {
-MONGO_FAIL_POINT_DECLARE(initialSyncFuzzerSynchronizationPoint1);
-MONGO_FAIL_POINT_DECLARE(initialSyncFuzzerSynchronizationPoint2);
 
 struct ConnectionPoolStats;
 class NetworkInterface;
@@ -74,6 +72,7 @@ public:
     void startup() override;
     void shutdown() override;
     void join() override;
+    SharedSemiFuture<void> joinAsync() override;
     void appendDiagnosticBSON(BSONObjBuilder* b) const override;
     Date_t now() override;
     StatusWith<EventHandle> makeEvent() override;
@@ -89,6 +88,10 @@ public:
         const RemoteCommandRequestOnAny& request,
         const RemoteCommandOnAnyCallbackFn& cb,
         const BatonHandle& baton = nullptr) override;
+    StatusWith<CallbackHandle> scheduleExhaustRemoteCommandOnAny(
+        const RemoteCommandRequestOnAny& request,
+        const RemoteCommandOnAnyCallbackFn& cb,
+        const BatonHandle& baton = nullptr);
     void cancel(const CallbackHandle& cbHandle) override;
     void wait(const CallbackHandle& cbHandle,
               Interruptible* interruptible = Interruptible::notInterruptible()) override;
@@ -100,11 +103,17 @@ public:
      */
     void dropConnections(const HostAndPort& hostAndPort);
 
+    /**
+     * Returns true if there are any tasks in any of _poolInProgressQueue, _networkInProgressQueue,
+     * or _sleepersQueue.
+     */
+    bool hasTasks();
+
 private:
     class CallbackState;
     class EventState;
-    using WorkQueue = stdx::list<std::shared_ptr<CallbackState>>;
-    using EventList = stdx::list<std::shared_ptr<EventState>>;
+    using WorkQueue = std::list<std::shared_ptr<CallbackState>>;
+    using EventList = std::list<std::shared_ptr<EventState>>;
 
     /**
      * Representation of the stage of life of a thread pool.
@@ -175,11 +184,23 @@ private:
                                  const WorkQueue::iterator& end,
                                  stdx::unique_lock<Latch> lk);
 
+    /**
+     * Schedules cbState into the thread pool and places it into _poolInProgressQueue. Does not
+     * remove the entry from the original queue.
+     */
+    void scheduleExhaustIntoPool_inlock(std::shared_ptr<CallbackState> cbState,
+                                        stdx::unique_lock<Latch> lk);
     /**
      * Executes the callback specified by "cbState".
      */
     void runCallback(std::shared_ptr<CallbackState> cbState);
 
+    /**
+     * Executes the callback specified by "cbState". Will not mark cbState as finished.
+     */
+    void runCallbackExhaust(std::shared_ptr<CallbackState> cbState,
+                            WorkQueue::iterator expectedExhaustIter);
+
     bool _inShutdown_inlock() const;
     void _setState_inlock(State newState);
     stdx::unique_lock<Latch> _join(stdx::unique_lock<Latch> lk);
