@@ -231,10 +231,8 @@ std::vector<AsyncRequestsSender::Response> scatterGatherOnlyVersionIfUnsharded(
     const std::set<ErrorCodes::Error>& ignorableErrors = {});
 
 /**
- * Utility for dispatching commands against the primary of a database and attach the appropriate
- * database version.
- *
- * Does not retry on StaleDbVersion.
+ * Utility for dispatching commands against the primary of a database and attaching the appropriate
+ * database version. Does not retry on StaleDbVersion.
  */
 AsyncRequestsSender::Response executeCommandAgainstDatabasePrimary(
     OperationContext* opCtx,
@@ -244,6 +242,18 @@ AsyncRequestsSender::Response executeCommandAgainstDatabasePrimary(
     const ReadPreferenceSetting& readPref,
     Shard::RetryPolicy retryPolicy);
 
+/**
+ * Utility for dispatching commands against the primary of a database. Does not attach a database or
+ * shard version to the command object, but instead issues it exactly as provided. Does not retry.
+ */
+AsyncRequestsSender::Response executeRawCommandAgainstDatabasePrimary(
+    OperationContext* opCtx,
+    StringData dbName,
+    const CachedDatabaseInfo& dbInfo,
+    const BSONObj& cmdObj,
+    const ReadPreferenceSetting& readPref,
+    Shard::RetryPolicy retryPolicy);
+
 /**
  * Utility for dispatching commands against the shard with the MinKey chunk for the namespace and
  * attaching the appropriate shard version.
