@@ -284,7 +284,7 @@ assert = (function() {
 
         if (count != arr.length) {
             doassert(_buildAssertionMessage(
-                msg, "None of values from " + tojson(arr) + " was in " + tojson(result)));
+                msg, "Not all of the values from " + tojson(arr) + " were in " + tojson(result)));
         }
     };
 
@@ -311,7 +311,7 @@ assert = (function() {
      * or more than 'timeout' milliseconds have elapsed. Throws an exception with
      * message 'msg' after timing out.
      */
-    assert.soon = function(func, msg, timeout, interval) {
+    assert.soon = function(func, msg, timeout, interval, {runHangAnalyzer = true} = {}) {
         _validateAssertionMessage(msg);
 
         var msgPrefix = "assert.soon failed: " + func;
@@ -323,9 +323,15 @@ assert = (function() {
         }
 
         var start = new Date();
-        timeout = timeout || 5 * 60 * 1000;
+
+        if (TestData && TestData.inEvergreen) {
+            timeout = timeout || 10 * 60 * 1000;
+        } else {
+            timeout = timeout || 90 * 1000;
+        }
+
         interval = interval || 200;
-        var last;
+
         while (1) {
             if (typeof (func) == "string") {
                 if (eval(func))
@@ -337,9 +343,20 @@ assert = (function() {
 
             diff = (new Date()).getTime() - start.getTime();
             if (diff > timeout) {
-                doassert(_buildAssertionMessage(msg, msgPrefix));
+                msg = _buildAssertionMessage(msg, msgPrefix);
+                if (runHangAnalyzer) {
+                    msg = msg +
+                        " The hang analyzer is automatically called in assert.soon functions." +
+                        " If you are *expecting* assert.soon to possibly fail, call assert.soon" +
+                        " with {runHangAnalyzer: false} as the fifth argument" +
+                        " (you can fill unused arguments with `undefined`).";
+                    print(msg + " Running hang analyzer from assert.soon.");
+                    MongoRunner.runHangAnalyzer();
+                }
+                doassert(msg);
+            } else {
+                sleep(interval);
             }
-            sleep(interval);
         }
     };
 
@@ -351,7 +368,28 @@ assert = (function() {
     assert.soonNoExcept = function(func, msg, timeout, interval) {
         var safeFunc =
             _convertExceptionToReturnStatus(func, "assert.soonNoExcept caught exception");
-        assert.soon(safeFunc, msg, timeout, interval);
+        var getFunc = () => {
+            // No TestData means not running from resmoke. Non-resmoke tests usually don't trace
+            // exceptions.
+            if (typeof TestData === "undefined") {
+                return safeFunc;
+            }
+            return () => {
+                // Turns off printing the JavaScript stacktrace in doassert() to avoid
+                // generating an overwhelming amount of log messages when handling transient
+                // errors.
+                const origTraceExceptions = TestData.traceExceptions;
+                TestData.traceExceptions = false;
+
+                const res = safeFunc();
+
+                // Restore it's value to original value.
+                TestData.traceExceptions = origTraceExceptions;
+                return res;
+            };
+        };
+
+        assert.soon(getFunc(), msg, timeout, interval);
     };
 
     /*
@@ -361,7 +399,7 @@ assert = (function() {
      * message 'msg' after all attempts are used up. If no 'intervalMS' argument is passed,
      * it defaults to 0.
      */
-    assert.retry = function(func, msg, num_attempts, intervalMS) {
+    assert.retry = function(func, msg, num_attempts, intervalMS, {runHangAnalyzer = true} = {}) {
         var intervalMS = intervalMS || 0;
         var attempts_made = 0;
         while (attempts_made < num_attempts) {
@@ -374,6 +412,15 @@ assert = (function() {
             }
         }
         // Used up all attempts
+        msg = _buildAssertionMessage(msg);
+        if (runHangAnalyzer) {
+            msg = msg + "The hang analyzer is automatically called in assert.retry functions. " +
+                "If you are *expecting* assert.soon to possibly fail, call assert.retry " +
+                "with {runHangAnalyzer: false} as the fifth argument " +
+                "(you can fill unused arguments with `undefined`).";
+            print(msg + " Running hang analyzer from assert.retry.");
+            MongoRunner.runHangAnalyzer();
+        }
         doassert(msg);
     };
 
@@ -412,7 +459,7 @@ assert = (function() {
         return res;
     };
 
-    assert.time = function(f, msg, timeout /*ms*/) {
+    assert.time = function(f, msg, timeout /*ms*/, {runHangAnalyzer = true} = {}) {
         _validateAssertionMessage(msg);
 
         var start = new Date();
@@ -427,7 +474,16 @@ assert = (function() {
         if (diff > timeout) {
             const msgPrefix =
                 "assert.time failed timeout " + timeout + "ms took " + diff + "ms : " + f + ", msg";
-            doassert(_buildAssertionMessage(msg, msgPrefix));
+            msg = _buildAssertionMessage(msg, msgPrefix);
+            if (runHangAnalyzer) {
+                msg = msg + "The hang analyzer is automatically called in assert.time functions. " +
+                    "If you are *expecting* assert.soon to possibly fail, call assert.time " +
+                    "with {runHangAnalyzer: false} as the fourth argument " +
+                    "(you can fill unused arguments with `undefined`).";
+                print(msg + " Running hang analyzer from assert.time.");
+                MongoRunner.runHangAnalyzer();
+            }
+            doassert(msg);
         }
         return res;
     };
