@@ -27,20 +27,22 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kNetwork
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/transport/service_state_machine.h"
 
+#include <memory>
+
 #include "mongo/config.h"
 #include "mongo/db/client.h"
 #include "mongo/db/dbmessage.h"
 #include "mongo/db/stats/counters.h"
 #include "mongo/db/traffic_recorder.h"
+#include "mongo/logv2/log.h"
 #include "mongo/rpc/message.h"
 #include "mongo/rpc/op_msg.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/transport/message_compressor_manager.h"
 #include "mongo/transport/service_entry_point.h"
 #include "mongo/transport/service_executor_task_names.h"
@@ -51,12 +53,13 @@
 #include "mongo/util/concurrency/thread_name.h"
 #include "mongo/util/debug_util.h"
 #include "mongo/util/exit.h"
-#include "mongo/util/log.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/net/socket_exception.h"
 #include "mongo/util/quick_exit.h"
 
 namespace mongo {
 namespace {
+MONGO_FAIL_POINT_DEFINE(doNotSetMoreToCome);
 /**
  * Creates and returns a legacy exhaust message, if exhaust is allowed. The returned message is to
  * be used as the subsequent 'synthetic' exhaust request. Returns an empty message if exhaust is not
@@ -66,10 +69,18 @@ Message makeLegacyExhaustMessage(Message* m, const DbResponse& dbresponse) {
     // OP_QUERY responses are always of type OP_REPLY.
     invariant(dbresponse.response.operation() == opReply);
 
-    if (dbresponse.exhaustNS.empty()) {
+    if (!dbresponse.shouldRunAgainForExhaust) {
         return Message();
     }
 
+    // Legacy find operations via the OP_QUERY/OP_GET_MORE network protocol never provide the next
+    // invocation for exhaust.
+    invariant(!dbresponse.nextInvocation);
+
+    DbMessage dbmsg(*m);
+    invariant(dbmsg.messageShouldHaveNs());
+    const char* ns = dbmsg.getns();
+
     MsgData::View header = dbresponse.response.header();
     QueryResult::View qr = header.view2ptr();
     long long cursorid = qr.getCursorId();
@@ -85,7 +96,7 @@ Message makeLegacyExhaustMessage(Message* m, const DbResponse& dbresponse) {
     b.appendNum(header.getResponseToMsgId());  // in response to
     b.appendNum(static_cast<int>(dbGetMore));  // opCode is OP_GET_MORE
     b.appendNum(static_cast<int>(0));          // Must be ZERO (reserved)
-    b.appendStr(dbresponse.exhaustNS);         // Namespace
+    b.appendStr(StringData(ns));               // Namespace
     b.appendNum(static_cast<int>(0));          // ntoreturn
     b.appendNum(cursorid);                     // cursor id from the OP_REPLY
 
@@ -96,12 +107,9 @@ Message makeLegacyExhaustMessage(Message* m, const DbResponse& dbresponse) {
 
 /**
  * Given a request and its already generated response, checks for exhaust flags. If exhaust is
- * allowed, modifies the given request message to produce the subsequent exhaust message, and
- * modifies the response message to indicate it is part of an exhaust stream. Returns the modified
- * request message for it to be used as the subsequent, 'synthetic' exhaust request. Returns an
- * empty message if exhaust is not allowed.
- *
- * Currently only supports exhaust for 'getMore' commands.
+ * allowed, produces the subsequent request message, and modifies the response message to indicate
+ * it is part of an exhaust stream. Returns the subsequent request message, which is known as a
+ * 'synthetic' exhaust request. Returns an empty message if exhaust is not allowed.
  */
 Message makeExhaustMessage(Message requestMsg, DbResponse* dbresponse) {
     if (requestMsg.operation() == dbQuery) {
@@ -112,36 +120,44 @@ Message makeExhaustMessage(Message requestMsg, DbResponse* dbresponse) {
         return Message();
     }
 
-    // Only support exhaust for 'getMore' commands.
-    auto request = OpMsgRequest::parse(requestMsg);
-    if (request.getCommandName() != "getMore"_sd) {
-        return Message();
-    }
-
-    // A returned cursor id of '0' indicates that the cursor is exhausted and so the exhaust stream
-    // should be terminated. Also make sure the cursor namespace is valid.
-    if (dbresponse->exhaustCursorId == 0 || dbresponse->exhaustNS.empty()) {
+    if (!dbresponse->shouldRunAgainForExhaust) {
         return Message();
     }
 
     const bool checksumPresent = OpMsg::isFlagSet(requestMsg, OpMsg::kChecksumPresent);
+    Message exhaustMessage;
+
+    if (auto nextInvocation = dbresponse->nextInvocation) {
+        // The command provided a new BSONObj for the next invocation.
+        OpMsgBuilder builder;
+        builder.setBody(*nextInvocation);
+        exhaustMessage = builder.finish();
+    } else {
+        // Reuse the previous invocation for the next invocation.
+        OpMsg::removeChecksum(&requestMsg);
+        exhaustMessage = requestMsg;
+    }
+
+    // The id of the response is used as the request id of this 'synthetic' request. Re-checksum
+    // if needed.
+    exhaustMessage.header().setId(dbresponse->response.header().getId());
+    exhaustMessage.header().setResponseToMsgId(dbresponse->response.header().getResponseToMsgId());
+    OpMsg::setFlag(&exhaustMessage, OpMsg::kExhaustSupported);
+    if (checksumPresent) {
+        OpMsg::appendChecksum(&exhaustMessage);
+    }
+
     OpMsg::removeChecksum(&dbresponse->response);
-    // Indicate that the response is part of an exhaust stream. Re-checksum if needed.
-    OpMsg::setFlag(&dbresponse->response, OpMsg::kMoreToCome);
+    // Indicate that the response is part of an exhaust stream (unless the 'doNotSetMoreToCome'
+    // failpoint is set). Re-checksum if needed.
+    if (!MONGO_unlikely(doNotSetMoreToCome.shouldFail())) {
+        OpMsg::setFlag(&dbresponse->response, OpMsg::kMoreToCome);
+    }
     if (checksumPresent) {
         OpMsg::appendChecksum(&dbresponse->response);
     }
 
-    // Return an augmented form of the initial request, which is to be used as the next request to
-    // be processed by the database. The id of the response is used as the request id of this
-    // 'synthetic' request. Re-checksum if needed.
-    OpMsg::removeChecksum(&requestMsg);
-    requestMsg.header().setId(dbresponse->response.header().getId());
-    requestMsg.header().setResponseToMsgId(dbresponse->response.header().getResponseToMsgId());
-    if (checksumPresent) {
-        OpMsg::appendChecksum(&requestMsg);
-    }
-    return requestMsg;
+    return exhaustMessage;
 }
 
 }  // namespace
@@ -364,16 +380,27 @@ void ServiceStateMachine::_sourceCallback(Status status) {
                                       transport::ServiceExecutorTaskName::kSSMProcessMessage);
     } else if (ErrorCodes::isInterruption(status.code()) ||
                ErrorCodes::isNetworkError(status.code())) {
-        LOG(2) << "Session from " << remote
-               << " encountered a network error during SourceMessage: " << status;
+        LOGV2_DEBUG(
+            22986,
+            2,
+            "Session from {remote} encountered a network error during SourceMessage: {error}",
+            "Session from remote encountered a network error during SourceMessage",
+            "remote"_attr = remote,
+            "error"_attr = status);
         _state.store(State::EndSession);
     } else if (status == TransportLayer::TicketSessionClosedStatus) {
         // Our session may have been closed internally.
-        LOG(2) << "Session from " << remote << " was closed internally during SourceMessage";
+        LOGV2_DEBUG(22987,
+                    2,
+                    "Session from {remote} was closed internally during SourceMessage",
+                    "remote"_attr = remote);
         _state.store(State::EndSession);
     } else {
-        log() << "Error receiving request from client: " << status << ". Ending connection from "
-              << remote << " (connection id: " << _session()->id() << ")";
+        LOGV2(22988,
+              "Error receiving request from client. Ending connection from remote",
+              "error"_attr = status,
+              "remote"_attr = remote,
+              "connectionId"_attr = _session()->id());
         _state.store(State::EndSession);
     }
 
@@ -395,8 +422,11 @@ void ServiceStateMachine::_sinkCallback(Status status) {
     // Otherwise, update the current state depending on whether we're in exhaust or not, and call
     // scheduleNext() to unwind the stack and do the next step.
     if (!status.isOK()) {
-        log() << "Error sending response to client: " << status << ". Ending connection from "
-              << _session()->remote() << " (connection id: " << _session()->id() << ")";
+        LOGV2(22989,
+              "Error sending response to client. Ending connection from remote",
+              "error"_attr = status,
+              "remote"_attr = _session()->remote(),
+              "connectionId"_attr = _session()->id());
         _state.store(State::EndSession);
         return _runNextInGuard(std::move(guard));
     } else if (_inExhaust) {
@@ -435,14 +465,20 @@ void ServiceStateMachine::_processMessage(ThreadGuard guard) {
 
     // Pass sourced Message to handler to generate response.
     auto opCtx = Client::getCurrent()->makeOperationContext();
+    if (_inExhaust) {
+        opCtx->markKillOnClientDisconnect();
+    }
 
     // The handleRequest is implemented in a subclass for mongod/mongos and actually all the
     // database work for this request.
     DbResponse dbresponse = _sep->handleRequest(opCtx.get(), _inMessage);
 
-    // opCtx must be destroyed here so that the operation cannot show
-    // up in currentOp results after the response reaches the client
-    opCtx.reset();
+    // opCtx must be killed and delisted here so that the operation cannot show up in currentOp
+    // results after the response reaches the client. The destruction is postponed for later to
+    // mitigate its performance impact on the critical path of execution.
+    _serviceContext->killAndDelistOperation(opCtx.get(), ErrorCodes::OperationIsKilledAndDelisted);
+    invariant(!_killedOpCtx);
+    _killedOpCtx = std::move(opCtx);
 
     // Format our response, if we have one
     Message& toSink = dbresponse.response;
@@ -463,12 +499,11 @@ void ServiceStateMachine::_processMessage(ThreadGuard guard) {
 #endif
         }
 
-        // If the incoming message has the exhaust flag set and is a 'getMore' command, then we
-        // bypass the normal RPC behavior. We will sink the response to the network, but we also
-        // synthesize a new 'getMore' request, as if we sourced a new message from the network. This
-        // new request is sent to the database once again to be processed. This cycle repeats as
-        // long as the associated cursor is not exhausted. Once it is exhausted, we will send a
-        // final response, terminating the exhaust stream.
+        // If the incoming message has the exhaust flag set, then we bypass the normal RPC behavior.
+        // We will sink the response to the network, but we also synthesize a new request, as if we
+        // sourced a new message from the network. This new request is sent to the database once
+        // again to be processed. This cycle repeats as long as the command indicates the exhaust
+        // stream should continue.
         _inMessage = makeExhaustMessage(_inMessage, &dbresponse);
         _inExhaust = !_inMessage.empty();
 
@@ -488,6 +523,7 @@ void ServiceStateMachine::_processMessage(ThreadGuard guard) {
     } else {
         _state.store(State::Source);
         _inMessage.reset();
+        _inExhaust = false;
         return _scheduleNextWithGuard(std::move(guard),
                                       ServiceExecutor::kDeferredTask,
                                       transport::ServiceExecutorTaskName::kSSMSourceMessage);
@@ -508,6 +544,12 @@ void ServiceStateMachine::_runNextInGuard(ThreadGuard guard) {
         _state.store(curState);
     }
 
+    // Destroy the opCtx (already killed) here, to potentially use the delay between clients'
+    // requests to hide the destruction cost.
+    if (MONGO_likely(_killedOpCtx)) {
+        _killedOpCtx.reset();
+    }
+
     // Make sure the current Client got set correctly
     dassert(Client::getCurrent() == _dbClientPtr);
     try {
@@ -527,7 +569,10 @@ void ServiceStateMachine::_runNextInGuard(ThreadGuard guard) {
 
         return;
     } catch (const DBException& e) {
-        log() << "DBException handling request, closing client connection: " << redact(e);
+        LOGV2(22990,
+              "DBException handling request, closing client connection: {error}",
+              "DBException handling request, closing client connection",
+              "error"_attr = redact(e));
     }
     // No need to catch std::exception, as std::terminate will be called when the exception bubbles
     // to the top of the stack
@@ -591,14 +636,16 @@ void ServiceStateMachine::terminateIfTagsDontMatch(transport::Session::TagMask t
     // If terminateIfTagsDontMatch gets called when we still are 'pending' where no tags have been
     // set, then skip the termination check.
     if ((sessionTags & tags) || (sessionTags & transport::Session::kPending)) {
-        log() << "Skip closing connection for connection # " << _session()->id();
+        LOGV2(22991,
+              "Skip closing connection for connection",
+              "connectionId"_attr = _session()->id());
         return;
     }
 
     terminate();
 }
 
-void ServiceStateMachine::setCleanupHook(stdx::function<void()> hook) {
+void ServiceStateMachine::setCleanupHook(std::function<void()> hook) {
     invariant(state() == State::Created);
     _cleanupHook = std::move(hook);
 }
@@ -609,12 +656,45 @@ ServiceStateMachine::State ServiceStateMachine::state() {
 
 void ServiceStateMachine::_terminateAndLogIfError(Status status) {
     if (!status.isOK()) {
-        warning(logger::LogComponent::kExecutor) << "Terminating session due to error: " << status;
+        LOGV2_WARNING_OPTIONS(22993,
+                              {logv2::LogComponent::kExecutor},
+                              "Terminating session due to error: {error}",
+                              "Terminating session due to error",
+                              "error"_attr = status);
         terminate();
     }
 }
 
+void ServiceStateMachine::_cleanupExhaustResources() noexcept try {
+    if (!_inExhaust) {
+        return;
+    }
+    auto request = OpMsgRequest::parse(_inMessage);
+    // Clean up cursor for exhaust getMore request.
+    if (request.getCommandName() == "getMore"_sd) {
+        auto cursorId = request.body["getMore"].Long();
+        auto opCtx = Client::getCurrent()->makeOperationContext();
+        // Fire and forget. This is a best effort attempt to immediately clean up the exhaust
+        // cursor. If the killCursors request fails here for any reasons, it will still be
+        // cleaned up once the cursor times out.
+        _sep->handleRequest(opCtx.get(), makeKillCursorsMessage(cursorId));
+    }
+} catch (const DBException& e) {
+    LOGV2(22992,
+          "Error cleaning up resources for exhaust requests: {error}",
+          "Error cleaning up resources for exhaust requests",
+          "error"_attr = e.toStatus());
+}
+
 void ServiceStateMachine::_cleanupSession(ThreadGuard guard) {
+    // Ensure the delayed destruction of opCtx always happens before doing the cleanup.
+    if (MONGO_likely(_killedOpCtx)) {
+        _killedOpCtx.reset();
+    }
+    invariant(!_killedOpCtx);
+
+    _cleanupExhaustResources();
+
     _state.store(State::Ended);
 
     _inMessage.reset();
