@@ -64,7 +64,7 @@ private:
 
     void _threadRoutine();
 
-    stdx::mutex _mutex;
+    Mutex _mutex = MONGO_MAKE_LATCH("AlarmRunnerBackgroundThread::_mutex");
     stdx::condition_variable _condVar;
     bool _running = false;
     Date_t _nextAlarm = Date_t::max();
