@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -30,16 +29,19 @@
 
 #pragma once
 
+#include <functional>
 #include <map>
 #include <string>
 #include <vector>
 
 #include "mongo/base/status.h"
-#include "mongo/stdx/functional.h"
+#include "mongo/util/duration.h"
 
 namespace mongo {
 namespace optionenvironment {
 
+constexpr Seconds kDefaultConfigExpandTimeout{30};
+
 class Environment;
 class OptionSection;
 class Value;
@@ -93,7 +95,7 @@ public:
      *  true - unknown config options will generate an error during parsing.
      *  false - unknow config options will be ignored during parsing.
      */
-    static stdx::function<bool()> useStrict;
+    static std::function<bool()> useStrict;
 
     OptionsParser() {}
     virtual ~OptionsParser() {}
@@ -122,6 +124,16 @@ public:
                          const std::map<std::string, std::string>& env,
                          Environment*);
 
+    /**
+     * Flags controlling whether or not __rest and/or __exec directives in a
+     * config file should be expanded via HttpClient/shellExec.
+     */
+    struct ConfigExpand {
+        bool rest = false;
+        bool exec = false;
+        Seconds timeout = kDefaultConfigExpandTimeout;
+    };
+
 private:
     /** Handles parsing of the command line and adds the results to the given Environment */
     Status parseCommandLine(const OptionSection&,
@@ -134,14 +146,19 @@ private:
 
     /** Handles parsing of either YAML or INI config and adds the results to the given Environment
      */
-    Status parseConfigFile(const OptionSection&, const std::string& argv, Environment*);
+    Status parseConfigFile(const OptionSection&,
+                           const std::string& argv,
+                           Environment*,
+                           const ConfigExpand& configExpand);
 
     /** Gets defaults from the OptionSection and adds them to the given Environment */
     Status addDefaultValues(const OptionSection&, Environment*);
 
     /** Reads the given config file into the output string.  This function is virtual for
      *  testing purposes only. */
-    virtual Status readConfigFile(const std::string& filename, std::string*);
+    virtual Status readConfigFile(const std::string& filename,
+                                  std::string*,
+                                  ConfigExpand configExpand);
 };
 
 }  // namespace optionenvironment
