@@ -39,12 +39,15 @@
 #include "mongo/bson/bsonelement.h"
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
+#include "mongo/logv2/log_attr.h"
 
 namespace mongo {
 
 namespace repl {
+class CollectionInfo;
 class OplogEntryBase;
 class DurableReplOperation;
+class InitialSyncIdDocument;
 }  // namespace repl
 
 namespace idl {
@@ -64,6 +67,7 @@ class UUID {
     friend class ConfigsvrShardCollectionResponse;
     friend class ConfigsvrCommitShardCollection;
     friend class ShardsvrShardCollectionResponse;
+    friend class ShardsvrRenameCollection;
     friend class DatabaseVersion;
     friend class DbCheckOplogCollection;
     friend class EncryptionPlaceholder;
@@ -74,10 +78,16 @@ class UUID {
     friend class LogicalSessionToClient;
     friend class LogicalSessionIdToClient;
     friend class LogicalSessionFromClient;
+    friend class MigrationCoordinatorDocument;
+    friend class MigrationDestinationManager;
+    friend class RangeDeletionTask;
     friend class ResolvedKeyId;
+    friend class repl::CollectionInfo;
     friend class repl::OplogEntryBase;
     friend class repl::DurableReplOperation;
+    friend class repl::InitialSyncIdDocument;
     friend class ResumeTokenInternal;
+    friend class ShardCollectionTypeBase;
     friend class VoteCommitIndexBuild;
 
 public:
@@ -194,6 +204,10 @@ public:
         }
     };
 
+    friend auto logAttrs(const UUID& uuid) {
+        return "uuid"_attr = uuid;
+    }
+
 private:
     UUID(const UUIDStorage& uuid) : _uuid(uuid) {}
 
